Package: medicalcoder
Title: A Unified and Longitudinally Aware Framework for ICD-Based
        Comorbidity Assessment
Version: 0.8.1
Authors@R: c(
    person(given = "Peter",   family = "DeWitt",    email = "peter.dewitt@cuanschutz.edu",      role = c("aut", "cre", "cov"), comment = c(ORCID = "0000-0002-6391-0795")),
    person(given = "Tell",    family = "Bennett",   email = "tell.bennett@cuanschutz.edu",      role = c("ctb"),               comment = c(ORCID = "0000-0003-1483-4236")),
    person(given = "Seth",    family = "Russell",   email = "seth.russell@cuanschutz.edu",      role = c("ctb"),               comment = c(ORCID = "0000-0002-2436-1367")),
    person(given = "Meg",     family = "Rebull",    email = "meg.rebull@cuanschutz.edu",        role = c("ctb"),               comment = c(ORCID = "0000-0003-0334-4223")),
    person(given = "Vincent", family = "Rubinetti", email = "vincent.rubinetti@cuanschutz.edu", role = c("cov"),               comment = c(ORCID = "0000-0002-4655-3773"))
    )
Description: Provides comorbidity classification algorithms such as the
  Pediatric Complex Chronic Conditions (PCCC), Charlson, and Elixhauser indices,
  supports longitudinal comorbidity flagging across encounters, and includes
  utilities for working with medical coding schemas such as the International
  Classification of Diseases (ICD).
Depends: R (>= 3.5.0)
License: BSD_3_clause + file LICENSE
Language: en-US
Encoding: UTF-8
URL: http://www.peteredewitt.com/medicalcoder/,
        https://github.com/dewittpe/medicalcoder/
BugReports: https://github.com/dewittpe/medicalcoder/issues
LazyData: true
Suggests: data.table, dplyr, kableExtra, knitr, R.utils, rmarkdown
VignetteBuilder: knitr
Config/Needs/website: pkgdown, comorbidity, icdcomorbid,
        multimorbidity, dplyr, odbc, DBI, RSQLite, pccc
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-05 01:25:31 UTC; dewittp
Author: Peter DeWitt [aut, cre, cov] (ORCID:
    <https://orcid.org/0000-0002-6391-0795>),
  Tell Bennett [ctb] (ORCID: <https://orcid.org/0000-0003-1483-4236>),
  Seth Russell [ctb] (ORCID: <https://orcid.org/0000-0002-2436-1367>),
  Meg Rebull [ctb] (ORCID: <https://orcid.org/0000-0003-0334-4223>),
  Vincent Rubinetti [cov] (ORCID:
    <https://orcid.org/0000-0002-4655-3773>)
Maintainer: Peter DeWitt <peter.dewitt@cuanschutz.edu>
Repository: CRAN
Date/Publication: 2026-05-05 05:00:03 UTC
Built: R 4.5.2; ; 2026-05-05 05:17:01 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(comorbidities,data.frame)
S3method(print,medicalcoder_comorbidities)
S3method(print,medicalcoder_comorbidities_with_subconditions)
S3method(summary,medicalcoder_comorbidities)
S3method(summary,medicalcoder_comorbidities_with_subconditions)
export(comorbidities)
export(get_charlson_codes)
export(get_charlson_index_scores)
export(get_elixhauser_codes)
export(get_elixhauser_index_scores)
export(get_elixhauser_poa)
export(get_icd_codes)
export(get_pccc_codes)
export(get_pccc_conditions)
export(icd_compact_to_full)
export(is_icd)
export(lookup_icd_codes)

<!-- README.md is generated from README.Rmd. Please edit that file -->



# medicalcoder: A Unified and Longitudinally Aware Framework for ICD-Based Comorbidity Assessment <img src="man/figures/hex.svg" width="200px" align="right" alt = "medicalcoder hex logo"/>

<!-- badges: start -->
[![Project Status: Active – The project has reached a stable, usable state and is being actively developed.](http://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![R-CMD-check](https://github.com/dewittpe/medicalcoder/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/dewittpe/medicalcoder/actions/workflows/R-CMD-check.yaml)
[![Codecov test coverage](https://codecov.io/gh/dewittpe/medicalcoder/graph/badge.svg)](https://app.codecov.io/gh/dewittpe/medicalcoder)
[![CRAN status](https://www.r-pkg.org/badges/version-last-release/medicalcoder)](https://CRAN.R-project.org/package=medicalcoder)
[![CRAN RStudio mirror downloads](http://cranlogs.r-pkg.org/badges/medicalcoder)](https://www.r-pkg.org/pkg/medicalcoder)
[![downloads](https://cranlogs.r-pkg.org/badges/grand-total/medicalcoder)](https://www.r-pkg.org/pkg/medicalcoder)
[![Ask DeepWiki](https://deepwiki.com/badge.svg)](https://deepwiki.com/dewittpe/medicalcoder)
<!-- badges: end -->

medicalcoder is a lightweight, base-R package for working with ICD-9 and
ICD-10 diagnosis and procedure codes. It implements widely used comorbidity
algorithms such as Charlson, Elixhauser, and the Pediatric Complex Chronic
Conditions (PCCC), supports longitudinal comorbidity flagging across encounters,
and provides fast, dependency-free utilities to look up, validate, and
manipulate ICD codes. Designed for portability and reproducibility, the package
avoids external dependencies—requiring only R ≥ 3.5.0—yet offers a rich set of
curated ICD code libraries from the United States' Centers for Medicare and
Medicaid Services (CMS), Centers for Disease Control (CDC), and the World Health
Organization (WHO).

The package balances performance with elegance: its internal caching, efficient
joins, and compact data structures make it practical for large-scale health data
analyses, while its clean design makes it easy to extend or audit. Whether you
need to flag comorbidities, explore ICD hierarchies, or standardize clinical
coding workflows, medicalcoder provides a robust, transparent foundation for
research and applied work in biomedical informatics.

The primary objectives of medicalcoder are:

1. **Fully self-contained**
   - Minimal Dependencies
     - No dependencies other than base R.
     - Requires R version ≥ 3.5.0 due to a [change in data
       serialization](https://cran.r-project.org/src/base/NEWS.3#:~:text=R%20has%20new,to%20version%203.5.0).
       R 3.5.0 was released in April 2018. The initial public release of
       medicalcoder was in 2025.
     - Several packages are listed in the *Suggests* section of the
       `DESCRIPTION` file. These are only needed for building vignettes, other
       documentation, and testing. They are not required to install the package.

   - No Imports
     - medicalcoder does not import any non-base namespaces. This improves
       ease of maintenance and usability.
     - Suggested packages are needed only for development work and building
       vignettes. They are not required for installation or use.
    - That said, there are non-trivial performance gains when passing a
      [`data.table`](https://cran.r-project.org/package=data.table) to the
      `comorbidities()` function. Passing a `tibble` is typically faster than a
      base `data.frame` but slower than a `data.table`.
      (See [benchmarking](https://github.com/dewittpe/medicalcoder/tree/main/benchmarking)).

   - Internal lookup tables
     - All required data are included in the package. If you have the `.tar.gz`
       source file and R ≥ 3.5.0, that is all you need to install and use the
       package.

2. **Efficient implementation of multiple comorbidity algorithms**
   - Implements three general algorithms, each with multiple variants. Details
     are provided below.
   - Supports flagging of subconditions within PCCC.
   - Supports longitudinal flagging of comorbidities. medicalcoder will flag
     comorbidities based on present-on-admission indicators for the
     current encounter and can look back in time for a patient to flag a
     comorbidity if reported in a prior encounter. See examples.

3. **Tools for working with ICD codes**
   - Lookup tables.
   - Ability to work with both full codes (ICD codes with decimal points) and
     compact codes (ICD codes with decimal points omitted).

## Why use medicalcoder

There are several tools for working with ICD codes and comorbidity algorithms.
medicalcoder provides novel features:

- Unified access to multiple comorbidity algorithms through a single function:
  `comorbidities()`.
- Support for both ICD-9 and ICD-10 diagnostic and procedure codes.
- Longitudinal patient-level comorbidity flagging using present-on-admission indicators.
- Fully self-contained package (no external dependencies).

## Install

### From CRAN

``` r
install.packages("medicalcoder")
```

### From GitHub


``` r
remotes::install_github("dewittpe/medicalcoder")
```

### From source

If you have the .tar.gz file for version X.Y.Z, e.g., `medicalcoder_X.Y.Z.tar.gz`
you can install from within R via:


``` r
install.packages(pkgs = "medicalcoder_X.Y.Z.tar.gz", repos = NULL, type = "source")
```

From the command line:

```
R CMD INSTALL medicalcoder_X.Y.Z.tar.gz
```

## Quick Start:

### Example Data

Input data for `comorbidities()` is expected to be in a 'long' format.  Each row
is one code with additional columns for patient and/or encounter id. There are
two example data sets in the package: `mdcr` and `mdcr_longitudinal`.

``` r
data(mdcr, mdcr_longitudinal, package = "medicalcoder")
```

The `mdcr` data set consists of 319 856 rows.
Each row contains one ICD code (`code`). The column `icdv` denotes
each code as ICD-9 or ICD-10, and the `dx` column denotes diagnostic (1) or
procedure (0) code. This data set contains diagnostic and procedure codes for
38 262 patients.

``` r
str(mdcr)
#> 'data.frame':	319856 obs. of  4 variables:
#>  $ patid: int  71412 71412 71412 71412 71412 17087 64424 64424 84361 84361 ...
#>  $ icdv : int  9 9 9 9 9 10 9 9 9 9 ...
#>  $ code : chr  "99931" "75169" "99591" "V5865" ...
#>  $ dx   : int  1 1 1 1 1 1 1 0 1 1 ...
head(mdcr)
#>   patid icdv  code dx
#> 1 71412    9 99931  1
#> 2 71412    9 75169  1
#> 3 71412    9 99591  1
#> 4 71412    9 V5865  1
#> 5 71412    9  V427  1
#> 6 17087   10  V441  1
```

The `mdcr_longitudinal` data set is distinct from the `mdcr` data set.  The major
difference is that this data set contains only diagnostic codes and there are
only 3 patients.  The `date` column
denotes the date of the diagnosis and allows us to look at changes in
comorbidities over time.

``` r
str(mdcr_longitudinal)
#> 'data.frame':	60 obs. of  4 variables:
#>  $ patid: int  9663901 9663901 9663901 9663901 9663901 9663901 9663901 9663901 9663901 9663901 ...
#>  $ date : IDate, format: "2016-03-18" "2016-03-24" ...
#>  $ icdv : int  10 10 10 10 10 10 10 10 10 10 ...
#>  $ code : chr  "Z77.22" "IMO0002" "V87.7XXA" "J95.851" ...
head(mdcr_longitudinal)
#>     patid       date icdv     code
#> 1 9663901 2016-03-18   10   Z77.22
#> 2 9663901 2016-03-24   10  IMO0002
#> 3 9663901 2016-03-24   10 V87.7XXA
#> 4 9663901 2016-03-25   10  J95.851
#> 5 9663901 2016-03-30   10  IMO0002
#> 6 9663901 2016-03-30   10    Z93.0
```

### Comorbidity Algorithms

There are three comorbidity methods, each with several variants, available in
medicalcoder.  All of which are accessible through the `comorbidities()`
method by specifying the `method` argument.

General examples and explanations for when conditions are flagged are in the
vignette

``` r
vignette(topic = "comorbidities", package = "medicalcoder")
```

#### Pediatric Complex Chronic Conditions (PCCC)

  * Version 2.0
    - BMC Pediatrics: [Feudtner et al. (2014)](https://doi.org/10.1186/1471-2431-14-199)
    - Consistent with R package [pccc](https://cran.r-project.org/package=pccc)

  * Version 2.1
    - Updated code base with the same assessment algorithm as version 2.0.

  * Version 3.0
    - JAMA Network Open: [Feinstein et al. (2024)](https://doi.org/10.1001/jamanetworkopen.2024.20579)
    - Children's Hospital Association [Toolkit](https://www.childrenshospitals.org/content/analytics/toolkit/complex-chronic-conditions)

  * Version 3.1
    - Updated code base with same assessment algorithm as version 3.0.

  * All variants can flag conditions and subconditions.


``` r
# PCCC v2.1 and v3.1 example
library(medicalcoder)
cmrbs2 <-
  comorbidities(
    data = mdcr,
    id.vars = "patid", # can use more than one column, e.g., site, patient, encounter
    icd.codes = "code",
    dx.var = "dx",
    poa = 1,  # consider all codes to be present on admission
    method = "pccc_v2.1"
  )
cmrbs3 <-
  comorbidities(
    data = mdcr,
    id.vars = "patid",
    icd.codes = "code",
    dx.var = "dx",
    poa = 1,  # consider all codes to be present on admission
    method = "pccc_v3.1"
  )
str(cmrbs2, max.level = 0)
#> Classes 'medicalcoder_comorbidities' and 'data.frame':	38262 obs. of  16 variables:
#>  - attr(*, "method")= chr "pccc_v2.1"
#>  - attr(*, "id.vars")= chr "patid"
#>  - attr(*, "flag.method")= chr "current"
str(cmrbs3, max.level = 0)
#> Classes 'medicalcoder_comorbidities' and 'data.frame':	38262 obs. of  49 variables:
#>  - attr(*, "method")= chr "pccc_v3.1"
#>  - attr(*, "id.vars")= chr "patid"
#>  - attr(*, "flag.method")= chr "current"
```

A summary of the flagged conditions is generated with a call to `summary()`.

``` r
s2 <- summary(cmrbs2)
str(s2)
```

For `pccc_v2.0` and `pccc_v2.1` the `data.frame` returned by `summary()`
reports the count (unique `id.vars` with the condition) and percentage.


``` r
s3 <- summary(cmrbs3)
str(s3)
```
For `pccc_v3.0` and `pccc_v3.1` the returned `data.frame` reports counts and
percentages for how the condition was flagged based on diagnosis/procedure codes
only, technology dependent codes only, or both.  The `dxpr_or_tech` columns
answer the question "did this patient have the condition".

Further detail, examples, and explanations are in the vignette.

``` r
vignette(topic = "pccc", package = "medicalcoder")
```

#### Charlson Comorbidities
There are four variants of Charlson comorbidities implemented in medicalcoder:

* [Deyo, Cherkin, and Ciol (1992)](https://doi.org/10.1016/0895-4356(92)90133-8)
* [Quan et al. (2005)](https://doi.org/10.1097/01.mlr.0000182534.19832.83)
* [Quan et al. (2011)](https://doi.org/10.1093/aje/kwq433)
* [Glasheen (2019)](https://pubmed.ncbi.nlm.nih.gov/31428236/)


``` r
# Charlson example
cmrbs <-
  comorbidities(
    data = mdcr,
    id.vars = "patid",
    icd.codes = "code",
    dx.var = "dx",
    poa = 1,        # assume all codes are present on admission
    primarydx = 0L, # assume all codes are secondary diagnosis codes
    method = "charlson_quan2005"
  )
```
A summary of the flagged conditions can be generated by calling `summary()`.
Where the summary for the PCCC method was a `data.frame` the return for the
Charlson comorbidities is a list of data frames summarizing the conditions, age
category, and the index score.

``` r
s <- summary(cmrbs)
str(s, max.level = 1)
```

More details and examples are provided in the vignette:

``` r
vignette(topic = "charlson", package = "medicalcoder")
```

#### Elixhauser Comorbidities

* [Elixhauser et al. (1998)](https://doi.org/10.1097/00005650-199801000-00004)
  * `method = elixhauser_elixhauser1988`
* [Quan et al. (2005)](https://doi.org/10.1097/01.mlr.0000182534.19832.83)
  * `method = elixhauser_quan2005`
* AHRQ (2017, 2022, 2023, 2024, 2025, 2026, ICD10)
  * [For ICD-9 codes](https://hcup-us.ahrq.gov/toolssoftware/comorbidity/comorbidity.jsp)
    * `method = elixhauser_ahrq_web`
  * [For ICD-10 codes](https://hcup-us.ahrq.gov/toolssoftware/comorbidityicd10/comorbidity_icd10.jsp)
    * `method = elixhauser_ahrq2022`
    * `method = elixhauser_ahrq2023`
    * `method = elixhauser_ahrq2024`
    * `method = elixhauser_ahrq2025`
    * `method = elixhauser_ahrq2026`
    * `method = elixhauser_ahrq_icd10`


``` r
# Elixhauser example
cmrbs <-
  comorbidities(
    data = mdcr,
    id.vars = "patid",
    icd.codes = "code",
    dx.var = "dx",
    poa = 1,
    primarydx = 0L,
    method = "elixhauser_ahrq_icd10"
  )
```
The summary for the results from `method = elixhauser_ahrq_icd10` is similar to those for
Charlson. A `data.frame` with the counts and percentages of distinct
`data[id.vars]` with the noted condition, and a summary of the index scores.

``` r
s <- summary(cmrbs)
str(s, max.level = 1)
```

More details and examples are provided in the vignette:

``` r
vignette(topic = "elixhauser", package = "medicalcoder")
```

### ICD

The package contains internal data sets with references for ICD-9 and ICD-10 US
based diagnostic and procedure codes. These codes are supplemented with
additional codes from the World Health Organization.

You can get a table of ICD codes via `get_icd_codes()`.

``` r
str(medicalcoder::get_icd_codes())
#> 'data.frame':	249819 obs. of  9 variables:
#>  $ icdv            : int  9 9 9 9 9 9 9 9 9 9 ...
#>  $ dx              : int  0 0 0 0 0 0 0 0 0 0 ...
#>  $ full_code       : chr  "00" "00" "00.0" "00.0" ...
#>  $ code            : chr  "00" "00" "000" "000" ...
#>  $ src             : chr  "cms" "cdc" "cms" "cdc" ...
#>  $ known_start     : int  2006 2003 2006 2003 2006 2003 2006 2003 2006 2003 ...
#>  $ known_end       : int  2015 2012 2015 2012 2015 2012 2015 2012 2015 2012 ...
#>  $ assignable_start: int  NA NA NA NA 2006 2003 2006 2003 2006 2003 ...
#>  $ assignable_end  : int  NA NA NA NA 2015 2012 2015 2012 2015 2012 ...
```

The columns are:

* `icdv`: integer value 9 or 10; for ICD-9 or ICD-10

* `dx`:   integer 0 or 1; 0 = procedure code, 1 = diagnostic code

* `full_code`: character string for the ICD code with any appropriate decimal point.

* `code`: character string for the compact ICD code, that is, the ICD code
  without any decimal point, e.g., the full code C00.1 has the compact code form
  C001.

* `src`: character string denoting the source of the ICD code information.
    * `cms`: The ICD-9-CM, ICD-9-PCS, ICD-10-CM, or ICD-10-PCS codes curated
      by the Centers for Medicare and Medicaid Services (CMS).
    * `cdc`: CDC mortality coding.
    * `who`: World Health Organization.

* `known_start`: The earliest (fiscal) year when source data for the code was
  available in the source code for medicalcoder.  Codes from CMS are for the
  United States fiscal year.  Codes from CDC and WHO are calendar year.  The
  United States fiscal year starts October 1 and concludes September 30.  For
  example, fiscal year 2013 started October 1 2012 and concluded September 30 2013.

  To reemphasize that the year is for the data within medicalcoder.  For
  ICD-9-CM, the codes went into effect for fiscal year 1980.  The source code
  only has documented source files for the codes dating back to
  1997.

* `known_end`: The latest (fiscal) year when the code was part of the ICD
  system and/or known within the medicalcoder lookup tables.

* Assignable codes.  Some codes are header codes, e.g., ICD-10-CM three-digit
  code Z94 is a header code because the four-digit codes Z94.0, Z94.1, Z94.2,
  Z94.3, Z94.4, Z94.5, Z94.6, Z94.7, Z94.8, and Z94.9 exist. All but Z94.8 are
  assignable codes because no five-digit codes with the same initial four-digits
  exist.  Z94.8 is a header code because the five-digit codes Z94.81, Z94.82,
  Z94.83, Z94.84, and Z94.89 exist.
    * `assignable_start`: Earliest (fiscal) year when the code was assignable.
    * `assignable_end`: Latest (fiscal) year when the code was assignable.


``` r
subset(
  x = lookup_icd_codes("^Z94", regex = TRUE, full.codes = TRUE, compact.codes = FALSE),
  subset = src == "cms",
  select = c("full_code", "known_start", "known_end", "assignable_start", "assignable_end")
)
#>    full_code known_start known_end assignable_start assignable_end
#> 1        Z94        2014      2026               NA             NA
#> 5      Z94.0        2014      2026             2014           2026
#> 9      Z94.1        2014      2026             2014           2026
#> 14     Z94.2        2014      2026             2014           2026
#> 17     Z94.3        2014      2026             2014           2026
#> 22     Z94.4        2014      2026             2014           2026
#> 25     Z94.5        2014      2026             2014           2026
#> 29     Z94.6        2014      2026             2014           2026
#> 33     Z94.7        2014      2026             2014           2026
#> 38     Z94.8        2014      2026               NA             NA
#> 41    Z94.81        2014      2026             2014           2026
#> 42    Z94.82        2014      2026             2014           2026
#> 43    Z94.83        2014      2026             2014           2026
#> 44    Z94.84        2014      2026             2014           2026
#> 45    Z94.89        2014      2026             2014           2026
#> 46     Z94.9        2014      2026             2014           2026
```

Additionally, the `get_icd_codes()` method can provide descriptions and the ICD
hierarchy by using the `with.descriptions` and/or `with.hierarchy` arguments.

Functions `lookup_icd_codes()`, `is_icd()`, and `icd_compact_to_full()` are also
provided for working with ICD codes.

More details and examples are in the vignette:

``` r
vignette(topic = "icd", package = "medicalcoder")
```

## Benchmarking

The major factors impacting the expected computation time for applying a
comorbidity algorithm to a data set are:

1. Data size: number of subjects/encounters.
2. Data storage class: medicalcoder has been built such that no imports of
   other namespaces is required. That said, when a `data.table` is passed to
   `comorbidities()` and the `data.table` namespace is available, then S3
   dispatch for `merge` is used, along with some other methods, to reduce memory
   use and reduce computation time. When a `tibble` is passed and the tidyverse
   namespaces are available, the tibble-aware paths improve performance over a
   base `data.frame`, but `data.table` remains fastest.
3. `flag.method`: "current" will take less time than the "cumulative" method.

Details on the benchmarking method, summary graphics, and tables,  can be found
on the medicalcoder GitHub
[benchmarking](https://github.com/dewittpe/medicalcoder/tree/main/benchmarking)
directory.

## Testing

Along with the GitHub actions and testing on current versions of R, the
[testing](https://github.com/dewittpe/medicalcoder/tree/main/testing)
directory in the medicalcoder GitHub repo reports the `R CMD check` results for
all R versions from 3.5.0 to latest.  Several with, and without Suggests.

## ----label = "setup", include = FALSE-----------------------------------------
# IMPORTANT SYNTAX NOTE:
#
# DO NOT USE the pipeOp `|>`
#
# While convenient, that is a R 4.1.0 feature at a minimum. Notable improvements
# to the pipeOp come in 4.2.0 and 4.2.1.  To keep this package dependent on R >=
# 3.5.0 do not use the pipeOp.

library(kableExtra)
options(qwraps2_markup = "markdown")
options(knitr.kable.NA = '')
knitr::opts_chunk$set(collapse = TRUE, fig.align = "center")

## ----label = 'medicalcoder-namespace'-----------------------------------------
library(medicalcoder)

## ----label = "tbl-syntactically-valid-conditions", echo = FALSE, results = "asis"----
CNDS <- subset(get_pccc_conditions(), select = c("condition", "condition_label"))
data.table::setDT(CNDS)
data.table::setkey(CNDS, condition)
CNDS <- unique(CNDS)

tab <-
  kbl(
    CNDS,
    caption = "Syntactically valid names for complex chronic conditions",
    row.names = TRUE
  )
tab <- kable_styling(tab, bootstrap_options = c("striped"), full_width = FALSE, font_size = 10)
tab

## ----label = "get-pccc-codes"-------------------------------------------------
pccc_codes <- get_pccc_codes()
str(pccc_codes)

## ----label = "define-pat1"----------------------------------------------------
pat1 <-
  data.frame(
    dx = c(1, 1, 1, 1, 0, 0),
    icdv = 9L,
    code = c("34590", "78065", "3432", "78065", "9929", "8606")
  )

## ----label = "inner-join-pat1-pccc-codes"-------------------------------------
merge(x = pccc_codes, y = pat1, all = FALSE, by = c("icdv", "dx", "code"))

## ----label = "pat1-pccc-v2"---------------------------------------------------
pat1_pccc_v2.0 <-
  comorbidities(
    data = pat1,
    icd.codes = "code",
    dx.var = "dx",
    icdv = 9,
    method = "pccc_v2.0",
    flag.method = "current", # default
    poa = 1                  # default for flag.method = 'current'
  )

pat1_pccc_v2.1 <-
  comorbidities(
    data = pat1,
    icd.codes = "code",
    dx.var = "dx",
    icdv = 9,
    method = "pccc_v2.1",
    flag.method = "current",
    poa = 1
  )

all.equal(pat1_pccc_v2.0, pat1_pccc_v2.1, check.attributes = FALSE)
pat1_pccc_v2.1

## ----label = "pat1-pccc-v3"---------------------------------------------------
pat1_pccc_v3.0 <-
  comorbidities(
    data = pat1,
    icd.codes = "code",
    dx.var = "dx",
    icdv = 9,
    method = "pccc_v3.0",
    flag.method = 'current',
    poa = 1
  )

pat1_pccc_v3.1 <-
  comorbidities(
    data = pat1,
    icd.codes = "code",
    dx.var = "dx",
    icdv = 9,
    method = "pccc_v3.1",
    flag.method = 'current',
    poa = 1
  )

all.equal(pat1_pccc_v3.0, pat1_pccc_v3.1, check.attributes = FALSE)

# retain the needed columns, there are four columns for each condition in v3
pat1_pccc_v3.0[, grep("^(cmrb_flag|num_cmrb|neuromus|metabolic|tech_dep_flag)", names(pat1_pccc_v3.0))]

## ----echo = FALSE, results = "asis"-------------------------------------------
tab <- data.table::fread(colClass = character(), text = "
cmrb_flag | num_cmrb | <condition>_dxpr_or_tech | <condition>_dxpr_only | <condition>_tech_only | <condition>_dxpr_and_tech | <other condition(s)>_dxpr_or_tech
0         | 0        | 0                        | 0                     | 0                     | 0                         | 0
1         | 1        | 0                        | 0                     | 0                     | 0                         | 1
1         | 1        | 1                        | 1                     | 0                     | 0                         | 0
1         | 1        | 1                        | 1                     | 1                     | 1                         | 0
1         | >1       | 0                        | 0                     | 0                     | 0                         | 1
1         | >1       | 1                        | 0                     | 1                     | 0                         | 1
1         | >1       | 1                        | 1                     | 0                     | 0                         | 1
1         | >1       | 1                        | 1                     | 1                     | 1                         | 1                                  ")

tab <-
  kbl(
    x = tab,
    align = rep("c", ncol(tab)),
    col.names = c("cmrb flag", "num cmrb", "_dxpr_or_tech", "_dxpr_only", "_tech_only", "_dxpr_and_tech", "_dxpr_or_tech")
  )
tab <- kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab <- add_header_above(tab, c("", "", "<condition>" = 4, "<other condition(s)>" = 1))
tab

## ----label = 'define-pat2'----------------------------------------------------
pat2 <- subset(pat1, code != "3432")

## ----label = "pat2-pccc-v2"---------------------------------------------------
pat2_pccc_v2.1 <-
  comorbidities(
    data = pat2,
    icd.codes = "code",
    dx.var = "dx",
    icdv = 9,
    method = "pccc_v2.1",
    flag.method = 'current',
    poa = 1
  )
Filter(f = function(x) x > 0, pat2_pccc_v2.1)

## ----label = "pat2-pccc-v3"---------------------------------------------------
pat2_pccc_v3.1 <-
  comorbidities(
    data = pat2,
    icd.codes = "code",
    dx.var = "dx",
    icdv = 9,
    method = "pccc_v3.1",
    flag.method = 'current',
    poa = 1
  )
Filter(f = function(x) x > 0, pat2_pccc_v3.1)

## ----label = "mdcr-data"------------------------------------------------------
head(mdcr)
str(mdcr)

## ----label = "mdcr-results-01"------------------------------------------------
mdcr_results_v2.1_01 <-
  comorbidities(
    data = mdcr,
    icd.codes = "code",
    id.vars = "patid",
    poa = 1,
    flag.method = 'current',
    method = "pccc_v2.1"
  )

mdcr_results_v3.1_01 <-
  comorbidities(
    data = mdcr,
    icd.codes = "code",
    id.vars = "patid",
    poa = 1,
    flag.method = 'current',
    method = "pccc_v3.1"
  )

## ----label = "comorbidities-summary-table-str"--------------------------------
str(summary(mdcr_results_v2.1_01))
str(summary(mdcr_results_v3.1_01))

## ----label = "mdcr-results-01-summary"----------------------------------------
x <-
  merge(
    summary(mdcr_results_v2.1_01),
    summary(mdcr_results_v3.1_01),
    all = TRUE,
    by = c("condition", "label"),
    sort = FALSE
  )
x[["condition"]] <- NULL

## ----label = "mdcr-results-01-summary-kable", echo = FALSE, results = "asis"----
tab <-
  kableExtra::kbl(
    x,
    digits = 1,
    col.names = c("", rep(c("count", "%"), times = 5)),
    caption = "Summary Table for `mdcr_results_v2.1_01` and `mdcr_results_v3.1_01`."
  )
tab <-
  kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab <-
  kableExtra::pack_rows(tab, group_label = c("Conditions"), start_row = 1, end_row = 11)
tab <-
  kableExtra::pack_rows(tab, group_label = c("Flags"), start_row = 12, end_row = 13)
tab <-
  kableExtra::pack_rows(tab, group_label = c("Total Conditions"), start_row = 14, end_row = 24)
tab <-
  kableExtra::add_header_above(tab, c("", "v2.1" = 2, "dxpr or tech" = 2, "dxpr only" = 2, "tech only" = 2, "dxpr and tech" = 2))
tab <-
  kableExtra::add_header_above(tab, c("", "", "", "v3.1" = 8))
tab

## ----label = "note-dx-pr-in-mdcr"---------------------------------------------
pccc_codes[pccc_codes$code == "3321", ]
table(mdcr[mdcr$code == "3321", "dx"])

## ----label = "mdcr-results-02"------------------------------------------------
mdcr_results_v2.1_02 <-
  comorbidities(
    data = mdcr,
    id.vars = "patid",
    icd.codes = "code",
    dx.var = "dx",
    flag.method = 'current',
    poa = 1,
    method = "pccc_v2.1"
  )

mdcr_results_v3.1_02 <-
  comorbidities(
    data = mdcr,
    id.vars = "patid",
    icd.codes = "code",
    dx.var = "dx",
    flag.method = 'current',
    poa = 1,
    method = "pccc_v3.1"
  )

## -----------------------------------------------------------------------------
# verify that the cmrb_flag and number of conditions is the same or less after
# accounting for the diagnostic/procedure flag in the comorbidities call
stopifnot(all(mdcr_results_v2.1_02$cmrb_flag <= mdcr_results_v2.1_01$cmrb_flag))
stopifnot(all(mdcr_results_v2.1_02$num_cmrb  <= mdcr_results_v2.1_01$num_cmrb))

sum(mdcr_results_v2.1_02$cmrb_flag != mdcr_results_v2.1_01$cmrb_flag)
sum(mdcr_results_v2.1_02$num_cmrb  != mdcr_results_v2.1_01$num_cmrb)

stopifnot(all(mdcr_results_v3.1_02$cmrb_flag <= mdcr_results_v3.1_01$cmrb_flag))
stopifnot(all(mdcr_results_v3.1_02$num_cmrb  <= mdcr_results_v3.1_01$num_cmrb))

sum(mdcr_results_v3.1_02$cmrb_flag != mdcr_results_v3.1_01$cmrb_flag)
sum(mdcr_results_v3.1_02$num_cmrb  != mdcr_results_v3.1_01$num_cmrb)

## ----include = FALSE----------------------------------------------------------
# this chunk is not included, just test that the results for patient 87420 has
# not changed.
#
# NOTE: vignettes are built in one R session.  As a result, in a prior version
# of this package, mdcr was modified in the icd.Rmd script. Those changes
# persisted into this vignette and resulted in an error.  The fix was to use an
# object in the icd.Rmd vignette called mdcr_copy.
subset(mdcr, patid %in% mdcr[mdcr$code == "5641" & mdcr$dx == 1, "patid"])
pat87420 <- subset(mdcr, patid == 87420)
stopifnot(
  isTRUE(
    all.equal(
      pat87420,
      structure(list(patid = c(87420L, 87420L), icdv = c(9L, 9L), code = c("78321", "5641"), dx = c(1L, 1L)), row.names = 4073:4074, class = "data.frame"),
      check.attributes = FALSE
    )
  )
)

## -----------------------------------------------------------------------------
subset(mdcr, patid == "87420")
subset(get_pccc_codes(), code %in% c("78321", "5641"))

## -----------------------------------------------------------------------------
subset(mdcr_results_v2.1_01, patid == "87420", select = c("cmrb_flag", "renal"))
subset(mdcr_results_v2.1_02, patid == "87420", select = c("cmrb_flag", "renal"))

subset(mdcr_results_v3.1_01, patid == "87420", select = c("cmrb_flag", "renal_dxpr_or_tech"))
subset(mdcr_results_v3.1_02, patid == "87420", select = c("cmrb_flag", "renal_dxpr_or_tech"))

subset(get_icd_codes(with.descriptions = TRUE), full_code %in% c("56.41", "564.1"))

## -----------------------------------------------------------------------------
merge(
  x = subset(mdcr, patid == "87420"),
  y = pccc_codes,
  by.x = c("code"),
  by.y = c("code"),
  suffixes = c(".mdcr", ".pccc_codes")
)

## -----------------------------------------------------------------------------
DF <-
  data.frame(
    id = c("full dx", "full pr", "compact dx", "compact pr"),
    code = c("564.1", "56.41", "5641", "5641"),
    dx = c(1, 0, 1, 0)
  )

# ideal: using the dx/pr status and matching on full and compact codes.
comorbidities(
  data = DF,
  id.vars = "id",
  dx.var = "dx",
  icd.codes = "code",
  poa = 1,
  method = "pccc_v3.1"
)[, c("id", "cmrb_flag", "renal_dxpr_or_tech")]

# false positive for the compact dx
comorbidities(
  data = DF,
  id.vars = "id",
  icd.codes = "code",
  poa = 1,
  method = "pccc_v3.1"
)[, c("id", "cmrb_flag", "renal_dxpr_or_tech")]

# false negative for compact pr
comorbidities(
  data = DF,
  id.vars = "id",
  icd.codes = "code",
  poa = 1,
  full.code = TRUE,
  compact.codes = FALSE,
  method = "pccc_v3.1"
)[, c("id", "cmrb_flag", "renal_dxpr_or_tech")]

# false positive for compact dx
comorbidities(
  data = DF,
  id.vars = "id",
  icd.codes = "code",
  poa = 1,
  full.code = FALSE,
  compact.codes = TRUE,
  method = "pccc_v3.1"
)[, c("id", "cmrb_flag", "renal_dxpr_or_tech")]

# false negatives for compact and full pr
comorbidities(
  data = DF,
  id.vars = "id",
  icd.codes = "code",
  dx.var = "dx",
  poa = 1,
  full.code = FALSE,
  compact.codes = TRUE,
  method = "pccc_v3.1"
)[, c("id", "cmrb_flag", "renal_dxpr_or_tech")]

## ----label = "patid95471"-----------------------------------------------------
subset(mdcr, patid == "95471")

# no flag becuse icdv = 9 which treats all input codes as ICD-9
comorbidities(
  data = subset(mdcr, patid == "95471"),
  icd.codes = "code",
  id.vars = 'patid',
  dx.var = "dx",
  icdv = 9L,
  poa = 1,
  method = "pccc_v3.1"
)[, c('patid', 'cmrb_flag')]

# flag because icdv = 10 - same as using `icdv.var = "icdv"`
comorbidities(
  data = subset(mdcr, patid == "95471"),
  icd.codes = "code",
  id.vars = 'patid',
  dx.var = "dx",
  icdv = 10L,
  poa = 1,
  method = "pccc_v3.1"
)[, c('patid', 'cmrb_flag')]

comorbidities(
  data = subset(mdcr, patid == "95471"),
  icd.codes = "code",
  id.vars = 'patid',
  dx.var = "dx",
  icdv.var = "icdv",
  poa = 1,
  method = "pccc_v3.0"
)[, c('patid', 'cmrb_flag')]

## -----------------------------------------------------------------------------
lookup_icd_codes("E030")
data <- data.frame(id = c("Ambiguous compact code", "Full ICD-9 code", "Full ICD-10 code"),
                   code  = c("E030", "E030", "E03.0"))
data

args <-
  list(
    data = data,
    id.vars = "id",
    icd.codes = "code",
    poa = 1,
    method = "pccc_v3.1"
  )

default <-
  do.call(comorbidities, c(args, list(full.codes = TRUE,  compact.codes = TRUE )))
full_only <-
  do.call(comorbidities, c(args, list(full.codes = TRUE,  compact.codes = FALSE)))
compact_only <-
  do.call(comorbidities, c(args, list(full.codes = FALSE, compact.codes = TRUE )))

default[,       c("id", "cmrb_flag")]
full_only[,     c("id", "cmrb_flag")]
compact_only[,  c("id", "cmrb_flag")]

## -----------------------------------------------------------------------------
head(mdcr_longitudinal)

## ----results = 'asis'---------------------------------------------------------
longitudinal_v2_patid <-
  comorbidities(
    data = mdcr_longitudinal,
    icd.codes = "code",
    id.vars = c("patid"),
    icdv.var = "icdv",
    method = "pccc_v2.1",
    flag.method = "current",
    poa = 1
  )
tab <- kableExtra::kbl(longitudinal_v2_patid)
tab <- kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab

## -----------------------------------------------------------------------------
longitudinal_v2_patid_date <-
  comorbidities(data = mdcr_longitudinal,
    icd.codes = "code",
    id.vars = c("patid", "date"),
    icdv.var = "icdv",
    method = "pccc_v2.1",
    flag.method = "current",
    poa = 1
  )

## ----echo = FALSE, results = 'asis'-------------------------------------------
tab <-
  kableExtra::kbl(
    subset(longitudinal_v2_patid_date, patid == "9663901"),
    row.names = FALSE
  )
tab <- kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab

## -----------------------------------------------------------------------------
longitudinal_v2_patid_date_cumulative_poa0 <-
  comorbidities(
    data = mdcr_longitudinal,
    icd.codes = "code",
    id.vars = c("patid", "date"),
    icdv.var = "icdv",
    method = "pccc_v2.1",
    flag.method = "cumulative",
    poa = 0
  )

## ----echo = FALSE, results = 'asis'-------------------------------------------
tab <-
  kableExtra::kbl(
    subset(longitudinal_v2_patid_date_cumulative_poa0, patid == "9663901"),
    row.names = FALSE
  )
tab <- kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab

## -----------------------------------------------------------------------------
longitudinal_v2_patid_date_cumulative_poa1 <-
  comorbidities(
    data = mdcr_longitudinal,
    icd.codes = "code",
    id.vars = c("patid", "date"),
    icdv.var = "icdv",
    method = "pccc_v2.1",
    flag.method = "cumulative",
    poa = 1
  )

## ----echo = FALSE, results = 'asis'-------------------------------------------
tab <-
  kableExtra::kbl(
    subset(longitudinal_v2_patid_date_cumulative_poa1, patid == "9663901"),
    row.names = FALSE
  )
tab <- kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab

## -----------------------------------------------------------------------------
codes <- c("H49.811", "J84.111", "Z96.41")
subset(get_pccc_codes(), full_code %in% codes)

## -----------------------------------------------------------------------------
permutations <-
  data.table::data.table(
    permutation = rep(1:6, each = 7),
    encounter_id = rep(1:7, times = 6),
    code =
      codes[c(NA, 1, NA, 2, NA, 3, NA,
              NA, 1, NA, 3, NA, 2, NA,
              NA, 2, NA, 1, NA, 3, NA,
              NA, 2, NA, 3, NA, 1, NA,
              NA, 3, NA, 1, NA, 2, NA,
              NA, 3, NA, 2, NA, 1, NA)]
  )

permutations[, plabel := paste(na.omit(code), collapse = ", "), by = .(permutation)]
permutations[, plabel := paste0("Permutation ", permutation, ": ", plabel)]
str(permutations, vec.len = 1)

## ----echo = FALSE, results = "asis"-------------------------------------------
cat(paste("*", permutations[, unique(plabel)]), sep = "\n")

## -----------------------------------------------------------------------------
rtn <-
  comorbidities(
    data = permutations,
    icd.codes = "code",
    id.vars = c("permutation", "plabel", "encounter_id"),
    icdv = 10L,
    compact.codes = FALSE,
    method = "pccc_v3.1",
    flag.method = "cumulative",
    poa = 1
  )

## ----label = "setup-rtn-for-discussion", include = FALSE----------------------
rtn_wide <-
  data.table::dcast(
    encounter_id ~ plabel,
    data = rtn,
    value.var = c("metabolic_dxpr_or_tech", "metabolic_dxpr_only",
                  "metabolic_tech_only", "metabolic_dxpr_and_tech",
                  "respiratory_dxpr_or_tech", "respiratory_dxpr_only",
                  "respiratory_tech_only", "respiratory_dxpr_and_tech",
                  "cmrb_flag", "num_cmrb")
  )

## ----label = "define-pkbl", include = FALSE-----------------------------------
pkbl <- function(permutation = 1) {
  stopifnot(length(permutation) == 1L)

  x <- rtn_wide[
        ,
        .SD,
        .SDcols = c("encounter_id",
                    grep(paste0("Permutation ", permutation), names(rtn_wide), value = TRUE))
       ]

  pl <- rtn[["plabel"]][rtn$permutation == permutation][1]

  tab <-
    kableExtra::kbl(
      x,
      col.names = c("encounter_id", rep(c("dxpr or tech", "dxpr only", "tech only", "dxpr and tech"), times = 2), "ccc flag", "num ccc")
    )
  tab <- kableExtra::kable_styling(kable_input = tab, bootstrap_options = c("striped"), font_size = 10)
  tab <- kableExtra::add_header_above(kable_input = tab, header = c("", c("Metabolic" = 4, "Respiratory" = 4), "", ""))
  tab <- kableExtra::add_header_above(kable_input = tab, header = c("", setNames(10, pl)))
  tab
}

## ----echo = FALSE, results = "asis"-------------------------------------------
pkbl(1)

## ----echo = FALSE, results = "asis"-------------------------------------------
pkbl(2)

## ----echo = FALSE, results = "asis"-------------------------------------------
pkbl(3)

## ----echo = FALSE, results = "asis"-------------------------------------------
pkbl(4)

## ----echo = FALSE, results = "asis"-------------------------------------------
pkbl(5)

## ----echo = FALSE, results = "asis"-------------------------------------------
pkbl(6)

## ----"tbl-syntactically-valid-subconditions", echo = FALSE, results = "asis"----
SCNDS <- get_pccc_conditions()
data.table::setDT(SCNDS)
data.table::setkey(SCNDS, condition, subcondition)
SCNDS[, condition := paste(condition, condition_label, sep = ": ")]

tab <-
  kableExtra::kbl(SCNDS[, .(subcondition, subcondition_label)],
      capttion = "Syntactically valid names for subconditions",
      row.names = FALSE
  )
tab <-
  kableExtra::pack_rows(tab, index = table(SCNDS$condition))
tab <-
  kableExtra::kable_styling(tab, bootstrap_options = c('striped'), fixed_thead = TRUE, font_size = 10)
tab <-
  kableExtra::scroll_box(tab, height = "300px")
tab

## -----------------------------------------------------------------------------
without_subconditions <-
  comorbidities(
    data = mdcr,
    id.vars = "patid",
    icd.codes = "code",
    icdv.var = "icdv",
    dx.var = "dx",
    poa = 1,
    method = "pccc_v3.1",
    subconditions = FALSE
  )

with_subconditions <-
  comorbidities(
    data = mdcr,
    id.vars = "patid",
    icd.codes = "code",
    icdv.var = "icdv",
    dx.var = "dx",
    poa = 1,
    method = "pccc_v3.1",
    subconditions = TRUE
  )

## -----------------------------------------------------------------------------
with_subconditions

all.equal(
  with_subconditions$conditions,
  without_subconditions,
  check.attributes = FALSE
)

## -----------------------------------------------------------------------------
str(summary(with_subconditions))

## ----include = FALSE----------------------------------------------------------
args <-
  list(
    data = mdcr,
    id.vars = "patid",
    icd.codes = "code",
    icdv.var = "icdv",
    dx.var = "dx",
    poa = 1,
    subconditions = TRUE
  )
with_subconditions_v2.0 <- do.call(comorbidities, c(args, list(method = "pccc_v2.0")))
with_subconditions_v2.1 <- do.call(comorbidities, c(args, list(method = "pccc_v2.1")))
with_subconditions_v3.0 <- do.call(comorbidities, c(args, list(method = "pccc_v3.0")))
with_subconditions_v3.1 <- do.call(comorbidities, c(args, list(method = "pccc_v3.1")))

## ----include = FALSE----------------------------------------------------------
rslts <-
  merge(
    merge(
      summary(with_subconditions_v2.0),
      summary(with_subconditions_v2.1),
      by = c("condition", "subcondition"),
      suffixes = c("_v2.0", "_v2.1"),
      sort = FALSE
    ),
    merge(
      summary(with_subconditions_v3.0),
      summary(with_subconditions_v3.1),
      by = c("condition", "subcondition"),
      suffixes = c("_v3.0", "_v3.1"),
      sort = FALSE
    ),
    by = c("condition", "subcondition"),
    sort = FALSE
  )

rslts$idx <- 1:nrow(rslts)

rslts <-
  merge(rslts,
        unique(get_pccc_conditions()[c("condition", "condition_label")]),
        all = TRUE,
        by = "condition",
        sort = FALSE)
rslts <-
  merge(rslts,
        unique(get_pccc_conditions()[c("subcondition", "subcondition_label")]),
        all = TRUE,
        by = "subcondition",
        sort = FALSE)
rslts$lab <- rslts$subcondition_label
rslts$lab[is.na(rslts$subcondition)] <- rslts$condition_label[is.na(rslts$subcondition)]
rslts <- rslts[order(rslts$idx), ]

rslts <- split(rslts, f = rslts[["condition"]])

cols_to_keep <-
  c(
    "lab",
    "count_v2.0",
    "percent_of_cohort_v2.0",
    "percent_of_those_with_condition_v2.0",
    "count_v2.1",
    "percent_of_cohort_v2.1",
    "percent_of_those_with_condition_v2.1",
    "count_v3.0",
    "percent_of_cohort_v3.0",
    "percent_of_those_with_condition_v3.0",
    "count_v3.1",
    "percent_of_cohort_v3.1",
    "percent_of_those_with_condition_v3.1"
  )

tabs <- lapply(rslts, function(x) {x[, cols_to_keep]})

tabs <-
  lapply(tabs,
    kableExtra::kbl,
    col.names = c("", rep(c("count", "% of cohort", "% of those with condition"), 4)),
    row.names = FALSE,
    digits = 1
  )
tabs <-
  Map(
    f = function(t,r) {
          x <- kableExtra::column_spec(t, column = 1, bold = is.na(r$subcondition))
          kableExtra::add_indent(x, which(!is.na(r$subcondition)))
        },
    t = tabs,
    r = rslts
  )
tabs <-
  lapply(tabs, kableExtra::kable_styling, bootstrap_options = "striped", font_size = 10)
tabs <-
  lapply(tabs, kableExtra::add_header_above, header = c("", "v2.0" = 3, "v2.1" = 3, "v3.0" = 3, "v3.1" = 3))
tabs

## ----echo = FALSE, results = "asis"-------------------------------------------
tabs[["congeni_genetic"]]

## ----echo = FALSE, results = "asis"-------------------------------------------
tabs[["cvd"]]

## ----echo = FALSE, results = "asis"-------------------------------------------
tabs[["gi"]]

## ----echo = FALSE, results = "asis"-------------------------------------------
tabs[["hemato_immu"]]

## ----echo = FALSE, results = "asis"-------------------------------------------
tabs[["malignancy"]]

## ----echo = FALSE, results = "asis"-------------------------------------------
tabs[["metabolic"]]

## ----echo = FALSE, results = "asis"-------------------------------------------
tabs[["misc"]]

## ----echo = FALSE, results = "asis"-------------------------------------------
tabs[["neonatal"]]

## ----echo = FALSE, results = "asis"-------------------------------------------
tabs[["neuromusc"]]

## ----echo = FALSE, results = "asis"-------------------------------------------
tabs[["renal"]]

## ----echo = FALSE, results = "asis"-------------------------------------------
tabs[["respiratory"]]

## -----------------------------------------------------------------------------
rslts <-
  comorbidities(
    data = permutations,
    icd.codes = "code",
    id.vars = c("permutation", "plabel", "encounter_id"),
    icdv = 10L,
    compact.codes = FALSE,
    method = "pccc_v3.1",
    flag.method = "cumulative",
    poa = 1,
    subconditions = TRUE
  )

## -----------------------------------------------------------------------------
all(rslts$subconditions$respiratory$chronic_respiratory_diseases == 1)
sapply(rslts$subconditions$respiratory[, -(1:3)], max)

# which encounters flag for primary condition respiratory?
cnd <-
  rslts$conditions[
    respiratory_dxpr_or_tech == 1,
    .(cencid = paste(encounter_id, collapse = ", ")),
    by = .(plabel)
  ]


# which encounters flag for the subcondition chronic_respiratory_diseases?
scnd <-
  rslts$subconditions$respiratory[
    ,
    .(sencid = paste(encounter_id, collapse = ", ")),
    by = .(plabel)
  ]

## ----echo = FALSE, results = "asis"-------------------------------------------
tab <-
  kableExtra::kbl(
    merge(cnd, scnd, all = TRUE, by = "plabel"),
    caption = "Encounters flagging for respiratory condition and the chronic respiratory disease subcondition.",
    col.names = c("", "Condition", "Subcondition")
  )
tab <- kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab <- kableExtra::add_header_above(tab, c("", "Encounters" = 2))
tab

## -----------------------------------------------------------------------------
# which encounters flag for primary condition metabolic?
cnd <-
  rslts$conditions[
    metabolic_dxpr_or_tech == 1,
    .(cencid = paste(encounter_id, collapse = ", ")),
    by = .(plabel)
  ]

# which encounters flag for the subconditions?
scnd <-
  data.table::melt(
    rslts$subconditions$metabolic,
    id.vars = c("plabel", "encounter_id"),
    measure.vars = c("device_and_technology_use", "other_metabolic_disorders"),
    variable.factor = FALSE,
    variable.name = "subcondition"
  )
scnd <- scnd[value == 1]
scnd <-
  scnd[
    ,
    .(sencid = paste(encounter_id, collapse = ", ")),
    by = .(plabel, subcondition)
  ]

scnd <-
  data.table::dcast(
    scnd,
    plabel ~ subcondition,
    value.var = "sencid"
  )

## ----echo = FALSE, results = "asis"-------------------------------------------
tab <-
  kableExtra::kbl(
    x = merge(cnd, scnd, all = TRUE, by = "plabel"),
    caption = "Encounters flagging for a metabolic condition and the encounters flagging for subconidtions device and technology use and/or other metabolic disorders.",
    col.names = c("", "Condition", "Device and Technology Use", "Other Metabolic Disorders")
  )
tab <- kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab <-
  kableExtra::add_header_above(tab, c("", "Encounters" = 3))
tab

## ----label = "setup", include = FALSE-----------------------------------------
# IMPORTANT SYNTAX NOTE:
#
# DO NOT USE the pipeOp `|>`
#
# While convenient, that is a R 4.1.0 feature at a minimum. Notable improvements
# to the pipeOp come in 4.2.0 and 4.2.1.  To keep this package dependent on R >=
# 3.5.0 do not use the pipeOp.

library(kableExtra)
options(qwraps2_markup = "markdown")
options(knitr.kable.NA = '')
knitr::opts_chunk$set(collapse = TRUE, fig.align = "center")

## ----label = 'medicalcoder-namespace'-----------------------------------------
library(medicalcoder)
packageVersion("medicalcoder")

## -----------------------------------------------------------------------------
str(get_charlson_codes())
str(get_charlson_index_scores())

## -----------------------------------------------------------------------------
mdcr_results <-
  comorbidities(
    data = mdcr,
    id.vars = "patid",
    icdv.var = "icdv",
    icd.codes = "code",
    dx.var = "dx",
    flag.method = "current",
    poa = 1,
    primarydx = 0,
    method = "charlson_quan2005"
  )

## -----------------------------------------------------------------------------
str(mdcr_results)

## -----------------------------------------------------------------------------
str(summary(mdcr_results))

## ----echo = FALSE, results = "asis"-------------------------------------------
x <- summary(mdcr_results)$conditions[, c("condition_description", "count", "percent")]
tab <-
  kableExtra::kbl(
    x = x,
    format = "html",
    caption = "Counts and percentages of patients in the mdcr example data sets with the @quan2005 comorbidities.",
    col.names = c("", "Count", "Percentage"),
    digits = 3
  )
tab <- kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab <- kableExtra::pack_rows(tab, group_label = "Comorbidity", start_row = 1, end_row = 17)
tab <- kableExtra::pack_rows(tab, group_label = "Total Comorbidities", start_row = 18, end_row = nrow(x))
tab

## ----label = "setup", include = FALSE-----------------------------------------
# IMPORTANT SYNTAX NOTE:
#
# DO NOT USE the pipeOp `|>`
#
# While convenient, that is a R 4.1.0 feature at a minimum. Notable improvements
# to the pipeOp come in 4.2.0 and 4.2.1.  To keep this package dependent on R >=
# 3.5.0 do not use the pipeOp.

library(kableExtra)
knitr::opts_chunk$set(collapse = TRUE, fig.align = "center")
options(qwraps2_markup = "markdown")

## ----label = 'medicalcoder-namespace'-----------------------------------------
library(medicalcoder)
packageVersion("medicalcoder")

## ----label = "valid-methods"--------------------------------------------------
medicalcoder:::comorbidities_methods()

## ----echo = TRUE, eval = FALSE------------------------------------------------
# vignette(topic = "pccc",       package = "medicalcoder")
# vignette(topic = "charlson",   package = "medicalcoder")
# vignette(topic = "elixhauser", package = "medicalcoder")

## ----label = 'test-that-the-args-have-not-changed', include = FALSE-----------
# IF THIS FAILS YOU NEED TO MAKE SURE THE DOCUMENTATION IN THIS VIGNETTE IS
# UPTO DATE
# dput(capture.output(args(comorbidities)))
stopifnot(
  capture.output(args(comorbidities))  ==
    c("function (data, icd.codes, method, id.vars = NULL, icdv.var = NULL, ",
      "    icdv = NULL, dx.var = NULL, dx = NULL, poa.var = NULL, poa = NULL, ",
      "    age.var = NULL, primarydx.var = NULL, primarydx = NULL, flag.method = c(\"current\", ",
      "        \"cumulative\"), full.codes = TRUE, compact.codes = TRUE, ",
      "    subconditions = FALSE) ", "NULL")
)

## ----label = "read-the-manual", eval = FALSE----------------------------------
# help(topic = "comorbidities", package = "medicalcoder")

## ----label = "comorbidities-args"---------------------------------------------
args(comorbidities)

## ----label = "example-data"---------------------------------------------------
head(mdcr)
head(mdcr_longitudinal)

## -----------------------------------------------------------------------------
str(get_elixhauser_poa())

## ----label = "example-setup", include = FALSE---------------------------------
lookup_icd_codes(c("C78.4", "I50.40"))

codes <- c("C78.4", "I50.40")
cols  <- c("icdv", "dx", "code", "full_code", "condition")

subset(
  get_pccc_codes(),
  subset = full_code %in% codes,
  select = c(cols, "pccc_v3.0")
)

subset(
  get_charlson_codes(),
  subset = full_code %in% codes,
  select = c(cols, "charlson_quan2011")
)

subset(
  get_elixhauser_codes(),
  subset = full_code %in% codes & elixhauser_ahrq2025 == 1L,
  select = c(cols, "elixhauser_ahrq2025")
)

record <-
  structure(
    list(
      patid = c("A", "A", "A", "A", "A", "A", "A"),
      encid = c(1L, 2L, 3L, 4L, 5L, 5L, 6L),
      code = c(NA, "C78.4", "I50.40", NA, "C78.4", "I50.40", NA),
      poa = c(NA, 0L, 1L, NA, 1L, 0L, NA)),
    row.names = c(NA, -7L),
    class = "data.frame"
  )

## ----include = FALSE----------------------------------------------------------
data.table::setDT(record)
args <-
  list(data = record,
       icd.codes = "code",
       id.vars = c("patid", "encid"),
       icdv = 10L,
       dx = 1
  )
args_current_poa0    <- c(args, poa = 0L,        flag.method = "current")
args_current_poa1    <- c(args, poa = 1L,        flag.method = "current")
args_current_poav    <- c(args, poa.var = "poa", flag.method = "current")
args_cumulative_poa0 <- c(args, poa = 0L,        flag.method = "cumulative")
args_cumulative_poa1 <- c(args, poa = 1L,        flag.method = "cumulative")
args_cumulative_poav <- c(args, poa.var = "poa", flag.method = "cumulative")

left_cols <-
  cbind(encid = 1L:6L,
        ICD = c("", "C78.4", paste0("I50.40", footnote_marker_symbol(1L)),
                "", paste0("C78.4", footnote_marker_symbol(1L), "; I50.40"), ""))

rtn <-
  rbind(
    do.call(cbind,
            list(
              left_cols,
              do.call(comorbidities, c(args_current_poa0,                 method = "pccc_v3.0"))[,           .(CVD = cvd_dxpr_or_tech, CANCER = malignancy_dxpr_or_tech)],
              do.call(comorbidities, c(args_current_poa0, primarydx = 0L, method = "charlson_quan2011"))[,   .(CVD = chf,              CANCER = mst)],
              do.call(comorbidities, c(args_current_poa0, primarydx = 0L, method = "elixhauser_ahrq2025"))[, .(CVD = HF,               CANCER = CANCER_METS)],
              do.call(comorbidities, c(args_current_poa1,                 method = "pccc_v3.0"))[,           .(CVD = cvd_dxpr_or_tech, CANCER = malignancy_dxpr_or_tech)],
              do.call(comorbidities, c(args_current_poa1, primarydx = 0L, method = "charlson_quan2011"))[,   .(CVD = chf,              CANCER = mst)],
              do.call(comorbidities, c(args_current_poa1, primarydx = 0L, method = "elixhauser_ahrq2025"))[, .(CVD = HF,               CANCER = CANCER_METS)],
              do.call(comorbidities, c(args_current_poav,                 method = "pccc_v3.0"))[,           .(CVD = cvd_dxpr_or_tech, CANCER = malignancy_dxpr_or_tech)],
              do.call(comorbidities, c(args_current_poav, primarydx = 0L, method = "charlson_quan2011"))[,   .(CVD = chf,              CANCER = mst)],
              do.call(comorbidities, c(args_current_poav, primarydx = 0L, method = "elixhauser_ahrq2025"))[, .(CVD = HF,               CANCER = CANCER_METS)]
    ))
  ,
    do.call(cbind,
            list(
              left_cols,
              do.call(comorbidities, c(args_cumulative_poa0,                 method = "pccc_v3.0"))[,           .(CVD = cvd_dxpr_or_tech, CANCER = malignancy_dxpr_or_tech)],
              do.call(comorbidities, c(args_cumulative_poa0, primarydx = 0L, method = "charlson_quan2011"))[,   .(CVD = chf,              CANCER = mst)],
              do.call(comorbidities, c(args_cumulative_poa0, primarydx = 0L, method = "elixhauser_ahrq2025"))[, .(CVD = HF,               CANCER = CANCER_METS)],
              do.call(comorbidities, c(args_cumulative_poa1,                 method = "pccc_v3.0"))[,           .(CVD = cvd_dxpr_or_tech, CANCER = malignancy_dxpr_or_tech)],
              do.call(comorbidities, c(args_cumulative_poa1, primarydx = 0L, method = "charlson_quan2011"))[,   .(CVD = chf,              CANCER = mst)],
              do.call(comorbidities, c(args_cumulative_poa1, primarydx = 0L, method = "elixhauser_ahrq2025"))[, .(CVD = HF,               CANCER = CANCER_METS)],
              do.call(comorbidities, c(args_cumulative_poav,                 method = "pccc_v3.0"))[,           .(CVD = cvd_dxpr_or_tech, CANCER = malignancy_dxpr_or_tech)],
              do.call(comorbidities, c(args_cumulative_poav, primarydx = 0L, method = "charlson_quan2011"))[,   .(CVD = chf,              CANCER = mst)],
              do.call(comorbidities, c(args_cumulative_poav, primarydx = 0L, method = "elixhauser_ahrq2025"))[, .(CVD = HF,               CANCER = CANCER_METS)]
    ))
  )

## ----echo = FALSE, results = "asis"-------------------------------------------
tab <-
  kbl(
    rtn,
    row.names = FALSE,
    escape = FALSE,
    align = rep("c", nrow(rtn)),
    caption = "Indicators for when a comorbidity is flagged based on the algorithm, present on admission (poa), and flag.method. The two ICD codes, C78.4 and I50.40, map to cancer and cardiovascular disease respectively."
  )
tab <-
  footnote(
    tab,
    symbol = c("Present on Admission"),
    general = "C78.4 does not need to be POA to count for Elixhauser. I50.40 does need to be POA to count for Elixhauser."
  )

tab <- kable_styling(tab, bootstrap_options = c("striped", "bordered"), full_width = FALSE, font_size = 8)
#tab <- scroll_box(tab, height = "600px")
tab <- pack_rows(tab, "flag.method = 'current'", 1L, 6L)
tab <- pack_rows(tab, "flag.method = 'cumulative'", 7L, 12L)
tab <- add_header_above(tab, c(" " = 2L, rep(c("PCCC" = 2L, "Charlson" = 2L, "Elixhauser" = 2L), 3L)))
tab <- add_header_above(tab, c(" " = 2L, c("POA = 0" = 6L, "POA = 1" = 6L, "poa.var = 'poa'" = 6L)))
tab

## ----label = "get-codes"------------------------------------------------------
str(get_pccc_codes())
str(get_charlson_codes())
str(get_elixhauser_codes())

## -----------------------------------------------------------------------------
cdmf_eg <-
  merge(x = mdcr,
        y = subset(get_charlson_codes(),
                   condition %in% c("aids", "hiv") &
                   charlson_cdmf2019 == 1),
        by = c("icdv", "dx", "code"))
data.table::setDT(cdmf_eg)

cdmf_eg <-
  data.table::dcast(data = cdmf_eg,
                    patid ~ condition,
                    value.var = "charlson_cdmf2019",
                    fun.aggregate = function(x) {as.integer(sum(x) > 0)})

cdmf_eg[, .N, keyby = .(hiv, aids)]

## -----------------------------------------------------------------------------
cmdf_mdcr <-
  comorbidities(data = mdcr,
                icd.codes = "code",
                id.vars = "patid",
                icdv.var = "icdv",
                dx.var = "dx",
                method = "charlson_cdmf2019",
                flag.method = "current",
                primarydx = 0L,
                poa = 1L)
data.table::setDT(cmdf_mdcr)

cmdf_mdcr[, .N, keyby = .(hiv, aids)]

#' Pediatric Complex Chronic Conditions ICD Codes
#'
#' Retrieve a copy of internal lookup tables for the ICD codes mapping to the
#' Pediatric Complex Chronic Conditions (PCCC) conditions and subconditions by
#' variant.
#'
#' @seealso
#' * [`get_pccc_conditions()`] for a reference of the PCCC conditions and
#'   subconditions.
#' * [`get_icd_codes()`] for the lookup table of all ICD codes.
#' * [`comorbidities()`] for applying comorbidity algorithms to a data set.
#'
#' @return a `data.frame` with the following columns
#' * `icdv`: Integer vector indicating if the code is from ICD-9 or ICD-10.
#' * `dx`: Integer vector.  1 if the code is a diagnostic, (ICD-9-CM,
#'   ICD-10-CM, WHO, CDC Mortality), or 0 if the code is procedural (ICD-9-PCS,
#'   ICD-10-PCS).
#' * `full_code`: Character vector with the ICD code and any relevant decimal point.
#' * `code`: Character vector with the compact ICD code omitting any relevant decimal point.
#' * `condition`: Character vector of the conditions.
#' * `subcondition`: Character vector of the subconditions.
#' * `transplant_flag`: Integer vector indicating if the code is associated with a transplant.
#' * `tech_dep_flag`: Integer vector indicating if the code is associated with technology dependence.
#' * `pccc_<variant>`: Integer vector indicating if the code is part of the
#'    v2.0, v2.1, v3.0, or v3.1 variant.
#'
#' @example examples/get_pccc_codes.R
#'
#' @export
get_pccc_codes <- function() {
  x <- get(x = "pccc_codes", envir = ..mdcr_data_env.., inherits = FALSE)
  unserialize(serialize(x, connection = NULL))
}

################################################################################
#                                 End of File                                  #
################################################################################

library(medicalcoder)
source('utilities.R')
################################################################################
# objective - the pccc_v2.0 in medicalcoder should reproduce the results from
# pccc version 1.0.7 _almost_ identically.
#
# There are a few differences.  The reason for this difference is that the old
# pccc package only mapped to conditions and did not map subconditions.  this
# creates a problem with a few codes that were not documented but in the
# software.  Those will be noted in this script.

# get the ICD codes from the medicalcoder package and add a code_id column
icd_codes <- get_icd_codes()
icd_codes[["code_id"]] <- seq_len(nrow(icd_codes))

# pccc_1.0.7 requires the input to be in a wide format and can only apply logic
# for ICD-9 or ICD-10 in one call.  Split the codes into the four sets ICD-9-CM,
# ICD-9-PCS, ICD-10-CM, and ICD-10-PCS.  Apply pccc::ccc to each and bind the
# results so we know when ICD codes map to which conditions.
#
# The following code generates the output object which is saved in the test
# directory such that the pccc_v1.0.7 package is not needed in the SUGGESTS
# section of the DESCRIPTION file.

### library(pccc)
### stopifnot(packageVersion("pccc") == "1.0.7")
###
### oldpccc <- rbind(
###   pccc::ccc(
###     data = icd_codes[icd_codes$icdv ==  9 & icd_codes$dx == 1, c("code_id", "code")],
###     id = code_id,
###     dx_cols = "code",
###     icdv = 9
###   )
###   ,
###   pccc::ccc(
###     data = icd_codes[icd_codes$icdv ==  9 & icd_codes$dx == 0, c("code_id", "code")],
###     id = code_id,
###     pc_cols = "code",
###     icdv = 9
###   )
###   ,
###   pccc::ccc(
###     data = icd_codes[icd_codes$icdv ==  10 & icd_codes$dx == 1, c("code_id", "code")],
###     id = code_id,
###     dx_cols = "code",
###     icdv = 10
###   )
###   ,
###   pccc::ccc(
###     data = icd_codes[icd_codes$icdv ==  10 & icd_codes$dx == 0, c("code_id", "code")],
###     id = code_id,
###     pc_cols = "code",
###     icdv = 10
###   )
### )
###
### saveRDS(oldpccc, file = "results_pccc_1.0.7.rds", compress = "xz")

oldpccc <- readRDS(file = "results_pccc_1.0.7.rds")

newpccc <-
  medicalcoder::comorbidities(
      data = icd_codes
    , icd.codes = "full_code"
    , id.vars = "code_id"
    , icdv.var = "icdv"
    , icdv = NULL
    , dx.var = "dx"
    , dx = NULL
    , poa = 1
    , poa.var = NULL
    , age.var = NULL
    , primarydx = NULL
    , primarydx.var = NULL
    , flag.method = "current"
    , full.codes = TRUE
    , compact.codes = TRUE
    , method = "pccc_v2.0"
  )

old_vs_mdcr <-
  merge(
    x = oldpccc,
    y = newpccc,
    all = TRUE,
    by = "code_id",
    suffixes = c("_old", "_mdcr")
  )

old_vs_mdcr <-
  merge(x = old_vs_mdcr,
    y = icd_codes[, c("code_id", "icdv", "dx", "full_code")],
    all = TRUE,
    by = "code_id"
  )

stopifnot(!any(is.na(old_vs_mdcr)))
stopifnot(nrow(old_vs_mdcr) == nrow(icd_codes))

# expect no difference in the following conditions:
#   cvd
#   neuromusc
#   respiratory
#   renal
#   gi
#   hemato_immu
#   metabolic
#   congeni_genetic
#   malignancy
#   neonatal
stopifnot(
  with(old_vs_mdcr, all(cvd_old == cvd_mdcr)),
  with(old_vs_mdcr, all(neuromusc_old == neuromusc_mdcr)),
  with(old_vs_mdcr, all(respiratory_old == respiratory_mdcr)),
  with(old_vs_mdcr, all(renal_old == renal_mdcr)),
  with(old_vs_mdcr, all(gi_old == gi_mdcr)),
  with(old_vs_mdcr, all(hemato_immu_old == hemato_immu_mdcr)),
  with(old_vs_mdcr, all(metabolic_old == metabolic_mdcr)),
  with(old_vs_mdcr, all(congeni_genetic_old == congeni_genetic_mdcr)),
  with(old_vs_mdcr, all(malignancy_old == malignancy_mdcr)),
  with(old_vs_mdcr, all(neonatal_old == neonatal_mdcr))
)

# we expect there is no difference in the ccc_flag (old) vs cmrb_flag (new)
stopifnot(
  with(old_vs_mdcr, all(ccc_flag == cmrb_flag))
)

#
# we _do_ expect differences in the tech_dep flag
#
# ICD-9-CM 349.1 - not in the v2 document, but in the software for neuromusc
# only. This raises a problem that the subcondition is not defined.  To allow
# for the methods in medicalcoder to work, a non-missing subcondition is needed.
# device and technology use seems appropriate
#
# ICD-9-CM V56 - same, not in v2 documents, in the software
#
# ICD-10-CM Z49 - same, not in v2 documents, in the software
#
# ICD-9-PCS 86.06 - this is in the document as metabolic (devices) but is only
# listed under metabolic in the software.  Again, because of the use of the
# subconditions in the implementation in medicalcoder the corrected mapping is
# needed in medicalcoder and thus the difference.
#
# ICD-9-CM V45.85 - same as ICD-9-PCS 86.06
#
# ICD-9-CM V53.3 - great, this is a header code used in cvd, the assignable
# codes, V53.31, V53.32, V53.39 are explicitly mapped to tech dep.  The result,
# the V53.3 needs to also have a tech dep flag
# subset(get_icd_codes(), grepl("^V53\\.3", full_code) & icdv == 9)
#
# ICD-9-CM V53.91 "Fitting and adjustment of insulin pump"
# This is listed in the documentation as metabolic (devices) but has been
# implemented in R as metabolic (transplant).  One major issue with this error is
# that there are no codes for metabolic (transplant). It doesn't make sense to
# retain this error so, method=pccc_v2.0 will differ from the R package
# pccc_1.0.7 for this code.
#
# subset(get_icd_codes(with.descriptions = TRUE), grepl("^V53\\.9", full_code) & icdv == 9)
#
# ICD-9-CM V65, and specifically V65.46 (encounter for insulin pump training),
# is not in the documentation but is in the R pccc_v1.0.7 implementation as
# metabolic.  The device flag has been added to this code.
# subset(get_icd_codes(with.descriptions = TRUE), grepl("^V65\\.4", full_code) & icdv == 9)

mismatch_tech_dep <- old_vs_mdcr[old_vs_mdcr$tech_dep != old_vs_mdcr$any_tech_dep, ]

stopifnot(nrow(mismatch_tech_dep) == 32L)

stopifnot(
  all(mismatch_tech_dep$tech_dep == 0L),
  all(mismatch_tech_dep$any_tech_dep == 1L)
)

stopifnot(
  isTRUE(
    identical(
      sort(unique(mismatch_tech_dep$full_code)),
      c("349.1",
        "86.06",
        "V45.85",
        "V53.3",
        "V53.91",
        "V56", "V56.0", "V56.1", "V56.2", "V56.3", "V56.31", "V56.32", "V56.8",
        "V65.46",
        "Z49.01", "Z49.02", "Z49.31", "Z49.32")
    )
  )
)

# we expect there are some differences in the transplant flag
#
# ICD-9-PCS 37.52 -- documented as cvd (device) implemented as cvd and
# transplant.

#subset(get_pccc_codes(), grepl("^37\\.52", full_code))
#subset(get_icd_codes(with.descriptions = TRUE), grepl("^37\\.52", full_code))

# ICD-9-CM V42.0
# documented as renal (transplant)
# missing from the transplant set in the pccc_1.0.7/src/pccc.cpp

#subset(get_pccc_codes(), grepl("^V42\\.0", full_code))
#subset(get_icd_codes(with.descriptions = TRUE), grepl("^V42\\.0", full_code) & icdv == 9)

# ICD-10-CM Z94
# specifically Z94.1, Z94.2, Z94.4, Z94.81, Z94.82, Z94.83, Z94.84
# All documented as subcondition transplant but are missing from the transplant
# set in the pccc_1.0.7/src/pccc.cpp
#
#subset(get_pccc_codes(), grepl("^Z94", full_code))
#subset(get_icd_codes(with.descriptions = TRUE), grepl("^Z94", full_code) & icdv == 10)

#
mismatch_transplant <- old_vs_mdcr[old_vs_mdcr$transplant != old_vs_mdcr$any_transplant, ]
stopifnot(nrow(mismatch_transplant) == 12L)
stopifnot(
  all(mismatch_transplant$transplant == 0L),
  all(mismatch_transplant$any_transplant == 1L)
)

stopifnot(
  isTRUE(
    identical(
      sort(unique(mismatch_transplant$full_code)),
      c("V42.0", "Z94.1", "Z94.2", "Z94.4", "Z94.81", "Z94.82", "Z94.83", "Z94.84")
    )
  )
)


################################################################################
#                                 End of File                                  #
################################################################################

#' Summaries of Comorbidities
#'
#' Build summaries (counts and percentages) for each comorbidity and other
#' summary statistics by method.
#'
#' @param object a `medicalcoder_comorbidities` object generated by calling
#' [comorbidities()]
#' @param ... additional parameters, not currently used
#'
#' @examples
#' pccc_v3.1_results <-
#'   comorbidities(data = mdcr,
#'                 icd.codes = "code",
#'                 id.vars = "patid",
#'                 dx.var = "dx",
#'                 method = "pccc_v3.1",
#'                 flag.method = 'current',
#'                 poa = 1)
#' summary(pccc_v3.1_results)
#'
#' charlson_results <-
#'   comorbidities(data = mdcr,
#'                 icd.codes = "code",
#'                 id.vars = "patid",
#'                 dx.var = "dx",
#'                 method = "charlson_quan2011",
#'                 flag.method = 'current',
#'                 poa = 1)
#' summary(charlson_results)
#'
#' elixhauser_results <-
#'   comorbidities(data = mdcr,
#'                 icd.codes = "code",
#'                 id.vars = "patid",
#'                 dx.var = "dx",
#'                 method = "elixhauser_ahrq2025",
#'                 primarydx = 1,
#'                 flag.method = 'current',
#'                 poa = 1)
#' summary(elixhauser_results)
#'
#' @return either a list or a data `data.frame`
#'
#' @export
summary.medicalcoder_comorbidities <- function(object, ...) {
  if (startsWith(attr(object, "method"), "pccc_v2")) {
    .pccc_v2_summary(object)
  } else if (startsWith(attr(object, "method"), "pccc_v3")) {
    .pccc_v3_summary(object)
  } else if (startsWith(attr(object, "method"), "charlson")) {
    .charlson_summary(object)
  } else if (startsWith(attr(object, "method"), "elixhauser")) {
    .elixhauser_summary(object)
  } else {
    stop(sprintf("No summary method for a medicalcoder_comorbidities object with method %s has been built", attr(object, "method")))
  }
}

#' Summaries of Comorbidities with Subconditions
#'
#' Build summaries (counts and percentages) for each Pediatric Complex Chronic
#' Condition (PCCC) condition and subcondition.
#'
#' @param object a `medicalcoder_comorbidities_with_subconditions` object generated by calling
#' [`comorbidities()`] with `subconditions = TRUE`.  This is currently only
#' applicable to PCCC.
#'
#' @param ... additional parameters, not currently used
#'
#' @seealso [`comorbidities()`],
#' `vignette(topic = "pccc", package = "medicalcoder")`
#'
#' @examples
#' pccc_v3.1_subcondition_results <-
#'   comorbidities(data = mdcr,
#'                 icd.codes = "code",
#'                 id.vars = "patid",
#'                 dx.var = "dx",
#'                 method = "pccc_v3.1",
#'                 flag.method = 'current',
#'                 poa = 1,
#'                 subconditions = TRUE)
#' summary(pccc_v3.1_subcondition_results)
#'
#' @return a `data.frame` with five columns.
#'
#' 1. `condition` the primary condition
#' 2. `subcondition` the subcondition(s) within the `condition`.  There will be
#'    a row where `subcondition` is `NA` which is used to report the `count` and
#'    `percent_of_cohort` for the `condition` overall.
#' 3. `count` the number of rows in `object` with the applicable `condition` and
#'    `subcondition`.
#' 4. `percent_of_cohort`: a numeric value within \[0, 100\] for the percent of
#'     rows in `object` with the flagged `condition` and `subcondition`.
#' 5. `percent_of_those_with_condition`: a numeric value within \[0, 100\] for the
#'    subset of rows in `object` with the primary `condition` and the flagged
#'    `subcondition`.  Will be `NA` for the primary `condition`.
#'
#' @export
summary.medicalcoder_comorbidities_with_subconditions <- function(object, ...) {
  if (attr(object, "flag.method") != "current") {
    warning(sprintf("Logic for pccc_summary_table has been implemented for flag.method = 'current'.  Using this function for flag.method = '%s' may not provide a meaningful summary.", attr(object, "flag.method")))
  }

  N <- nrow(object[["conditions"]])

  conditions <- ..mdcr_internal_pccc_conditions..[c("condition", "condition_label")]
  conditions <- unique(conditions)
  conditions <- conditions[order(conditions[["condition"]]), ]

  cols <- conditions[["condition"]]
  if (startsWith(attr(object, "method"), "pccc_v3")) {
    cols <- paste0(cols, "_dxpr_or_tech")
  }
  counts <- colSums(mdcr_select(object[["conditions"]], cols))
  names(counts) <- conditions[["condition"]]

  scounts <-
    lapply(object[["subconditions"]],
           function(x) {
             cols <- names(x)
             cols <- cols[-which(cols %in% attr(object, "id.vars"))]
             colSums(mdcr_select(x, cols))
           })

  rtn <-
    lapply(conditions[["condition"]],
           function(cnd) {
             x1 <- data.frame(
               condition = cnd,
               subcondition = NA_character_,
               count = counts[cnd],
               percent_of_cohort = if (N > 0) {100 * counts[cnd] / N} else {NA_real_},
               percent_of_those_with_condition = NA_real_,
               stringsAsFactors = FALSE
             )
             x2 <- data.frame(
               condition = cnd,
               subcondition = names(scounts[[cnd]]),
               count = scounts[[cnd]],
               percent_of_cohort = if (N > 0) {100 * scounts[[cnd]] / N} else {NA_real_},
               percent_of_those_with_condition = ifelse(scounts[[cnd]] > 0, 100 * scounts[[cnd]] / counts[cnd], NA_real_),
               stringsAsFactors = FALSE
             )
             rbind(x1, x2)
           }
           )
  rtn <- do.call(rbind, rtn)
  rownames(rtn) <- NULL
  rtn
}

################################################################################
# internal summaries by method, each of these are called by the summary methods
# above

.pccc_v2_summary <- function(object) {
  if (attr(object, "flag.method") != "current") {
    warning(sprintf("Logic for pccc_summary_table has been implemented for flag.method = 'current'.  Using this function for flag.method = '%s' may not provide a meaningful summary.", attr(object, "flag.method")))
  }

  conditions <- ..mdcr_internal_pccc_conditions..[c("condition", "condition_label")]
  conditions <- unique(conditions)
  conditions <- conditions[order(conditions[["condition"]]), ]

  cnds <- mdcr_select(object, c(conditions[["condition"]], "any_tech_dep", "any_transplant", "cmrb_flag"))

  # Track running counts for patients meeting at least N conditions
  # so the summary can report distribution thresholds (>=2, >=3, ...).
  tlts <- lapply(2:11, function(x) { as.integer(object[["num_cmrb"]] >= x)})
  tlts <- do.call(cbind, tlts)
  colnames(tlts) <- paste(">=", 2:11, "conditions")

  p <- 100 * c(colMeans(cnds), colMeans(tlts))
  p <- ifelse(is.nan(p), NA_real_, p)

  rtn <-
    data.frame(
      condition = c(names(cnds), rep("num_cmrb", ncol(tlts))),
      label     = c(conditions[["condition_label"]], "Any Technology Dependence", "Any Transplantation", "Any Condition", colnames(tlts)),
      count     = as.integer(c(colSums(cnds), colSums(tlts))),
      percent   = p,
      stringsAsFactors = FALSE
    )

  rownames(rtn) <- NULL

  rtn
}

.pccc_v3_summary <- function(object) {
  if (attr(object, "flag.method") != "current") {
    warning(sprintf("Logic for pccc_summary_table has been implemented for flag.method = 'current'.  Using this function for flag.method = '%s' may not provide a meaningful summary.", attr(object, "flag.method")))
  }

  conditions <- ..mdcr_internal_pccc_conditions..[c("condition", "condition_label")]
  conditions <- unique(conditions)
  conditions <- conditions[order(conditions[["condition"]]), ]

  # Track running counts for patients meeting at least N conditions
  # so the summary can report distribution thresholds (>=2, >=3, ...).
  tlts <- lapply(2:11, function(x) { as.integer(object[["num_cmrb"]] >= x)})
  tlts <- do.call(cbind, tlts)
  colnames(tlts) <- paste(">=", 2:11, "conditions")

  sets <-
    list(
      dxpr_or_tech  = mdcr_select(object, paste0(conditions[["condition"]], "_dxpr_or_tech")),
      dxpr_only     = mdcr_select(object, paste0(conditions[["condition"]], "_dxpr_only"   )),
      tech_only     = mdcr_select(object, paste0(conditions[["condition"]], "_tech_only"   )),
      dxpr_and_tech = mdcr_select(object, paste0(conditions[["condition"]], "_dxpr_and_tech")),
      flags         = mdcr_select(object, c("any_tech_dep", "any_transplant", "cmrb_flag")),
      totals        = tlts
    )

  counts <- lapply(sets, colSums, na.rm = TRUE)
  N <- nrow(object)
  percents <- lapply(counts,
    function(x) {
      y <- 100 * x / N
      ifelse(is.nan(y), NA_real_, y)
    })

  rtn <-
    data.frame(
      condition = conditions[["condition"]],
      label = conditions[["condition_label"]],
      dxpr_or_tech_count = as.integer(counts[["dxpr_or_tech"]]),
      dxpr_or_tech_percent = percents[["dxpr_or_tech"]],
      dxpr_only_count = as.integer(counts[["dxpr_only"]]),
      dxpr_only_percent = percents[["dxpr_only"]],
      tech_only_count = as.integer(counts[["tech_only"]]),
      tech_only_percent = percents[["tech_only"]],
      dxpr_and_tech_count = as.integer(counts[["dxpr_and_tech"]]),
      dxpr_and_tech_percent = percents[["dxpr_and_tech"]],
      stringsAsFactors = FALSE
    )

  rtn <-
    rbind(rtn,
      data.frame(
        condition = c(names(counts[["flags"]]), rep("num_cmrb", length(counts[["totals"]]))),
        label = c("Any Technology Dependence", "Any Transplantation", "Any Condition", names(counts[["totals"]])),
        dxpr_or_tech_count = as.integer(c(counts[["flags"]], counts[["totals"]])),
        dxpr_or_tech_percent = c(percents[["flags"]], percents[["totals"]]),
        dxpr_only_count = NA_integer_,
        dxpr_only_percent = NA_real_,
        tech_only_count = NA_integer_,
        tech_only_percent = NA_real_,
        dxpr_and_tech_count = NA_integer_,
        dxpr_and_tech_percent = NA_real_,
        stringsAsFactors = FALSE
      )
  )

  rownames(rtn) <- NULL

  rtn
}

.charlson_summary <- function(object) {
  if (attr(object, "flag.method") != "current") {
    warning(sprintf("Logic for charlson summary table has been implemented for flag.method = 'current'.  Using this function for flag.method = '%s' may not provide a meaningful summary.", attr(object, "flag.method")))
  }

  cmrbs <- ..mdcr_internal_charlson_index_scores..[!is.na( ..mdcr_internal_charlson_index_scores..[[attr(object, "method")]]), c("condition_description", "condition")]

  cmrbs[["count"]]   <- colSums(object[cmrbs[["condition"]]])
  cmrbs[["percent"]] <- 100 * colMeans(object[cmrbs[["condition"]]])

  num_cmrbs <-
    lapply(seq_len(max(c(1L, object[["num_cmrb"]]))),
      function(x) {
        y <- object[["num_cmrb"]] >= x
        data.frame(
          condition_description = paste(">=", x),
          condition = NA_character_,
          count = sum(y),
          percent = 100 * mean(y),
          stringsAsFactors = FALSE
        )
      })
  num_cmrbs <- do.call(rbind, num_cmrbs)
  cmrbs <- rbind(cmrbs, num_cmrbs)
  rownames(cmrbs) <- NULL

  # set percent to NA instead of NaN
  cmrbs[["percent"]][is.nan(cmrbs[["percent"]])] <- NA_real_

  index_summary <-
    if (length(object[["cci"]]) > 0L) {
      data.frame(
        min       = min(object[["cci"]]),
        q1        = stats::quantile(object[["cci"]], prob = 0.25),
        median    = stats::median(object[["cci"]]),
        q3        = stats::quantile(object[["cci"]], prob = 0.75),
        max       = max(object[["cci"]]),
        row.names = NULL,
        stringsAsFactors = FALSE
      )
    } else {
      data.frame(
        min       = NA_integer_,
        q1        = NA_real_,
        median    = NA_real_,
        q3        = NA_real_,
        max       = NA_integer_,
        row.names = NULL,
        stringsAsFactors = FALSE
      )
    }

  age_summary <-
    {
      age_count <- table(object[["age_score"]], useNA = "always")
      age_percent <- 100 * prop.table(age_count)
      data.frame(
        age_score = unname(names(age_count)),
        count = unname(as.integer(age_count)),
        percent = unname(as.numeric(age_percent)),
        stringsAsFactors = FALSE
      )
    }

  rownames(age_summary) <- NULL

  # set NA instead of NaN
  age_summary[["percent"]][is.nan(age_summary[["percent"]])] <- NA_real_

  list(
    conditions = cmrbs,
    age_summary = age_summary,
    index_summary = index_summary
  )
}

.elixhauser_summary <- function(object) {
  if (attr(object, "flag.method") != "current") {
    warning(sprintf("Logic for Elixhauser summary has been implemented for flag.method = 'current'.  Using this function for flag.method = '%s' may not provide a meaningful summary.", attr(object, "flag.method")))
  }

  cmrbs <- ..mdcr_internal_elixhauser_index_scores..[!is.na( ..mdcr_internal_elixhauser_index_scores..[[attr(object, "method")]]), "condition", drop = FALSE]
  cmrbs <- unique(cmrbs)

  cmrbs[["count"]]   <- colSums(object[cmrbs[["condition"]]])
  cmrbs[["percent"]] <- 100 * colMeans(object[cmrbs[["condition"]]])

  num_cmrbs <-
    lapply(seq_len(max(c(1L, object[["num_cmrb"]]))),
      function(x) {
        y <- object[["num_cmrb"]] >= x
        data.frame(condition = paste(">=", x), count = sum(y), percent = 100 * mean(y), stringsAsFactors = FALSE)
      })
  num_cmrbs <- do.call(rbind, num_cmrbs)
  cmrbs <- rbind(cmrbs, num_cmrbs)
  rownames(cmrbs) <- NULL
  # set percent to NA instead of NaN
  cmrbs[["percent"]][is.nan(cmrbs[["percent"]])] <- NA_real_

  index_summary <-
    if (length(object[["readmission_index"]]) > 0L) {
      data.frame(
        index  = c("readmission", "mortality"),
        min    = c(min(object[["readmission_index"]]),
          min(object[["mortality_index"]])),
        q1     = c(stats::quantile(object[["readmission_index"]], prob = 0.25),
          stats::quantile(object[["mortality_index"]], prob = 0.25)),
        median = c(stats::median(object[["readmission_index"]]),
          stats::median(object[["mortality_index"]])),
        q3     = c(stats::quantile(object[["readmission_index"]], prob = 0.75),
          stats::quantile(object[["mortality_index"]], prob = 0.75)),
        max    = c(max(object[["readmission_index"]]),
          max(object[["mortality_index"]])),
        row.names = NULL,
        stringsAsFactors = FALSE
      )
    } else {
      data.frame(
        index  = c("readmission", "mortality"),
        min    = c(NA_integer_, NA_integer_),
        q1     = c(NA_real_, NA_real_),
        median = c(NA_real_, NA_real_),
        q3     = c(NA_real_, NA_real_),
        max    = c(NA_integer_, NA_integer_),
        row.names = NULL,
        stringsAsFactors = FALSE
      )
    }

  list(
    conditions = cmrbs,
    index_summary = index_summary
  )
}

################################################################################
#                                 End of File                                  #
################################################################################

source('utilities.R')
# test the construction and results of the icd to pccc mappings.  the code to
# build the data sets is in
# inst/mappings/icd-codes-and-mappings.R
#
# ANY CHANGES IN THE CODES AND THESE TESTS MAY IMPACT THE WRITE UP AND THESE TESTS
if (!requireNamespace("data.table", quietly = TRUE)) {
  message("SKIP: data.table not available; skipping test-pccc-deltas.R")
  quit(save = "no", status = 0, runLast = FALSE)
}

#
library(medicalcoder)
library(data.table)
setDTthreads(threads = 1L) # to prevent CPU time exceeding elapsed time on CRAN

source(
  system.file("mappings", "icd-codes-and-mappings.R", package = "medicalcoder"),
  verbose = interactive()
)

stopifnot("pccc_deltas" %in% ls())
stopifnot("pccc_pers" %in% ls())

# Expected counts in pccc_pers
stopifnot(
  pccc_pers[per ==  1, N == 198521],
  pccc_pers[per ==  2, N ==      3],
  pccc_pers[per ==  3, N ==      0],
  pccc_pers[per ==  4, N ==      0],
  pccc_pers[per ==  5, N ==      7],
  pccc_pers[per ==  6, N ==     37],
  pccc_pers[per ==  7, N ==    962],
  pccc_pers[per ==  8, N ==      0],
  pccc_pers[per ==  9, N ==     48],
  pccc_pers[per == 10, N ==      0],
  pccc_pers[per == 11, N ==      0],
  pccc_pers[per == 12, N ==      0],
  pccc_pers[per == 13, N ==      0],
  pccc_pers[per == 14, N ==      0],
  pccc_pers[per == 15, N ==      0],
  pccc_pers[per == 16, N ==     49],
  pccc_pers[per == 17, N ==      0],
  pccc_pers[per == 18, N ==      0],
  pccc_pers[per == 19, N ==      0],
  pccc_pers[per == 20, N ==      0],
  pccc_pers[per == 21, N ==      0],
  pccc_pers[per == 22, N ==      0],
  pccc_pers[per == 23, N ==      0],
  pccc_pers[per == 24, N ==      0],
  pccc_pers[per == 25, N ==      0],
  pccc_pers[per == 26, N ==      0],
  pccc_pers[per == 27, N ==      0],
  pccc_pers[per == 28, N ==      7],
  pccc_pers[per == 29, N ==      0],
  pccc_pers[per == 30, N ==      0],
  pccc_pers[per == 31, N ==      0],
  pccc_pers[per == 32, N ==      0],
  pccc_pers[per == 33, N ==      0],
  pccc_pers[per == 34, N ==     24],
  pccc_pers[per == 35, N ==      0],
  pccc_pers[per == 36, N ==      4],
  pccc_pers[per == 37, N ==      0],
  pccc_pers[per == 38, N ==      0],
  pccc_pers[per == 39, N ==      0],
  pccc_pers[per == 40, N ==      0],
  pccc_pers[per == 41, N ==      1],
  pccc_pers[per == 42, N ==      0],
  pccc_pers[per == 43, N ==     64],
  pccc_pers[per == 44, N ==      0],
  pccc_pers[per == 45, N ==   1032],
  pccc_pers[per == 46, N ==      0],
  pccc_pers[per == 47, N ==      3],
  pccc_pers[per == 48, N ==     25],
  pccc_pers[per == 49, N ==      0],
  pccc_pers[per == 50, N ==      0],
  pccc_pers[per == 51, N ==      3],
  pccc_pers[per == 52, N ==   5451]
)

# Expected counts in pccc_deltas, this should be the same as pccc_pers, but you
# need to count the rows.
stopifnot(
  pccc_deltas[per ==  1, .N == 198521],
  pccc_deltas[per ==  2, .N ==      3],
  pccc_deltas[per ==  3, .N ==      0],
  pccc_deltas[per ==  4, .N ==      0],
  pccc_deltas[per ==  5, .N ==      7],
  pccc_deltas[per ==  6, .N ==     37],
  pccc_deltas[per ==  7, .N ==    962],
  pccc_deltas[per ==  8, .N ==      0],
  pccc_deltas[per ==  9, .N ==     48],
  pccc_deltas[per == 10, .N ==      0],
  pccc_deltas[per == 11, .N ==      0],
  pccc_deltas[per == 12, .N ==      0],
  pccc_deltas[per == 13, .N ==      0],
  pccc_deltas[per == 14, .N ==      0],
  pccc_deltas[per == 15, .N ==      0],
  pccc_deltas[per == 16, .N ==     49],
  pccc_deltas[per == 17, .N ==      0],
  pccc_deltas[per == 18, .N ==      0],
  pccc_deltas[per == 19, .N ==      0],
  pccc_deltas[per == 20, .N ==      0],
  pccc_deltas[per == 21, .N ==      0],
  pccc_deltas[per == 22, .N ==      0],
  pccc_deltas[per == 23, .N ==      0],
  pccc_deltas[per == 24, .N ==      0],
  pccc_deltas[per == 25, .N ==      0],
  pccc_deltas[per == 26, .N ==      0],
  pccc_deltas[per == 27, .N ==      0],
  pccc_deltas[per == 28, .N ==      7],
  pccc_deltas[per == 29, .N ==      0],
  pccc_deltas[per == 30, .N ==      0],
  pccc_deltas[per == 31, .N ==      0],
  pccc_deltas[per == 32, .N ==      0],
  pccc_deltas[per == 33, .N ==      0],
  pccc_deltas[per == 34, .N ==     24],
  pccc_deltas[per == 35, .N ==      0],
  pccc_deltas[per == 36, .N ==      4],
  pccc_deltas[per == 37, .N ==      0],
  pccc_deltas[per == 38, .N ==      0],
  pccc_deltas[per == 39, .N ==      0],
  pccc_deltas[per == 40, .N ==      0],
  pccc_deltas[per == 41, .N ==      1],
  pccc_deltas[per == 42, .N ==      0],
  pccc_deltas[per == 43, .N ==     64],
  pccc_deltas[per == 44, .N ==      0],
  pccc_deltas[per == 45, .N ==   1032],
  pccc_deltas[per == 46, .N ==      0],
  pccc_deltas[per == 47, .N ==      3],
  pccc_deltas[per == 48, .N ==     25],
  pccc_deltas[per == 49, .N ==      0],
  pccc_deltas[per == 50, .N ==      0],
  pccc_deltas[per == 51, .N ==      3],
  pccc_deltas[per == 52, .N ==   5451]
)

# Verify the per_desc are as expected.
stopifnot(
  pccc_deltas[per ==  1, per_desc == "[v2.0, v2.1, v3.0, v3.1]"],
  pccc_deltas[per ==  2, per_desc == "[v2.0, v2.1, v3.0]; \\{v3.1\\}"],
  pccc_deltas[per ==  3, per_desc == "[v2.0, v2.1, v3.1]; \\{v3.0\\}"],
  pccc_deltas[per ==  4, per_desc == "[v2.0, v3.0, v3.1]; \\{v2.1\\}"],
  pccc_deltas[per ==  5, per_desc == "[v2.1, v3.0, v3.1]; \\{v2.0\\}"],
  pccc_deltas[per ==  6, per_desc == "[v2.0, v2.1]; \\{v3.0\\}; \\{v3.1\\}"],
  pccc_deltas[per ==  8, per_desc == "[v2.0], \\{v2.1\\}; [v3.0]; \\{v3.1\\}"],
  pccc_deltas[per ==  7, per_desc == "[v2.0, v2.1]; \\{v3.0, v3.1\\}"],
  pccc_deltas[per ==  9, per_desc == "[v2.0, v3.0]; \\{v2.1, v3.1\\}"],
  pccc_deltas[per == 10, per_desc == "[v2.0, v3.1]; \\{v2.1\\}; \\{v3.0\\}"],
  pccc_deltas[per == 11, per_desc == "[v2.0]; \\{v2.1\\}; \\{v3.0\\}; \\{v3.1\\}"],
  pccc_deltas[per == 12, per_desc == "[v2.0]; \\{v2.1\\}; \\{v3.0, v3.1\\}"],
  pccc_deltas[per == 13, per_desc == "[v2.0]; \\{v2.1, v3.1\\}, \\{v3.0\\}"],
  pccc_deltas[per == 14, per_desc == "[v2.0, v3.1]; \\{v2.1, v3.0\\}"],
  pccc_deltas[per == 15, per_desc == "[v2.0]; \\{v2.1, v3.0\\}, \\{v3.1\\}"],
  pccc_deltas[per == 16, per_desc == "[v2.0]; \\{v2.1, v3.0, v3.1\\}"],
  pccc_deltas[per == 17, per_desc == "\\{v2.0\\}; [v2.1, v3.0], \\{v3.1\\}"],
  pccc_deltas[per == 18, per_desc == "\\{v2.0, v3.1\\}; [v2.1, v3.0]"],
  pccc_deltas[per == 19, per_desc == "\\{v2.0\\}; [v2.1, v3.1]; \\{v3.0\\}"],
  pccc_deltas[per == 20, per_desc == "\\{v2.0\\}; [v2.1], \\{v3.0\\}, \\{v3.1\\}"],
  pccc_deltas[per == 21, per_desc == "\\{v2.0\\}; [v2.1], \\{v3.0, v3.1\\}"],
  pccc_deltas[per == 22, per_desc == "\\{v2.0, v3.1\\}; [v2.1], \\{v3.0\\}"],
  pccc_deltas[per == 23, per_desc == "\\{v2.0, v3.0\\}; [v2.1, v3.1]"],
  pccc_deltas[per == 24, per_desc == "\\{v2.0, v3.0\\}; [v2.1]; \\{v3.1\\}"],
  pccc_deltas[per == 25, per_desc == "\\{v2.0, v3.0, v3.1\\}; [v2.1]"],
  pccc_deltas[per == 26, per_desc == "\\{v2.0\\}; \\{v2.1\\}; [v3.0, v3.1]"],
  pccc_deltas[per == 27, per_desc == "\\{v2.0\\}; \\{v2.1\\}; [v3.0], \\{v3.1\\}"],
  pccc_deltas[per == 28, per_desc == "\\{v2.0\\}; \\{v2.1, v3.1\\}; [v3.0]"],
  pccc_deltas[per == 29, per_desc == "\\{v2.0, v3.1\\}; \\{v2.1\\} [v3.0]"],
  pccc_deltas[per == 30, per_desc == "\\{v2.0\\}; \\{v2.1\\}; \\{v3.0\\}, [v3.1]"],
  pccc_deltas[per == 31, per_desc == "\\{v2.0\\}; \\{v2.1, v3.0\\}; [v3.1]"],
  pccc_deltas[per == 32, per_desc == "\\{v2.0\\}; \\{v2.1\\}; \\{v3.0\\}; \\{v3.1\\}"],
  pccc_deltas[per == 33, per_desc == "\\{v2.0\\}; \\{v2.1\\}; \\{v3.0, v3.1\\}"],
  pccc_deltas[per == 34, per_desc == "\\{v2.0\\}; \\{v2.1, v3.1\\}; \\{v3.0\\}"],
  pccc_deltas[per == 35, per_desc == "\\{v2.0\\}; \\{v2.1, v3.0\\}; \\{v3.1\\}"],
  pccc_deltas[per == 36, per_desc == "\\{v2.0\\}; \\{v2.1, v3.0, v3.1\\}"],
  pccc_deltas[per == 37, per_desc == "\\{v2.0, v3.1\\}; \\{v2.1\\}; \\{v3.0\\}"],
  pccc_deltas[per == 38, per_desc == "\\{v2.0, v3.1\\}, \\{v2.1, v3.0\\}"],
  pccc_deltas[per == 39, per_desc == "\\{v2.0, v3.0\\}, \\{v2.1\\}; [v3.1]"],
  pccc_deltas[per == 40, per_desc == "\\{v2.0, v3.0\\}; \\{v2.1\\}; \\{v3.1\\}"],
  pccc_deltas[per == 41, per_desc == "\\{v2.0, v3.0\\}; \\{v2.1, v3.1\\}"],
  pccc_deltas[per == 42, per_desc == "\\{v2.0, v3.0, v3.1\\}; \\{v2.1\\}"],
  pccc_deltas[per == 43, per_desc == "\\{v2.0, v2.1\\}, [v3.0, v3.1]"],
  pccc_deltas[per == 44, per_desc == "\\{v2.0, v2.1\\}; [v3.0]; \\{v3.1\\}"],
  pccc_deltas[per == 45, per_desc == "\\{v2.0, v2.1, v3.1\\}; [v3.0]"],
  pccc_deltas[per == 46, per_desc == "\\{v2.0, v2.1\\}; \\{v3.0\\}, [v3.1]"],
  pccc_deltas[per == 47, per_desc == "\\{v2.0, v2.1\\}; \\{v3.0\\}; \\{v3.1\\}"],
  pccc_deltas[per == 48, per_desc == "\\{v2.0, v2.1\\}; \\{v3.0, v3.1\\}"],
  pccc_deltas[per == 49, per_desc == "\\{v2.0, v2.1, v3.1\\}; \\{v3.0\\}"],
  pccc_deltas[per == 50, per_desc == "\\{v2.0, v2.1, v3.0\\}; [v3.1]"],
  pccc_deltas[per == 51, per_desc == "\\{v2.0, v2.1, v3.0\\}; \\{v3.1\\}"],
  pccc_deltas[per == 52, per_desc == "\\{v2.0, v2.1, v3.0, v3.1\\}"]
)

# Tests related to PER 1
# All ICD-10 codes starting with a D mapping to a condition are diagnostic
# codes.  There are ICD-10 procedure codes starting with D which should not be
# in any mapping.
stopifnot(
  pccc_deltas[per != 1 & icdv == 10 & dx == 0 & grepl("^D", code), .N == 0]
)

# Tests related to PER 2
# for per 2 verify that the codes are as expected
stopifnot(pccc_deltas[per == 2, full_code == c("E72.530", "E72.538", "E72.539")])

# Tests related to PER 3

# Tests related to PER 4
# for per 4 verify that the codes are as expected
stopifnot(pccc_deltas[per == 4, full_code == c("56.7")])

# Tests related to PER 5

# Tests related to PER 6
stopifnot(
  pccc_deltas[per == 6, all(pccc_v3.1 == "cvd (other); neuromusc (occlusion_of_cerebral_arteries)")],
  pccc_deltas[per == 6, all(pccc_v3.0 == "neuromusc (occlusion_of_cerebral_arteries)")]
)

# Tests related to PER 7

# Tests related to PER 8

# Tests related to PER 9

# Tests related to PER 10

# Tests related to PER 11

# Tests related to PER 12

# Tests related to PER 13

# Tests related to PER 14

# Tests related to PER 15

# Tests related to PER 16

# Tests related to PER 17

# Tests related to PER 18

# Tests related to PER 19

# Tests related to PER 20

# Tests related to PER 21

# Tests related to PER 22

# Tests related to PER 23

# Tests related to PER 24

# Tests related to PER 25

# Tests related to PER 26

# Tests related to PER 27

# Tests related to PER 28

# Tests related to PER 29

# Tests related to PER 30

# Tests related to PER 31

# Tests related to PER 32

# Tests related to PER 33

# Tests related to PER 34

# Tests related to PER 35

# Tests related to PER 36

# Tests related to PER 37

# Tests related to PER 38

# Tests related to PER 39

# Tests related to PER 40

# Tests related to PER 41

# Tests related to PER 42

# Tests related to PER 43
not_documented_in_eTable3_per43 <-
  c("737.3",  # header that needs to be omitted because 737.3[049] were removed from V3
    "02H70KZ", # possible typo in eTable3 - might have needed to be 02H70JZ
    "02UA0JZ", # possible typo in eTable3 - might have needed to be 02UA07Z
    "C97",   # not a CMS code, in cdc_mortality and WHO
    "D84", # headers and multiple subconditions
    "D86", "D86.8", # headers and multiple subconditions
    "E88", "E88.8", "E88.82", "E88.A", # headers and multiple subconditions
    "G25.4",
    "G31.8",
    "G31.80",
    "G31.86",
    "G31.87",
    "G80",
    "M41.2",  # omit this header - all other codes removed from v3
    "M41.8",  # omit this header - all other codes removed from v3
    "P21.0",  # not a cms code - is in cdc mortality and who
    "P21.9",  # not a cms code - is in cdc mortality and who
    "P84",
    "Q43", "Q43.0", # not in the mappings for v3.0 -- that appears to be an missing code from v3 documents
    "T84.049A" # not in eTable3, but should likely be omitted from v3 based on similarity to other codes.
  )

#"M41.34",   # documented in eTable3 but not in this per
#"M41.35",   # documented in eTable3 but not in this per

documented_in_eTable3_per43 <-
  c(
    "737.30",
    "737.34",
    "737.39",
    "770.4",
    "G25.3",
    "G40.301",
    "M41.20",
    "M41.22",
    "M41.23",
    "M41.24",
    "M41.25",
    "M41.26",
    "M41.27",
    "M41.30",
    "M41.80",
    "M41.82",
    "M41.83",
    "M41.84",
    "M41.85",
    "M41.86",
    "M41.87",
    "M41.9",
    "P28.0",
    "T84.019A",
    "T84.029A",
    "T84.039A",
    "T84.059A",
    "T84.069A",
    "T84.099A",
    "T84.119A",
    "T84.129A",
    "T84.199A",
    "T84.498A",
    "T84.50XA",
    "T84.60XA",
    "T84.7XXA",
    "T85.79XA",
    "Z46.6",
    "Z95.9")

stopifnot(
  pccc_deltas[per == 43][, all(documented_in_eTable3_per43 %in% full_code)]
  ,
  pccc_deltas[per == 43][, all(not_documented_in_eTable3_per43 %in% full_code)]
  ,
  pccc_deltas[per == 43][, all(full_code %in% c(not_documented_in_eTable3_per43, documented_in_eTable3_per43))]
)

# Tests related to PER 44

# Tests related to PER 45

# Tests related to PER 46

# Tests related to PER 47

# Tests related to PER 48

# Tests related to PER 49

# Tests related to PER 50

# Tests related to PER 51

# Tests related to PER 52

################################################################################
#                                 End of File                                  #
################################################################################

#' Get Elixhauser Codes
#'
#' Retrieve  copy of internal lookup tables for the ICD codes used in assessing
#' Elixhauser comorbidities.
#'
#' @seealso
#' * [`get_elixhauser_index_scores()`] for the lookup table of the condition by
#'   condition scores for mortality and readmission indices.
#' * [`get_elixhauser_poa()`] for the lookup table of the conditions which do an
#'   do not require associated ICD codes to be present-on-admission to flag the
#'   comorbidity.
#' * [`get_icd_codes()`] for the lookup table of all ICD codes.
#' * [`get_pccc_codes()`] for the lookup table of ICD codes used for the PCCC.
#' * [`get_charlson_codes()`] for the lookup table of ICD codes used for the Charlson comorbidities.
#' * [`comorbidities()`] for applying comorbidity algorithms to a data set.
#' 
#' @return A `data.frame` with the following columns:
#' * `icdv`: Integer vector indicating if the code is from ICD-9 or ICD-10
#' * `dx`: Integer vector.  1 if the code is a diagnostic, (ICD-9-CM, ICD-10-CM, WHO, CDC Mortality), or 0 if the code is procedural (ICD-9-PCS, ICD-10-PCS)
#' * `full_code`: Character vector with the ICD code and any relevant decimal point
#' * `code`: Character vector with the compact ICD code omitting any relevant decimal point
#' * `condition`: Character vector of the conditions
#' * `elixhauser_<variant>`: Integer vector indicating if the code is part of the `<variant>` of the Elixhauser comorbidities.
#'
#' @examples
#' head(get_elixhauser_codes())
#' str(get_elixhauser_codes())
#'
#' @export
get_elixhauser_codes <- function() {
  x <- get("elixhauser_codes", envir = ..mdcr_data_env.., inherits = FALSE)
  unserialize(serialize(x, connection = NULL))
}

################################################################################
#                                 End of File                                  #
################################################################################

..mdcr_data_env.. <- new.env(parent = emptyenv())

.onLoad <- function(libname, pkgname) {
  # Build icd_codes
  # NOTE: code_id is a distinct key mapping to: icdv, dx, full_code, code.
  # See data-raw/icd/icd_codes.R
  idx <- match(..mdcr_internal_known_and_assignable_start_stop..[["code_id"]],
               ..mdcr_internal_icd_codes..[["code_id"]])
  cols <- c("icdv","dx","full_code","code","chap_id","subchap_id")
  icd_codes <- cbind(..mdcr_internal_known_and_assignable_start_stop..,
                     ..mdcr_internal_icd_codes..[idx, cols, drop = FALSE])
  icd_codes <- icd_codes[, c("code_id", cols, "src","known_start","known_end",
                           "assignable_start","assignable_end"), drop = FALSE]

  # build icd_codes_with_desc
  dkeys <- do.call(paste, c(..mdcr_internal_desc_start_stop..[c("code_id", "src")], sep = "\r"))
  ckeys <- do.call(paste, c(icd_codes[c("code_id", "src")], sep = "\r"))
  idx <- match(dkeys, unique(ckeys))

  icd_codes_with_desc <- cbind(..mdcr_internal_desc_start_stop..,
                               icd_codes[idx, setdiff(names(icd_codes), c("code_id", "src")), drop = FALSE])

  idx <- match(icd_codes_with_desc[["desc_id"]], ..mdcr_internal_icd_descs..[["desc_id"]])
  icd_codes_with_desc <- cbind(icd_codes_with_desc,
                               ..mdcr_internal_icd_descs..[idx, "desc", drop = FALSE])

  # place the icd_codes and icd_codes_with_desc in the cache
  icd_codes[["src"]] <- as.character(icd_codes[["src"]])
  icd_codes_with_desc[["src"]] <- as.character(icd_codes_with_desc[["src"]])

  rownames(icd_codes) <- NULL
  assign(x = "icd_codes", value = icd_codes, envir = ..mdcr_data_env..)
  lockBinding("icd_codes", env = ..mdcr_data_env..)

  rownames(icd_codes_with_desc) <- NULL
  assign(x = "icd_codes_with_desc", value = icd_codes_with_desc, envir = ..mdcr_data_env..)
  lockBinding("icd_codes_with_desc", env = ..mdcr_data_env..)

  # build the pccc codes and load into the cache
  ix <- match(..mdcr_internal_pccc_codes..[["code_id"]], ..mdcr_internal_icd_codes..[["code_id"]])
  pccc_codes <-
    cbind(..mdcr_internal_icd_codes..[ix, c("icdv", "dx", "full_code", "code"), drop = FALSE],
          ..mdcr_internal_pccc_codes..[, setdiff(names(..mdcr_internal_pccc_codes..), "code_id"), drop = FALSE])
  rownames(pccc_codes) <- NULL
  assign(x = "pccc_codes", value = pccc_codes, envir = ..mdcr_data_env..)
  lockBinding("pccc_codes", env = ..mdcr_data_env..)

  # build the charlson_codes and load into the cache
  ix <- match(..mdcr_internal_charlson_codes..[["code_id"]], ..mdcr_internal_icd_codes..[["code_id"]])
  charlson_codes <-
    cbind(..mdcr_internal_icd_codes..[ix, c("icdv", "dx", "full_code", "code"), drop = FALSE],
          ..mdcr_internal_charlson_codes..[, setdiff(names(..mdcr_internal_charlson_codes..), "code_id"), drop = FALSE])
  rownames(charlson_codes) <- NULL
  assign(x = "charlson_codes", value = charlson_codes, envir = ..mdcr_data_env..)
  lockBinding("charlson_codes", env = ..mdcr_data_env..)

  # build the elixhauser_codes and load into the cache
  ix <- match(..mdcr_internal_elixhauser_codes..[["code_id"]], ..mdcr_internal_icd_codes..[["code_id"]])
  elixhauser_codes <-
    cbind(..mdcr_internal_icd_codes..[ix, c("icdv", "dx", "full_code", "code"), drop = FALSE],
          ..mdcr_internal_elixhauser_codes..[, setdiff(names(..mdcr_internal_elixhauser_codes..), "code_id"), drop = FALSE])
  rownames(elixhauser_codes) <- NULL
  assign(x = "elixhauser_codes", value = elixhauser_codes, envir = ..mdcr_data_env..)
  lockBinding("elixhauser_codes", env = ..mdcr_data_env..)

  lockEnvironment(..mdcr_data_env.., bindings = TRUE)
}

library(medicalcoder)
source("utilities.R")
################################################################################
# testing assert_scaler_logical
#
# Tests:
#
#   t01:
#     a: verify the method is in the namespace
#     b: verify the method is not exported
#
#   t02: verify an error is thrown when `full.codes` is not a length-1
#        non-missing logical when calling comorbidities()
#
#   t03: verify an error is thrown when `compact.codes` is not a length-1
#        non-missing logical when calling comorbidities()
#
#   t04: verify an error is thrown when subconditions is not a length-1
#        non-missing logical (using pccc_v3.1 as the method to avoid warning
#        messages) when calling comorbidities()
#
#   t05: verify an error is thrown when with.descriptions is not a length-1
#        non-missing logical when calling get_icd_codes()
#
#   t06: verify an error is thrown when with.hierarchy is not a length-1
#        non-missing logical when calling get_icd_codes()

stopifnot(
  t01a = "assert_scalar_logical" %in% ls(getNamespace("medicalcoder"), all.names = TRUE),
  t01b = !("assert_scalar_logical" %in% getNamespaceExports("medicalcoder"))
)

common_args <- list(data = mdcr, method = "pccc_v3.1", icd.codes = "code", poa = 1L)

t02a <- tryCatchError(do.call(comorbidities, args = c(common_args, list(full.codes = TRUE))))
t02b <- tryCatchError(do.call(comorbidities, args = c(common_args, list(full.codes = FALSE))))
t02c <- tryCatchError(do.call(comorbidities, args = c(common_args, list(full.codes = logical(0)))))
t02d <- tryCatchError(do.call(comorbidities, args = c(common_args, list(full.codes = NA))))
t02e <- tryCatchError(do.call(comorbidities, args = c(common_args, list(full.codes = "yes"))))
t02f <- tryCatchError(do.call(comorbidities, args = c(common_args, list(full.codes = 3))))

t03a <- tryCatchError(do.call(comorbidities, args = c(common_args, list(compact.codes = TRUE))))
t03b <- tryCatchError(do.call(comorbidities, args = c(common_args, list(compact.codes = FALSE))))
t03c <- tryCatchError(do.call(comorbidities, args = c(common_args, list(compact.codes = logical(0)))))
t03d <- tryCatchError(do.call(comorbidities, args = c(common_args, list(compact.codes = NA))))
t03e <- tryCatchError(do.call(comorbidities, args = c(common_args, list(compact.codes = "yes"))))
t03f <- tryCatchError(do.call(comorbidities, args = c(common_args, list(compact.codes = 3))))

t04a <- tryCatchError(do.call(comorbidities, args = c(common_args, list(subconditions = TRUE))))
t04b <- tryCatchError(do.call(comorbidities, args = c(common_args, list(subconditions = FALSE))))
t04c <- tryCatchError(do.call(comorbidities, args = c(common_args, list(subconditions = logical(0)))))
t04d <- tryCatchError(do.call(comorbidities, args = c(common_args, list(subconditions = NA))))
t04e <- tryCatchError(do.call(comorbidities, args = c(common_args, list(subconditions = "yes"))))
t04f <- tryCatchError(do.call(comorbidities, args = c(common_args, list(subconditions = 3))))

t05a <- tryCatchError(get_icd_codes(with.descriptions = TRUE))
t05b <- tryCatchError(get_icd_codes(with.descriptions = FALSE))
t05c <- tryCatchError(get_icd_codes(with.descriptions = logical(0)))
t05d <- tryCatchError(get_icd_codes(with.descriptions = NA))
t05e <- tryCatchError(get_icd_codes(with.descriptions = "yes"))
t05f <- tryCatchError(get_icd_codes(with.descriptions = 3))

t06a <- tryCatchError(get_icd_codes(with.hierarchy = TRUE))
t06b <- tryCatchError(get_icd_codes(with.hierarchy = FALSE))
t06c <- tryCatchError(get_icd_codes(with.hierarchy = logical(0)))
t06d <- tryCatchError(get_icd_codes(with.hierarchy = NA))
t06e <- tryCatchError(get_icd_codes(with.hierarchy = "yes"))
t06f <- tryCatchError(get_icd_codes(with.hierarchy = 3))

stopifnot(
  inherits(t02a, "medicalcoder_comorbidities"),
  inherits(t02b, "medicalcoder_comorbidities"),
  inherits(t02c, "error"),
  inherits(t02d, "error"),
  inherits(t02e, "error"),
  inherits(t02f, "error"),
  t02c[["message"]] == "The value passed to 'full.codes' is expected to be a length-1 non-missing logical.",
  t02d[["message"]] == "The value passed to 'full.codes' is expected to be a length-1 non-missing logical.",
  t02e[["message"]] == "The value passed to 'full.codes' is expected to be a length-1 non-missing logical.",
  t02f[["message"]] == "The value passed to 'full.codes' is expected to be a length-1 non-missing logical.",
  inherits(t03a, "medicalcoder_comorbidities"),
  inherits(t03b, "medicalcoder_comorbidities"),
  inherits(t03c, "error"),
  inherits(t03d, "error"),
  inherits(t03e, "error"),
  inherits(t03f, "error"),
  t03c[["message"]] == "The value passed to 'compact.codes' is expected to be a length-1 non-missing logical.",
  t03d[["message"]] == "The value passed to 'compact.codes' is expected to be a length-1 non-missing logical.",
  t03e[["message"]] == "The value passed to 'compact.codes' is expected to be a length-1 non-missing logical.",
  t03f[["message"]] == "The value passed to 'compact.codes' is expected to be a length-1 non-missing logical.",
  inherits(t04a, "medicalcoder_comorbidities"),
  inherits(t04b, "medicalcoder_comorbidities"),
  inherits(t04c, "error"),
  inherits(t04d, "error"),
  inherits(t04e, "error"),
  inherits(t04f, "error"),
  t04c[["message"]] == "The value passed to 'subconditions' is expected to be a length-1 non-missing logical.",
  t04d[["message"]] == "The value passed to 'subconditions' is expected to be a length-1 non-missing logical.",
  t04e[["message"]] == "The value passed to 'subconditions' is expected to be a length-1 non-missing logical.",
  t04f[["message"]] == "The value passed to 'subconditions' is expected to be a length-1 non-missing logical.",
  inherits(t05a, "data.frame"),
  inherits(t05b, "data.frame"),
  inherits(t05c, "error"),
  inherits(t05d, "error"),
  inherits(t05e, "error"),
  inherits(t05f, "error"),
  t05c[["message"]] == "The value passed to 'with.descriptions' is expected to be a length-1 non-missing logical.",
  t05d[["message"]] == "The value passed to 'with.descriptions' is expected to be a length-1 non-missing logical.",
  t05e[["message"]] == "The value passed to 'with.descriptions' is expected to be a length-1 non-missing logical.",
  t05f[["message"]] == "The value passed to 'with.descriptions' is expected to be a length-1 non-missing logical.",
  inherits(t06a, "data.frame"),
  inherits(t06b, "data.frame"),
  inherits(t06c, "error"),
  inherits(t06d, "error"),
  inherits(t06e, "error"),
  inherits(t06f, "error"),
  t06c[["message"]] == "The value passed to 'with.hierarchy' is expected to be a length-1 non-missing logical.",
  t06d[["message"]] == "The value passed to 'with.hierarchy' is expected to be a length-1 non-missing logical.",
  t06e[["message"]] == "The value passed to 'with.hierarchy' is expected to be a length-1 non-missing logical.",
  t06f[["message"]] == "The value passed to 'with.hierarchy' is expected to be a length-1 non-missing logical."
)

################################################################################
#                                 End of File                                  #
################################################################################

#' Comorbidities
#'
#' Apply established comorbidity algorithms to ICD-coded data. Supported
#' methods include several variants of the Charlson comorbidity system,
#' Elixhauser, and the Pediatric Complex Chronic Conditions (PCCC).
#'
#' @template params-comorbidities
#'
#' @details
#' When `flag.method = "current"`, only codes from the index encounter
#' contribute to flags. When a longitudinal method is selected (e.g.,
#' `"cumulative"`), prior encounters for the same `id.vars`
#' combination may contribute to condition flags. For the cumulative method to
#' work, `id.vars` needs to be a character vector of length 2 or more. The last
#' element is treated as the encounter identifier and must be sortable. For
#' example, say you have data with a hospital, patient, and encounter id. The
#' `id.vars` could be one of two entries: `c("hospital", "patient", "encounter")`
#' or `c("patient", "hospital", "encounter")`. In both cases the return will be
#' the same because the encounter identifier is unchanged regardless of whether
#' hospital or patient is listed first.
#'
#' It is critically important that the `data[[tail(id.vars, 1)]]` variable can
#' be sorted.  Just because your data is sorted in temporal order does not mean
#' that the results will be correct if the `tail(id.vars, 1)` is not in the same
#' order as the data.  For example, say you had the following:
#'
#' | patid | enc_id   | date     |
#' |:---:  |:---:     | :---:    |
#' | P1    | 10823090 | Aug 2023 |
#' | P1    | 10725138 | Jul 2025 |
#'
#' `id.vars = c("patid", "enc_id")` will give the wrong result as enc_id
#' 10725138 would be sorted to come before enc_id 10823090.  `id.vars =
#' c("patid", "date")` would be sufficient input, assuming that `date` has been
#' correctly stored. Adding a column `enc_seq`, e.g.,
#'
#' | patid | enc_id   | date     | enc_seq |
#' |:---:  |:---:     | :---:    | :---:   |
#' | P1    | 10823090 | Aug 2023 | 1       |
#' | P1    | 10725138 | Jul 2025 | 2       |
#'
#' and calling `comorbidities()` with `id.vars = c("patid", "enc_seq")` will
#' have better performance than using the date and will clear up any possible
#' issues with non-sequential encounter ids from the source data.
#'
#' **Cumulative + POA defaults:**
#'
#' When `flag.method = "cumulative"` and neither
#' `poa` nor `poa.var` is supplied, the first encounter for a condition is
#' treated as `poa = 0`. Subsequent encounters for that condition are flagged as
#' `poa = 1`.
#'
#' When `flag.method = "current"` and neither `poa` nor `poa.var` is supplied,
#' then all codes will be considered present-on-admission.  If poa was assumed
#' to be 0, then in this case the only conditions that could be flagged are the
#' Elixhauser conditions which are poa-exempt.
#'
#' @return
#'
#' The return object will be slightly different depending on the value of
#' `method` and `subconditions`.
#'
#' * When `subconditions = FALSE`, a `medicalcoder_comorbidities` object (a
#'   `data.frame` with attributes) is returned.  Column(s) for `id.vars`, if
#'   defined in the function call.  For all methods there will be the following
#'   columns:
#'   * `num_cmrb` a count of comorbidities/conditions flagged
#'   * `cmrb_flag` a 0/1 integer indicator for at least one
#'   comorbidity/condition.
#'
#'   Additional columns:
#'
#'   * PCCC methods:
#'     * For `method = "pccc_v2.0"` and `method = "pccc_v2.1"`, there is one
#'     indicator column per condition.
#'
#'     * For `method = "pccc_v3.0"` and `method = "pccc_v3.1"`,
#'       there are four columns per condition:
#'       * `<condition>_dxpr_or_tech`: the condition was flagged due to the
#'         presence of either a diagnostic or procedure code, or was flagged due to
#'         the presence of a technology dependence code along with at least one
#'         comorbidity being flagged by a diagnostic or procedure code.
#'       * `<condition>_dxpr_only`: the condition was flagged due to the
#'         presence of a non-technology dependent diagnostic or procedure code
#'         only.
#'       * `<condition>_tech_only`: the condition was flagged due to the
#'         presence of a technology dependent code only and at least one other
#'         comorbidity was flagged by a non-technology dependent code.
#'       * `<condition>_dxpr_and_tech`: The patient had both diagnostic or
#'         procedure codes and a technology dependence code for the condition.
#'
#'   * For Charlson variants, indicator columns are returned for the relevant
#'     conditions, `cci` (Charlson Comorbidity Index), and `age_score`.
#'
#'   * For Elixhauser variants, indicator columns are returned for all relevant
#'     comorbidities, mortality, and readmission indices.
#'
#' * When `subconditions = TRUE` and the method is a PCCC variant,
#'   a list of length two is returned: the first element contains condition
#'   indicators; the second element is a named list of `data.frame`s with
#'   indicators for subconditions within each condition.
#'
#' @references
#'
#' * Pediatric Complex Chronic Conditions:
#'
#'   * Feudtner, C., Feinstein, J.A., Zhong, W. et al. Pediatric complex
#'     chronic conditions classification system version 2: updated for ICD-10
#'     and complex medical technology dependence and transplantation. BMC
#'     Pediatr 14, 199 (2014). https://doi.org/10.1186/1471-2431-14-199
#'
#'   * Feinstein JA, Hall M, Davidson A, Feudtner C. Pediatric Complex Chronic
#'     Condition System Version 3. JAMA Netw Open. 2024;7(7):e2420579.
#'     https://doi.org/10.1001/jamanetworkopen.2024.20579
#'
#' * Charlson Comorbidities:
#'
#'   * Mary E. Charlson, Peter Pompei, Kathy L. Ales, C.Ronald
#'       MacKenzie, A new method of classifying prognostic comorbidity in
#'       longitudinal studies: Development and validation, Journal of Chronic
#'       Diseases, Volume 40, Issue 5, 1987, Pages 373-383, ISSN 0021-9681,
#'       https://doi.org/10.1016/0021-9681(87)90171-8.
#'   * Deyo RA, Cherkin DC, Ciol MA. Adapting a clinical comorbidity index
#'       for use with ICD-9-CM administrative databases. J Clin Epidemiol. 1992
#'       Jun;45(6):613-9. https://doi.org/10.1016/0895-4356(92)90133-8. PMID: 1607900.
#'   * Quan H, Sundararajan V, Halfon P, Fong A, Burnand B, Luthi JC,
#'       Saunders LD, Beck CA, Feasby TE, Ghali WA. Coding algorithms for defining
#'       comorbidities in ICD-9-CM and ICD-10 administrative data. Med Care. 2005
#'       Nov;43(11):1130-9. https://doi.org/10.1097/01.mlr.0000182534.19832.83. PMID:
#'       16224307.
#'   * Quan H, Li B, Couris CM, Fushimi K, Graham P, Hider P, Januel JM,
#'       Sundararajan V. Updating and validating the Charlson comorbidity index
#'       and score for risk adjustment in hospital discharge abstracts using
#'       data from 6 countries. Am J Epidemiol. 2011 Mar 15;173(6):676-82.
#'       https://doi.org/10.1093/aje/kwq433. Epub 2011 Feb 17. PMID: 21330339.
#'   * Glasheen WP, Cordier T, Gumpina R, Haugh G, Davis J, Renda A.
#'       Charlson Comorbidity Index: ICD-9 Update and ICD-10 Translation. Am Health
#'       Drug Benefits. 2019 Jun-Jul;12(4):188-197. PMID: 31428236; PMCID:
#'       PMC6684052.
#'
#' * Elixhauser Comorbidities:
#'
#'   * Agency for Healthcare Research and Quality (AHRQ). Elixhauser
#'     Comorbidity Software Refined for ICD-10-CM Diagnoses, v2026.1 \[Internet\].
#'     2026. Available from:
#'     https://www.hcup-us.ahrq.gov/toolssoftware/comorbidityicd10/comorbidity_icd10.jsp
#'
#' @seealso
#' * `vignettes(topic = "comorbidities", package = "medicalcoder")`
#' * `vignettes(topic = "pccc", package = "medicalcoder")`
#' * `vignettes(topic = "charlson", package = "medicalcoder")`
#' * `vignettes(topic = "elixhauser", package = "medicalcoder")`
#'
#' @example examples/comorbidities.R
#'
#' @export
comorbidities <- function(data,
                          icd.codes,
                          method,
                          id.vars = NULL,
                          icdv.var = NULL, icdv = NULL,
                          dx.var = NULL, dx = NULL,
                          poa.var = NULL,  poa = NULL,
                          age.var = NULL,
                          primarydx.var = NULL, primarydx = NULL,
                          flag.method = c("current", "cumulative"),
                          full.codes = TRUE,
                          compact.codes = TRUE,
                          subconditions = FALSE
                          ) {
  UseMethod("comorbidities")
}

#' @export
comorbidities.data.frame <- function(data,
                                     icd.codes,
                                     method,
                                     id.vars = NULL,
                                     icdv.var = NULL, icdv = NULL,
                                     dx.var = NULL, dx = NULL,
                                     poa.var = NULL,  poa = NULL,
                                     age.var = NULL,
                                     primarydx.var = NULL, primarydx = NULL,
                                     flag.method = c("current", "cumulative"),
                                     full.codes = TRUE,
                                     compact.codes = TRUE,
                                     subconditions = FALSE) {

  ##############################################################################
  # verify input arguments
  assert_scalar_logical(full.codes)
  assert_scalar_logical(compact.codes)
  stopifnot(full.codes | compact.codes)

  method <-
    match.arg(
      method,
      choices = comorbidities_methods(),
      several.ok = FALSE
    )

  is_a_column <- function(x, cols) {
    stopifnot(is.character(x) && length(x) == 1L && x %in% cols)
  }

  is_a_column(icd.codes, names(data))

  if (!is.null(id.vars)) {
    for (x in id.vars) {
      is_a_column(x, names(data))
    }
    pn <- which(id.vars %in% ..protected_names..)
    if (length(pn)) {
      stop(sprintf("The value(s) \"%s\" in 'id.vars' are protected name(s).  It is ill-advised to use a protected name as medicalcoder is expecting to use them internally to apply the comorbidity algorithms.  Sorry for the inconvenience, but you will need to rename the column(s) in your data set.  Protected names that you should not use for 'id.vars' are: %s.",
        paste(id.vars[pn], collapse = ", "),
        paste(..protected_names.., collapse = ", ")
        )
      )
    }
  }

  if (!is.null(poa.var)) {
    is_a_column(poa.var, names(data))
    if (!is.numeric(data[[poa.var]])) {
      stop(sprintf("Column '%s' must be numeric (0/1/NA) when supplied as poa.var.", poa.var), call. = FALSE)
    }
    pn <- poa.var %in% ..protected_names..
    if (pn) {
      stop(
        sprintf("The value \"%s\" in 'poa.var' is a protected name.  It is ill-advised to use a protected name as medicalcoder is expecting to use them internally to apply the comorbidity algorithms.  Sorry for the inconvenience, but you will need to rename the column in your data set.  Protected names that you should not use for 'poa.var' are: %s.",
          poa.var,
          paste(..protected_names.., collapse = ", ")
        )
      )
    }
  }

  if ((startsWith(method, "elixhauser") | startsWith(method, "charlson")) & !is.null(primarydx.var)) {
    is_a_column(primarydx.var, names(data))
    if (!is.numeric(data[[primarydx.var]])) {
      stop(sprintf("Column '%s' must be numeric (0/1/NA) when supplied as primarydx.var.", primarydx.var), call. = FALSE)
    }
    pn <- primarydx.var %in% ..protected_names..
    if (pn) {
      stop(
        sprintf("The value \"%s\" in 'primarydx.var' is a protected name.  It is ill-advised to use a protected name as medicalcoder is expecting to use them internally to apply the comorbidity algorithms.  Sorry for the inconvenience, but you will need to rename the column in your data set.  Protected names that you should not use for 'primarydx.var' are: %s.",
          primarydx.var,
          paste(..protected_names.., collapse = ", ")
        )
      )
    }
  } else if (startsWith(method, "pccc") & (!is.null(primarydx.var) | !is.null(primarydx))) {
    warning(sprintf("primarydx.var and primarydx are ignored when method = '%s'", method), call. = FALSE)
    primarydx.var <- primarydx <- NULL
  }

  flag.method <- match.arg(flag.method, choices = c("current", "cumulative"), several.ok = FALSE)

  if (startsWith(method, "charlson") && !is.null(age.var)) {
    is_a_column(age.var, names(data))
  }

  assert_scalar_logical(subconditions)
  if (subconditions & !startsWith(method, "pccc")) {
    warning("subconditions only implemented for PCCC", call. = FALSE)
    subconditions <- FALSE
  }

  # Cumulative flagging carries a condition forward from its first encounter.
  # Identify the earliest encounter per condition (and subcondition for PCCC),
  # replicate that flag across later encounters for the same id.vars stack, and
  # flip poa to 1L after the first occurrence so downstream methods treat the
  # condition as persistent.
  if (flag.method == "cumulative" & length(id.vars) < 2L) {
    stop("When using `flag.method = 'cumulative'` the `id.vars` are expected to be provided and have a minimum length of 2, e.g., c('subject_id', 'encounter_number')", call. = FALSE)
  }

  # Check if icdv.var and/or icdv have been specified and check for expected
  # values.  if icdv is specified and icdv.var is NULL then then the icdv will
  # be used to subset the lookup table of ICD codes and conditions by method
  if (!is.null(icdv.var)) {
    if (!is.null(icdv)) {
      warning("'icdv.var' and 'icdv' were both specified; ignoring 'icdv'", call. = FALSE)
      icdv <- NULL
    } else {
      is_a_column(icdv.var, names(data))
      if (!is.numeric(data[[icdv.var]])) {
        stop(sprintf("Column '%s' must be numeric (9/10/NA) when supplied as icdv.var.", icdv.var), call. = FALSE)
      }
    }
  } else {
    if (!is.null(icdv)) {
      stopifnot(inherits(icdv, "numeric") | inherits(icdv, "integer"))
      stopifnot(length(icdv) == 1L)
      icdv <- as.integer(icdv)
      stopifnot(icdv %in% c(9L, 10L))
    } else {
      # both icdv.var and icdv are NULL
      # do nothing
    }
  }

  # Check is dx.var and/or dx have been specified.  If dx has been specified and
  # dx.var is NULL, then the value of dx will be used to subset the lookup
  # table.
  if (!is.null(dx.var)) {
    if (!is.null(dx)) {
      warning("'dx.var' and 'dx' were both specified; ignoring 'dx'", call. = FALSE)
      dx <- NULL
    } else{
      is_a_column(dx.var, names(data))
      if (!is.numeric(data[[dx.var]])) {
        stop(sprintf("Column '%s' must be numeric (0/1/NA) when supplied as dx.var.", dx.var), call. = FALSE)
      }
    }
  } else {
    if (!is.null(dx)) {
      stopifnot(inherits(dx, "numeric") | inherits(dx, "integer"))
      stopifnot(length(dx) == 1L)
      dx <- as.integer(dx)
      stopifnot(dx %in% c(0L, 1L))
    } else {
      # both dx.var and dx are NULL
      # do nothing
    }
  }

  ##############################################################################
  # Determine how to join the data and the look up table
  # the by_y will be prepended by full_code or code in the merge calls that
  # follow so here that part of the by_y character vector is not defined.
  by_x <- icd.codes
  by_y <- character(0)
  if (!is.null(icdv.var)) {
    by_x <- c(by_x, icdv.var)
    by_y <- c(by_y, "icdv")
  }
  if (!is.null(dx.var)) {
    by_x <- c(by_x, dx.var)
    by_y <- c(by_y, "dx")
  }

  ##############################################################################
  # Determine the lookup table and the columns for the lookup table to keep
  lookup_to_keep <- c("condition")
  if (startsWith(method, "pccc")) {
    lookup <- get(x = "pccc_codes", envir = ..mdcr_data_env.., inherits = FALSE)
    lookup_to_keep <- c(lookup_to_keep, "subcondition", "transplant_flag", "tech_dep_flag")
  } else if (startsWith(method, "charlson")) {
    lookup <- get("charlson_codes", envir = ..mdcr_data_env.., inherits = FALSE)
    lookup_to_keep <- c(lookup_to_keep)
  } else if (startsWith(method, "elixhauser")) {
    lookup <- get("elixhauser_codes", envir = ..mdcr_data_env.., inherits = FALSE)
    lookup_to_keep <- c(lookup_to_keep, "poaexempt")
  }

  idx <- lookup[[method]] == 1L

  if (!is.null(dx)) {
    idx <- idx & (lookup[["dx"]] == dx)
  }

  if (!is.null(icdv)) {
    idx <- idx & (lookup[["icdv"]] == icdv)
  }

  lookup <- mdcr_subset(lookup, i = idx)

  ##############################################################################
  # inner join the data with the lookup table
  on_full <-
    mdcr_inner_join(
      x = if (full.codes) {data} else {data[0, ]},
      y = lookup,
      by.x = by_x,
      by.y = c("full_code", by_y),
      suffixes = c("", ".y")
    )

  on_comp <-
    mdcr_inner_join(
      x = if (compact.codes) {data} else {data[0, ]},
      y = lookup,
      by.x = by_x,
      by.y = c("code", by_y),
      suffixes = c("", ".y")
    )

  ##############################################################################
  # Now determine if the id.vars, poa.var, and primarydx.var need to be
  # constructed and added to the on_full and on_comp sets
  id.vars.created <- is.null(id.vars)

  build_name <- function(stem, names) {
    while(stem %in% names) {
      stem <- paste0(".", stem, ".")
    }
    stem
  }
  nms <- unique(c(names(on_full), names(on_comp)))

  if (id.vars.created) {
    id.vars <- build_name("..medicalcoder_id..", nms)
    on_full <- mdcr_set(on_full, j = id.vars, value = rep(1L, nrow(on_full)))
    on_comp <- mdcr_set(on_comp, j = id.vars, value = rep(1L, nrow(on_comp)))
  }

  if (is.null(poa.var) & is.null(poa)) {
    poa <- as.integer(flag.method == "current")
    warning(
      sprintf("'poa.var' and 'poa' are both NULL.  With flag.method = '%s' poa is set to %d.",
        flag.method, poa),
      call. = FALSE
    )
  }

  if (is.null(poa.var)) {
    stopifnot(inherits(poa, "numeric") | inherits(poa, "integer"))
    stopifnot(length(poa) == 1L)
    poa <- as.integer(poa)
    stopifnot(poa %in% c(0L, 1L))
    poa.var <- build_name("..medicalcoder_poa..", nms)
    on_full <- mdcr_set(on_full, j = poa.var, value = rep(poa, nrow(on_full)))
    on_comp <- mdcr_set(on_comp, j = poa.var, value = rep(poa, nrow(on_comp)))
  } else {
    if (!is.null(poa)) {
      warning("'poa.var' and 'poa' were both specified; ignoring 'poa'", call. = FALSE)
    }
    is_a_column(poa.var, nms)
  }

  if (startsWith(method, "elixhauser") | startsWith(method, "charlson")) {
    if (is.null(primarydx.var)) {
      if (!is.null(primarydx)) {
        stopifnot(inherits(primarydx, "numeric") | inherits(primarydx, "integer"))
        stopifnot(length(primarydx) == 1L)
        primarydx <- as.integer(primarydx)
        stopifnot(primarydx %in% c(0L, 1L))
      } else {
        warning("Assuming all codes provided are secondary diagnostic codes.  Define `primarydx.var` or `primarydx` if this assumption is incorrect.", call. = FALSE)
        primarydx <- 0L
      }

      primarydx.var <- build_name("..medicalcoder_primarydx..", nms)

      on_full <- mdcr_set(on_full, j = primarydx.var, value = rep(primarydx, nrow(on_full)))
      on_comp <- mdcr_set(on_comp, j = primarydx.var, value = rep(primarydx, nrow(on_comp)))

    } else {
      if (!is.null(primarydx)) {
        warning("'primarydx.var' and 'primarydx' were both specified; ignoring 'primarydx'", call. = FALSE)
      }
      is_a_column(primarydx.var, nms)
    }
  }

  cmrb <-
    rbind(
      mdcr_select(on_full, c(id.vars, poa.var, primarydx.var, method, lookup_to_keep)),
      mdcr_select(on_comp, c(id.vars, poa.var, primarydx.var, method, lookup_to_keep))
    )

  # retain only meaningful rows, that is, unique rows.  If a condition is
  # reported more than once with the same information except for poa, then keep
  # a row for poa = 1 and omit the other poa = 1 row(s) and any poa = 0 row(s).
  cmrb <- mdcr_setorder(cmrb, by = c(names(cmrb)[names(cmrb) != poa.var], poa.var))
  keep <- !mdcr_duplicated(cmrb, by = names(cmrb)[names(cmrb) != poa.var], fromLast = TRUE)
  cmrb <- mdcr_subset(cmrb, keep)

  ##############################################################################
  # create a data.frame with one unique row for the id.vars
  if (!id.vars.created) {
    iddf <- mdcr_unique(mdcr_select(data, cols = id.vars))
  } else {
    iddf <- mdcr_unique(mdcr_select(cmrb, cols = id.vars))
    if (nrow(iddf) == 0) {
      iddf <- stats::setNames(data.frame(1L, stringsAsFactors = FALSE), id.vars)
    }
  }

  iddf <- mdcr_setorder(iddf, id.vars)

  ##############################################################################
  # create additional rows in cmrb based on the flag.method
  #
  # only need the if statement for flag.method cumulative.  The match.arg call
  # above should make sure that only valid and expected arguments for
  # flag.method would get to this point in the code.
  #
  # Also, expected, and verified above, that there are at least two id.vars.
  if (flag.method == "cumulative" & nrow(cmrb) > 0L) {
    id.vars2 <- id.vars[-length(id.vars)]
    encid <- id.vars[length(id.vars)]

    # find the first occurance of each condition
    grps <- c(id.vars2, "condition")
    byconditions <- c("condition")
    if (startsWith(method, "pccc")) {
      grps <- c(grps, "subcondition")
      byconditions <- c(byconditions, "subcondition")
    }
    # identify first occurrence per id/condition then retain encounters on/after it
    tmp <- mdcr_select(cmrb, c(grps, encid))
    tmp <- mdcr_setorder(tmp, c(grps, encid))
    keep <- !mdcr_duplicated(tmp, by = grps)
    foc <- mdcr_subset(tmp, keep)

    # add the first occurrence on to the cmrb data.frame
    foc <-
      mdcr_left_join(
        x = cmrb,
        y = foc,
        by = c(id.vars2, encid, byconditions)
      )
    foc <- mdcr_setnames(foc, old = encid, new = "first_occurrance")

    iddf2 <-
      mdcr_inner_join(
        x = mdcr_unique(mdcr_select(iddf, id.vars)),
        y = mdcr_unique(mdcr_select(foc, id.vars2)),
        by = id.vars2)
    iddf2 <- mdcr_unique(iddf2)

    if (startsWith(method, "pccc")) {
      foc <- split(foc, f = mdcr_select(foc, c("condition", "subcondition")), drop = TRUE)
    } else {
      foc <- split(foc, f = mdcr_select(foc, c("condition")), drop = TRUE)
    }
    foc <- lapply(foc, mdcr_unique)


    foc <-
      lapply(foc,
             function(y) {
               rtn <- mdcr_left_join(x = iddf2, y = y, by = c(id.vars2))
               rtn <- mdcr_subset(rtn, i = !is.na(rtn[["condition"]]))
               i <- rtn[[encid]] >= rtn[["first_occurrance"]]
               mdcr_subset(rtn, i = i)
             })

    cmrb <- do.call(rbind, foc)

    # Carry condition forward after first occurrence: set poa to 1 and
    # primarydx to 0 on later encounters so downstream POA filtering keeps
    # all post-first-occurrence rows and the first-occurrence row only if poa =
    # 1 (via poa.var or poa) for the first-occurrence
    idx <- cmrb[[encid]] > cmrb[["first_occurrance"]]
    cmrb[[poa.var]][idx] <- 1L
    if (!is.null(primarydx.var)) {
      cmrb[[primarydx.var]][cmrb[[encid]] > cmrb[["first_occurrance"]]] <- 0L
    }
    cmrb <- mdcr_set(cmrb, j = "first_occurrance", value =  NULL)

    cmrb <- mdcr_unique(cmrb)
  }

  ##############################################################################
  # retain only the row for present on admission for pccc and charlson.
  # elixhauser conditions may or may not need poa, so do not subset in that
  # case.
  if (startsWith(method, "charlson") | startsWith(method, "pccc")) {
    cmrb <- mdcr_subset(cmrb, i = cmrb[[poa.var]] == 1L)
  }

  ##############################################################################
  # apply the needed method
  if (startsWith(method, "pccc_v2")) {
    ccc <- .pccc_v2(id.vars = id.vars, iddf = iddf, cmrb = cmrb, subconditions = subconditions)
  } else if (startsWith(method, "pccc_v3")) {
    ccc <- .pccc_v3(id.vars = id.vars, iddf = iddf, cmrb = cmrb, subconditions = subconditions)
  } else if (startsWith(method, "charlson")) {
    ccc <- .charlson(id.vars = id.vars, iddf = iddf, cmrb = cmrb, primarydx.var = primarydx.var, method = method)
    if (!is.null(age.var)) {
      if (id.vars.created) {
        ages <- mdcr_unique(mdcr_select(data, cols = c(age.var)))
        ages <- mdcr_set(ages, j = id.vars, value = rep(1L, nrow(ages)))
        warn_about_age <- nrow(ages) > 1
      } else {
        ages <- mdcr_unique(mdcr_select(data, cols = c(id.vars, age.var)))
        warn_about_age <- any(mdcr_duplicated(ages, by = id.vars))
      }

      if (warn_about_age) {
        if (id.vars.created) {
          msg <- "There is more than one unique value for age.  Since `id.vars = NULL` the expectation is there would be one unique age value.  The return will have more than one row, one for each unique age."
        } else {
          msg <- "There is at least one set of id.vars with more than one age value.  The expectation is that there is only one age value for each unique set of id.vars.  The return will have more than one row for each unique set of id.vars."
        }
        warning(msg, call. = FALSE)
      }

      ages[["age_score"]] <- as.integer(cut(ages[[age.var]], breaks = c(-Inf, 50, 60, 70, 80, Inf), right = TRUE)) - 1L
      ccc <- merge(ccc, mdcr_select(ages, cols = c(id.vars, "age_score")), all.x = TRUE, by = id.vars, sort = FALSE)
      nonmissing <- which(!is.na(ccc[["age_score"]]))
      ccc[["cci"]][nonmissing] <- ccc[["cci"]][nonmissing] + ccc[["age_score"]][nonmissing]

    } else {
      ccc[["age_score"]] <- rep(NA_integer_, nrow(ccc))
    }
  } else if (startsWith(method, "elixhauser")) {
    ccc <- .elixhauser(id.vars = id.vars, iddf = iddf, cmrb = cmrb, poa.var = poa.var, primarydx.var = primarydx.var, method = method)
  } else {
    stop(sprintf("method '%s' has not yet been implemented", method))
  }

  ##############################################################################
  # Remove the id.vars if it was created
  if (id.vars.created) {
    if (subconditions) {
      ccc[["conditions"]] <- mdcr_set(ccc[["conditions"]], j = id.vars, value = NULL)
        if (nrow(data) == 0) {
          ccc[["conditions"]] <- ccc[["conditions"]][0, ]
        }
      for (i in seq_along(ccc[["subconditions"]])) {
        ccc[["subconditions"]][[i]] <- mdcr_set(ccc[["subconditions"]][[i]], j = id.vars, value = NULL)
        if (nrow(data) == 0) {
          ccc[["subconditions"]][[i]] <- ccc[["subconditions"]][[i]][0, ]
        }
      }
    } else {
      ccc <- mdcr_set(ccc, j = id.vars, value = NULL)
      if (nrow(data) == 0) {
        ccc <- ccc[0, ]
      }
    }
  }

  ##############################################################################
  # set attributes and return
  attr(ccc, "method") <- method
  attr(ccc, "id.vars") <- id.vars
  attr(ccc, "flag.method") <- flag.method
  class(ccc) <- c("medicalcoder_comorbidities", class(ccc))

  if (subconditions) {
    class(ccc) <- c("medicalcoder_comorbidities_with_subconditions", class(ccc))
    rownames(ccc[["conditions"]]) <- NULL
    for (i in seq_along(ccc[["subconditions"]])) {
      rownames(ccc[["subconditions"]][[i]]) <- NULL
    }
  } else {
    rownames(ccc) <- NULL
  }

  ccc
}

#' @export
print.medicalcoder_comorbidities <- function(x, ...) {
  cat(sprintf("\nComorbidities via %s\n\n", attr(x, "method")))
  NextMethod(generic = "print", object = x, ...)
  invisible(x)
}

#' @export
print.medicalcoder_comorbidities_with_subconditions <- function(x, ...) {
  cat(sprintf("\nComorbidities and Subconditions via %s\n\n", attr(x, "method")))
  l1 <- utils::capture.output(utils::str(x, max.level = 1, give.attr = FALSE))
  l2 <- utils::capture.output(utils::str(x[["subconditions"]], max.level = 1, give.attr = FALSE))
  l2 <- sub("^\\s\\$", "  ..$", l2)
  cat(c(l1, l2[-1], "\n"), sep = "\n")
  invisible(x)
}

################################################################################
# Non-exported functions
#
comorbidities_methods <- function() {
    c("pccc_v2.0", "pccc_v2.1", "pccc_v3.0", "pccc_v3.1",
      "charlson_deyo1992", "charlson_quan2011", "charlson_quan2005",
      "charlson_cdmf2019",
      "elixhauser_elixhauser1988", "elixhauser_ahrq_web", "elixhauser_quan2005",
      "elixhauser_ahrq2022", "elixhauser_ahrq2023", "elixhauser_ahrq2024",
      "elixhauser_ahrq2025", "elixhauser_ahrq2026", "elixhauser_ahrq_icd10")
}


# protected names... throw and error and tell end users that it is ill-advised
# to use these names for id.vars, poa.var, primarydx.var
# dput(unique( c( names(get_icd_codes()), names(get_pccc_codes()), names(get_elixhauser_codes()), names(get_charlson_codes()))))
..protected_names.. <-
  c("icdv", "dx", "full_code", "code", "src", "known_start", "known_end",
    "assignable_start", "assignable_end", "condition", "subcondition",
    "transplant_flag", "tech_dep_flag",
    "pccc_v3.1", "pccc_v3.0", "pccc_v2.1", "pccc_v2.0",
    "elixhauser_ahrq_web", "elixhauser_elixhauser1988", "elixhauser_quan2005",
    "elixhauser_ahrq2022", "elixhauser_ahrq2023", "elixhauser_ahrq2024", "elixhauser_ahrq2025", "elixhauser_ahrq2026",
    "elixhauser_ahrq_icd10",
    "charlson_cdmf2019", "charlson_deyo1992", "charlson_quan2005", "charlson_quan2011"
  )

# No need to load and attach the namespace, everything in this test script is
# non-exported
# library(medicalcoder)
source('utilities.R')

dataframetools <-
  c("mdcr_set",
    "mdcr_select",
    "mdcr_subset",
    "mdcr_setorder",
    "mdcr_setnames",
    "mdcr_duplicated",
    "mdcr_unique",
    "mdcr_inner_join",
    "mdcr_full_outer_join",
    "mdcr_left_join",
    "mdcr_cbind"
  )

mdcr <- getNamespace("medicalcoder")

# are all the dataframetools in the namespcae
stopifnot(all(dataframetools %in% names(mdcr)))

# check that there are not unaccounted for data sets.  the ..mdcr_internal_
# prefix and .. suffix is expected.  noted in the data-raw/build_sysdata.R
stopifnot(
  all(
    grep("^mdcr_", names(mdcr), value = TRUE) %in% dataframetools
  )
)

################################################################################
# Set up data for testing
DF <- data.frame(A = 1:10, C = NA_integer_, B = LETTERS[1:10], stringsAsFactors = FALSE)
if (requireNamespace("dplyr", quietly = TRUE)) {
  TBL <- getExportedValue(name = "as_tibble", ns = "dplyr")(DF)
} else {
  TBL <- DF
}
if (requireNamespace("data.table", quietly = TRUE)) {
  DT <- getExportedValue(name = "copy", ns = "data.table")(DF)
  getExportedValue(name = "setDT", ns = "data.table")(DT)
} else {
  DT <- DF
}

################################################################################
# set the value of column C in row 5
DF  <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(DF,  i = 5L, j = "C", value = 3L)
TBL <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(TBL, i = 5L, j = "C", value = 3L)
DT  <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(DT,  i = 5L, j = "C", value = 3L)

stopifnot(
  identical(DF[["C"]],  c(rep(NA_integer_, 4L), 3L, rep(NA_integer_, 5L))),
  identical(TBL[["C"]], c(rep(NA_integer_, 4L), 3L, rep(NA_integer_, 5L))),
  identical(DT[["C"]],  c(rep(NA_integer_, 4L), 3L, rep(NA_integer_, 5L)))
)

# set the value in two rows at a time with one value
DF  <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(DF,  i = c(1L, 10L), j = "C", value = 8L)
TBL <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(TBL, i = c(1L, 10L), j = "C", value = 8L)
DT  <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(DT,  i = c(1L, 10L), j = "C", value = 8L)

stopifnot(
  identical(DF[["C"]],  c(8L, rep(NA_integer_, 3L), 3L, rep(NA_integer_, 4L), 8L)),
  identical(TBL[["C"]], c(8L, rep(NA_integer_, 3L), 3L, rep(NA_integer_, 4L), 8L)),
  identical(DT[["C"]],  c(8L, rep(NA_integer_, 3L), 3L, rep(NA_integer_, 4L), 8L))
)

# set the value in three rows at a time with three values
DF  <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(DF,  i = c(2L, 3L, 4L), j = "C", value = c(21L, 22L, 23L))
TBL <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(TBL, i = c(2L, 3L, 4L), j = "C", value = c(21L, 22L, 23L))
DT  <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(DT,  i = c(2L, 3L, 4L), j = "C", value = c(21L, 22L, 23L))

stopifnot(
  identical(DF[["C"]],  c(8L, 21L, 22L, 23L, 3L, rep(NA_integer_, 4L), 8L)),
  identical(TBL[["C"]], c(8L, 21L, 22L, 23L, 3L, rep(NA_integer_, 4L), 8L)),
  identical(DT[["C"]],  c(8L, 21L, 22L, 23L, 3L, rep(NA_integer_, 4L), 8L))
)

# set a full column
DF  <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(DF,  j = "A", value = as.integer(11:20))
TBL <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(TBL, j = "A", value = as.integer(11:20))
DT  <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(DT,  j = "A", value = as.integer(11:20))

stopifnot(
  identical(DF[["A"]],  as.integer(11:20)),
  identical(TBL[["A"]], as.integer(11:20)),
  identical(DT[["A"]],  as.integer(11:20))
)

# create a new column
x <- paste("v", c(0:5, 5, 6:8))
DF  <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(DF,  j = "D", value = x)
TBL <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(TBL, j = "D", value = x)
DT  <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(DT,  j = "D", value = x)

stopifnot(
  identical(DF[["D"]],  x),
  identical(TBL[["D"]], x),
  identical(DT[["D"]],  x)
)

# create a new column and only add a value to rows 2, 5, and 7
x <- c(NA_character_, "first", NA_character_, NA_character_, "second", NA_character_, "third", rep(NA_character_, nrow(DF) - 7L))
DF  <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(DF,  i = c(2L, 5L, 7L), j = "newC", value = c("first", "second", "third"))
TBL <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(TBL, i = c(2L, 5L, 7L), j = "newC", value = c("first", "second", "third"))
DT  <- getFromNamespace(x = "mdcr_set", ns = "medicalcoder")(DT,  i = c(2L, 5L, 7L), j = "newC", value = c("first", "second", "third"))

stopifnot(
  identical(DF[["newC"]],  x),
  identical(TBL[["newC"]], x),
  identical(DT[["newC"]],  x)
)

################################################################################
# testing mdcr_select
# set colummns - change the order of the columns
DF  <- getFromNamespace(x = "mdcr_select", ns = "medicalcoder")(DF,  col = c("D", "B", "C", "A"))
TBL <- getFromNamespace(x = "mdcr_select", ns = "medicalcoder")(TBL, col = c("D", "B", "C", "A"))
DT  <- getFromNamespace(x = "mdcr_select", ns = "medicalcoder")(DT,  col = c("D", "B", "C", "A"))

stopifnot(
  identical(names(DF),  c("D", "B", "C", "A")),
  identical(names(TBL), c("D", "B", "C", "A")),
  identical(names(DT),  c("D", "B", "C", "A"))
)

# retun the object if col is missing
stopifnot(
  identical(getFromNamespace(x = "mdcr_select", ns = "medicalcoder")(DF), DF),
  identical(getFromNamespace(x = "mdcr_select", ns = "medicalcoder")(TBL), TBL),
  identical(getFromNamespace(x = "mdcr_select", ns = "medicalcoder")(DT), DT)
)

# check for one column
DFD  <- getFromNamespace(x = "mdcr_select", ns = "medicalcoder")(DF,  col = c("D"))
TBLD <- getFromNamespace(x = "mdcr_select", ns = "medicalcoder")(TBL, col = c("D"))
DTD  <- getFromNamespace(x = "mdcr_select", ns = "medicalcoder")(DT,  col = c("D"))

stopifnot(
  identical(names(DFD),  c("D")),
  identical(names(TBLD), c("D")),
  identical(names(DTD),  c("D")),
  identical(DFD[["D"]], paste("v", c(0:5, 5:8))),
  identical(TBLD[["D"]], paste("v", c(0:5, 5:8))),
  identical(DTD[["D"]], paste("v", c(0:5, 5:8))),
  identical(class(DFD), class(DF)),
  identical(class(TBLD), class(TBL)),
  identical(class(DTD), class(DT))
)

################################################################################
# testing mdcr_subset

# if arg i and cols are missing then the object is retruned
stopifnot(
  identical(getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(DF), DF),
  identical(getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(TBL), TBL),
  identical(getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(DT), DT)
)

# with no row specfied, it is the same as calling mdcr_select
stopifnot(
  identical(
    getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(DF,  col = c("D", "B", "C", "A")),
    getFromNamespace(x = "mdcr_select", ns = "medicalcoder")(DF,  col = c("D", "B", "C", "A"))
  ),
  identical(
    getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(TBL, col = c("D", "B", "C", "A")),
    getFromNamespace(x = "mdcr_select", ns = "medicalcoder")(TBL, col = c("D", "B", "C", "A"))
  ),
  identical(
    getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(DT,  col = c("D", "B", "C", "A")),
    getFromNamespace(x = "mdcr_select", ns = "medicalcoder")(DT,  col = c("D", "B", "C", "A"))
  )
)

# select one column and a subset of rows
DF0 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(DF, i = seq(1, 10, by = 2), cols = c("B"))
DF1 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(DF, i = seq(2, 10, by = 2), cols = c("B"))
TBL0 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(TBL, i = seq(1, 10, by = 2), cols = c("B"))
TBL1 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(TBL, i = seq(2, 10, by = 2), cols = c("B"))
DT0 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(DT, i = seq(1, 10, by = 2), cols = c("B"))
DT1 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(DT, i = seq(2, 10, by = 2), cols = c("B"))

stopifnot(
  inherits(DF0, "data.frame"),
  inherits(DF1, "data.frame"),
  inherits(TBL0, "data.frame"),
  inherits(TBL1, "data.frame"),
  inherits(DT0, "data.frame"),
  inherits(DT1, "data.frame"),
  identical(names(DF0), "B"),
  identical(names(DF1), "B"),
  identical(names(TBL0), "B"),
  identical(names(TBL1), "B"),
  identical(names(DT0), "B"),
  identical(names(DT1), "B")
)

if (requireNamespace("data.table", quietly = TRUE)) {
  stopifnot(inherits(DT0, "data.table"), inherits(DT1, "data.table"))
}

if (requireNamespace("dplyr", quietly = TRUE)) {
  stopifnot(inherits(TBL0, "tbl_df"), inherits(TBL1, "tbl_df"))
}

# select two columns
DF0 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(DF, i = seq(1, 10, by = 2), cols = c("B", "A"))
DF1 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(DF, i = seq(2, 10, by = 2), cols = c("B", "A"))
TBL0 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(TBL, i = seq(1, 10, by = 2), cols = c("B", "A"))
TBL1 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(TBL, i = seq(2, 10, by = 2), cols = c("B", "A"))
DT0 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(DT, i = seq(1, 10, by = 2), cols = c("B", "A"))
DT1 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(DT, i = seq(2, 10, by = 2), cols = c("B", "A"))

stopifnot(
  inherits(DF0, "data.frame"),
  inherits(DF1, "data.frame"),
  inherits(TBL0, "data.frame"),
  inherits(TBL1, "data.frame"),
  inherits(DT0, "data.frame"),
  inherits(DT1, "data.frame"),
  identical(names(DF0), c("B", "A")),
  identical(names(DF1), c("B", "A")),
  identical(names(TBL0), c("B", "A")),
  identical(names(TBL1), c("B", "A")),
  identical(names(DT0), c("B", "A")),
  identical(names(DT1), c("B", "A"))
)

if (requireNamespace("data.table", quietly = TRUE)) {
  stopifnot(inherits(DT0, "data.table"), inherits(DT1, "data.table"))
}

if (requireNamespace("dplyr", quietly = TRUE)) {
  stopifnot(inherits(TBL0, "tbl_df"), inherits(TBL1, "tbl_df"))
}

# if only i is supplied, all the columns are returned
DF0 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(DF, i = seq(1, 10, by = 2))
DF1 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(DF, i = seq(2, 10, by = 2))
TBL0 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(TBL, i = seq(1, 10, by = 2))
TBL1 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(TBL, i = seq(2, 10, by = 2))
DT0 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(DT, i = seq(1, 10, by = 2))
DT1 <- getFromNamespace(x = "mdcr_subset", ns = "medicalcoder")(DT, i = seq(2, 10, by = 2))

stopifnot(
  inherits(DF0, "data.frame"),
  inherits(DF1, "data.frame"),
  inherits(TBL0, "data.frame"),
  inherits(TBL1, "data.frame"),
  inherits(DT0, "data.frame"),
  inherits(DT1, "data.frame"),
  identical(names(DF0), c("D", "B", "C", "A")),
  identical(names(DF1), c("D", "B", "C", "A")),
  identical(names(TBL0), c("D", "B", "C", "A")),
  identical(names(TBL1), c("D", "B", "C", "A")),
  identical(names(DT0), c("D", "B", "C", "A")),
  identical(names(DT1), c("D", "B", "C", "A"))
)

if (requireNamespace("data.table", quietly = TRUE)) {
  stopifnot(inherits(DT0, "data.table"), inherits(DT1, "data.table"))
}

if (requireNamespace("dplyr", quietly = TRUE)) {
  stopifnot(inherits(TBL0, "tbl_df"), inherits(TBL1, "tbl_df"))
}


################################################################################
# testing mdcr_setorder
DF <- rbind(DF0, DF1)
TBL <- rbind(TBL0, TBL1)
DT <- rbind(DT0, DT1)

# verify that the data.frames are not currently ordered by "A"

stopifnot(
  identical(DF[["A"]],  as.integer(c(11, 13, 15, 17, 19, 12, 14, 16, 18, 20))),
  identical(TBL[["A"]], as.integer(c(11, 13, 15, 17, 19, 12, 14, 16, 18, 20))),
  identical(DT[["A"]],  as.integer(c(11, 13, 15, 17, 19, 12, 14, 16, 18, 20))),
  identical(DF[["D"]],  paste("v", c(0, 2, 4, 5, 7, 1, 3, 5, 6, 8))),
  identical(TBL[["D"]], paste("v", c(0, 2, 4, 5, 7, 1, 3, 5, 6, 8))),
  identical(DT[["D"]],  paste("v", c(0, 2, 4, 5, 7, 1, 3, 5, 6, 8)))
)

DF <-  getFromNamespace(x = "mdcr_setorder", ns = "medicalcoder")(DF, by = c("A"))
TBL <- getFromNamespace(x = "mdcr_setorder", ns = "medicalcoder")(TBL, by = c("A"))
DT <-  getFromNamespace(x = "mdcr_setorder", ns = "medicalcoder")(DT, by = c("A"))

stopifnot(
  identical(DF[["A"]],  as.integer(11:20)),
  identical(TBL[["A"]], as.integer(11:20)),
  identical(DT[["A"]],  as.integer(11:20)),
  identical(DF[["D"]],  paste("v", c(0:5, 5, 6:8))),
  identical(TBL[["D"]], paste("v", c(0:5, 5, 6:8))),
  identical(DT[["D"]],  paste("v", c(0:5, 5, 6:8)))
)

################################################################################
# testing mdcr_setnames
DF  <- getFromNamespace(x = "mdcr_setnames", ns = "medicalcoder")(DF,  old = "A", new = "Column A")
TBL <- getFromNamespace(x = "mdcr_setnames", ns = "medicalcoder")(TBL, old = "A", new = "Column A")
DT  <- getFromNamespace(x = "mdcr_setnames", ns = "medicalcoder")(DT,  old = "A", new = "Column A")

stopifnot(
  identical(names(DF),  c("D", "B", "C", "Column A")),
  identical(names(TBL), c("D", "B", "C", "Column A")),
  identical(names(DT),  c("D", "B", "C", "Column A"))
)

################################################################################
# testing mdcr_duplicated

stopifnot(
  !any(getFromNamespace(x = "mdcr_duplicated", ns = "medicalcoder")(DF)),
  !any(getFromNamespace(x = "mdcr_duplicated", ns = "medicalcoder")(TBL)),
  !any(getFromNamespace(x = "mdcr_duplicated", ns = "medicalcoder")(DT))
)

expected <- rep(FALSE, 10)
expected[7] <- TRUE
stopifnot(
  identical(getFromNamespace(x = "mdcr_duplicated", ns = "medicalcoder")(DF, by = "D"), expected),
  identical(getFromNamespace(x = "mdcr_duplicated", ns = "medicalcoder")(TBL, by = "D"), expected),
  identical(getFromNamespace(x = "mdcr_duplicated", ns = "medicalcoder")(DT, by = "D"), expected)
)

# check fromLast
expected[6:7] <- !expected[6:7]
stopifnot(
  identical(getFromNamespace(x = "mdcr_duplicated", ns = "medicalcoder")(DF, by = "D", fromLast = TRUE), expected),
  identical(getFromNamespace(x = "mdcr_duplicated", ns = "medicalcoder")(TBL, by = "D", fromLast = TRUE), expected),
  identical(getFromNamespace(x = "mdcr_duplicated", ns = "medicalcoder")(DT, by = "D", fromLast = TRUE), expected)
)

################################################################################
# testing mdcr_inner_join
#
# without specifying the by
t0 <- getFromNamespace(x = "mdcr_inner_join", ns = "medicalcoder")(DF, DF[2, ])
t1 <- getFromNamespace(x = "mdcr_inner_join", ns = "medicalcoder")(DT, DT[2, ])
t2 <- getFromNamespace(x = "mdcr_inner_join", ns = "medicalcoder")(TBL, TBL[2, ])

stopifnot(
  isTRUE( all.equal(t0, DF[2, ], check.attributes = FALSE)),
  isTRUE( all.equal(t1, DT[2, ], check.attributes = FALSE)),
  isTRUE( all.equal(t2, TBL[2, ], check.attributes = FALSE))
)

t3 <- getFromNamespace(x = "mdcr_inner_join", ns = "medicalcoder")(DF, DF[c(2, 5, 1), ])
t4 <- getFromNamespace(x = "mdcr_inner_join", ns = "medicalcoder")(DT, DT[c(2, 5, 1), ])
t5 <- getFromNamespace(x = "mdcr_inner_join", ns = "medicalcoder")(TBL, TBL[c(2, 5, 1), ])

stopifnot(
  isTRUE( all.equal( t3, DF[c(1, 2, 5), ], check.attributes = FALSE)),
  isTRUE( all.equal( t4, DT[c(1, 2, 5), ], check.attributes = FALSE)),
  isTRUE( all.equal( t5, TBL[c(1, 2, 5), ], check.attributes = FALSE))
  )

# test with a by statement and suffixes statement
expected_df <-
  data.frame(
    x1 = c(1L, 2L, 8L),
    x2.right = c("A", "B", "C"),
    x2.left  = c("a", "b", "c"),
    stringsAsFactors = FALSE
  )
r <- data.frame(
  x1 = as.integer(1:10),
  x2 = c("A", "B", "D", "E", "F", "T", "A", "C", "9", "ten"),
  stringsAsFactors = FALSE
)
l <- data.frame(
  x1 = as.integer(c(1, 2, 33, 44, 55, 66, 77, 8, 99, 1100)),
  x2 = c("a", "b", "d", "e", "f", "t", "a", "c", "9", "TEN"),
  stringsAsFactors = FALSE
)
outDF <- getFromNamespace(x = "mdcr_inner_join", ns = "medicalcoder")(r, l, by = "x1", suffixes = c(".right", ".left"))
stopifnot(identical(outDF, expected_df))

if (requireNamespace("data.table", quietly = TRUE)) {
  data.table::setDT(r)
  data.table::setDT(l)
  expected_dt <- data.table::copy(expected_df)
  data.table::setDT(expected_dt)
} else {
  r <- as.data.frame(r, stringsAsFactors = FALSE)
  l <- as.data.frame(l, stringsAsFactors = FALSE)
  expected_dt <- expected_df
}
outDT <- getFromNamespace(x = "mdcr_inner_join", ns = "medicalcoder")(r, l, by = "x1", suffixes = c(".right", ".left"))
stopifnot(identical(outDT, expected_dt))

if (requireNamespace("dplyr", quietly = TRUE)) {
  r <- dplyr::as_tibble(r)
  l <- dplyr::as_tibble(l)
  expected_tb <- dplyr::as_tibble(expected_df)
} else {
  r <- as.data.frame(r, stringsAsFactors = FALSE)
  l <- as.data.frame(l, stringsAsFactors = FALSE)
  expected_tb <- expected_df
}
outTBL <- getFromNamespace(x = "mdcr_inner_join", ns = "medicalcoder")(r, l, by = "x1", suffixes = c(".right", ".left"))
stopifnot(identical(outTBL, expected_tb))

# test with by.x and by.y statements
r <- data.frame(
  x1 = as.integer(1:10),
  x2 = c("A", "B", "D", "E", "F", "T", "A", "C", "9", "ten"),
  stringsAsFactors = FALSE
)
l <- data.frame(
  z = as.integer(c(1, 2, 33, 44, 55, 66, 77, 8, 99, 1100)),
  x2 = c("a", "b", "d", "e", "f", "t", "a", "c", "9", "TEN"),
  stringsAsFactors = FALSE
)
outDF <- getFromNamespace(x = "mdcr_inner_join", ns = "medicalcoder")(r, l, by.x = "x1", by.y = "z", suffixes = c(".right", ".left"))
stopifnot(identical(outDF, expected_df))

if (requireNamespace("data.table", quietly = TRUE)) {
  data.table::setDT(r)
  data.table::setDT(l)
  expected_dt <- data.table::copy(expected_df)
  data.table::setDT(expected_dt)
} else {
  r <- as.data.frame(r, stringsAsFactors = FALSE)
  l <- as.data.frame(l, stringsAsFactors = FALSE)
  expected_dt <- expected_df
}
outDT <- getFromNamespace(x = "mdcr_inner_join", ns = "medicalcoder")(r, l, by.x = "x1", by.y = "z", suffixes = c(".right", ".left"))
stopifnot(identical(outDT, expected_dt))

if (requireNamespace("dplyr", quietly = TRUE)) {
  r <- dplyr::as_tibble(r)
  l <- dplyr::as_tibble(l)
  expected_tb <- dplyr::as_tibble(expected_df)
} else {
  r <- as.data.frame(r, stringsAsFactors = FALSE)
  l <- as.data.frame(l, stringsAsFactors = FALSE)
  expected_tb <- expected_df
}
outTBL <- getFromNamespace(x = "mdcr_inner_join", ns = "medicalcoder")(r, l, by.x = "x1", by.y = "z", suffixes = c(".right", ".left"))
stopifnot(identical(outTBL, expected_tb))

################################################################################
# testing mdcr_left_join

# These wrappers around merge have sort = FALSE and dplyr::left_join doesn't
# sort the return by default.  So, build the merge, sort the result, and then
# test for the outcome.  The first set of tests for a single

t0 <- getFromNamespace(x = "mdcr_left_join", ns = "medicalcoder")(DF, DF[2, ])
t0 <- t0[do.call(order, t0), ]

t1 <- getFromNamespace(x = "mdcr_left_join", ns = "medicalcoder")(DF, DF[c(2, 5, 2), ])
t1 <- t1[do.call(order, t1), ]

t2 <- getFromNamespace(x = "mdcr_left_join", ns = "medicalcoder")(DT, DT[2, ])
t2 <- t2[do.call(order, t2), ]

t3 <- getFromNamespace(x = "mdcr_left_join", ns = "medicalcoder")(DT, DT[c(2, 5, 2), ])
t3 <- t3[do.call(order, t3), ]

t4 <- getFromNamespace(x = "mdcr_left_join", ns = "medicalcoder")(TBL, TBL[2, ])
t4 <- t4[do.call(order, t4), ]

t5 <- getFromNamespace(x = "mdcr_left_join", ns = "medicalcoder")(TBL, TBL[c(2, 5, 2), ])
t5 <- t5[do.call(order, t5), ]

# without specifying the by
stopifnot(
  isTRUE( all.equal( t0, DF, check.attributes = FALSE)),
  isTRUE( all.equal( t1, DF[c(1, 2, 2, 3:nrow(DF)), ], check.attributes = FALSE)),
  isTRUE( all.equal( t2, DT, check.attributes = FALSE)),
  isTRUE( all.equal( t3, DT[c(1, 2, 2, 3:nrow(DF)), ], check.attributes = FALSE)),
  isTRUE( all.equal( t4, TBL, check.attributes = FALSE)),
  isTRUE( all.equal( t5, TBL[c(1, 2, 2, 3:nrow(DF)), ], check.attributes = FALSE))
)

# tests with by statements and suffixes
expected_df <-
  data.frame(
    x1 = as.integer(1:10),
    x2.right = c("A", "B", "D", "E", "F", "T", "A", "C", "9", "ten"),
    x2.left  = c("a", "b", rep(NA_character_, 5), "c", rep(NA_character_, 2)),
    stringsAsFactors = FALSE
  )
r <- data.frame(
  x1 = as.integer(1:10),
  x2 = c("A", "B", "D", "E", "F", "T", "A", "C", "9", "ten"),
  stringsAsFactors = FALSE
)
l <- data.frame(
  x1 = as.integer(c(1, 2, 33, 44, 55, 66, 77, 8, 99, 1100)),
  x2 = c("a", "b", "d", "e", "f", "t", "a", "c", "9", "TEN"),
  stringsAsFactors = FALSE
)
outDF <- getFromNamespace(x = "mdcr_left_join", ns = "medicalcoder")(r, l, by = "x1", suffixes = c(".right", ".left"))
outDF <- outDF[order(outDF$x1), ]
rownames(outDF) <- NULL
stopifnot(identical(outDF, expected_df))

if (requireNamespace("data.table", quietly = TRUE)) {
  data.table::setDT(r)
  data.table::setDT(l)
  expected_dt <- data.table::copy(expected_df)
  data.table::setDT(expected_dt)
} else {
  r <- as.data.frame(r, stringsAsFactors = FALSE)
  l <- as.data.frame(l, stringsAsFactors = FALSE)
  expected_dt <- expected_df
}
outDT <- getFromNamespace(x = "mdcr_left_join", ns = "medicalcoder")(r, l, by = "x1", suffixes = c(".right", ".left"))
outDT <- outDT[order(outDT$x1), ]
rownames(outDT) <- NULL
stopifnot(identical(outDT, expected_dt))

if (requireNamespace("dplyr", quietly = TRUE)) {
  r <- dplyr::as_tibble(r)
  l <- dplyr::as_tibble(l)
  expected_tb <- dplyr::as_tibble(expected_df)
} else {
  r <- as.data.frame(r, stringsAsFactors = FALSE)
  l <- as.data.frame(l, stringsAsFactors = FALSE)
  expected_tb <- expected_df
}
outTBL <- getFromNamespace(x = "mdcr_left_join", ns = "medicalcoder")(r, l, by = "x1", suffixes = c(".right", ".left"))
outTBL <- outTBL[order(outTBL$x1), ]
rownames(outTBL) <- NULL
stopifnot(identical(outTBL, expected_tb))

# tests with by.x and by.x statements and suffixes
expected_df <-
  data.frame(
    x1 = as.integer(1:10),
    x2.right = c("A", "B", "D", "E", "F", "T", "A", "C", "9", "ten"),
    x2.left  = c("a", "b", rep(NA_character_, 5), "c", rep(NA_character_, 2)),
    stringsAsFactors = FALSE
  )
r <- data.frame(
  x1 = as.integer(1:10),
  x2 = c("A", "B", "D", "E", "F", "T", "A", "C", "9", "ten"),
  stringsAsFactors = FALSE
)
l <- data.frame(
  l1 = as.integer(c(1, 2, 33, 44, 55, 66, 77, 8, 99, 1100)),
  x2 = c("a", "b", "d", "e", "f", "t", "a", "c", "9", "TEN"),
  stringsAsFactors = FALSE
)
outDF <- getFromNamespace(x = "mdcr_left_join", ns = "medicalcoder")(r, l, by.x = "x1", by.y = "l1", suffixes = c(".right", ".left"))
outDF <- outDF[order(outDF$x1), ]
rownames(outDF) <- NULL
stopifnot(identical(outDF, expected_df))

if (requireNamespace("data.table", quietly = TRUE)) {
  data.table::setDT(r)
  data.table::setDT(l)
  expected_dt <- data.table::copy(expected_df)
  data.table::setDT(expected_dt)
} else {
  r <- as.data.frame(r, stringsAsFactors = FALSE)
  l <- as.data.frame(l, stringsAsFactors = FALSE)
  expected_dt <- expected_df
}
outDT <- getFromNamespace(x = "mdcr_left_join", ns = "medicalcoder")(r, l, by.x = "x1", by.y = "l1", suffixes = c(".right", ".left"))
outDT <- outDT[order(outDT$x1), ]
rownames(outDT) <- NULL
stopifnot(identical(outDT, expected_dt))

if (requireNamespace("dplyr", quietly = TRUE)) {
  r <- dplyr::as_tibble(r)
  l <- dplyr::as_tibble(l)
  expected_tb <- dplyr::as_tibble(expected_df)
} else {
  r <- as.data.frame(r, stringsAsFactors = FALSE)
  l <- as.data.frame(l, stringsAsFactors = FALSE)
  expected_tb <- expected_df
}
outTBL <- getFromNamespace(x = "mdcr_left_join", ns = "medicalcoder")(r, l, by.x = "x1", by.y = "l1", suffixes = c(".right", ".left"))
outTBL <- outTBL[order(outTBL$x1), ]
rownames(outTBL) <- NULL
stopifnot(identical(outTBL, expected_tb))

################################################################################
# testing mdcr_full_outer_join

expected_df <-
  data.frame(
    x1 = c(1L, 1L, 1L, 2L, 2L, 3L, 4L, 5L, 6L, 7L, 8L, 8L, 9L, 10L, 33L, 44L, 55L, 66L, 77L, 99L, 1100L),
    x2 = c("A", "a", "a1", "B", "b", "D", "E", "F", "T", "A", "C", "c", "9", "ten", "d", "e", "f", "t", "a", "9", "TEN"),
    stringsAsFactors = FALSE
  )
r <- data.frame(
  x1 = as.integer(1:10),
  x2 = c("A", "B", "D", "E", "F", "T", "A", "C", "9", "ten"),
  stringsAsFactors = FALSE
)
l <- data.frame(
  x1 = as.integer(c(1, 1, 2, 33, 44, 55, 66, 77, 8, 99, 1100)),
  x2 = c("a", "a1", "b", "d", "e", "f", "t", "a", "c", "9", "TEN"),
  stringsAsFactors = FALSE
)
outDF <- getFromNamespace(x = "mdcr_full_outer_join", ns = "medicalcoder")(r, l)
outDF <- outDF[order(outDF$x1), ]
rownames(outDF) <- NULL
stopifnot(identical(outDF, expected_df))

if (requireNamespace("data.table", quietly = TRUE)) {
  data.table::setDT(r)
  data.table::setDT(l)
  expected_dt <- data.table::copy(expected_df)
  data.table::setDT(expected_dt)
} else {
  r <- as.data.frame(r, stringsAsFactors = FALSE)
  l <- as.data.frame(l, stringsAsFactors = FALSE)
  expected_dt <- expected_df
}
outDT <- getFromNamespace(x = "mdcr_full_outer_join", ns = "medicalcoder")(r, l)
outDT <- outDT[order(outDT$x1), ]
rownames(outDT) <- NULL
stopifnot(identical(outDT, expected_dt))

if (requireNamespace("dplyr", quietly = TRUE)) {
  r <- dplyr::as_tibble(r)
  l <- dplyr::as_tibble(l)
  expected_tb <- dplyr::as_tibble(expected_df)
} else {
  r <- as.data.frame(r, stringsAsFactors = FALSE)
  l <- as.data.frame(l, stringsAsFactors = FALSE)
  expected_tb <- expected_df
}
outTBL <- getFromNamespace(x = "mdcr_full_outer_join", ns = "medicalcoder")(r, l)
outTBL <- outTBL[order(outTBL$x1), ]
rownames(outTBL) <- NULL
stopifnot(identical(outTBL, expected_tb))

# tests with by statements and suffixes
expected_df <-
  data.frame(
    x1 = as.integer(c(1, 1:10, 33, 44, 55, 66, 77, 99, 1100)),
    x2.right = c("A", "A", "B", "D", "E", "F", "T", "A", "C", "9", "ten", rep(NA_character_, 7)),
    x2.left  = c("a", "a1", "b", rep(NA_character_, 5), "c", rep(NA_character_, 2), "d", "e", "f", "t", "a", "9", "TEN"),
    stringsAsFactors = FALSE
  )
r <- data.frame(
  x1 = as.integer(1:10),
  x2 = c("A", "B", "D", "E", "F", "T", "A", "C", "9", "ten"),
  stringsAsFactors = FALSE
)
l <- data.frame(
  x1 = as.integer(c(1, 1, 2, 33, 44, 55, 66, 77, 8, 99, 1100)),
  x2 = c("a", "a1", "b", "d", "e", "f", "t", "a", "c", "9", "TEN"),
  stringsAsFactors = FALSE
)
outDF <- getFromNamespace(x = "mdcr_full_outer_join", ns = "medicalcoder")(r, l, by = "x1", suffixes = c(".right", ".left"))
outDF <- outDF[order(outDF$x1), ]
rownames(outDF) <- NULL
stopifnot(identical(outDF, expected_df))

if (requireNamespace("data.table", quietly = TRUE)) {
  data.table::setDT(r)
  data.table::setDT(l)
  expected_dt <- data.table::copy(expected_df)
  data.table::setDT(expected_dt)
} else {
  r <- as.data.frame(r, stringsAsFactors = FALSE)
  l <- as.data.frame(l, stringsAsFactors = FALSE)
  expected_dt <- expected_df
}
outDT <- getFromNamespace(x = "mdcr_full_outer_join", ns = "medicalcoder")(r, l, by = "x1", suffixes = c(".right", ".left"))
outDT <- outDT[order(outDT$x1), ]
rownames(outDT) <- NULL
stopifnot(identical(outDT, expected_dt))

if (requireNamespace("dplyr", quietly = TRUE)) {
  r <- dplyr::as_tibble(r)
  l <- dplyr::as_tibble(l)
  expected_tb <- dplyr::as_tibble(expected_df)
} else {
  r <- as.data.frame(r, stringsAsFactors = FALSE)
  l <- as.data.frame(l, stringsAsFactors = FALSE)
  expected_tb <- expected_df
}
outTBL <- getFromNamespace(x = "mdcr_full_outer_join", ns = "medicalcoder")(r, l, by = "x1", suffixes = c(".right", ".left"))
outTBL <- outTBL[order(outTBL$x1), ]
rownames(outTBL) <- NULL
stopifnot(identical(outTBL, expected_tb))

################################################################################
# testing mdcr_unique
DF <- expand.grid(x1 = LETTERS, x2 = LETTERS)
DF <- rbind(DF, DF, DF, DF, DF, DF)
DF <- DF[sample(seq_len(nrow(DF))), ]
rownames(DF) <- NULL
if (requireNamespace("dplyr", quietly = TRUE)) {
  TBL <- getExportedValue(ns = "dplyr", name = "as_tibble")(DF)
} else {
  TBL <- DF
  class(TBL) <- c("tbl_df", class(TBL))
}
if (requireNamespace("data.table", quietly = TRUE)) {
  DT <- getExportedValue(name = "copy", ns = "data.table")(DF)
  getExportedValue(name = "setDT", ns = "data.table")(DT)
} else {
  DT <- DF
  class(DT) <- c("data.table", class(DT))
}

uDF <- unique(DF)

stopifnot(
  all.equal(uDF, getFromNamespace(x = "mdcr_unique", ns = "medicalcoder")(DF), check.attributes = FALSE),
  all.equal(uDF, getFromNamespace(x = "mdcr_unique", ns = "medicalcoder")(DT), check.attributes = FALSE),
  all.equal(uDF, getFromNamespace(x = "mdcr_unique", ns = "medicalcoder")(TBL), check.attributes = FALSE)
)


################################################################################
# testing mdcr_cbind

################################################################################
#                                 End of File                                  #
################################################################################

#' Get Elixhauser Present-on-Admission Requirements
#'
#' Retrieve a copy of internal lookup table with details on which Elixhauser
#' comorbidities do and do not require the associated ICD codes to be
#' present-on-admission to be flagged.
#'
#' @seealso
#' * [`get_elixhauser_index_scores()`] for the lookup table of the condition by
#'   condition scores for mortality and readmission indices.
#' * [`get_elixhauser_codes()`] for the lookup table of ICD codes mapping to the
#'   Elixhauser comorbidities.
#' * [comorbidities()] for applying comorbidity algorithms to a data set.
#' 
#' @return A `data.frame` with the following columns:
#' * `condition`: Character vector of the conditions
#' * `desc`: Character vector with a verbose description of the condition
#' * `poa_required`: Integer indicators if the code needs to present on admission to be considered a comorbidity
#' * `elixhauser_<variant>`: indicators for the Elixhauser `<variant>`
#'
#' @examples
#' head(get_elixhauser_poa())
#' str(get_elixhauser_poa())
#'
#' @export
get_elixhauser_poa <- function() {
  unserialize(serialize(..mdcr_internal_elixhauser_poa.., connection = NULL))
}

################################################################################
#                                 End of File                                  #
################################################################################

#' Get Charlson Index Scores
#'
#' Retrieve a copy of internal lookup tables of index scores used in assessing
#' Charlson comorbidities.
#'
#' @seealso
#' * [`get_charlson_codes()`] for a lookup table of the ICD codes mapping to the
#'   Charlson comorbidities.
#' * [comorbidities()] for applying comorbidity algorithms to a data set.
#'
#' @return A `data.frame` with the following columns:
#' 
#' * `condition`: Character vector of the conditions
#' * `index`: Character vector indicating if the score is for the mortality or the readmission index score
#' * `charlson_<variant>`:  the index scores for the variant
#'
#' @examples
#' head(get_charlson_index_scores())
#' str(get_charlson_index_scores())
#'
#' @export
get_charlson_index_scores <- function() {
  unserialize(serialize(..mdcr_internal_charlson_index_scores.., connection = NULL))
}

################################################################################
#                                 End of File                                  #
################################################################################

source('utilities.R')
################################################################################
library(medicalcoder)
# build and test the longitudinal flagging of conditions.  The renal code will
# be provided twice, once as poa and once as not poa.   This should flag on the
# encounter as expected.  the duplication is here to see if there is a any bug
# in the code that would get messed up by a 0 and a 1 poa for a conditon on an
# encounter.

# C78.4: Cancer
#   - Charlson (Quan 2005)
#     - mst
#   - Elixhauser (AHRQ 2025)
#     - does not need to be POA to count for Elixhauser
#     - CANCER_METS
#   - PCCC v3.1
#     - malignancy_dxpr_only
#     - malignancy_dxpr_tech_only
#     - malignancy_dxpr_and_tech
#     - malignancy_dxpr_or_tech
#
# I50.40: heart failure, or cardiovasular disease
#   - Charlson (Quan 2005)
#     - chf
#   - Elixhauser (AHRQ 2025)
#     - does need to be POA to count for Elixhauser
#     - HF
#   - PCCC v3.1
#     - cvd_dxpr_only
#     - cvd_dxpr_tech_only
#     - cvd_dxpr_and_tech
#     - cvd_dxpr_or_tech
#
# N18.4: renal
#   - Charlson (Quan 2005)
#     - rnd
#   - Elixhauser (AHRQ 2025)
#     - does need to be POA to count for Elixhauser
#     - RENLFL_SEV
#   - PCCC v3.1
#     - renal_dxpr_only
#     - renal_dxpr_tech_only
#     - renal_dxpr_and_tech
#     - renal_dxpr_or_tech

record <-
  structure(
    list(
      patid = c("A", "A", "A", "A", "A", "A", "A", "A", "A", "B", "B", "B", "B", "B", "B", "B"),
      encid = c(1L, 2L, 3L, 4L, 4L, 5L, 5L, 6L, 7L, 1L, 1L, 1L, 2L, 3L, 4L, 5L),
      code  = c(NA, "C78.4", "I50.40", "N18.4", "N18.4", "C78.4", "I50.40", NA, "-", NA, "N18.4", "N18.4", NA, NA, "N18.4", NA),
      poa = c(NA, 0L, 1L, 1L, 0L, 1L, 0L, NA, 1L, NA, 1L, 0L, NA, NA, 0L, NA)
    ),
  class = "data.frame",
  row.names = c(NA, -16L)
)


# set the data in an unsorted order to verify that the output will be sorted and
# as expected.
set.seed(42)
rws <- c(sample(seq_len(nrow(record))),
         sample(seq_len(nrow(record))),
         sample(seq_len(nrow(record))))
record <- record[rws, ]

recordDT <- record
recordTBL <- record

class(recordDT) <- c("data.table", class(recordDT))
class(recordTBL) <- c("tbl_df", "tbl", class(recordTBL))

################################################################################
# Expected results

expected_zeros <- rep(0L, 12L)
expected_patid <- rep(c("A", "B"), times = c(7, 5))
expected_encid <- c(1:7, 1:5)

################################################################################
# arguments for the comorbidities call
args <-
  list(
       data = record,
       icd.codes = "code",
       id.vars = c("patid", "encid"),
       icdv = 10L,
       dx = 1L,
       full.codes = TRUE,
       compact.codes = FALSE
  )

CMRBS <-
  list(
       charlson_current_0 = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "current", primarydx = 0L, poa = 0))),
       charlson_current_1 = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "current", primarydx = 0L, poa = 1))),
       charlson_current_v = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "current", primarydx = 0L, poa.var = "poa"))),

       charlson_cumulative_0 = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "cumulative", primarydx = 0L, poa = 0))),
       charlson_cumulative_1 = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "cumulative", primarydx = 0L, poa = 1))),
       charlson_cumulative_v = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "cumulative", primarydx = 0L, poa.var = "poa"))),

       pccc_current_0 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa = 0))),
       pccc_current_1 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa = 1))),
       pccc_current_v = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa.var = "poa"))),

       pccc_cumulative_0 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa = 0))),
       pccc_cumulative_1 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa = 1))),
       pccc_cumulative_v = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa.var = "poa"))),

       spccc_current_0 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa = 0, subconditions = TRUE))),
       spccc_current_1 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa = 1, subconditions = TRUE))),
       spccc_current_v = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa.var = "poa", subconditions = TRUE))),

       spccc_cumulative_0 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa = 0, subconditions = TRUE))),
       spccc_cumulative_1 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa = 1, subconditions = TRUE))),
       spccc_cumulative_v = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa.var = "poa", subconditions = TRUE))),

       elixhauser_current_0 = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "current", primarydx = 0L, poa = 0))),
       elixhauser_current_1 = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "current", primarydx = 0L, poa = 1))),
       elixhauser_current_v = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "current", primarydx = 0L, poa.var = "poa"))),

       elixhauser_cumulative_0 = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "cumulative", primarydx = 0L, poa = 0))),
       elixhauser_cumulative_1 = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "cumulative", primarydx = 0L, poa = 1))),
       elixhauser_cumulative_v = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "cumulative", primarydx = 0L, poa.var = "poa")))
)

args[["data"]] <- recordDT
CMRBSDT <-
  list(
       charlson_current_0 = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "current", primarydx = 0L, poa = 0))),
       charlson_current_1 = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "current", primarydx = 0L, poa = 1))),
       charlson_current_v = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "current", primarydx = 0L, poa.var = "poa"))),

       charlson_cumulative_0 = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "cumulative", primarydx = 0L, poa = 0))),
       charlson_cumulative_1 = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "cumulative", primarydx = 0L, poa = 1))),
       charlson_cumulative_v = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "cumulative", primarydx = 0L, poa.var = "poa"))),

       pccc_current_0 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa = 0))),
       pccc_current_1 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa = 1))),
       pccc_current_v = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa.var = "poa"))),

       pccc_cumulative_0 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa = 0))),
       pccc_cumulative_1 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa = 1))),
       pccc_cumulative_v = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa.var = "poa"))),

       spccc_current_0 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa = 0, subconditions = TRUE))),
       spccc_current_1 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa = 1, subconditions = TRUE))),
       spccc_current_v = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa.var = "poa", subconditions = TRUE))),

       spccc_cumulative_0 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa = 0, subconditions = TRUE))),
       spccc_cumulative_1 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa = 1, subconditions = TRUE))),
       spccc_cumulative_v = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa.var = "poa", subconditions = TRUE))),

       elixhauser_current_0 = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "current", primarydx = 0L, poa = 0))),
       elixhauser_current_1 = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "current", primarydx = 0L, poa = 1))),
       elixhauser_current_v = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "current", primarydx = 0L, poa.var = "poa"))),

       elixhauser_cumulative_0 = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "cumulative", primarydx = 0L, poa = 0))),
       elixhauser_cumulative_1 = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "cumulative", primarydx = 0L, poa = 1))),
       elixhauser_cumulative_v = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "cumulative", primarydx = 0L, poa.var = "poa")))
)

args[["data"]] <- recordTBL
CMRBSTBL <-
  list(
       charlson_current_0 = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "current", primarydx = 0L, poa = 0))),
       charlson_current_1 = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "current", primarydx = 0L, poa = 1))),
       charlson_current_v = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "current", primarydx = 0L, poa.var = "poa"))),

       charlson_cumulative_0 = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "cumulative", primarydx = 0L, poa = 0))),
       charlson_cumulative_1 = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "cumulative", primarydx = 0L, poa = 1))),
       charlson_cumulative_v = do.call(comorbidities, c(args, list(method = "charlson_quan2005", flag.method = "cumulative", primarydx = 0L, poa.var = "poa"))),

       pccc_current_0 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa = 0))),
       pccc_current_1 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa = 1))),
       pccc_current_v = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa.var = "poa"))),

       pccc_cumulative_0 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa = 0))),
       pccc_cumulative_1 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa = 1))),
       pccc_cumulative_v = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa.var = "poa"))),

       spccc_current_0 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa = 0, subconditions = TRUE))),
       spccc_current_1 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa = 1, subconditions = TRUE))),
       spccc_current_v = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "current", poa.var = "poa", subconditions = TRUE))),

       spccc_cumulative_0 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa = 0, subconditions = TRUE))),
       spccc_cumulative_1 = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa = 1, subconditions = TRUE))),
       spccc_cumulative_v = do.call(comorbidities, c(args, list(method = "pccc_v3.1", flag.method = "cumulative", poa.var = "poa", subconditions = TRUE))),

       elixhauser_current_0 = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "current", primarydx = 0L, poa = 0))),
       elixhauser_current_1 = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "current", primarydx = 0L, poa = 1))),
       elixhauser_current_v = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "current", primarydx = 0L, poa.var = "poa"))),

       elixhauser_cumulative_0 = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "cumulative", primarydx = 0L, poa = 0))),
       elixhauser_cumulative_1 = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "cumulative", primarydx = 0L, poa = 1))),
       elixhauser_cumulative_v = do.call(comorbidities, c(args, list(method = "elixhauser_ahrq2025", flag.method = "cumulative", primarydx = 0L, poa.var = "poa")))
)

CMRBS <- list(DF = CMRBS, DT = CMRBSDT, TBL = CMRBSTBL)

################################################################################
# for the testing - check for the expected results and then remove the object
# from the CMRBS until it is empty.

# PCCC Subconditions
#
# The PCCC with subconditions should have the same comorbidities as the objects
# without subconditions.
for (i in seq_len(length(CMRBS))) {
  for (x in grep("^pccc_", names(CMRBS[[i]]), value = TRUE)) {
    check <-
      all.equal(
        target = CMRBS[[i]][[x]],
        current = CMRBS[[i]][[paste0("s", x)]][["conditions"]],
        check.attributes = FALSE
      )
    if (check) {
      CMRBS[[i]][[paste0("s", x)]][["conditions"]] <- NULL
    } else {
      stop(sprintf('CMRBS[[%d]][[s%s]][["conditions"]] is not all.equal to CMRBS[[%s]]', i, x, x))
    }
  }
}

# Check each of the subconditions.  For the following conditions, all of which
# should not be flagged, the number of rows in the output should be zero.
for (i in seq_len(length(CMRBS))) {
  for (cnd in c("respiratory", "neuromusc", "neonatal", "misc", "metabolic", "hemato_immu", "gi", "congeni_genetic")) {
    stopifnot(
      nrow(CMRBS[[i]][["spccc_current_0"]][["subconditions"]][[cnd]]) == 0L,
      nrow(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][[cnd]]) == 0L,
      nrow(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][[cnd]]) == 0L,
      nrow(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][[cnd]]) == 0L,
      nrow(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][[cnd]]) == 0L,
      nrow(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][[cnd]]) == 0L
    )
    CMRBS[[i]][["spccc_current_0"]][["subconditions"]][[cnd]] <- NULL
    CMRBS[[i]][["spccc_current_1"]][["subconditions"]][[cnd]] <- NULL
    CMRBS[[i]][["spccc_current_v"]][["subconditions"]][[cnd]] <- NULL
    CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][[cnd]] <- NULL
    CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][[cnd]] <- NULL
    CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][[cnd]] <- NULL
  }
}

# Specific checks for cvd
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(nrow(CMRBS[[i]][["spccc_current_0"]][["subconditions"]][["cvd"]]), 0L))
  CMRBS[[i]][["spccc_current_0"]][["subconditions"]][["cvd"]] <- NULL

  stopifnot(
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["cvd"]][["patid"]], c("A", "A")),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["cvd"]][["encid"]], c(3L, 5L)),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["cvd"]][["cardiomyopathies"]], c(0L, 0L)),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["cvd"]][["conduction_disorder"]], c(0L, 0L)),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["cvd"]][["device_and_technology_use"]], c(0L, 0L)),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["cvd"]][["dysrhythmias"]], c(0L, 0L)),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["cvd"]][["endocardium_diseases"]], c(0L, 0L)),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["cvd"]][["heart_and_great_vessel_malformations"]], c(0L, 0L)),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["cvd"]][["transplantation"]], c(0L, 0L)),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["cvd"]][["other"]], c(1L, 1L))
  )
  CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["cvd"]] <- NULL

  stopifnot(
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["cvd"]][["patid"]], c("A")),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["cvd"]][["encid"]], c(3L)),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["cvd"]][["cardiomyopathies"]], c(0L)),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["cvd"]][["conduction_disorder"]], c(0L)),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["cvd"]][["device_and_technology_use"]], c(0L)),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["cvd"]][["dysrhythmias"]], c(0L)),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["cvd"]][["endocardium_diseases"]], c(0L)),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["cvd"]][["heart_and_great_vessel_malformations"]], c(0L)),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["cvd"]][["transplantation"]], c(0L)),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["cvd"]][["other"]], c(1L))
  )
  CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["cvd"]] <- NULL

  stopifnot(
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["cvd"]][["patid"]], rep("A", 4)),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["cvd"]][["encid"]], 4:7),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["cvd"]][["other"]], rep(c(1L), 4L)),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["cvd"]][["cardiomyopathies"]], rep(c(0L), 4L)),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["cvd"]][["conduction_disorder"]], rep(c(0L), 4L)),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["cvd"]][["device_and_technology_use"]], rep(c(0L), 4L)),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["cvd"]][["dysrhythmias"]], rep(c(0L), 4L)),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["cvd"]][["endocardium_diseases"]], rep(c(0L), 4L)),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["cvd"]][["heart_and_great_vessel_malformations"]], rep(c(0L), 4L)),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["cvd"]][["transplantation"]], rep(c(0L), 4L)),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["cvd"]][["other"]], rep(c(1L), 4L))
  )
  CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["cvd"]] <- NULL

  stopifnot(
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["cvd"]][["patid"]], rep("A", 5)),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["cvd"]][["encid"]], 3:7),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["cvd"]][["cardiomyopathies"]], rep(c(0L), 5L)),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["cvd"]][["conduction_disorder"]], rep(c(0L), 5L)),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["cvd"]][["device_and_technology_use"]], rep(c(0L), 5L)),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["cvd"]][["dysrhythmias"]], rep(c(0L), 5L)),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["cvd"]][["endocardium_diseases"]], rep(c(0L), 5L)),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["cvd"]][["heart_and_great_vessel_malformations"]], rep(c(0L), 5L)),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["cvd"]][["transplantation"]], rep(c(0L), 5L)),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["cvd"]][["other"]], rep(c(1L), 5L))
  )
  CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["cvd"]] <- NULL

  stopifnot(
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["cvd"]][["patid"]], rep("A", 5)),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["cvd"]][["encid"]], 3:7),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["cvd"]][["cardiomyopathies"]], rep(c(0L), 5L)),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["cvd"]][["conduction_disorder"]], rep(c(0L), 5L)),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["cvd"]][["device_and_technology_use"]], rep(c(0L), 5L)),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["cvd"]][["dysrhythmias"]], rep(c(0L), 5L)),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["cvd"]][["endocardium_diseases"]], rep(c(0L), 5L)),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["cvd"]][["heart_and_great_vessel_malformations"]], rep(c(0L), 5L)),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["cvd"]][["transplantation"]], rep(c(0L), 5L)),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["cvd"]][["other"]], rep(c(1L), 5L))
  )
  CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["cvd"]] <- NULL
}

################################################################################
# for cancer/malignancy
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(nrow(CMRBS[[i]][["spccc_current_0"]][["subconditions"]][["malignancy"]]), 0L))
  CMRBS[[i]][["spccc_current_0"]][["subconditions"]][["malignancy"]] <- NULL

  stopifnot(
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["malignancy"]][["patid"]], c("A", "A")),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["malignancy"]][["encid"]], c(2L, 5L)),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["malignancy"]][["neoplasms"]], c(1L, 1L)),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["malignancy"]][["transplantation"]], c(0L, 0L))
  )
  CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["malignancy"]] <- NULL

  stopifnot(
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["malignancy"]][["patid"]], c("A")),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["malignancy"]][["encid"]], c(5L)),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["malignancy"]][["neoplasms"]], c(1L)),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["malignancy"]][["transplantation"]], c(0L))
  )
  CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["malignancy"]] <- NULL

  stopifnot(
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["malignancy"]][["patid"]], rep("A", 5)),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["malignancy"]][["encid"]], 3:7),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["malignancy"]][["neoplasms"]], rep(1L, 5)),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["malignancy"]][["transplantation"]], rep(0L, 5))
  )
  CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["malignancy"]] <- NULL

  stopifnot(
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["malignancy"]][["patid"]], rep("A", 6)),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["malignancy"]][["encid"]], 2:7),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["malignancy"]][["neoplasms"]], rep(1L, 6)),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["malignancy"]][["transplantation"]], rep(0L, 6))
  )
  CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["malignancy"]] <- NULL

  stopifnot(
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["malignancy"]][["patid"]], rep("A", 5)),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["malignancy"]][["encid"]], 3:7),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["malignancy"]][["neoplasms"]], rep(1L, 5)),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["malignancy"]][["transplantation"]], rep(0L, 5))
  )
  CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["malignancy"]] <- NULL
}

################################################################################
# for renal
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(nrow(CMRBS[[i]][["spccc_current_0"]][["subconditions"]][["renal"]]), 0L))
  CMRBS[[i]][["spccc_current_0"]][["subconditions"]][["renal"]] <- NULL

  stopifnot(
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["renal"]][["patid"]], c("A", "B", "B")),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["renal"]][["encid"]], c(4L, 1L, 4L)),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["renal"]][["chronic_renal_failure"]], c(1L, 1L, 1L)),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["renal"]][["chronic_bladder_diseases"]], c(0L, 0L, 0L)),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["renal"]][["congenital_anomalies"]], c(0L, 0L, 0L)),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["renal"]][["device_and_technology_use"]], c(0L, 0L, 0L)),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["renal"]][["other"]], c(0L, 0L, 0L)),
    identical(CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["renal"]][["transplantation"]], c(0L, 0L, 0L))
  )
  CMRBS[[i]][["spccc_current_1"]][["subconditions"]][["renal"]] <- NULL

  stopifnot(
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["renal"]][["patid"]], c("A", "B")),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["renal"]][["encid"]], c(4L, 1L)),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["renal"]][["chronic_renal_failure"]], c(1L, 1L)),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["renal"]][["chronic_bladder_diseases"]], c(0L, 0L)),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["renal"]][["congenital_anomalies"]], c(0L, 0L)),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["renal"]][["device_and_technology_use"]], c(0L, 0L)),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["renal"]][["other"]], c(0L, 0L)),
    identical(CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["renal"]][["transplantation"]], c(0L, 0L))
  )
  CMRBS[[i]][["spccc_current_v"]][["subconditions"]][["renal"]] <- NULL

  stopifnot(
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["renal"]][["patid"]], rep(c("A", "B"), times = c(3, 4))),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["renal"]][["encid"]], c(5:7, 2:5)),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["renal"]][["chronic_renal_failure"]], rep(1L, 7)),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["renal"]][["chronic_bladder_diseases"]], rep(0L, 7L)),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["renal"]][["congenital_anomalies"]], rep(0L, 7L)),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["renal"]][["device_and_technology_use"]], rep(0L, 7L)),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["renal"]][["other"]], rep(0L, 7L)),
    identical(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["renal"]][["transplantation"]], rep(0L, 7L))
  )
  CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]][["renal"]] <- NULL

  stopifnot(
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["renal"]][["patid"]], rep(c("A", "B"), times = c(4, 5))),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["renal"]][["encid"]], c(4:7, 1:5)),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["renal"]][["chronic_renal_failure"]], rep(1L, 9L)),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["renal"]][["chronic_bladder_diseases"]], rep(0L, 9L)),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["renal"]][["congenital_anomalies"]], rep(0L, 9L)),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["renal"]][["device_and_technology_use"]], rep(0L, 9L)),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["renal"]][["other"]], rep(0L, 9L)),
    identical(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["renal"]][["transplantation"]], rep(0L, 9L))
  )
  CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]][["renal"]] <- NULL

  stopifnot(
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["renal"]][["patid"]], rep(c("A", "B"), times = c(4, 5))),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["renal"]][["encid"]], c(4:7, 1:5)),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["renal"]][["chronic_renal_failure"]], rep(1L, 9)),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["renal"]][["chronic_bladder_diseases"]], rep(0L, 9L)),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["renal"]][["congenital_anomalies"]], rep(0L, 9L)),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["renal"]][["device_and_technology_use"]], rep(0L, 9L)),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["renal"]][["other"]], rep(0L, 9L)),
    identical(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["renal"]][["transplantation"]], rep(0L, 9L))
  )
  CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]][["renal"]] <- NULL
}

################################################################################
# remove the subcondtions after verifying they are all empty
for( i in seq_len(length(CMRBS)) ) {
  stopifnot(identical(length(CMRBS[[i]][["spccc_current_0"]][["subconditions"]]), 0L))
  CMRBS[[i]][["spccc_current_0"]][["subconditions"]] <- NULL
  stopifnot(identical(length(CMRBS[[i]][["spccc_current_0"]]), 0L))
  CMRBS[[i]][["spccc_current_0"]] <- NULL

  stopifnot(identical(length(CMRBS[[i]][["spccc_current_1"]][["subconditions"]]), 0L))
  CMRBS[[i]][["spccc_current_1"]][["subconditions"]] <- NULL
  stopifnot(identical(length(CMRBS[[i]][["spccc_current_1"]]), 0L))
  CMRBS[[i]][["spccc_current_1"]] <- NULL

  stopifnot(identical(length(CMRBS[[i]][["spccc_current_v"]][["subconditions"]]), 0L))
  CMRBS[[i]][["spccc_current_v"]][["subconditions"]] <- NULL
  stopifnot(identical(length(CMRBS[[i]][["spccc_current_v"]]), 0L))
  CMRBS[[i]][["spccc_current_v"]] <- NULL

  stopifnot(identical(length(CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]]), 0L))
  CMRBS[[i]][["spccc_cumulative_0"]][["subconditions"]] <- NULL
  stopifnot(identical(length(CMRBS[[i]][["spccc_cumulative_0"]]), 0L))
  CMRBS[[i]][["spccc_cumulative_0"]] <- NULL

  stopifnot(identical(length(CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]]), 0L))
  CMRBS[[i]][["spccc_cumulative_1"]][["subconditions"]] <- NULL
  stopifnot(identical(length(CMRBS[[i]][["spccc_cumulative_1"]]), 0L))
  CMRBS[[i]][["spccc_cumulative_1"]] <- NULL

  stopifnot(identical(length(CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]]), 0L))
  CMRBS[[i]][["spccc_cumulative_v"]][["subconditions"]] <- NULL
  stopifnot(identical(length(CMRBS[[i]][["spccc_cumulative_v"]]), 0L))
  CMRBS[[i]][["spccc_cumulative_v"]] <- NULL
}

################################################################################
# Check the conditions for charlson current poa 0
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["charlson_current_0"]][["age_score"]], rep(NA_integer_, 12)))
  CMRBS[[i]][["charlson_current_0"]][["age_score"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_current_0"]][["patid"]], expected_patid))
  CMRBS[[i]][["charlson_current_0"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_current_0"]][["encid"]], expected_encid))
  CMRBS[[i]][["charlson_current_0"]][["encid"]] <- NULL
  stopifnot(all(CMRBS[[i]][["charlson_current_0"]] == 0))
  CMRBS[[i]][["charlson_current_0"]] <- NULL
}

################################################################################
# Check the conditions for pccc current poa 0
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["pccc_current_0"]][["patid"]], expected_patid))
  CMRBS[[i]][["pccc_current_0"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_0"]][["encid"]], expected_encid))
  CMRBS[[i]][["pccc_current_0"]][["encid"]] <- NULL
  stopifnot(all(CMRBS[[i]][["pccc_current_0"]] == 0))
  CMRBS[[i]][["pccc_current_0"]] <- NULL
}

################################################################################
# Check the conditions for elixhauser current poa 0
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_0"]][["patid"]], expected_patid))
  CMRBS[[i]][["elixhauser_current_0"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_0"]][["encid"]], expected_encid))
  CMRBS[[i]][["elixhauser_current_0"]][["encid"]] <- NULL

  stopifnot(identical(CMRBS[[i]][["elixhauser_current_0"]][["CANCER_METS"]], c(0L, 1L, 0L, 0L, 1L, rep(0L, 7))))
  CMRBS[[i]][["elixhauser_current_0"]][["CANCER_METS"]] <- NULL

  stopifnot(identical(CMRBS[[i]][["elixhauser_current_0"]][["mortality_index"]], 22L * c(0L, 1L, 0L, 0L, 1L, rep(0L, 7))))
  CMRBS[[i]][["elixhauser_current_0"]][["mortality_index"]] <- NULL

  stopifnot(identical(CMRBS[[i]][["elixhauser_current_0"]][["readmission_index"]], 11L * c(0L, 1L, 0L, 0L, 1L, rep(0L, 7))))
  CMRBS[[i]][["elixhauser_current_0"]][["readmission_index"]] <- NULL

  stopifnot(identical(CMRBS[[i]][["elixhauser_current_0"]][["cmrb_flag"]], c(0L, 1L, 0L, 0L, 1L, rep(0L, 7))))
  CMRBS[[i]][["elixhauser_current_0"]][["cmrb_flag"]] <- NULL

  stopifnot(identical(CMRBS[[i]][["elixhauser_current_0"]][["num_cmrb"]], c(0L, 1L, 0L, 0L, 1L, rep(0L, 7))))
  CMRBS[[i]][["elixhauser_current_0"]][["num_cmrb"]] <- NULL

  stopifnot(all(CMRBS[[i]][["elixhauser_current_0"]] == 0))
  CMRBS[[i]][["elixhauser_current_0"]] <- NULL
}

################################################################################
# Charlson, current, poa 1
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["charlson_current_1"]][["patid"]], expected_patid))
  CMRBS[[i]][["charlson_current_1"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_current_1"]][["encid"]], expected_encid))
  CMRBS[[i]][["charlson_current_1"]][["encid"]] <- NULL

  stopifnot(identical(CMRBS[[i]][["charlson_current_1"]][["age_score"]], rep(NA_integer_, 12)))
  CMRBS[[i]][["charlson_current_1"]][["age_score"]] <- NULL

  stopifnot(identical(CMRBS[[i]][["charlson_current_1"]][["chf"]], c(0L, 0L, 1L, 0L, 1L, rep(0L, 7L))))
  CMRBS[[i]][["charlson_current_1"]][["chf"]] <- NULL

  stopifnot(identical(CMRBS[[i]][["charlson_current_1"]][["mst"]], c(0L, 1L, 0L, 0L, 1L, rep(0L, 7L))))
  CMRBS[[i]][["charlson_current_1"]][["mst"]] <- NULL

  stopifnot(identical(CMRBS[[i]][["charlson_current_1"]][["rnd"]], c(0L, 0L, 0L, 1L, 0L, 0L, 0L, 1L, 0L, 0L, 1L, 0L)))
  CMRBS[[i]][["charlson_current_1"]][["rnd"]] <- NULL

  stopifnot(identical(CMRBS[[i]][["charlson_current_1"]][["cmrb_flag"]], c(0L, 1L, 1L, 1L, 1L, 0L, 0L, 1L, 0L, 0L, 1L, 0L)))
  CMRBS[[i]][["charlson_current_1"]][["cmrb_flag"]] <- NULL

  stopifnot(identical(CMRBS[[i]][["charlson_current_1"]][["num_cmrb"]], c(0L, 1L, 1L, 1L, 2L, 0L, 0L, 1L, 0L, 0L, 1L, 0L)))
  CMRBS[[i]][["charlson_current_1"]][["num_cmrb"]] <- NULL

  stopifnot(identical(CMRBS[[i]][["charlson_current_1"]][["cci"]], c(0L, 6L, 1L, 2L, 7L, 0L, 0L, 2L, 0L, 0L, 2L, 0L)))
  CMRBS[[i]][["charlson_current_1"]][["cci"]] <- NULL

  stopifnot(all(CMRBS[[i]][["charlson_current_1"]] == 0))
  CMRBS[[i]][["charlson_current_1"]] <- NULL
}

################################################################################
# charlson_current_v
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["charlson_current_v"]][["patid"]], expected_patid))
  CMRBS[[i]][["charlson_current_v"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_current_v"]][["encid"]], expected_encid))
  CMRBS[[i]][["charlson_current_v"]][["encid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_current_v"]][["age_score"]], rep(NA_integer_, 12L)))
  CMRBS[[i]][["charlson_current_v"]][["age_score"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_current_v"]][["chf"]], c(rep(0L, 2L), 1L, rep(0L, 9L))))
  CMRBS[[i]][["charlson_current_v"]][["chf"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_current_v"]][["rnd"]], c(rep(0L, 3L), 1L, rep(0L, 3L), 1L, rep(0L, 4L))))
  CMRBS[[i]][["charlson_current_v"]][["rnd"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_current_v"]][["mst"]], c(rep(0L, 4L), 1L, rep(0L, 7L))))
  CMRBS[[i]][["charlson_current_v"]][["mst"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_current_v"]][["num_cmrb"]], c(0L, 0L, 1L, 1L, 1L, 0L, 0L, 1L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["charlson_current_v"]][["num_cmrb"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_current_v"]][["cmrb_flag"]], c(0L, 0L, 1L, 1L, 1L, 0L, 0L, 1L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["charlson_current_v"]][["cmrb_flag"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_current_v"]][["cci"]], c(0L, 0L, 1L, 2L, 6L, 0L, 0L, 2L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["charlson_current_v"]][["cci"]] <- NULL
  stopifnot(all(CMRBS[[i]][["charlson_current_v"]] == 0))
  CMRBS[[i]][["charlson_current_v"]] <- NULL
}

################################################################################
# charlson_cumulative_0
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_0"]][["patid"]], expected_patid))
  CMRBS[[i]][["charlson_cumulative_0"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_0"]][["encid"]], expected_encid))
  CMRBS[[i]][["charlson_cumulative_0"]][["encid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_0"]][["age_score"]], rep(NA_integer_, 12L)))
  CMRBS[[i]][["charlson_cumulative_0"]][["age_score"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_0"]][["chf"]], c(0L, 0L, 0L, rep(1L, 4L), rep(0L, 5L))))
  CMRBS[[i]][["charlson_cumulative_0"]][["chf"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_0"]][["rnd"]], c(rep(0L, 4L), rep(1L, 3L), 0L, rep(1L, 4L))))
  CMRBS[[i]][["charlson_cumulative_0"]][["rnd"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_0"]][["mst"]], c(0L, 0L, rep(1L, 5L), rep(0L, 5))))
  CMRBS[[i]][["charlson_cumulative_0"]][["mst"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_0"]][["num_cmrb"]], c(0L, 0L, 1L, 2L, 3L, 3L, 3L, 0L, 1L, 1L, 1L, 1L)))
  CMRBS[[i]][["charlson_cumulative_0"]][["num_cmrb"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_0"]][["cmrb_flag"]], c(0L, 0L, 1L, 1L, 1L, 1L, 1L, 0L, 1L, 1L, 1L, 1L)))
  CMRBS[[i]][["charlson_cumulative_0"]][["cmrb_flag"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_0"]][["cci"]], c(0L, 0L, 6L, 7L, 9L, 9L, 9L, 0L, 2L, 2L, 2L, 2L)))
  CMRBS[[i]][["charlson_cumulative_0"]][["cci"]] <- NULL

  stopifnot(all(CMRBS[[i]][["charlson_cumulative_0"]] == 0))
  CMRBS[[i]][["charlson_cumulative_0"]] <- NULL
}

################################################################################
# charlson_cumulative_1
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_1"]][["patid"]], expected_patid))
  CMRBS[[i]][["charlson_cumulative_1"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_1"]][["encid"]], expected_encid))
  CMRBS[[i]][["charlson_cumulative_1"]][["encid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_1"]][["age_score"]], rep(NA_integer_, 12L)))
  CMRBS[[i]][["charlson_cumulative_1"]][["age_score"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_1"]][["chf"]], c(0L, 0L, rep(1L, 5L), rep(0L, 5L))))
  CMRBS[[i]][["charlson_cumulative_1"]][["chf"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_1"]][["rnd"]], c(rep(0L, 3L), rep(1L, 4L), rep(1L, 5L))))
  CMRBS[[i]][["charlson_cumulative_1"]][["rnd"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_1"]][["mst"]], c(0L, rep(1L, 6L), rep(0L, 5))))
  CMRBS[[i]][["charlson_cumulative_1"]][["mst"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_1"]][["num_cmrb"]], c(0L, 1L, 2L, 3L, 3L, 3L, 3L, rep(1L, 5L))))
  CMRBS[[i]][["charlson_cumulative_1"]][["num_cmrb"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_1"]][["cmrb_flag"]], c(0L, rep(1L, 11L))))
  CMRBS[[i]][["charlson_cumulative_1"]][["cmrb_flag"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_1"]][["cci"]], c(0L, 6L, 7L, 9L, 9L, 9L, 9L, 2L, 2L, 2L, 2L, 2L)))
  CMRBS[[i]][["charlson_cumulative_1"]][["cci"]] <- NULL

  stopifnot(all(CMRBS[[i]][["charlson_cumulative_1"]] == 0))
  CMRBS[[i]][["charlson_cumulative_1"]] <- NULL
}

################################################################################
# charlson_cumulative_v
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_v"]][["patid"]], expected_patid))
  CMRBS[[i]][["charlson_cumulative_v"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_v"]][["encid"]], expected_encid))
  CMRBS[[i]][["charlson_cumulative_v"]][["encid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_v"]][["age_score"]], rep(NA_integer_, 12L)))
  CMRBS[[i]][["charlson_cumulative_v"]][["age_score"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_v"]][["chf"]], c(0L, 0L, rep(1L, 5L), rep(0L, 5L))))
  CMRBS[[i]][["charlson_cumulative_v"]][["chf"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_v"]][["rnd"]], c(rep(0L, 3L), rep(1L, 4L), rep(1L, 5L))))
  CMRBS[[i]][["charlson_cumulative_v"]][["rnd"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_v"]][["mst"]], c(0L, 0L, rep(1L, 5L), rep(0L, 5))))
  CMRBS[[i]][["charlson_cumulative_v"]][["mst"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_v"]][["num_cmrb"]], c(0L, 0L, 2L, 3L, 3L, 3L, 3L, rep(1L, 5L))))
  CMRBS[[i]][["charlson_cumulative_v"]][["num_cmrb"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_v"]][["cmrb_flag"]], c(0L, 0L, rep(1L, 10L))))
  CMRBS[[i]][["charlson_cumulative_v"]][["cmrb_flag"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["charlson_cumulative_v"]][["cci"]], c(0L, 0L, 7L, 9L, 9L, 9L, 9L, 2L, 2L, 2L, 2L, 2L)))
  CMRBS[[i]][["charlson_cumulative_v"]][["cci"]] <- NULL

  stopifnot(all(CMRBS[[i]][["charlson_cumulative_v"]] == 0))
  CMRBS[[i]][["charlson_cumulative_v"]] <- NULL
}

################################################################################
# pccc_current_1
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["pccc_current_1"]][["patid"]], expected_patid))
  CMRBS[[i]][["pccc_current_1"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_1"]][["encid"]], expected_encid))
  CMRBS[[i]][["pccc_current_1"]][["encid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_1"]][["cvd_dxpr_or_tech"]], c(0L, 0L, 1L, 0L, 1L, rep(0L, 7L))))
  CMRBS[[i]][["pccc_current_1"]][["cvd_dxpr_or_tech"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_1"]][["cvd_dxpr_only"]], c(0L, 0L, 1L, 0L, 1L, rep(0L, 7L))))
  CMRBS[[i]][["pccc_current_1"]][["cvd_dxpr_only"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_1"]][["malignancy_dxpr_or_tech"]], c(0L, 1L, 0L, 0L, 1L, rep(0L, 7L))))
  CMRBS[[i]][["pccc_current_1"]][["malignancy_dxpr_or_tech"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_1"]][["malignancy_dxpr_only"]], c(0L, 1L, 0L, 0L, 1L, rep(0L, 7L))))
  CMRBS[[i]][["pccc_current_1"]][["malignancy_dxpr_only"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_1"]][["renal_dxpr_or_tech"]], c(0L, 0L, 0L, 1L, 0L, 0L, 0L, 1L, 0L, 0L, 1L, 0L)))
  CMRBS[[i]][["pccc_current_1"]][["renal_dxpr_or_tech"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_1"]][["renal_dxpr_only"]], c(0L, 0L, 0L, 1L, 0L, 0L, 0L, 1L, 0L, 0L, 1L, 0L)))
  CMRBS[[i]][["pccc_current_1"]][["renal_dxpr_only"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_1"]][["cmrb_flag"]], c(0L, 1L, 1L, 1L, 1L, 0L, 0L, 1L, 0L, 0L, 1L, 0L)))
  CMRBS[[i]][["pccc_current_1"]][["cmrb_flag"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_1"]][["num_cmrb"]], c(0L, 1L, 1L, 1L, 2L, 0L, 0L, 1L, 0L, 0L, 1L, 0L)))
  CMRBS[[i]][["pccc_current_1"]][["num_cmrb"]] <- NULL

  stopifnot(all(CMRBS[[i]][["pccc_current_1"]] == 0))
  CMRBS[[i]][["pccc_current_1"]] <- NULL
}

################################################################################
# pccc_current_v
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["pccc_current_v"]][["patid"]], expected_patid))
  CMRBS[[i]][["pccc_current_v"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_v"]][["encid"]], expected_encid))
  CMRBS[[i]][["pccc_current_v"]][["encid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_v"]][["cvd_dxpr_or_tech"]], c(0L, 0L, 1L, 0L, 0L, rep(0L, 7L))))
  CMRBS[[i]][["pccc_current_v"]][["cvd_dxpr_or_tech"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_v"]][["cvd_dxpr_only"]], c(0L, 0L, 1L, 0L, 0L, rep(0L, 7L))))
  CMRBS[[i]][["pccc_current_v"]][["cvd_dxpr_only"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_v"]][["malignancy_dxpr_or_tech"]], c(0L, 0L, 0L, 0L, 1L, rep(0L, 7L))))
  CMRBS[[i]][["pccc_current_v"]][["malignancy_dxpr_or_tech"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_v"]][["malignancy_dxpr_only"]], c(0L, 0L, 0L, 0L, 1L, rep(0L, 7L))))
  CMRBS[[i]][["pccc_current_v"]][["malignancy_dxpr_only"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_v"]][["renal_dxpr_or_tech"]], c(0L, 0L, 0L, 1L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["pccc_current_v"]][["renal_dxpr_or_tech"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_v"]][["renal_dxpr_only"]], c(0L, 0L, 0L, 1L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["pccc_current_v"]][["renal_dxpr_only"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_v"]][["cmrb_flag"]], c(0L, 0L, 1L, 1L, 1L, 0L, 0L, 1L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["pccc_current_v"]][["cmrb_flag"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_current_v"]][["num_cmrb"]], c(0L, 0L, 1L, 1L, 1L, 0L, 0L, 1L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["pccc_current_v"]][["num_cmrb"]] <- NULL

  stopifnot(all(CMRBS[[i]][["pccc_current_v"]] == 0))
  CMRBS[[i]][["pccc_current_v"]] <- NULL
}

################################################################################
# pccc_cumulative_0
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_0"]][["patid"]], expected_patid))
  CMRBS[[i]][["pccc_cumulative_0"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_0"]][["encid"]], expected_encid))
  CMRBS[[i]][["pccc_cumulative_0"]][["encid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_0"]][["cvd_dxpr_or_tech"]], c(0L, 0L, 0L, rep(1L, 4L), rep(0L, 5L))))
  CMRBS[[i]][["pccc_cumulative_0"]][["cvd_dxpr_or_tech"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_0"]][["cvd_dxpr_only"]], c(0L, 0L, 0L, rep(1L, 4L), rep(0L, 5L))))
  CMRBS[[i]][["pccc_cumulative_0"]][["cvd_dxpr_only"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_0"]][["renal_dxpr_or_tech"]], c(rep(0L, 4L), rep(1L, 3L), 0L, rep(1L, 4L))))
  CMRBS[[i]][["pccc_cumulative_0"]][["renal_dxpr_or_tech"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_0"]][["renal_dxpr_only"]], c(rep(0L, 4L), rep(1L, 3L), 0L, rep(1L, 4L))))
  CMRBS[[i]][["pccc_cumulative_0"]][["renal_dxpr_only"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_0"]][["malignancy_dxpr_or_tech"]], c(0L, 0L, rep(1L, 5L), rep(0L, 5))))
  CMRBS[[i]][["pccc_cumulative_0"]][["malignancy_dxpr_or_tech"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_0"]][["malignancy_dxpr_only"]], c(0L, 0L, rep(1L, 5L), rep(0L, 5))))
  CMRBS[[i]][["pccc_cumulative_0"]][["malignancy_dxpr_only"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_0"]][["num_cmrb"]], c(0L, 0L, 1L, 2L, 3L, 3L, 3L, 0L, 1L, 1L, 1L, 1L)))
  CMRBS[[i]][["pccc_cumulative_0"]][["num_cmrb"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_0"]][["cmrb_flag"]], c(0L, 0L, 1L, 1L, 1L, 1L, 1L, 0L, 1L, 1L, 1L, 1L)))
  CMRBS[[i]][["pccc_cumulative_0"]][["cmrb_flag"]] <- NULL

  stopifnot(all(CMRBS[[i]][["pccc_cumulative_0"]] == 0))
  CMRBS[[i]][["pccc_cumulative_0"]] <- NULL
}

################################################################################
# pccc_cumulative_1
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_1"]][["patid"]], expected_patid))
  CMRBS[[i]][["pccc_cumulative_1"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_1"]][["encid"]], expected_encid))
  CMRBS[[i]][["pccc_cumulative_1"]][["encid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_1"]][["cvd_dxpr_or_tech"]], c(0L, 0L, rep(1L, 5L), rep(0L, 5L))))
  CMRBS[[i]][["pccc_cumulative_1"]][["cvd_dxpr_or_tech"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_1"]][["renal_dxpr_or_tech"]], c(rep(0L, 3L), rep(1L, 4L), rep(1L, 5L))))
  CMRBS[[i]][["pccc_cumulative_1"]][["renal_dxpr_or_tech"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_1"]][["malignancy_dxpr_or_tech"]], c(0L, rep(1L, 6L), rep(0L, 5))))
  CMRBS[[i]][["pccc_cumulative_1"]][["malignancy_dxpr_or_tech"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_1"]][["cvd_dxpr_only"]], c(0L, 0L, rep(1L, 5L), rep(0L, 5L))))
  CMRBS[[i]][["pccc_cumulative_1"]][["cvd_dxpr_only"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_1"]][["renal_dxpr_only"]], c(rep(0L, 3L), rep(1L, 4L), rep(1L, 5L))))
  CMRBS[[i]][["pccc_cumulative_1"]][["renal_dxpr_only"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_1"]][["malignancy_dxpr_only"]], c(0L, rep(1L, 6L), rep(0L, 5))))
  CMRBS[[i]][["pccc_cumulative_1"]][["malignancy_dxpr_only"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_1"]][["num_cmrb"]], c(0L, 1L, 2L, 3L, 3L, 3L, 3L, rep(1L, 5L))))
  CMRBS[[i]][["pccc_cumulative_1"]][["num_cmrb"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_1"]][["cmrb_flag"]], c(0L, rep(1L, 11L))))
  CMRBS[[i]][["pccc_cumulative_1"]][["cmrb_flag"]] <- NULL

  stopifnot(all(CMRBS[[i]][["pccc_cumulative_1"]] == 0))
  CMRBS[[i]][["pccc_cumulative_1"]] <- NULL
}

################################################################################
# pccc_cumulative_v
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_v"]][["patid"]], expected_patid))
  CMRBS[[i]][["pccc_cumulative_v"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_v"]][["encid"]], expected_encid))
  CMRBS[[i]][["pccc_cumulative_v"]][["encid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_v"]][["cvd_dxpr_or_tech"]], c(0L, 0L, rep(1L, 5L), rep(0L, 5L))))
  CMRBS[[i]][["pccc_cumulative_v"]][["cvd_dxpr_or_tech"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_v"]][["renal_dxpr_or_tech"]], c(rep(0L, 3L), rep(1L, 4L), rep(1L, 5L))))
  CMRBS[[i]][["pccc_cumulative_v"]][["renal_dxpr_or_tech"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_v"]][["malignancy_dxpr_or_tech"]], c(0L, 0L, rep(1L, 5L), rep(0L, 5))))
  CMRBS[[i]][["pccc_cumulative_v"]][["malignancy_dxpr_or_tech"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_v"]][["cvd_dxpr_only"]], c(0L, 0L, rep(1L, 5L), rep(0L, 5L))))
  CMRBS[[i]][["pccc_cumulative_v"]][["cvd_dxpr_only"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_v"]][["renal_dxpr_only"]], c(rep(0L, 3L), rep(1L, 4L), rep(1L, 5L))))
  CMRBS[[i]][["pccc_cumulative_v"]][["renal_dxpr_only"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_v"]][["malignancy_dxpr_only"]], c(0L, 0L, rep(1L, 5L), rep(0L, 5))))
  CMRBS[[i]][["pccc_cumulative_v"]][["malignancy_dxpr_only"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_v"]][["num_cmrb"]], c(0L, 0L, 2L, 3L, 3L, 3L, 3L, rep(1L, 5L))))
  CMRBS[[i]][["pccc_cumulative_v"]][["num_cmrb"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["pccc_cumulative_v"]][["cmrb_flag"]], c(0L, 0L, rep(1L, 10L))))
  CMRBS[[i]][["pccc_cumulative_v"]][["cmrb_flag"]] <- NULL

  stopifnot(all(CMRBS[[i]][["pccc_cumulative_v"]] == 0))
  CMRBS[[i]][["pccc_cumulative_v"]] <- NULL
}

################################################################################
# elixhauser_current_1
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_1"]][["patid"]], expected_patid))
  CMRBS[[i]][["elixhauser_current_1"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_1"]][["encid"]], expected_encid))
  CMRBS[[i]][["elixhauser_current_1"]][["encid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_1"]][["cmrb_flag"]], c(0L, 1L, 1L, 1L, 1L, 0L, 0L, 1L, 0L, 0L, 1L, 0L)))
  CMRBS[[i]][["elixhauser_current_1"]][["cmrb_flag"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_1"]][["num_cmrb"]], c(0L, 1L, 1L, 1L, 2L, 0L, 0L, 1L, 0L, 0L, 1L, 0L)))
  CMRBS[[i]][["elixhauser_current_1"]][["num_cmrb"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_1"]][["RENLFL_SEV"]], c(0L, 0L, 0L, 1L, 0L, 0L, 0L, 1L, 0L, 0L, 1L, 0L)))
  CMRBS[[i]][["elixhauser_current_1"]][["RENLFL_SEV"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_1"]][["HF"]], c(0L, 0L, 1L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["elixhauser_current_1"]][["HF"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_1"]][["CANCER_METS"]], c(0L, 1L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["elixhauser_current_1"]][["CANCER_METS"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_1"]][["mortality_index"]], c(0L, 22L, 14L, 7L, 36L, 0L, 0L, 7L, 0L, 0L, 7L, 0L)))
  CMRBS[[i]][["elixhauser_current_1"]][["mortality_index"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_1"]][["readmission_index"]], c(0L, 11L, 7L, 8L, 18L, 0L, 0L, 8L, 0L, 0L, 8L, 0L)))
  CMRBS[[i]][["elixhauser_current_1"]][["readmission_index"]] <- NULL

  stopifnot(all(CMRBS[[i]][["elixhauser_current_1"]] == 0))
  CMRBS[[i]][["elixhauser_current_1"]] <- NULL
}

################################################################################
# elixhauser_current_v
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_v"]][["patid"]], expected_patid))
  CMRBS[[i]][["elixhauser_current_v"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_v"]][["encid"]], expected_encid))
  CMRBS[[i]][["elixhauser_current_v"]][["encid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_v"]][["cmrb_flag"]], c(0L, 1L, 1L, 1L, 1L, 0L, 0L, 1L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["elixhauser_current_v"]][["cmrb_flag"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_v"]][["num_cmrb"]], c(0L, 1L, 1L, 1L, 1L, 0L, 0L, 1L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["elixhauser_current_v"]][["num_cmrb"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_v"]][["RENLFL_SEV"]], c(0L, 0L, 0L, 1L, 0L, 0L, 0L, 1L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["elixhauser_current_v"]][["RENLFL_SEV"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_v"]][["HF"]], c(0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["elixhauser_current_v"]][["HF"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_v"]][["CANCER_METS"]], c(0L, 1L, 0L, 0L, 1L, 0L, 0L, 0L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["elixhauser_current_v"]][["CANCER_METS"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_v"]][["mortality_index"]], c(0L, 22L, 14L, 7L, 22L, 0L, 0L, 7L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["elixhauser_current_v"]][["mortality_index"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_current_v"]][["readmission_index"]], c(0L, 11L, 7L, 8L, 11L, 0L, 0L, 8L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["elixhauser_current_v"]][["readmission_index"]] <- NULL

  stopifnot(all(CMRBS[[i]][["elixhauser_current_v"]] == 0))
  CMRBS[[i]][["elixhauser_current_v"]] <- NULL
}

################################################################################
# elixhauser_cumulative_0
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_0"]][["patid"]], expected_patid))
  CMRBS[[i]][["elixhauser_cumulative_0"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_0"]][["encid"]], expected_encid))
  CMRBS[[i]][["elixhauser_cumulative_0"]][["encid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_0"]][["cmrb_flag"]], c(0L, 1L, 1L, 1L, 1L, 1L, 1L, 0L, 1L, 1L, 1L, 1L)))
  CMRBS[[i]][["elixhauser_cumulative_0"]][["cmrb_flag"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_0"]][["num_cmrb"]],  c(0L, 1L, 1L, 2L, 3L, 3L, 3L, 0L, 1L, 1L, 1L, 1L)))
  CMRBS[[i]][["elixhauser_cumulative_0"]][["num_cmrb"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_0"]][["RENLFL_SEV"]], c(0L, 0L, 0L, 0L, 1L, 1L, 1L, 0L, 1L, 1L, 1L, 1L)))
  CMRBS[[i]][["elixhauser_cumulative_0"]][["RENLFL_SEV"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_0"]][["HF"]], c(0L, 0L, 0L, 1L, 1L, 1L, 1L, 0L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["elixhauser_cumulative_0"]][["HF"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_0"]][["CANCER_METS"]], c(0L, 1L, 1L, 1L, 1L, 1L, 1L, 0L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["elixhauser_cumulative_0"]][["CANCER_METS"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_0"]][["mortality_index"]], c(0L, 22L, 22L, 36L, 43L, 43L, 43L, 0L, 7L, 7L, 7L, 7L)))
  CMRBS[[i]][["elixhauser_cumulative_0"]][["mortality_index"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_0"]][["readmission_index"]], c(0L, 11L, 11L, 18L, 26L, 26L, 26L, 0L, 8L, 8L, 8L, 8L)))
  CMRBS[[i]][["elixhauser_cumulative_0"]][["readmission_index"]] <- NULL

  stopifnot(all(CMRBS[[i]][["elixhauser_cumulative_0"]] == 0))
  CMRBS[[i]][["elixhauser_cumulative_0"]] <- NULL
}

################################################################################
# elixhauser_cumulative_1
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_1"]][["patid"]], expected_patid))
  CMRBS[[i]][["elixhauser_cumulative_1"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_1"]][["encid"]], expected_encid))
  CMRBS[[i]][["elixhauser_cumulative_1"]][["encid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_1"]][["cmrb_flag"]], c(0L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L)))
  CMRBS[[i]][["elixhauser_cumulative_1"]][["cmrb_flag"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_1"]][["num_cmrb"]], c(0L, 1L, 2L, 3L, 3L, 3L, 3L, 1L, 1L, 1L, 1L, 1L)))
  CMRBS[[i]][["elixhauser_cumulative_1"]][["num_cmrb"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_1"]][["RENLFL_SEV"]], c(0L, 0L, 0L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L)))
  CMRBS[[i]][["elixhauser_cumulative_1"]][["RENLFL_SEV"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_1"]][["HF"]], c(0L, 0L, 1L, 1L, 1L, 1L, 1L, 0L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["elixhauser_cumulative_1"]][["HF"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_1"]][["CANCER_METS"]], c(0L, 1L, 1L, 1L, 1L, 1L, 1L, 0L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["elixhauser_cumulative_1"]][["CANCER_METS"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_1"]][["mortality_index"]], c(0L, 22L, 36L, 43L, 43L, 43L, 43L, 7L, 7L, 7L, 7L, 7L)))
  CMRBS[[i]][["elixhauser_cumulative_1"]][["mortality_index"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_1"]][["readmission_index"]], c(0L, 11L, 18L, 26L, 26L, 26L, 26L, 8L, 8L, 8L, 8L, 8L)))
  CMRBS[[i]][["elixhauser_cumulative_1"]][["readmission_index"]] <- NULL

  stopifnot(all(CMRBS[[i]][["elixhauser_cumulative_1"]] == 0))
  CMRBS[[i]][["elixhauser_cumulative_1"]] <- NULL
}

################################################################################
# elixhauser_cumulative_v
for (i in seq_len(length(CMRBS))) {
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_v"]][["patid"]], expected_patid))
  CMRBS[[i]][["elixhauser_cumulative_v"]][["patid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_v"]][["encid"]], expected_encid))
  CMRBS[[i]][["elixhauser_cumulative_v"]][["encid"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_v"]][["cmrb_flag"]], c(0L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L)))
  CMRBS[[i]][["elixhauser_cumulative_v"]][["cmrb_flag"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_v"]][["num_cmrb"]], c(0L, 1L, 2L, 3L, 3L, 3L, 3L, 1L, 1L, 1L, 1L, 1L)))
  CMRBS[[i]][["elixhauser_cumulative_v"]][["num_cmrb"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_v"]][["RENLFL_SEV"]], c(0L, 0L, 0L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L)))
  CMRBS[[i]][["elixhauser_cumulative_v"]][["RENLFL_SEV"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_v"]][["HF"]], c(0L, 0L, 1L, 1L, 1L, 1L, 1L, 0L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["elixhauser_cumulative_v"]][["HF"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_v"]][["CANCER_METS"]], c(0L, 1L, 1L, 1L, 1L, 1L, 1L, 0L, 0L, 0L, 0L, 0L)))
  CMRBS[[i]][["elixhauser_cumulative_v"]][["CANCER_METS"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_v"]][["mortality_index"]], c(0L, 22L, 36L, 43L, 43L, 43L, 43L, 7L, 7L, 7L, 7L, 7L)))
  CMRBS[[i]][["elixhauser_cumulative_v"]][["mortality_index"]] <- NULL
  stopifnot(identical(CMRBS[[i]][["elixhauser_cumulative_v"]][["readmission_index"]], c(0L, 11L, 18L, 26L, 26L, 26L, 26L, 8L, 8L, 8L, 8L, 8L)))
  CMRBS[[i]][["elixhauser_cumulative_v"]][["readmission_index"]] <- NULL

  stopifnot(all(CMRBS[[i]][["elixhauser_cumulative_v"]] == 0))
  CMRBS[[i]][["elixhauser_cumulative_v"]] <- NULL
}

################################################################################
#summary(CMRBS)
stopifnot(identical(length(CMRBS[["DF"]]), 0L))
CMRBS[["DF"]] <- NULL

stopifnot(identical(length(CMRBS[["DT"]]), 0L))
CMRBS[["DT"]] <- NULL

stopifnot(identical(length(CMRBS[["TBL"]]), 0L))
CMRBS[["TBL"]] <- NULL

stopifnot(identical(length(CMRBS), 0L))

################################################################################
#                                 End of File                                  #
################################################################################

#' Internal Implementation of the Elixhauser comorbidities
#' medicalcoder
#'
#' Internal.  Called from [`comorbidities()`].  Not expected to be called from
#' any other context.
#'
#' @param id.vars Character vector of ID column names.
#' @param iddf A `data.frame` of unique IDs
#' @param cmrb A `data.frame` containing at least `id.vars` and
#' `condition` columns; i.e., the 'comorbidity' `data.frame`.
#' @param poa.var Character denoting the name of the column in the `cmrb`
#' `data.frame` denoting if the condition was identified by an ICD code that
#' was or was not present on admission.
#' @param primarydx.var Character (scalar) with the name of the column in
#' `cmrb` denoting if the condition was flaged as a primary diagnostic or not.
#' @param method Character scalar; name of the Charlson variant to assess
#'
#' @return A `data.frame` with `id.vars`, per-condition 0/1
#' indicators, `num_cmrb`, `cmrb_flag`, and `cci`.
#'
#' @family internal comorbidity functions
#' @noRd
#' @keywords internal
.elixhauser <- function(id.vars, iddf, cmrb, poa.var, primarydx.var, method) {
  ccc <- mdcr_select(cmrb, cols = c(id.vars, "condition", "poaexempt", poa.var, primarydx.var))
  ccc <- mdcr_unique(ccc)

  # omit primary dx
  idx <- ccc[[primarydx.var]] == 0L
  ccc <- mdcr_subset(ccc, i = idx)

  if (startsWith(method, "elixhauser_ahrq20") | method == "elixhauser_ahrq_icd10") {
    results <- .elixhauser_post2022(ccc, id.vars, iddf, cmrb, poa.var, primarydx.var, method)
  } else {
    results <- .elixhauser_pre2022(ccc, id.vars, iddf, cmrb, poa.var, primarydx.var, method)
  }

  # build the return object
  rtn <- mdcr_cbind(iddf, as.data.frame(results$X, check.names = FALSE, stringsAsFactors = FALSE))
  rtn <- mdcr_set(rtn, j = "num_cmrb", value = results$num_cmrb)
  rtn <- mdcr_set(rtn, j = "cmrb_flag", value = results$cmrb_flag)
  rtn <- mdcr_set(rtn, j = "mortality_index", value = results$mortality_index)
  rtn <- mdcr_set(rtn, j = "readmission_index", value = results$readmission_index)
  rownames(rtn) <- NULL
  rtn
}

.elixhauser_post2022 <- function(ccc, id.vars, iddf, cmrb, poa.var, primarydx.var, method) {
  conditions <- ..mdcr_internal_elixhauser_codes..[["condition"]][which(..mdcr_internal_elixhauser_codes..[[method]] == 1L)]
  conditions <- sort(unique(conditions))

  # need to consider how the conditions are flagged POA, POAEXEMPT, or ANYPOA.
  XPOA <- matrix(0L, nrow = nrow(iddf), ncol = length(conditions), dimnames = list(NULL, conditions))
  XNPOA <- XPOAEXEMPT <- XANYPOA <- XPOA

  POA       <- mdcr_subset(ccc, ccc[[poa.var]] == 1L)
  POAEXEMPT <- mdcr_subset(ccc, ccc[["poaexempt"]] == 1L)
  NPOA      <- mdcr_subset(ccc, ccc[["poaexempt"]] == 0L & ccc[[poa.var]] == 0L)
  # ANYPOA == ccc

  key_iddf      <- do.call(paste, c(iddf, sep = "\r"))
  key_poa       <- do.call(paste, c(mdcr_select(POA, id.vars), sep = "\r"))
  key_poaexempt <- do.call(paste, c(mdcr_select(POAEXEMPT, id.vars), sep = "\r"))
  key_npoa      <- do.call(paste, c(mdcr_select(NPOA, id.vars), sep = "\r"))
  key_anypoa    <- do.call(paste, c(mdcr_select(ccc, id.vars), sep = "\r"))

  ri <- match(key_poa, key_iddf)
  ci <- match(POA[["condition"]], conditions)
  keep <- !(is.na(ri) | is.na(ci))
  if (any(keep)) {
    XPOA[cbind(ri[keep], ci[keep])] <- 1L
  }

  ri <- match(key_poaexempt, key_iddf)
  ci <- match(POAEXEMPT[["condition"]], conditions)
  keep <- !(is.na(ri) | is.na(ci))
  if (any(keep)) {
    XPOAEXEMPT[cbind(ri[keep], ci[keep])] <- 1L
  }

  ri <- match(key_npoa, key_iddf)
  ci <- match(NPOA[["condition"]], conditions)
  keep <- !(is.na(ri) | is.na(ci))
  if (any(keep)) {
    XNPOA[cbind(ri[keep], ci[keep])] <- 1L
  }

  ri <- match(key_anypoa, key_iddf)
  ci <- match(ccc[["condition"]], conditions)
  keep <- !(is.na(ri) | is.na(ci))
  if (any(keep)) {
    XANYPOA[cbind(ri[keep], ci[keep])] <- 1L
  }

  # Assign comorbidities which are neutral to POA
  from_to <-
    c("DRUG_ABUSEPSYCHOSES" = "DRUG_ABUSE",
      "HFHTN_CX" = "HTN_CX",
      "HTN_CXRENLFL_SEV"= "HTN_CX",
      "HFHTN_CXRENLFL_SEV"= "HTN_CX",
      "ALCOHOLLIVER_MLD" = "ALCOHOL",
      "VALVE_AUTOIMMUNE" = "AUTOIMMUNE",
      "LIVER_MLD_NEURO" = "LIVER_MLD",
      "LIVER_MLD_NEURO" = "NEURO_OTH",
      "NEURO_OTH_SEIZ" = "NEURO_OTH",
      "NEURO_OTH_SEIZ" = "NEURO_SEIZ",
      "LIVER_MLD_PULMCIRC" = "LIVER_MLD",
      "LIVER_MLD_PULMCIRC" = "PULMCIRC"
    )

  for (i in seq_len(length(from_to))) {
    f <- names(from_to)[i]
    t <- unname(from_to[i])
    if (f %in% colnames(XPOA)) {
      XPOA[XPOA[, f] == 1L, t] <- 1L
    }
    if (f %in% colnames(XPOAEXEMPT)) {
      XPOAEXEMPT[XPOAEXEMPT[, f] == 1L, t] <- 1L
    }
    if (f %in% colnames(XANYPOA)) {
      XANYPOA[XANYPOA[, f] == 1L, t] <- 1L
    }
    if (f %in% colnames(XNPOA)) {
      XNPOA[XNPOA[, f] == 1L, t] <- 1L
    }
  }

  # flag if POA exempt or POA
  from_to <-
    c("DRUG_ABUSEPSYCHOSES" = "PSYCHOSES",
      "HFHTN_CX" = "HF",
      "HTN_CXRENLFL_SEV"= "RENLFL_SEV",
      "HFHTN_CXRENLFL_SEV"= "HF",
      "HFHTN_CXRENLFL_SEV"= "RENLFL_SEV",
      "CBVD_SQLAPARALYSIS"= "PARALYSIS",
      "CBVD_SQLAPARALYSIS"= "CBVD_SQLA",
      "ALCOHOLLIVER_MLD"= "LIVER_MLD",
      "VALVE_AUTOIMMUNE"= "VALVE"
    )

  for (i in seq_len(length(from_to))) {
    f <- names(from_to)[i]
    t <- unname(from_to[i])
    if (f %in% colnames(XPOA)) {
      XPOA[XPOA[, f] == 1L, t] <- 1L
    }
    if (f %in% colnames(XPOAEXEMPT)) {
      XPOAEXEMPT[XPOAEXEMPT[, f] == 1L, t] <- 1L
    }
  }

  # CBVD_NPOA is unique in that it requires that the condition is not POA
  XNPOA <- cbind(XNPOA, "CBVD_NPOA" = rep(0L, nrow(XNPOA)))
  XNPOA[XNPOA[, "CBVD_POA"] == 1L, "CBVD_NPOA"] <- 1L

  ##############################################################################
  # Build one matrix with the conditions
  apoaconditions <-
    ..mdcr_internal_elixhauser_poa..[["condition"]][
      ..mdcr_internal_elixhauser_poa..[[method]] == 1L &
      ..mdcr_internal_elixhauser_poa..[["poa_required"]] == 0L
    ]

  X <- (XPOA | XPOAEXEMPT)
  X[, apoaconditions] <- X[, apoaconditions] | XANYPOA[, apoaconditions]
  # CBVD clean up
  idx <-
    (X[, "CBVD_POA"]) |
    (!X[, "CBVD_POA"] & X[, "CBVD_SQLA"] == 1L & XNPOA[, "CBVD_NPOA"] == 0L)
  X <- cbind(X, "CBVD" = rep(0L, nrow(X)))
  X[idx, "CBVD"] <- 1L

  storage.mode(X) <- "integer"

  ##############################################################################
  # exclusions for for less severe condiitons
  sev_less <-
    c("DIAB_CX" = "DIAB_UNCX",
      "HTN_CX" = "HTN_UNCX",
      "CANCER_METS" = "CANCER_SOLID",
      "CANCER_METS" = "CANCER_NSITU",
      "CANCER_SOLID"= "CANCER_NSITU",
      "LIVER_SEV" = "LIVER_MLD",
      "RENLFL_SEV" = "RENLFL_MOD"
    )
  for (i in seq_len(length(sev_less))) {
    s <- names(sev_less)[i]
    l <- unname(sev_less[i])
    X[X[, s] == 1L, l] <- 0L
  }

  ##############################################################################
  # index weights
  mortality_weights <-
    stats::setNames(
      ..mdcr_internal_elixhauser_index_scores..[[method]],
      ..mdcr_internal_elixhauser_index_scores..[["condition"]]
    )[..mdcr_internal_elixhauser_index_scores..[["index"]] == "mortality"]

  readmission_weights <-
    stats::setNames(
      ..mdcr_internal_elixhauser_index_scores..[[method]],
      ..mdcr_internal_elixhauser_index_scores..[["condition"]]
    )[..mdcr_internal_elixhauser_index_scores..[["index"]] == "readmission"]

  mortality_weights   <- mortality_weights[!is.na(mortality_weights)]
  readmission_weights <- readmission_weights[!is.na(readmission_weights)]

  # Set counts, flags, and index scores
  if (nrow(X)) {
    num_cmrb          <- as.integer(rowSums(X[, names(mortality_weights), drop = FALSE]))
    cmrb_flag         <- as.integer(num_cmrb > 0L)
    mortality_index   <- as.integer(as.vector(X[, names(mortality_weights), drop = FALSE] %*% mortality_weights))
    readmission_index <- as.integer(as.vector(X[, names(readmission_weights), drop = FALSE] %*% readmission_weights))
  } else {
    num_cmrb          <- integer(0)
    cmrb_flag         <- integer(0)
    mortality_index   <- integer(0)
    readmission_index <- integer(0)
  }

  list(X = X, num_cmrb = num_cmrb, cmrb_flag = cmrb_flag, mortality_index = mortality_index, readmission_index = readmission_index)
}

.elixhauser_pre2022 <- function(ccc, id.vars, iddf, cmrb, poa.var, primarydx.var, method) {
  # what are the relevent coniditions
  conditions <-
    unique(..mdcr_internal_elixhauser_codes..[["condition"]][which(..mdcr_internal_elixhauser_codes..[[method]] == 1L)])

  # build indicator matrix
  X <- matrix(0L, nrow = nrow(iddf), ncol = length(conditions) + 1L)
  colnames(X) <- c(conditions, "HTN_C")

  key_iddf <- do.call(paste, c(iddf, sep = "\r"))
  key_ccc  <- do.call(paste, c(mdcr_select(ccc, id.vars), sep = "\r"))
  ri <- match(key_ccc, key_iddf)
  ci <- match(ccc[["condition"]], conditions)
  keep <- !(is.na(ri) | is.na(ci))
  if (any(keep)) {
    X[cbind(ri[keep], ci[keep])] <- 1L
  }

  # severity corrections
  X[X[, "HTN_UNCX"] == 1L | X[, "HTN_CX"] == 1L, "HTN_C"] <- 1L
  X[X[, "HTN_CX"] == 1L, "HTN_UNCX"] <- 0L
  X[X[, "DMCX"] == 1, "DM"] <- 0L
  X[X[, "METS"] == 1, "TUMOR"] <- 0L

  mortality_weights <-
    stats::setNames(
      ..mdcr_internal_elixhauser_index_scores..[[method]],
      ..mdcr_internal_elixhauser_index_scores..[["condition"]]
    )[..mdcr_internal_elixhauser_index_scores..[["index"]] == "mortality"]

  readmission_weights <-
    stats::setNames(
      ..mdcr_internal_elixhauser_index_scores..[[method]],
      ..mdcr_internal_elixhauser_index_scores..[["condition"]]
    )[..mdcr_internal_elixhauser_index_scores..[["index"]] == "readmission"]

  mortality_weights <- mortality_weights[!is.na(mortality_weights)]
  readmission_weights <- readmission_weights[!is.na(readmission_weights)]

  # Set counts, flags, and index scores
  storage.mode(X) <- "integer"
  if (nrow(X)) {
    num_cmrb          <- as.integer(rowSums(X[, names(mortality_weights), drop = FALSE]))
    cmrb_flag         <- as.integer(num_cmrb > 0L)
    mortality_index   <- as.integer(as.vector(X[, names(mortality_weights), drop = FALSE] %*% mortality_weights))
    readmission_index <- as.integer(as.vector(X[, names(readmission_weights), drop = FALSE] %*% readmission_weights))
  } else {
    num_cmrb          <- integer(0)
    cmrb_flag         <- integer(0)
    mortality_index   <- integer(0)
    readmission_index <- integer(0)
  }

  list(X = X, num_cmrb = num_cmrb, cmrb_flag = cmrb_flag, mortality_index = mortality_index, readmission_index = readmission_index)
}

################################################################################
#                                 End of File                                  #
################################################################################

library(medicalcoder)
source('utilities.R')

# Verify that when a data.frame with zero rows is passed to comorbidities() the
# return will be a zero row data.frame with the same general structure that
# would be returned if a data.frame with at least one row was passed to
# comorbidities().

# build a zero-row data.frame for
DF <- data.frame(code = c("B", "A"), patid = 1:2)
common_args <- list(data = DF[0, ], id.vars = "patid", icd.codes = "code", poa = 1)

# fit all methods without subconditions
methods <- medicalcoder:::comorbidities_methods()
args <- lapply(methods, function(x) { if (startsWith(x, "pccc")) {c(common_args, list(method = x))} else {c(common_args, list(method = x, primarydx = 0L))} } )
args <- setNames(args, methods)
rtns <- lapply(args, do.call, what = comorbidities)

# pccc with subconditions
common_args[["subconditions"]] <- TRUE
args <- lapply(methods[startsWith(methods, "pccc")], function(x) c(common_args, list(method = x)))
args <- setNames(args, paste0(methods[startsWith(methods, "pccc")], "s"))
rtns <- c(rtns, lapply(args, do.call, what = comorbidities))

# build rtns1 with nrow = nrow(DF), this will be used to check that the names
# and general structure of the returns are the same with zero input rows or more
# than zero input rows
common_args <- list(data = DF, id.vars = "patid", icd.codes = "code", poa = 1)
args <- lapply(methods, function(x) { if (startsWith(x, "pccc")) {c(common_args, list(method = x))} else {c(common_args, list(method = x, primarydx = 0L))} } )
args <- setNames(args, methods)
rtns1 <- lapply(args, do.call, what = comorbidities)

# pccc with subconditions
common_args[["subconditions"]] <- TRUE
args <- lapply(methods[startsWith(methods, "pccc")], function(x) c(common_args, list(method = x)))
args <- setNames(args, paste0(methods[startsWith(methods, "pccc")], "s"))
rtns1 <- c(rtns1, lapply(args, do.call, what = comorbidities))

################################################################################
# verify that all the returned elements inherit the expected classes
for (m in names(rtns)) {
  if (!inherits(rtns[[m]], "medicalcoder_comorbidities")) {
    stop(sprintf("rtns[[%s]] does not inherit `medicalcoder_comorbidities`", m))
  }

  if (!startsWith(m, "pccc") & inherits(rtns[[m]], "medicalcoder_comorbidities_with_subconditions")) {
    stop(sprintf("rtns[[%s]] incorrectly inherits `medicalcoder_comorbidities_with_subconditions`", m))
  } else {
    if (endsWith(m, "s") & !inherits(rtns[[m]], "medicalcoder_comorbidities_with_subconditions")) {
      stop(sprintf("rtns[[%s]] does not inherit `medicalcoder_comorbidities_with_subconditions`", m))
    } else if (!endsWith(m, "s") & inherits(rtns[[m]], "medicalcoder_comorbidities_with_subconditions")) {
      stop(sprintf("rtns[[%s]] incorrectly inherits `medicalcoder_comorbidities_with_subconditions`", m))
    }
  }
}

# verify that all the non subcondition rtns are data.frames
for (m in names(rtns)) {
  if (!startsWith(m, "pccc")) {
    if (!inherits(rtns[[m]], "data.frame")) {
      stop(sprintf("rtns[[%s]] is not a data.frame", m))
    }
    if (nrow(rtns[[m]]) != 0L) {
      stop(sprintf("nrow(rtns[[%s]]) != 0", m))
    }
  } else {
    if (endsWith(m, "s")) {

      if (!inherits(rtns[[m]], "list")) {
        stop(sprintf("rtns[[%s]] is not a list", m))
      }

      if (length(rtns[[m]]) != 2L) {
        stop(sprintf("length(rtns[[%s]]) != 2L", m))
      }

      if (!inherits(rtns[[m]][['conditions']], "data.frame")) {
        stop(sprintf("rtns[[%s]][['conditions']] is not a data.frame", m))
      }

      if (nrow(rtns[[m]][['conditions']]) != 0L) {
        stop(sprintf("nrow(rtns[[%s]][['conditions']]) != 0", m))
      }

      if (!inherits(rtns[[m]][['subconditions']], "list")) {
        stop(sprintf("rtns[[%s]][['subconditions']] is not a list", m))
      }

      if (length(rtns[[m]][['subconditions']]) != 11L) {
        stop(sprintf("length(rtns[[%s]][['subconditions']]) != 11L", m))
      }

      if (!all(sapply(rtns[[m]][['subconditions']], nrow) == 0L)) {
        stop(sprintf("!all(sapply(rtns[[%s]][['subconditions']], nrow) == 0L)", m))
      }

    } else {
      if (!inherits(rtns[[m]], "data.frame")) {
        stop(sprintf("rtns[[%s]] is not a data.frame", m))
      }
      if (nrow(rtns[[m]]) != 0L) {
        stop(sprintf("nrow(rtns[[%s]]) != 0", m))
      }
    }
  }
}

stopifnot(identical(lapply(rtns, names), lapply(rtns1, names)))

stopifnot(
  identical(
    lapply(rtns[grep("pccc_v.+s$", names(rtns))], lapply, names),
    lapply(rtns1[grep("pccc_v.+s$", names(rtns))], lapply, names)
  )
)

################################################################################
#                                 End of File                                  #
################################################################################

#' Get Elixhauser Index Scores
#'
#' Functions to get a copy of internal lookup tables for the ICD codes and index
#' scores used in assessing Elixhauser comorbidities.
#'
#' @seealso
#' * [`get_elixhauser_codes()`] for the lookup table of ICD codes mapping to the
#'   Elixhauser comorbidities.
#' * [`get_elixhauser_poa()`] for the lookup table of the conditions which do an
#'   do not require associated ICD codes to be present-on-admission to flag the
#'   comorbidity.
#' * [comorbidities()] for applying comorbidity algorithms to a data set.
#' 
#' @return A `data.frame` with the following columns:
#' * `condition`: Character vector of the conditions
#' * `index`: Character vector indicating if the score is for the mortality or the readmission index score
#' * `elixhauser_<variant>`: integer vector of the scores 
#'
#' @examples
#' head(get_elixhauser_index_scores())
#' str(get_elixhauser_index_scores())
#' 
#' @export
get_elixhauser_index_scores <- function() {
  unserialize(serialize(..mdcr_internal_elixhauser_index_scores.., connection = NULL))
}

################################################################################
#                                 End of File                                  #
################################################################################

source('utilities.R')
library(medicalcoder)

# use three codes:
codes <-
  c(
    "H49.811", # metabolic (other)
    "J84.111", # respiratory (chronic respiratory disease)
    "Z96.41"   # metabolic (device and technology use)
  )

# verify that these codes are as expected
x <- subset(get_pccc_codes(), full_code %in% codes)
rownames(x) <- NULL
expected <- structure(list(icdv = c(10L, 10L, 10L), dx = c(1L, 1L, 1L), full_code = c("H49.811", "J84.111", "Z96.41"), code = c("H49811", "J84111", "Z9641"), condition = c("metabolic", "respiratory", "metabolic"), subcondition = c("other_metabolic_disorders", "chronic_respiratory_diseases", "device_and_technology_use"), transplant_flag = c(0L, 0L, 0L), tech_dep_flag = c(0L, 0L, 1L), pccc_v3.1 = c(1L, 1L, 1L), pccc_v3.0 = c(1L, 1L, 1L), pccc_v2.1 = c(1L, 0L, 1L), pccc_v2.0 = c(1L, 0L, 1L)), row.names = c(NA, -3L), class = "data.frame")
stopifnot(identical(x, expected))

# build six permutations of these codes.  The codes will be considered poa on
# encounters 2, 4, and 6.  There will be seven encounters for each permutation.
permutations <-
  data.frame(
    permutation = rep(1:6, each = 7),
    encounter_id = rep(1:7, times = 6),
    code =
      codes[c(NA, 1, NA, 2, NA, 3, NA,
        NA, 1, NA, 3, NA, 2, NA,
        NA, 2, NA, 1, NA, 3, NA,
        NA, 2, NA, 3, NA, 1, NA,
        NA, 3, NA, 1, NA, 2, NA,
        NA, 3, NA, 2, NA, 1, NA)]
  )


permutations <- split(permutations, f = permutations[["permutation"]])
permutations <-
  lapply(
    permutations,
    function(x) {
      x[["plabel"]] <- with(x, paste(na.omit(code), collapse = ", "))
      x[["plabel"]] <- with(x, paste0("Permutation ", permutation, ": ", plabel))
      x
    }
  )
permutations <- do.call(rbind, permutations)

# flag cumulative conditions
args <-
  list(
       data = permutations,
       id.vars = c("permutation", "plabel", "encounter_id"),
       icd.codes = "code",
       method = "pccc_v3.1",
       flag.method = "cumulative",
       poa = 1,
       icdv.var = NULL,
       icdv = NULL,
       dx.var = NULL,
       dx = NULL,
       poa.var = NULL,
       age.var = NULL,
       primarydx.var = NULL,
       primarydx = NULL,
       full.codes = TRUE,
       compact.codes = FALSE,
       subconditions = FALSE
  )
#list2env(args, .GlobalEnv)

rtn <- do.call(comorbidities, args)

################################################################################
# verify that the permutations are as expected
stopifnot(rtn[rtn$permutation == 1, "plabel"] == "Permutation 1: H49.811, J84.111, Z96.41")
stopifnot(rtn[rtn$permutation == 2, "plabel"] == "Permutation 2: H49.811, Z96.41, J84.111")
stopifnot(rtn[rtn$permutation == 3, "plabel"] == "Permutation 3: J84.111, H49.811, Z96.41")
stopifnot(rtn[rtn$permutation == 4, "plabel"] == "Permutation 4: J84.111, Z96.41, H49.811")
stopifnot(rtn[rtn$permutation == 5, "plabel"] == "Permutation 5: Z96.41, H49.811, J84.111")
stopifnot(rtn[rtn$permutation == 6, "plabel"] == "Permutation 6: Z96.41, J84.111, H49.811")

# Permutation 1
#   The sequnce of conditions:
#     2. metabolic (other) (H49.811)
#     4. respiratory       (J84.111)
#     6. metabolic (tech)  (Z96.41)
expected_metabolic_dxpr_or_tech_1  = c(0L, 1L, 1L, 1L, 1L, 1L, 1L)
expected_metabolic_dxpr_only_1     = c(0L, 1L, 1L, 1L, 1L, 0L, 0L)
expected_metabolic_tech_only_1     = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)
expected_metabolic_dxpr_and_tech_1 = c(0L, 0L, 0L, 0L, 0L, 1L, 1L)

expected_respiratory_dxpr_or_tech_1  = c(0L, 0L, 0L, 1L, 1L, 1L, 1L)
expected_respiratory_dxpr_only_1     = c(0L, 0L, 0L, 1L, 1L, 1L, 1L)
expected_respiratory_tech_only_1     = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)
expected_respiratory_dxpr_and_tech_1 = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)

# Permutation 2
#   The sequnce of conditions:
#     2. metabolic (other) (H49.811)
#     4. metabolic (tech)  (Z96.41)
#     6. respiratory       (J84.111)
expected_metabolic_dxpr_or_tech_2  = c(0L, 1L, 1L, 1L, 1L, 1L, 1L)
expected_metabolic_dxpr_only_2     = c(0L, 1L, 1L, 0L, 0L, 0L, 0L)
expected_metabolic_tech_only_2     = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)
expected_metabolic_dxpr_and_tech_2 = c(0L, 0L, 0L, 1L, 1L, 1L, 1L)

expected_respiratory_dxpr_or_tech_2  = c(0L, 0L, 0L, 0L, 0L, 1L, 1L)
expected_respiratory_dxpr_only_2     = c(0L, 0L, 0L, 0L, 0L, 1L, 1L)
expected_respiratory_tech_only_2     = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)
expected_respiratory_dxpr_and_tech_2 = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)

# Permutation 3
#   The sequnce of conditions:
#     2. respiratory       (J84.111)
#     4. metabolic (other) (H49.811)
#     6. metabolic (tech)  (Z96.41)
expected_metabolic_dxpr_or_tech_3  = c(0L, 0L, 0L, 1L, 1L, 1L, 1L)
expected_metabolic_dxpr_only_3     = c(0L, 0L, 0L, 1L, 1L, 0L, 0L)
expected_metabolic_tech_only_3     = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)
expected_metabolic_dxpr_and_tech_3 = c(0L, 0L, 0L, 0L, 0L, 1L, 1L)

expected_respiratory_dxpr_or_tech_3  = c(0L, 1L, 1L, 1L, 1L, 1L, 1L)
expected_respiratory_dxpr_only_3     = c(0L, 1L, 1L, 1L, 1L, 1L, 1L)
expected_respiratory_tech_only_3     = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)
expected_respiratory_dxpr_and_tech_3 = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)

# Permutation 4
#   The sequnce of conditions:
#     2. respiratory       (J84.111)
#     4. metabolic (tech)  (Z96.41)
#     6. metabolic (other) (H49.811)
expected_metabolic_dxpr_or_tech_4  = c(0L, 0L, 0L, 1L, 1L, 1L, 1L)
expected_metabolic_dxpr_only_4     = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)
expected_metabolic_tech_only_4     = c(0L, 0L, 0L, 1L, 1L, 0L, 0L)
expected_metabolic_dxpr_and_tech_4 = c(0L, 0L, 0L, 0L, 0L, 1L, 1L)

expected_respiratory_dxpr_or_tech_4  = c(0L, 1L, 1L, 1L, 1L, 1L, 1L)
expected_respiratory_dxpr_only_4     = c(0L, 1L, 1L, 1L, 1L, 1L, 1L)
expected_respiratory_tech_only_4     = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)
expected_respiratory_dxpr_and_tech_4 = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)

# Permutation 5
#   The sequnce of conditions:
#     2. metabolic (tech)  (Z96.41)
#     4. metabolic (other) (H49.811)
#     6. respiratory       (J84.111)
expected_metabolic_dxpr_or_tech_5  = c(0L, 0L, 0L, 1L, 1L, 1L, 1L)
expected_metabolic_dxpr_only_5     = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)
expected_metabolic_tech_only_5     = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)
expected_metabolic_dxpr_and_tech_5 = c(0L, 0L, 0L, 1L, 1L, 1L, 1L)

expected_respiratory_dxpr_or_tech_5  = c(0L, 0L, 0L, 0L, 0L, 1L, 1L)
expected_respiratory_dxpr_only_5     = c(0L, 0L, 0L, 0L, 0L, 1L, 1L)
expected_respiratory_tech_only_5     = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)
expected_respiratory_dxpr_and_tech_5 = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)

# Permutation 6
#   The sequnce of conditions:
#     2. metabolic (tech)  (Z96.41)
#     4. respiratory       (J84.111)
#     6. metabolic (other) (H49.811)
expected_metabolic_dxpr_or_tech_6  = c(0L, 0L, 0L, 1L, 1L, 1L, 1L)
expected_metabolic_dxpr_only_6     = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)
expected_metabolic_tech_only_6     = c(0L, 0L, 0L, 1L, 1L, 0L, 0L)
expected_metabolic_dxpr_and_tech_6 = c(0L, 0L, 0L, 0L, 0L, 1L, 1L)

expected_respiratory_dxpr_or_tech_6  = c(0L, 0L, 0L, 1L, 1L, 1L, 1L)
expected_respiratory_dxpr_only_6     = c(0L, 0L, 0L, 1L, 1L, 1L, 1L)
expected_respiratory_tech_only_6     = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)
expected_respiratory_dxpr_and_tech_6 = c(0L, 0L, 0L, 0L, 0L, 0L, 0L)

for (cnd in c("metabolic", "respiratory")) {
  for (col in c("dxpr_or_tech", "dxpr_only", "tech_only", "dxpr_and_tech")) {
    for (prm in 1:6) {
      check <-
        rtn[rtn[["permutation"]] == prm, paste(cnd, col, sep = "_")] ==
        get(paste("expected", cnd, col, prm, sep = "_"))
      if (!all(check)) {
        stop(sprintf("test for %s failed", paste("expected", cnd, col, prm, sep = "_")))
      }
    }
  }
}

################################################################################
#                                 End of File                                  #
################################################################################

#' Get ICD Codes
#'
#' Retrieve a copy of the internal lookup table for all known ICD codes.
#'
#' @details
#'
#' ## Sources
#' There are three sources of ICD codes.
#' * `cms`: Codes from the ICD-9-CM, ICD-9-PCS, ICD-10-CM, and ICD-10-PCS standards.
#' * `who`: Codes from World Health Organization.
#' * `cdc`: Codes from CDC Mortality coding standard.
#'
#' ## Fiscal and Calendar Years
#'
#' When reporting years there is a mix of fiscal and calendar years.
#'
#' Fiscal years are the United States Federal Government fiscal years, running
#' from October 1 to September 30. For example, fiscal year 2013 started October
#' 1 2012 and ended on September 30 2013.
#'
#' Calendar years run January 1 to December 31.
#'
#' Within the ICD data there are columns
#' `known_start`, `known_end`, `assignable_start`, `assignable_end`,
#' `desc_start` and `desc_end`.  For ICD codes with `src == "cms"`, these are
#' fiscal years.  For codes with `src == "cdc"` or `src == "who"` these are
#' calendar years.
#'
#' `known_start` is the first fiscal or calendar year (depending on source) that
#' the medicalcoder package as definitive source data for.  ICD-9-CM started in
#' the United States in fiscal year 1980.  The CDC extracts included in
#' medicalcoder span fiscal years 1997--2012; the CMS ICD-9-CM/PCS extracts
#' start in fiscal year 2006 and run through fiscal year 2015.  As such 1997 is
#' the earliest "known start" for ICD-9 within medicalcoder.
#'
#' `known_end` is the last fiscal or calendar year (depending on source)
#' for which we have definitive source data for.  For ICD-9-CM and ICD-9-PCS,
#' CMS provides data through fiscal year 2015, while the CDC extracts stop at
#' fiscal year 2012.  For ICD-10-CM and ICD-10-PCS, which are active, it is just
#' the last year of known data.  ICD-10 from the WHO ends in 2019.
#'
#' ## Header and Assignable Codes
#'
#' "Assignable" indicates that the code is the most granular for the source.
#' Ideally codes are reported with the greatest level of detail but that is not
#' always the case.  Also, the greatest level of detail can differ between
#' sources.
#
#' Example: C86 is a header code for `cms` and `who` because codes C86.0, C86.1,
#' C86.2, C86.3, C86.4, C86.5, and C86.6 all exist in both standards.  No code
#' with a fifth digit exists in the `who` so all these four digit codes are
#' 'assignable.'  In the `cms` standard, C86.0 was assignable through fiscal
#' year 2024. In fiscal year 2025 codes C86.00 and C86.01 were added making
#' C86.0 a header code and C86.00 and C86.01 assignable codes.
#'
#' @param with.descriptions Logical scalar, if `TRUE` include the
#'   description of the codes.
#' @param with.hierarchy Logical scalar, if `TRUE` include the ICD
#'   hierarchy.
#'
#' @seealso [is_icd()], [lookup_icd_codes()],
#'   `vignette(topic = "icd", package = "medicalcoder")`
#'
#' @family icd tools
#'
#' @returns a `data.frame`
#'
#' The default return has the following columns:
#'
#' * `icdv`: Integer vector indicating if the code is from ICD-9 or ICD-10
#' * `dx`: Integer vector.  1 if the code is a diagnostic, (ICD-9-CM, ICD-10-CM, WHO, CDC Mortality), or 0 if the code is procedural (ICD-9-PCS, ICD-10-PCS)
#' * `full_code`: Character vector with the ICD code and any relevant decimal point
#' * `code`: Character vector with the compact ICD code omitting any relevant decimal point
#' * `src`: Character vector reporting the source of the information.  See Details.
#' * `known_start`: Integer vector reporting the first known year of use.  See Details.
#' * `known_end`: Integer vector reporting the last known year of use.  See Details.
#' * `assignable_start`: Integer vector reporting the first known year the code was assignable.  See Details.
#' * `assignable_end`: Integer vector reporting the last known year the code was assignable.  See Details.
#'
#' When `with.descriptions = TRUE` there are the following additional columns:
#'
#' * `desc`: Character vector of descriptions.  For `cms` codes descriptions from CMS are used preferentially over CDC.
#' * `desc_start`: Integer vector of the first year the description was used.
#' * `desc_end`: Integer vector of the last year the description was used.
#'
#' When `with.hierarchy = TRUE` there are the following additional columns:
#'
#' * `chapter`
#' * `subchapter`
#' * `category`
#' * `subcategory`
#' * `subclassification`
#' * `subsubclassification`
#' * `extension`
#'
#' @example examples/get_icd_codes.R
#'
#' @export
get_icd_codes <- function(with.descriptions = FALSE, with.hierarchy = FALSE) {
  assert_scalar_logical(with.descriptions)
  assert_scalar_logical(with.hierarchy)

  column_order <-
    c("icdv", "dx", "full_code", "code", "src",
      "known_start", "known_end", "assignable_start", "assignable_end",
      "desc", "desc_start", "desc_end",
      "chapter", "subchapter", "category", "subcategory", "subclassification", "subsubclassification", "extension")

  if (!with.hierarchy) {
    if (!with.descriptions) {
      x <- get("icd_codes", envir = ..mdcr_data_env.., inherits = FALSE)
    } else {
      x <- get("icd_codes_with_desc", envir = ..mdcr_data_env.., inherits = FALSE)
    }
  } else {
    hier  <- icd_hierarchy()
    z <- get_icd_codes(with.descriptions = with.descriptions, with.hierarchy = FALSE)
    x <- merge(z, hier, all = TRUE, by = intersect(names(z), names(hier)))
  }

  x <- x[, column_order[column_order %in% names(x)], drop = FALSE]
  unserialize(serialize(x, connection = NULL))
}

################################################################################
# NON-Exported Functions
#
# Add more hierarchy to the data set
#
# FROM https://www.medicalbillingandcoding.org/
#
# ICD-9-CM codes are three-to-five digit numeric and, in certain cases, alpha-
# numeric codes. The first three digits in a code are called the “category.”
# The category describes the general illness, injury, or condition of the
# patient. In many cases, the category is not specific enough to describe the
# full extent of the patient’s condition. Take dementia, for example. The
# basic ICD-9-CM code for dementia is 290. These first three numbers are the
# category, but since dementia is such a complicated condition, you would
# almost never stop coding at this level.
#
# In cases where more specificity is needed, a decimal point is added after
# the category and one or two more digits are added. The fourth digit of the
# ICD-9 code is called the “subcategory,” and the fifth digit is called the
# “subclassification.”
#
# FROM https://www.medicalbillingandcoding.org/icd-10-cm/
#
# Layout and Organization
#
# ICD-10-CM is a seven-character, alphanumeric code. Each code begins with a
# letter, and that letter is followed by two numbers. The first three characters
# of ICD-10-CM are the “category.” The category describes the general type of
# the injury or disease. The category is followed by a decimal point and the
# subcategory. This is followed by up to two subclassifications, which further
# explain the cause, manifestation, location, severity, and type of injury or
# disease. The last character is the extension.
#
# The extension describes the type of encounter this is. That is, if this is the
# first time a healthcare provider has seen the patient for this
# condition/injury/disease, it’s listed as the “initial encounter.” Every
# encounter after the first is listed as a “subsequent encounter.” Patient
# visits related to the effects of a previous injury or disease are listed with
# the term “sequela.”
#
# To review: the first digit of an ICD-10-CM code is always an alpha, the second
# digit is always numeric, and digits three through seven may be alpha or
# numeric. Here’s a simplified look at ICD-10-CM’s format.
#
# A01 – {Disease}
#
# A01.0 {Disease] of the lungs
# A01.01 … simple
# A01.02 … complex
# A01.020 … affecting the trachea
# A01.021 … affecting the cardiopulmonary system
# A01.021A … initial encounter
# A01.021D … subsequent encounter
# A01.021S … sequela
#'
#' @keywords internal
icd_hierarchy <- function() {
  codes <- ..mdcr_internal_icd_codes..

  codes <- merge(x = codes, y = ..mdcr_internal_icd_chapters.., all = TRUE, by = "chap_id")
  codes <- merge(x = codes, y = ..mdcr_internal_icd_subchapters.., all = TRUE, by = "subchap_id")
  codes[["chap_id"]] <- NULL
  codes[["subchap_id"]] <- NULL

  codes[["category"]]             <- NA_character_
  codes[["subcategory"]]          <- NA_character_
  codes[["subclassification"]]    <- NA_character_
  codes[["subsubclassification"]] <- NA_character_
  codes[["extension"]]            <- NA_character_

  nchars <- nchar(codes[["code"]])

  icd10 <- codes[["icdv"]] == 10L

  i <- icd10 & nchars > 2; codes[["category"]][i]             <- substr(codes[["code"]][i], start = 1L, stop = 3L)
  i <- icd10 & nchars > 3; codes[["subcategory"]][i]          <- substr(codes[["code"]][i], start = 1L, stop = 4L)
  i <- icd10 & nchars > 4; codes[["subclassification"]][i]    <- substr(codes[["code"]][i], start = 1L, stop = 5L)
  i <- icd10 & nchars > 5; codes[["subsubclassification"]][i] <- substr(codes[["code"]][i], start = 1L, stop = 6L)
  i <- icd10 & nchars > 6; codes[["extension"]][i]            <- substr(codes[["code"]][i], start = 1L, stop = 7L)

  icd9dx <- codes[["icdv"]] == 9L & codes[["dx"]] == 1L

  i <- icd9dx & nchars > 2; codes[["category"]][i]          <- substr(codes[["code"]][i], start = 1L, stop = 3L)
  i <- icd9dx & nchars > 3; codes[["subcategory"]][i]       <- substr(codes[["code"]][i], start = 1L, stop = 4L)
  i <- icd9dx & nchars > 4; codes[["subclassification"]][i] <- substr(codes[["code"]][i], start = 1L, stop = 5L)

  icd9pr <- codes[["icdv"]] == 9L & codes[["dx"]] == 0L
  i <- icd9pr & nchars > 1; codes[["category"]][i]          <- substr(codes[["code"]][i], start = 1L, stop = 2L)
  i <- icd9pr & nchars > 2; codes[["subcategory"]][i]       <- substr(codes[["code"]][i], start = 1L, stop = 3L)
  i <- icd9pr & nchars > 3; codes[["subclassification"]][i] <- substr(codes[["code"]][i], start = 1L, stop = 4L)

  codes
}

################################################################################
#                                 End of File                                  #
################################################################################

#' Synthetic Data
#'
#' @format
#' `mdcr` is a `data.frame` with 4 columns, Each row is for one ICD id.
#'
#' * `patid`: patient identifier, integer values
#' * `icdv`: ICD version; integer values, 9 or 10
#' * `dx`: indicator column for ICD diagnostic (1) or procedure (0) codes
#' * `code`: ICD code; character values
#'
#' @family datasets
#'
"mdcr"

#' Synthetic Longitudinal Data
#'
#' @format
#' `mdcr_longitudinal` is a `data.frame` with 4 columns.  The codes are
#' expected to be treated as diagnostic codes. Warning: there are a few ICD-9
#' codes which could match to procedure codes.
#'
#' * `patid`: patient identifier, integer values
#' * `date`: date the diagnostic code was recorded
#' * `icdv`: ICD version 9 or 10, integer valued
#' * `code`: ICD codes; character values
#'
#' @family datasets
#'
"mdcr_longitudinal"

source('utilities.R')
################################################################################
# Tests for the charlson comorbidities
library(medicalcoder)

################################################################################
# verify the list of possible charlson methods
m <- grep("charlson_", medicalcoder:::comorbidities_methods(), value = TRUE)
m <- sort(m)
stopifnot(
  identical(
    m,
    c("charlson_cdmf2019", "charlson_deyo1992", "charlson_quan2005", "charlson_quan2011")
  )
)

# add an age variable
mdcr$age <- as.integer(substr(as.character(mdcr$patid), 1, 2))

cdmf         <- comorbidities(data = mdcr, id.vars = "patid", icdv.var = "icdv", icd.codes = "code", dx.var = "dx", method = "charlson_cdmf2019", poa = 1, primarydx = 0L)
cdmf_withage <- comorbidities(data = mdcr, id.vars = "patid", icdv.var = "icdv", icd.codes = "code", dx.var = "dx", method = "charlson_cdmf2019", poa = 1, age.var = "age", primarydx = 0L)

deyo         <- comorbidities(data = mdcr, id.vars = "patid", icdv.var = "icdv", icd.codes = "code", dx.var = "dx", method = "charlson_deyo1992", poa = 1, primarydx = 0L)
deyo_withage <- comorbidities(data = mdcr, id.vars = "patid", icdv.var = "icdv", icd.codes = "code", dx.var = "dx", method = "charlson_deyo1992", poa = 1, age.var = "age", primarydx = 0L)

quan2005         <- comorbidities(data = mdcr, id.vars = "patid", icdv.var = "icdv", icd.codes = "code", dx.var = "dx", method = "charlson_quan2005", poa = 1, primarydx = 0L)
quan2005_withage <- comorbidities(data = mdcr, id.vars = "patid", icdv.var = "icdv", icd.codes = "code", dx.var = "dx", method = "charlson_quan2005", poa = 1, age.var = "age", primarydx = 0L)

quan2011         <- comorbidities(data = mdcr, id.vars = "patid", icdv.var = "icdv", icd.codes = "code", dx.var = "dx", method = "charlson_quan2011", poa = 1, primarydx = 0L)
quan2011_withage <- comorbidities(data = mdcr, id.vars = "patid", icdv.var = "icdv", icd.codes = "code", dx.var = "dx", method = "charlson_quan2011", poa = 1, age.var = "age", primarydx = 0L)

# verify that a row is returned for each and every id.var
npatid <- length(unique(mdcr[["patid"]]))
stopifnot(
  nrow(cdmf) == npatid,
  nrow(cdmf_withage) == npatid,
  nrow(deyo) == npatid,
  nrow(deyo_withage) == npatid,
  nrow(quan2005) == npatid,
  nrow(quan2005_withage) == npatid,
  nrow(quan2011) == npatid,
  nrow(quan2011_withage) == npatid
)

# verify that age_score are NA or known
stopifnot(
   all(is.na(cdmf$age_score)),
  !any(is.na(cdmf_withage$age_score)),
   all(is.na(deyo$age_score)),
  !any(is.na(deyo_withage$age_score)),
   all(is.na(quan2005$age_score)),
  !any(is.na(quan2005_withage$age_score)),
   all(is.na(quan2011$age_score)),
  !any(is.na(quan2011_withage$age_score))
)

# verify that the cci is known and that the cci sans age <= cci with age
CDMF <- merge(cdmf, cdmf_withage, by = "patid")
stopifnot(
  !any(is.na(CDMF[["cci.x"]])),
  !any(is.na(CDMF[["cci.y"]])),
  all(CDMF[["cci.x"]] <= CDMF[["cci.y"]])
)

DEYO <- merge(deyo, deyo_withage, by = "patid")
stopifnot(
  !any(is.na(DEYO[["cci.x"]])),
  !any(is.na(DEYO[["cci.y"]])),
  all(DEYO[["cci.x"]] <= DEYO[["cci.y"]])
)

QUAN05 <- merge(quan2005, quan2005_withage, by = "patid")
stopifnot(
  !any(is.na(QUAN05[["cci.x"]])),
  !any(is.na(QUAN05[["cci.y"]])),
  all(QUAN05[["cci.x"]] <= QUAN05[["cci.y"]])
)

QUAN11 <- merge(quan2011, quan2011_withage, by = "patid")
stopifnot(
  !any(is.na(QUAN11[["cci.x"]])),
  !any(is.na(QUAN11[["cci.y"]])),
  all(QUAN11[["cci.x"]] <= QUAN11[["cci.y"]])
)

################################################################################
# Test that the aids and hiv flags as expected in charlson_cdmf2019

# Note: if this fails and needs to be updated there is a section in the
# comorbidities vignette that will need to be updated too.

cdmf_eg <-
  merge(x = mdcr,
        y = subset(get_charlson_codes(),
                   condition %in% c("aids", "hiv") &
                   charlson_cdmf2019 == 1),
        by = c("icdv", "dx", "code"))

cdmf_eg <-
  aggregate(
    x = cdmf_eg[["charlson_cdmf2019"]],
    by = cdmf_eg[c("patid", "condition")],
    FUN = function(x) as.integer(sum(x) > 0)
  )

cdmf_eg <-
  merge(
    x = cdmf_eg[cdmf_eg$condition == "aids", c("patid", "x")],
    y = cdmf_eg[cdmf_eg$condition == "hiv",  c("patid", "x")],
    all = TRUE,
    suffixes = c(".aids", ".hiv"),
    by = c("patid")
  )

stopifnot(
  identical(
    table(cdmf_eg[, c("x.aids", "x.hiv")], useNA = "always"),
    structure(c(1L, 6L, 2716L, 0L), dim = c(2L, 2L), dimnames = list(x.aids = c("1", NA), x.hiv = c("1", NA)), class = "table")
  )
)

cmdf_mdcr <-
  comorbidities(
    data = mdcr,
    icd.codes = "code",
    id.vars = "patid",
    icdv.var = "icdv",
    dx.var = "dx",
    method = "charlson_cdmf2019",
    flag.method = "current",
    poa = 1L,
    primarydx = 0L
  )

stopifnot(
  identical(
    table(cmdf_mdcr[, c("hiv", "aids")]),
    structure(c(38255L, 6L, 0L, 1L), dim = c(2L, 2L), dimnames = list(hiv = c("0", "1"), aids = c("0", "1")), class = "table")
  )
)

################################################################################
# tests for different sets of inputs
# with or with icdv
# with or without dx

mdcr2 <- mdcr

# add a patient and row to this data set that will result in a false positive
# for charlson_quan2005 when dx is not specified correctly.  This can be an
# ICD-9 procedure code which has the same compact form as an ICD-9 diagnostic
# code.  Found a good code for this.  ICD-9 5829, as a diagnostic code maps to
# rnd, as a procedure code it would map to nothing.  Add a patient to mdcr2 with
# this procedure code.
if (interactive()) {
  merge(
    get_icd_codes(),
    subset(get_charlson_codes(), charlson_quan2005 == 1 & icdv == 9, "code"),
    by = "code"
  )

  lookup_icd_codes("5829")
  subset(get_charlson_codes(), charlson_quan2005 == 1 & code == "5829")
  mdcr[mdcr$code == "5829", ]
}

mdcr2 <-
  rbind(
    mdcr2,
    data.frame(patid = 0, icdv = 9, code = "5829", dx = 0L, age = 42)
  )

#common arguments for the calls to comorbidities for just this bespoke patient
cargs <-
  list(
    data = data.frame(patid = 0, icdv = 9, code = "5829", dx = 0L, age = 42),
    id.vars = "patid",
    icd.codes = "code",
    poa = 1L,
    primarydx = 0L,
    method = "charlson_quan2005"
  )

out00 <- do.call(comorbidities, c(cargs, list(icdv.var = "icdv", dx.var = "dx")))
out01 <- do.call(comorbidities, c(cargs, list(                   dx.var = "dx")))
out02 <- do.call(comorbidities, c(cargs, list(icdv = 9,          dx.var = "dx")))
out03 <- do.call(comorbidities, c(cargs, list(icdv = 10,         dx.var = "dx")))
out04 <- do.call(comorbidities, c(cargs, list(icdv = 9)))
out05 <- do.call(comorbidities, c(cargs, list(icdv = 10)))
out06 <- do.call(comorbidities, c(cargs, list(icdv.var = "icdv"        )))
out07 <- do.call(comorbidities, c(cargs, list(icdv.var = "icdv", dx = 0)))
out08 <- do.call(comorbidities, c(cargs, list(icdv.var = "icdv", dx = 1)))
out09 <- do.call(comorbidities, c(cargs, list(                   dx = 0)))
out10 <- do.call(comorbidities, c(cargs, list(                   dx = 1)))
out11 <- do.call(comorbidities, cargs)

expected_out_false_positive <- structure(list(patid = 0, aidshiv = 0L, mal = 0L, cebvd = 0L, copd = 0L, chf = 0L, dem = 0L, dmc = 0L, dm = 0L, hp = 0L, mld = 0L, msld = 0L, mst = 0L, mi = 0L, pud = 0L, pvd = 0L, rnd = 1L, rhd = 0L, num_cmrb = 1L, cmrb_flag = 1L, cci = 2L, age_score = NA_integer_), row.names = c(NA, -1L), class = c("medicalcoder_comorbidities", "data.frame"), method = "charlson_quan2005", id.vars = "patid", flag.method = "current")
expected_out <- structure(list(patid = 0, aidshiv = 0L, mal = 0L, cebvd = 0L, copd = 0L, chf = 0L, dem = 0L, dmc = 0L, dm = 0L, hp = 0L, mld = 0L, msld = 0L, mst = 0L, mi = 0L, pud = 0L, pvd = 0L, rnd = 0L, rhd = 0L, num_cmrb = 0L, cmrb_flag = 0L, cci = 0L, age_score = NA_integer_), row.names = c(NA, -1L), class = c("medicalcoder_comorbidities", "data.frame"), method = "charlson_quan2005", id.vars = "patid", flag.method = "current")

stopifnot(
  identical(out00, expected_out),
  identical(out01, expected_out),
  identical(out02, expected_out),
  identical(out03, expected_out),
  identical(out04, expected_out_false_positive),
  identical(out05, expected_out),
  identical(out06, expected_out_false_positive),
  identical(out07, expected_out),
  identical(out08, expected_out_false_positive),
  identical(out09, expected_out),
  identical(out10, expected_out_false_positive),
  identical(out11, expected_out_false_positive)
)

# More general: common arguments for the calls to comorbidities
cargs <-
  list(
    data = mdcr2,
    id.vars = "patid",
    icd.codes = "code",
    poa = 1L,
    primarydx = 0L,
    method = "charlson_quan2005"
  )

out00 <- do.call(comorbidities, c(cargs, list(icdv.var = "icdv", dx.var = "dx")))
out01 <- do.call(comorbidities, c(cargs, list(                   dx.var = "dx")))
out02 <- do.call(comorbidities, c(cargs, list(icdv = 9,          dx.var = "dx")))
out03 <- do.call(comorbidities, c(cargs, list(icdv = 10,         dx.var = "dx")))
out04 <- do.call(comorbidities, c(cargs, list(icdv.var = "icdv"               )))
out05 <- do.call(comorbidities, c(cargs, list(icdv.var = "icdv", dx = 0       )))
out06 <- do.call(comorbidities, c(cargs, list(icdv.var = "icdv", dx = 1       )))

# all the outputs should be unique _except_ out04 and out06: charlson is only
# defined on diagnostic codes, so when nothing is passed for dx.var and dx then
# any diagnostic or procedure code will flag, in the same way that when dx = 1
# will treat al the input codes as diagnostic and will have the same false
# positives.
stopifnot(
  !isTRUE(all.equal(out00, out01)),
  !isTRUE(all.equal(out00, out02)),
  !isTRUE(all.equal(out00, out03)),
  !isTRUE(all.equal(out00, out04)),
  !isTRUE(all.equal(out00, out05)),
  !isTRUE(all.equal(out00, out06)),
  !isTRUE(all.equal(out01, out02)),
  !isTRUE(all.equal(out01, out03)),
  !isTRUE(all.equal(out01, out04)),
  !isTRUE(all.equal(out01, out05)),
  !isTRUE(all.equal(out01, out06)),
  !isTRUE(all.equal(out02, out03)),
  !isTRUE(all.equal(out02, out04)),
  !isTRUE(all.equal(out02, out05)),
  !isTRUE(all.equal(out02, out06)),
  !isTRUE(all.equal(out03, out04)),
  !isTRUE(all.equal(out03, out05)),
  !isTRUE(all.equal(out03, out06)),
  !isTRUE(all.equal(out04, out05)),
   isTRUE(all.equal(out04, out06)),
  !isTRUE(all.equal(out05, out06))
)

################################################################################
#                                 End of File                                  #
################################################################################

################################################################################
# file: test-vs-ahrq.R
#
# objective:
#
#   compare medicalcoder::comorbidities(..., method = "elixhauser_ahrqYYYY")
#   against the official AHRQ SAS mapping results when fixtures are available.
#
# Note: using stats::reshape instead of data.table::melt so that this test can
# run when no-suggests is TRUE.
#
################################################################################
library(medicalcoder)
source('utilities.R')

# ahrq results
ahrq_results <- readRDS("expected-ahrq-results.rds")

codes <-
  stats::reshape(
    data = ahrq_results,
    direction = "long",
    idvar = c("CMR_VERSION", "PATID"),
    varying = list(
      grep("^I10_DX", names(ahrq_results), value = TRUE),
      grep("^DXPOA",  names(ahrq_results), value = TRUE)
      ),
    v.names = c("code", "poa"),
    times = seq_along(grep("^I10_DX", names(ahrq_results)))
  )

codes[["primarydx"]] <- as.integer(codes[["time"]] == "1")
codes[["poa"]] <- as.integer(codes[["poa"]] == "Y")

# apply medicalcoder::comorbidities
common_args <-
  list(
    icd.codes = "code",
    id.vars = c("CMR_VERSION", "PATID"),
    dx = 1,
    icdv = 10,
    poa.var = "poa",
    primarydx.var = "primarydx",
    flag.method = "current"
  )

mdcr_2022 <- do.call(comorbidities, c(common_args, list(data = subset(codes, CMR_VERSION == 2022.1), method = "elixhauser_ahrq2022")))
mdcr_2023 <- do.call(comorbidities, c(common_args, list(data = subset(codes, CMR_VERSION == 2023.1), method = "elixhauser_ahrq2023")))
mdcr_2024 <- do.call(comorbidities, c(common_args, list(data = subset(codes, CMR_VERSION == 2024.1), method = "elixhauser_ahrq2024")))
mdcr_2025 <- do.call(comorbidities, c(common_args, list(data = subset(codes, CMR_VERSION == 2025.1), method = "elixhauser_ahrq2025")))
mdcr_2026 <- do.call(comorbidities, c(common_args, list(data = subset(codes, CMR_VERSION == 2026.1), method = "elixhauser_ahrq2026")))

mdcr_vs_ahrq_2022 <- merge(x = mdcr_2022, y = ahrq_results, all.x = TRUE, by = c("CMR_VERSION", "PATID"))
mdcr_vs_ahrq_2023 <- merge(x = mdcr_2023, y = ahrq_results, all.x = TRUE, by = c("CMR_VERSION", "PATID"))
mdcr_vs_ahrq_2024 <- merge(x = mdcr_2024, y = ahrq_results, all.x = TRUE, by = c("CMR_VERSION", "PATID"))
mdcr_vs_ahrq_2025 <- merge(x = mdcr_2025, y = ahrq_results, all.x = TRUE, by = c("CMR_VERSION", "PATID"))
mdcr_vs_ahrq_2026 <- merge(x = mdcr_2026, y = ahrq_results, all.x = TRUE, by = c("CMR_VERSION", "PATID"))

stopifnot("same number of rows (2022)" = nrow(mdcr_2022) == nrow(subset(ahrq_results, CMR_VERSION == 2022.1)) & nrow(mdcr_2022) == nrow(mdcr_vs_ahrq_2022))
stopifnot("same number of rows (2023)" = nrow(mdcr_2023) == nrow(subset(ahrq_results, CMR_VERSION == 2023.1)) & nrow(mdcr_2023) == nrow(mdcr_vs_ahrq_2023))
stopifnot("same number of rows (2024)" = nrow(mdcr_2024) == nrow(subset(ahrq_results, CMR_VERSION == 2024.1)) & nrow(mdcr_2024) == nrow(mdcr_vs_ahrq_2024))
stopifnot("same number of rows (2025)" = nrow(mdcr_2025) == nrow(subset(ahrq_results, CMR_VERSION == 2025.1)) & nrow(mdcr_2025) == nrow(mdcr_vs_ahrq_2025))
stopifnot("same number of rows (2026)" = nrow(mdcr_2026) == nrow(subset(ahrq_results, CMR_VERSION == 2026.1)) & nrow(mdcr_2026) == nrow(mdcr_vs_ahrq_2026))

# check each condition - this can change year to year
cnds_2022 <- subset(get_elixhauser_index_scores(), !is.na(elixhauser_ahrq2022), select = "condition", drop = TRUE)
cnds_2022 <- unique(cnds_2022)

cnds_2023 <- subset(get_elixhauser_index_scores(), !is.na(elixhauser_ahrq2023), select = "condition", drop = TRUE)
cnds_2023 <- unique(cnds_2023)

cnds_2024 <- subset(get_elixhauser_index_scores(), !is.na(elixhauser_ahrq2024), select = "condition", drop = TRUE)
cnds_2024 <- unique(cnds_2024)

cnds_2025 <- subset(get_elixhauser_index_scores(), !is.na(elixhauser_ahrq2025), select = "condition", drop = TRUE)
cnds_2025 <- unique(cnds_2025)

cnds_2026 <- subset(get_elixhauser_index_scores(), !is.na(elixhauser_ahrq2026), select = "condition", drop = TRUE)
cnds_2026 <- unique(cnds_2026)

for (j in cnds_2022) {
  t <- identical(mdcr_vs_ahrq_2022[[j]], mdcr_vs_ahrq_2022[[paste0("CMR_", j)]])
  if (!t) {# & !(j %in% c("CBVD", "HF"))) {
    stop(sprintf('identical(mdcr_vs_ahrq_2022[["%s"]], mdcr_vs_ahrq_2022[["%s"]]) is not true', j, paste0("CMR_", j)))
  }
}

for (j in cnds_2023) {
  t <- identical(mdcr_vs_ahrq_2023[[j]], mdcr_vs_ahrq_2023[[paste0("CMR_", j)]])
  if (!t) {
    stop(sprintf('identical(mdcr_vs_ahrq_2023[["%s"]], mdcr_vs_ahrq_2023[["%s"]]) is not true', j, paste0("CMR_", j)))
  }
}

for (j in cnds_2024) {
  t <- identical(mdcr_vs_ahrq_2024[[j]], mdcr_vs_ahrq_2024[[paste0("CMR_", j)]])
  if (!t) {
    stop(sprintf('identical(mdcr_vs_ahrq_2024[["%s"]], mdcr_vs_ahrq_2024[["%s"]]) is not true', j, paste0("CMR_", j)))
  }
}

for (j in cnds_2025) {
  t <- identical(mdcr_vs_ahrq_2025[[j]], mdcr_vs_ahrq_2025[[paste0("CMR_", j)]])
  if (!t) {
    stop(sprintf('identical(mdcr_vs_ahrq_2025[["%s"]], mdcr_vs_ahrq_2025[["%s"]]) is not true', j, paste0("CMR_", j)))
  }
}

for (j in cnds_2026) {
  t <- identical(mdcr_vs_ahrq_2026[[j]], mdcr_vs_ahrq_2026[[paste0("CMR_", j)]])
  if (!t) {
    stop(sprintf('identical(mdcr_vs_ahrq_2026[["%s"]], mdcr_vs_ahrq_2026[["%s"]]) is not true', j, paste0("CMR_", j)))
  }
}

stopifnot(mdcr_vs_ahrq_2022[["mortality_index"]]   == mdcr_vs_ahrq_2022[["CMR_Index_Mortality"]])
stopifnot(mdcr_vs_ahrq_2022[["readmission_index"]] == mdcr_vs_ahrq_2022[["CMR_Index_Readmission"]])

stopifnot(mdcr_vs_ahrq_2023[["mortality_index"]]   == mdcr_vs_ahrq_2023[["CMR_Index_Mortality"]])
stopifnot(mdcr_vs_ahrq_2023[["readmission_index"]] == mdcr_vs_ahrq_2023[["CMR_Index_Readmission"]])

stopifnot(mdcr_vs_ahrq_2024[["mortality_index"]]   == mdcr_vs_ahrq_2024[["CMR_Index_Mortality"]])
stopifnot(mdcr_vs_ahrq_2024[["readmission_index"]] == mdcr_vs_ahrq_2024[["CMR_Index_Readmission"]])

stopifnot(mdcr_vs_ahrq_2025[["mortality_index"]]   == mdcr_vs_ahrq_2025[["CMR_Index_Mortality"]])
stopifnot(mdcr_vs_ahrq_2025[["readmission_index"]] == mdcr_vs_ahrq_2025[["CMR_Index_Readmission"]])

stopifnot(mdcr_vs_ahrq_2026[["mortality_index"]]   == mdcr_vs_ahrq_2026[["CMR_Index_Mortality"]])
stopifnot(mdcr_vs_ahrq_2026[["readmission_index"]] == mdcr_vs_ahrq_2026[["CMR_Index_Readmission"]])

################################################################################
#                                 End of File                                  #
################################################################################

#' Internal Implementation of Pediatric Complex Chronic Conditions (PCCC) V2 and V3
#'
#' Internal.  Called from [`comorbidities()`].  Not expected to be called from
#' any other context.
#'
#' @param id.vars A character vector of the id variables
#'
#' @param iddf A `data.frame` with the unique `id.vars` columns from
#'   the `data` passed to [comorbidities()].
#'
#' @param cmrb A `data.frame` with the unique conditions and possible
#'   subconditions flagged by ICD in [comorbidities()].
#'
#' @param subconditions Logic scalar, if TRUE build subcondition results.
#'
#' @family internal comorbidity functions
#'
#' @noRd
#' @keywords internal
.pccc_v3 <- function(id.vars, iddf, cmrb, subconditions) {
  # The v3 schema tracks four mutually exclusive condition flags: codes flagged
  # by diagnostic/procedure codes only, tech-dependent codes only, the
  # intersection, and the union. Build the matrices separately so tech-only
  # signals can be handled after checking whether a non-tech condition was
  # flagged for the patient.
  any_transplant <- mdcr_unique(mdcr_subset(x = cmrb, i = cmrb[["transplant_flag"]] == 1L, cols = c(id.vars)))

  # Build ccc indicators based on only non-tech_dep codes
  # _or_ a tech_dep transplant.  There are codes which are both tech_dep and
  # transplant.  since transplant is end stage organ failure these should be
  # flagged too.
  #
  # This will be done in two steps. Build on indicator matrix based on non-tech
  # codes and a second on based on tech and transplant codes.  We also build an
  # indicator matrix for tech related.
  dxpr_iddf <- mdcr_subset(cmrb, i = cmrb[["tech_dep_flag"]] == 0L, cols = id.vars)
  dxpr_cmrb <- mdcr_subset(cmrb, i = cmrb[["tech_dep_flag"]] == 0L, cols = c(id.vars, "condition"))

  techtrans_iddf <-
    mdcr_subset(cmrb,
                i = (cmrb[["tech_dep_flag"]] * cmrb[["transplant_flag"]] == 1L),
                cols = id.vars)

  techtrans_cmrb <-
    mdcr_subset(cmrb,
                i = (cmrb[["tech_dep_flag"]] * cmrb[["transplant_flag"]] == 1L),
                cols = c(id.vars, "condition"))

  tech_iddf <-
    mdcr_subset(cmrb,
                i = (cmrb[["tech_dep_flag"]] == 1L),
                cols = id.vars)

  tech_cmrb <-
    mdcr_subset(cmrb,
                i = (cmrb[["tech_dep_flag"]] == 1L),
                cols = c(id.vars, "condition"))

  conditions <- sort(unique(..mdcr_internal_pccc_conditions..[["condition"]]))

  # build indicator matrix for conditions
  dxpr_X      <- matrix(0L, nrow = nrow(iddf), ncol = length(conditions))
  techtrans_X <- matrix(0L, nrow = nrow(iddf), ncol = length(conditions))
  tech_X      <- matrix(0L, nrow = nrow(iddf), ncol = length(conditions))
  colnames(dxpr_X) <- colnames(techtrans_X) <- colnames(tech_X) <- conditions

  key_iddf      <- do.call(paste, c(iddf, sep = "\r"))
  key_dxpr      <- do.call(paste, c(dxpr_iddf, sep = "\r"))
  key_techtrans <- do.call(paste, c(techtrans_iddf, sep = "\r"))
  key_tech      <- do.call(paste, c(tech_iddf, sep = "\r"))

  ri <- match(key_dxpr, key_iddf)
  ci <- match(dxpr_cmrb[["condition"]], conditions)
  keep <- !(is.na(ri) | is.na(ci))
  if (any(keep)) {
    dxpr_X[cbind(ri[keep], ci[keep])] <- 1L
  }

  ri <- match(key_techtrans, key_iddf)
  ci <- match(techtrans_cmrb[["condition"]], conditions)
  keep <- !(is.na(ri) | is.na(ci))
  if (any(keep)) {
    techtrans_X[cbind(ri[keep], ci[keep])] <- 1L
  }

  ri <- match(key_tech, key_iddf)
  ci <- match(tech_cmrb[["condition"]], conditions)
  keep <- !(is.na(ri) | is.na(ci))
  if (any(keep)) {
    tech_X[cbind(ri[keep], ci[keep])] <- 1L
  }

  # create a dxpr_or_tech matrix
  dxpr_or_tech_X <- (dxpr_X | techtrans_X)

  # If at least one condition is flagged in dxpr_or_tech_X then the tech_X
  # indicator will remain as a 0 or 1.  If there is no condition flagged in
  # dxpr_or_tech_X, then the corresponding rows in tech_X are set to
  # NA_integer_.  For the moment, keep the 0/1 in the tech_X and use it to build
  # the updated dxpr_or_tech_X.
  non_tech_condition <- rowSums(dxpr_or_tech_X) > 0L
  tech_X[!non_tech_condition, ] <- 0L
  dxpr_or_tech_X <- (dxpr_or_tech_X | tech_X)
  storage.mode(dxpr_or_tech_X) <- "integer"

  tech_X <- (techtrans_X | tech_X)
  storage.mode(tech_X) <- "integer"
  any_tech_dep <- as.integer(rowSums(tech_X) > 0L)

  dxpr_and_tech_X <- (dxpr_X & tech_X)
  storage.mode(dxpr_and_tech_X) <- "integer"

  tech_X <- xor(tech_X, dxpr_and_tech_X)
  dxpr_X <- xor(dxpr_X, dxpr_and_tech_X)
  storage.mode(dxpr_X) <- "integer"
  storage.mode(tech_X) <- "integer"

  colnames(dxpr_X) <- paste0(colnames(dxpr_X), "_dxpr_only")
  colnames(tech_X) <- paste0(colnames(tech_X), "_tech_only")
  colnames(dxpr_or_tech_X) <- paste0(colnames(dxpr_or_tech_X), "_dxpr_or_tech")
  colnames(dxpr_and_tech_X) <- paste0(colnames(dxpr_and_tech_X), "_dxpr_and_tech")

  X <- cbind(dxpr_X, tech_X, dxpr_and_tech_X, dxpr_or_tech_X, any_tech_dep)
  X <- cbind(X, any_transplant = rep(0L, nrow(X)))
  X <- cbind(X, num_cmrb = rep(NA_integer_, nrow(X)))
  X <- cbind(X, cmrb_flag = rep(NA_integer_, nrow(X)))

  key_tran <- do.call(paste, c(any_transplant, sep = "\r"))
  X[match(key_tran, key_iddf), "any_transplant"] <- 1L

  X[, "num_cmrb"] <- as.integer(rowSums(dxpr_or_tech_X))
  X[, "cmrb_flag"] <- as.integer(X[, "num_cmrb"] > 0L)

  colorder <-
    c(
      paste0(rep(conditions, each = 4L),
             c("_dxpr_only", "_tech_only", "_dxpr_and_tech", "_dxpr_or_tech"))
      , "any_tech_dep", "any_transplant", "num_cmrb", "cmrb_flag")

  X <- X[, colorder, drop = FALSE]

  rtn <- mdcr_cbind(iddf, as.data.frame(X, check.names = FALSE, stringsAsFactors = FALSE))

  if (subconditions) {
    rtn <- list(conditions = rtn, subconditions = list())
    for (cnd in conditions) {
      scnd <- mdcr_unique(mdcr_subset(cmrb, i = cmrb[["condition"]] == cnd, cols = c(id.vars, "subcondition")))

      # subset the uiddf to the set of those with the condition
      uiddf <- mdcr_subset(rtn[["conditions"]],
                           i = rtn[["conditions"]][[paste0(cnd, "_dxpr_or_tech")]] == 1,
                           cols = id.vars)

      key_uiddf <- do.call(paste, c(uiddf, sep = "\r"))

      key_ciddf <- do.call(paste, c(mdcr_select(scnd, id.vars), sep = "\r"))
      ri <- match(key_ciddf, key_uiddf)

      scnds <- sort(unique(..mdcr_internal_pccc_conditions..[..mdcr_internal_pccc_conditions..[["condition"]] == cnd, "subcondition"]))
      ci <- match(scnd[["subcondition"]], scnds)
      X <- matrix(0L, nrow = nrow(uiddf), ncol = length(scnds))
      colnames(X) <- scnds

      keep <- !(is.na(ri) | is.na(ci))
      if (any(keep)) {
        X[cbind(ri[keep], ci[keep])] <- 1L
      }

      rtn[["subconditions"]][[cnd]] <- mdcr_cbind(uiddf, as.data.frame(X, check.names = FALSE, stringsAsFactors = FALSE))
    }
  }

  rtn
}

#' @family internal comorbidity functions
#' @noRd
#' @keywords internal
.pccc_v2 <- function(id.vars, iddf, cmrb, subconditions) {
  any_tech_dep   <- mdcr_unique(mdcr_subset(cmrb, i = cmrb[["tech_dep_flag"]] == 1L,   cols = c(id.vars)))
  any_transplant <- mdcr_unique(mdcr_subset(cmrb, i = cmrb[["transplant_flag"]] == 1L, cols = c(id.vars)))

  CMRB <- mdcr_select(cmrb, cols = c(id.vars, "condition"))
  CMRB <- mdcr_unique(CMRB)

  conditions <- sort(unique(..mdcr_internal_pccc_conditions..[["condition"]]))

  # build indicator matrix for conditions
  X <- matrix(0L, nrow = nrow(iddf), ncol = length(conditions))
  colnames(X) <- conditions

  key_iddf <- do.call(paste, c(iddf, sep = "\r"))
  key_CMRB <- do.call(paste, c(mdcr_select(CMRB, id.vars), sep = "\r"))
  key_tech <- do.call(paste, c(any_tech_dep, sep = "\r"))
  key_tran <- do.call(paste, c(any_transplant, sep = "\r"))
  ri <- match(key_CMRB, key_iddf)
  ci <- match(CMRB[["condition"]], conditions)
  keep <- !(is.na(ri) | is.na(ci))
  if (any(keep)) {
    X[cbind(ri[keep], ci[keep])] <- 1L
  }

  num_cmrb <- as.integer(rowSums(X))
  cmrb_flag <- as.integer(num_cmrb > 0L)

  X <- cbind(X, any_tech_dep = rep(0L, nrow(X)), any_transplant = rep(0L, nrow(X)))

  X[match(key_tech, key_iddf), "any_tech_dep"] <- 1L
  X[match(key_tran, key_iddf), "any_transplant"] <- 1L
  X <- cbind(X, num_cmrb, cmrb_flag)

  rtn <- mdcr_cbind(iddf, as.data.frame(X, check.names = FALSE, stringsAsFactors = FALSE))

  if (subconditions) {
    rtn <- list(conditions = rtn, subconditions = list())
    for (cnd in conditions) {
      scnd <- mdcr_unique(mdcr_subset(cmrb, i = cmrb[["condition"]] == cnd, cols = c(id.vars, "subcondition")))

      # subset the uiddf to the set of those with the condition
      uiddf <- mdcr_subset(rtn[["conditions"]],
                           i = rtn[["conditions"]][[cnd]] == 1,
                           cols = id.vars)
      key_uiddf <- do.call(paste, c(uiddf, sep = "\r"))

      key_ciddf <- do.call(paste, c(mdcr_select(scnd, id.vars), sep = "\r"))
      ri <- match(key_ciddf, key_uiddf)

      scnds <- sort(unique(..mdcr_internal_pccc_conditions..[..mdcr_internal_pccc_conditions..[["condition"]] == cnd, "subcondition"]))
      ci <- match(scnd[["subcondition"]], scnds)
      X <- matrix(0L, nrow = nrow(uiddf), ncol = length(scnds))
      colnames(X) <- scnds

      keep <- !(is.na(ri) | is.na(ci))
      if (any(keep)) {
        X[cbind(ri[keep], ci[keep])] <- 1L
      }

      rtn[["subconditions"]][[cnd]] <- mdcr_cbind(uiddf, as.data.frame(X, check.names = FALSE, stringsAsFactors = FALSE))
    }
  }

  rtn
}

source('utilities.R')
library(medicalcoder)

################################################################################
# verify the list of possible elixhauser methods
m <- grep("elixhauser_", medicalcoder:::comorbidities_methods(), value = TRUE)
stopifnot(
  identical(
    m,
    c("elixhauser_elixhauser1988",
      "elixhauser_ahrq_web",
      "elixhauser_quan2005",
      "elixhauser_ahrq2022",
      "elixhauser_ahrq2023",
      "elixhauser_ahrq2024",
      "elixhauser_ahrq2025",
      "elixhauser_ahrq2026",
      "elixhauser_ahrq_icd10"
    )
  )
)

################################################################################
# verify that the elixhauser_ahrq_icd10 covers all the codes for prior
# elixhauser_ahrqYYYY methods
ec <- get_elixhauser_codes()

ahrqYYYY <- ec[, grep("elixhauser_ahrq\\d{4}", names(ec), value = TRUE)]
stopifnot(
  sum(as.integer(rowSums(ahrqYYYY, na.rm = TRUE) > 0)) == sum(ec[["elixhauser_ahrq_icd10"]], na.rm = TRUE)
)

################################################################################
#                                 End of File                                  #
################################################################################

################################################################################
# Verify primary diagnoses are excluded from comorbidity flagging
library(medicalcoder)

df <- data.frame(
  patid = c(1L, 2L),
  icdv  = c(10L, 10L),
  dx    = c(1L, 1L),
  code  = c("I252", "I252"), # MI code that should flag Charlson when counted
  primarydx = c(0L, 1L),
  stringsAsFactors = FALSE
)

res0 <- comorbidities(
  data      = df,
  id.vars   = "patid",
  icdv.var  = "icdv",
  dx.var    = "dx",
  icd.codes = "code",
  method    = "charlson_quan2011",
  poa       = 1L,
  primarydx = 0L
)

res1 <- comorbidities(
  data      = df,
  id.vars   = "patid",
  icdv.var  = "icdv",
  dx.var    = "dx",
  icd.codes = "code",
  method    = "charlson_quan2011",
  poa       = 1L,
  primarydx = 1L
)

resv <- comorbidities(
  data      = df,
  id.vars   = "patid",
  icdv.var  = "icdv",
  dx.var    = "dx",
  icd.codes = "code",
  method    = "charlson_quan2011",
  poa       = 1L,
  primarydx.var = "primarydx"
)

stopifnot(identical(res0[["cmrb_flag"]], c(1L, 1L)))
stopifnot(identical(res1[["cmrb_flag"]], c(0L, 0L)))
stopifnot(identical(resv[["cmrb_flag"]], c(1L, 0L)))

################################################################################
#                                 End of File                                  #
################################################################################

#' Convert ICD Compact Codes to Full Codes
#'
#' Take an assumed ICD compact code string and convert to a full code based on
#' the ICD version (9 or 10) and type (diagnostic or procedure).  This method
#' only formats strings and does not validate the code(s).
#'
#' @param x Character vector
#' @template params-icd-icdv
#' @template params-icd-dx
#'
#' @seealso
#' * [`get_icd_codes()`] to retrieve the internal lookup table of ICD codes.
#' * [`lookup_icd_codes()`] for retrieving details on a specific set of ICD
#'   codes.
#' * [`is_icd()`] to test if a string is a known ICD code.
#'
#' @return A character vector the same length as `x`.
#'
#' @family ICD tools
#' @export
icd_compact_to_full <- function(x, icdv, dx) {
  # given a compact ICD code, a code with no decimal point, and knowing if the
  # code is ICD-9 or ICD-10, and diagnostic or procedure, return the full code
  # with the decimal point
  stopifnot(inherits(x, "character"))

  # standardize icdv/dx lengths
  # icdv and dx need to be length 1 or the same length as x
  stopifnot(length(icdv) == 1L || length(icdv) == length(x))
  stopifnot(length(dx)   == 1L || length(dx)   == length(x))
  if (length(icdv) == 1L) icdv <- rep_len(icdv, length(x))
  if (length(dx)   == 1L) dx   <- rep_len(dx,   length(x))

  # leave entries with an existing decimal untouched
  has_dot <- grepl("\\.", x, useBytes = TRUE)
  nc <- nchar(x, type = "bytes", allowNA = FALSE, keepNA = FALSE)

  # patterns apply only to compact forms (no dot)
  iscompact <- !has_dot & !is.na(x)

  # There are four sets of codes which could have a decimal
  #   * ICD-9-CM non-E codes
  #   * ICD-9-CM E codes
  #   * ICD-9 Procedure codes
  #   * ICD-10-CM
  #     - NOTE: \\D\\d{2} will not match all the codes.  Z3A.29 is a valid code
  #       for 29 weeks gestation of pregnancy. Just use nchar > 3
  #
  # ICD-10 Procedure codes do not have a decimal

  icd9cm_nonE <- which(iscompact & (icdv ==  9 & dx == 1) & grepl("^(\\d{4,5}|V\\d{3,4})$", x))
  icd9cm_E    <- which(iscompact & (icdv ==  9 & dx == 1) & grepl("^(E\\d{4})$", x))
  icd9pr      <- which(iscompact & (icdv ==  9 & dx == 0) & grepl("^\\d{3,4}$", x))
  icd10cm     <- which(iscompact & (icdv == 10 & dx == 1) & nc > 3L)

  if (length(icd9cm_nonE) > 0L) {
    x[icd9cm_nonE] <-
      paste0(substr(x[icd9cm_nonE], start = 1, stop = 3), ".",
             substr(x[icd9cm_nonE], start = 4, stop = nchar(x[icd9cm_nonE])))
  }

  if (length(icd9cm_E) > 0L) {
    x[icd9cm_E] <-
      paste0(substr(x[icd9cm_E], start = 1, stop = 4), ".",
             substr(x[icd9cm_E], start = 5, stop = nchar(x[icd9cm_E])))
  }

  if (length(icd9pr) > 0L) {
    x[icd9pr] <-
      paste0(substr(x[icd9pr], start = 1, stop = 2), ".",
             substr(x[icd9pr], start = 3, stop = 4))
  }

  if (length(icd10cm) > 0L) {
    x[icd10cm] <-
      paste0(substr(x[icd10cm], start = 1, stop = 3), ".",
             substr(x[icd10cm], start = 4, stop = nchar(x[icd10cm])))
  }

  # return the modified x vector as the full codes
  x
}

################################################################################
#                                 End of File                                  #
################################################################################

#' Tools for working with data.frames.
#'
#' An objective of the medicalcoder package is to use only base R methods.  That
#' said, when data.tables are passed into the methods within the package there
#' are operations where the data.table set functions should be used or the
#' subsetting needs to be specific to data.tables. These internal (non-exported)
#' functions provide the needed method by data.table or data.frame.
#'
#' `mdcr_select()` deep-copies data.table subsets (via data.table::copy()) to
#' avoid aliasing when downstream code mutates; this intentionally trades some
#' performance for isolation.
#'
#' @param x a data.frame or data.table
#' @param i Optional. Indicates the rows on which the values must be updated. If
#'   not `NULL`, implies all rows.
#' @param j Column name (character).  For `mdcr_set` this is the column assigned
#'   `value`, update values if it exits. If `x[[j]]` does note exist it will be
#'   created.
#' @param value replacement values
#'
#' @family data.frame tools
#' @noRd
#' @name mdcr_data_frame_tools
NULL

#' @rdname mdcr_data_frame_tools
#' @family data.frame tools
#' @noRd
#' @keywords internal
mdcr_set <- function(x, i = NULL, j, value) {
  stopifnot(is.data.frame(x))
  if (requireNamespace(package = "data.table", quietly = TRUE) && inherits(x, "data.table")) {
    # calling data.table::setDT to make sure that the object can be modified by
    # reference. Without data.table::setDT here we see an error if the table was
    # read from disk (e.g., readRDS()) or hand-constructed:
    #
    #   This data.table has either been loaded from disk (e.g. using
    #   readRDS()/load()) or constructed manually (e.g. using structure()).
    #   Please run setDT() or setalloccol() on it first (to pre-allocate space
    #   for new columns) before assigning by reference to it.
    #
    # data.tables read via readRDS()/load() or hand‑constructed can have an
    # invalid .internal.selfref, so the first by‑reference op (set()/:= ) errors.
    # data.table::setDT() reinitializes the selfref so data.table::set() can work
    # by reference. This wrapper keeps by-ref semantics consistent across backends.

    getExportedValue(name = "setDT", ns = "data.table")(x = x)
    getExportedValue(name = "set", ns = "data.table")(x = x, i = i, j = j, value = if (nrow(x) == 0L) value[0] else value)
  } else if (requireNamespace(package = "dplyr", quietly = TRUE) && inherits(x, "tbl_df")) {
    mutate <- getExportedValue(name = "mutate", ns = "dplyr")
    if (is.null(i)) {
      newcol <- if (nrow(x) == 0L) value[0] else value
    } else {
      newcol <- x[[j]]
      if (is.null(newcol)) {
        newcol <- rep(NA, length = nrow(x))
        storage.mode(newcol) <- typeof(value)
      }
      newcol[i] <- value
    }
    x <- do.call(mutate, c(list(.data = x), stats::setNames(list(newcol), j)))
  } else {
    if (is.null(i)) {
      x[[j]] <- value
    } else {
      if (is.null(x[[j]])) {
        x[[j]] <- rep(NA, nrow(x))
        storage.mode(x[[j]]) <- typeof(value)
      }
      x[[j]][i] <- value
    }
  }
  x
}

#'
#' @rdname mdcr_data_frame_tools
#' @family data.frame tools
#' @noRd
#' @keywords internal
mdcr_select <- function(x, cols) {
  stopifnot(is.data.frame(x))
  if (missing(cols)) {
    return(x)
  }

  #  # By making sure that cols is a character vector the `with = FALSE` is not
  #  # needed for data.tables which will allow for a simple call.  This is
  #  # important because `[.data.frame` will error if `with = FALSE` is passed.
  #  # `[.data.table` does not need `with = FALSE` if `j` is a character vector.
  #  stopifnot(inherits(cols, "character"))
  #  #x[, cols, drop = FALSE, with = FALSE]
  #  x[, cols, drop = FALSE]

  if (requireNamespace(package = "data.table", quietly = TRUE) && inherits(x, "data.table")) {
    # note: the data.table::copy() is needed here because x[, cols] returns a
    # shallow copy of the columns. The use of mdcr_select in the package
    # implicitly assumes deep copies. Downstream setorder()/setnames() mutate in
    # place, so copying here preserves the original. This pays a copy cost to
    # protect callers who expect an isolated subset.
    return(getExportedValue(name = "copy", ns = "data.table")(x[, cols, drop = FALSE, with = FALSE]))
  } else if (requireNamespace(package = "dplyr", quietly = TRUE) && inherits(x, "tbl_df")) {
    select <- getExportedValue(name = "select", ns = "dplyr")
    all_of <- getExportedValue(name = "all_of", ns = "dplyr")
    return(select(x, all_of(cols)))
  } else {
    return(x[, cols, drop = FALSE])
  }
}

#'
#' @rdname mdcr_data_frame_tools
#' @family data.frame tools
#' @noRd
#' @keywords internal
mdcr_subset <- function(x, i, cols) {
  stopifnot(is.data.frame(x))

  if (missing(i)) {
    if (missing(cols)) {
      return(x)
    } else {
      return(mdcr_select(x, cols = cols))
    }
  } else {
    # match base/data.table semantics: logical i is converted to positions
    rows <- if (is.logical(i)) which(i) else i

    if (missing(cols)) {
      if (requireNamespace(package = "data.table", quietly = TRUE) && inherits(x, "data.table")) {
        return(x[rows, , drop = FALSE, with = FALSE])
      } else if (requireNamespace(package = "dplyr", quietly = TRUE) && inherits(x, "tbl_df")) {
        slice <- getExportedValue(name = "slice", ns = "dplyr")
        return(slice(x, rows))
      } else {
        return(x[rows, , drop = FALSE])
      }
    } else {
      if (requireNamespace(package = "data.table", quietly = TRUE) && inherits(x, "data.table")) {
        return(x[rows, cols, drop = FALSE, with = FALSE])
      } else if (requireNamespace(package = "dplyr", quietly = TRUE) && inherits(x, "tbl_df")) {
        slice  <- getExportedValue(name = "slice",  ns = "dplyr")
        select <- getExportedValue(name = "select", ns = "dplyr")
        all_of <- getExportedValue(name = "all_of", ns = "dplyr")
        x <- slice(x, rows)
        return(select(x, all_of(cols)))
      } else {
        cols_idx <- match(cols, names(x))
        return(x[rows, cols_idx, drop = FALSE])
      }
    }
  }
}

#'
#' @rdname mdcr_data_frame_tools
#' @family data.frame tools
#' @noRd
#' @keywords internal
mdcr_setorder <- function(x, by) {
  stopifnot(is.data.frame(x))
  if (requireNamespace(package = "data.table", quietly = TRUE) && inherits(x, "data.table")) {
    getExportedValue(name = "setorderv", ns = "data.table")(x, by)
  } else if (requireNamespace(package = "dplyr", quietly = TRUE) && inherits(x, "tbl_df")) {
    arrange <- getExportedValue(name = "arrange", ns = "dplyr")
    x <- do.call(arrange, c(list(.data = x), lapply(by, as.name)))
  } else {
    x <- x[do.call(order, x[by]), , drop = FALSE]
  }
  x
}

#'
#' @rdname mdcr_data_frame_tools
#' @family data.frame tools
#' @noRd
#' @keywords internal
mdcr_setnames <- function(x, old, new, ...) {
  stopifnot(is.data.frame(x))
  stopifnot(is.character(old), is.character(new))
  stopifnot(length(old) == length(new))
  if (requireNamespace(package = "data.table", quietly = TRUE) && inherits(x, "data.table")) {
    getExportedValue(name = "setnames", ns = "data.table")(x, old, new, ...)
  } else if (requireNamespace(package = "dplyr", quietly = TRUE) && inherits(x, "tbl_df")) {
    rename <- getExportedValue(name = "rename", ns = "dplyr")
    args <- c(list(.data = x), stats::setNames(lapply(old, as.name), new))
    x <- do.call(rename, args)
  } else {
    for (i in seq_len(length(old))) {
      names(x)[names(x) == old[i]] <- new[i]
    }
  }
  x
}

#'
#' @rdname mdcr_data_frame_tools
#' @family data.frame tools
#' @noRd
#' @keywords internal
mdcr_duplicated <- function(x, by = seq_along(x), ...) {
  stopifnot(is.data.frame(x))
  if (requireNamespace(package = "data.table", quietly = TRUE) && inherits(x, "data.table")) {
    # Flag this frame as data.table-aware so duplicated.data.table uses its
    # optimized path instead of falling back to duplicated.data.frame.
    .datatable.aware <- TRUE
    rtn <- utils::getFromNamespace(x = 'duplicated.data.table', ns = "data.table")(x, by = by, ...)
  } else {  # this is for base R data.frames and tidyverse tbl_df
    rtn <- duplicated(x[, by, drop = FALSE], ...)
  }
  rtn
}

#'
#' @rdname mdcr_data_frame_tools
#' @family data.frame tools
#' @noRd
#' @keywords internal
mdcr_unique <- function(x, ...) {
  stopifnot(is.data.frame(x))
  if (requireNamespace(package = "data.table", quietly = TRUE) && inherits(x, "data.table")) {
    # Flag this frame as data.table-aware so unique.data.table uses its
    # optimized path instead of falling back to unique.data.frame.
    .datatable.aware <- TRUE
    rtn <- utils::getFromNamespace(x = 'unique.data.table', ns = "data.table")(x, ...)
  } else if (requireNamespace(package = "dplyr", quietly = TRUE) && inherits(x, "tbl_df")) {
    rtn <- getExportedValue(name = "distinct", ns = "dplyr")(.data = x, ...)
  } else {
    rtn <- unique(x, ...)
  }
  rtn
}

#'
#' @rdname mdcr_data_frame_tools
#' @family data.frame tools
#' @noRd
#' @keywords internal
mdcr_left_join <- function(x, y, ...) {
  stopifnot(is.data.frame(x), is.data.frame(y))

  if (requireNamespace(package = "dplyr", quietly = TRUE) && inherits(x, "tbl_df")) {
    lj <- getExportedValue(name = "left_join", ns = "dplyr")
    dots <- list(...)
    if (!is.null(dots$by.x) & !is.null(dots$by.y)) {
      by <- stats::setNames(dots$by.y, dots$by.x)
      dots$by.x <- NULL
      dots$by.y <- NULL
    } else if (!is.null(dots$by)) {
      by <- dots$by
      dots$by   <- NULL
    } else {
      by <- NULL
    }
    if (!is.null(dots$suffixes)) {
      suffix <- dots$suffixes
      dots$suffixes <- NULL
    } else {
      suffix <- c(".x", ".y")
    }
    # normalize to dplyr's by/suffix arguments to mirror base/data.table defaults
    rtn <- do.call(what = lj, args = c(list(x = x, y = y, by = by, suffix = suffix), dots))
  } else {
    # if x is a data.table and the data.table namespace is available then the
    # data.table:::merge.data.table method will be called and a specific block
    # for data.table is not needed here
    rtn <- merge(x = x, y = y, all.x = TRUE, all.y = FALSE, sort = FALSE, allow.cartesian = TRUE, ...)
  }
  rtn
}

#'
#' @rdname mdcr_data_frame_tools
#' @family data.frame tools
#' @noRd
#' @keywords internal
mdcr_full_outer_join <- function(x, y, ...) {
  stopifnot(is.data.frame(x), is.data.frame(y))

  if (requireNamespace(package = "dplyr", quietly = TRUE) && inherits(x, "tbl_df")) {
    fj <- getExportedValue(name = "full_join", ns = "dplyr")
    dots <- list(...)
    if (!is.null(dots$by.x) & !is.null(dots$by.y)) {
      by <- stats::setNames(dots$by.y, dots$by.x)
      dots$by.x <- NULL
      dots$by.y <- NULL
    } else if (!is.null(dots$by)) {
      by <- dots$by
      dots$by   <- NULL
    } else {
      by <- NULL
    }
    if (!is.null(dots$suffixes)) {
      suffix <- dots$suffixes
      dots$suffixes <- NULL
    } else {
      suffix <- c(".x", ".y")
    }
    # normalize to dplyr's by/suffix arguments to mirror base/data.table defaults
    rtn <- do.call(what = fj, args = c(list(x = x, y = y, by = by, suffix = suffix), dots))
  } else {
    # if x is a data.table and the data.table namespace is available then the
    # data.table:::merge.data.table method will be called and a specific block
    # for data.table is not needed here
    rtn <- merge(x = x, y = y, all.x = TRUE, all.y = TRUE, sort = FALSE, allow.cartesian = TRUE, ...)
  }
  rtn
}

#'
#' @rdname mdcr_data_frame_tools
#' @family data.frame tools
#' @noRd
#' @keywords internal
mdcr_inner_join <- function(x, y, ...) {
  stopifnot(is.data.frame(x), is.data.frame(y))

  if (requireNamespace(package = "dplyr", quietly = TRUE) && inherits(x, "tbl_df")) {
    ij <- getExportedValue(name = "inner_join", ns = "dplyr")
    dots <- list(...)
    if (!is.null(dots$by.x) & !is.null(dots$by.y)) {
      by <- stats::setNames(dots$by.y, dots$by.x)
      dots$by.x <- NULL
      dots$by.y <- NULL
    } else if (!is.null(dots$by)) {
      by <- dots$by
      dots$by   <- NULL
    } else {
      by <- NULL
    }
    if (!is.null(dots$suffixes)) {
      suffix <- dots$suffixes
      dots$suffixes <- NULL
    } else {
      suffix <- c(".x", ".y")
    }
    # normalize to dplyr's by/suffix arguments to mirror base/data.table defaults
    rtn <- do.call(what = ij, args = c(list(x = x, y = y, by = by, suffix = suffix), dots))
  } else {
    # if x is a data.table and the data.table namespace is available then the
    # data.table:::merge.data.table method will be called and a specific block
    # for data.table is not needed here
    rtn <- merge(x = x, y = y, all.x = FALSE, all.y = FALSE, sort = FALSE, ...)
  }
  rtn
}

#'
#' @rdname mdcr_data_frame_tools
#' @family data.frame tools
#' @noRd
#' @keywords internal
mdcr_cbind <- function(x, ...) {
  stopifnot(is.data.frame(x))
  if (requireNamespace(package = "dplyr", quietly = TRUE) && inherits(x, "tbl_df")) {
    cb <- getExportedValue(name = "bind_cols", ns = "dplyr")
    rtn <- cb(x, ...)
  } else {
    # if x is a data.table and the data.table namespace is available then the
    # data.table:::cbind.data.table method will be called and a specific block
    # for data.table is not needed here
    rtn <- cbind(x, ...)
  }
  rtn
}


################################################################################

#' Assert
#'
#' A set of functions used to check function arguments.
#'
#' @details
#'
#' ## assert_scalar_logical
#' tests if the input is a length-1, non-missing logical.
#'
#' **Developer example (not executed):**
#' ```
#' medicalcoder:::assert_scalar_logical(TRUE)          # pass, return TRUE
#' medicalcoder:::assert_scalar_logical(FALSE)         # pass, return TRUE
#' medicalcoder:::assert_scalar_logical(NA)            # fail, throw error
#' medicalcoder:::assert_scalar_logical(1L)            # fail, throw error
#' medicalcoder:::assert_scalar_logical(c(TRUE, TRUE)) # fail, throw error
#' ```
#'
#' @param x a R object
#'
#' @return `TRUE`, invisibly if the assertion is TRUE, otherwise an error is
#' thrown.
#'
#' @noRd
#' @keywords internal
assert_scalar_logical <- function(x) {
  z <- is.logical(x) && length(x) == 1L && !is.na(x)
  if (!z) {
    stop(sprintf("The value passed to '%s' is expected to be a length-1 non-missing logical.", deparse(substitute(x))), call. = FALSE)
  }
  invisible(z)
}

################################################################################
#                                 End of File                                  #
################################################################################

source('utilities.R')
################################################################################
# Tests for summarizing PCCC comorbidities with subconditions
library(medicalcoder)

################################################################################
# Build PCCC with subconditions and compute the summary
pccc_sub <- comorbidities(
  data        = mdcr,
  id.vars     = "patid",
  icd.codes   = "code",
  icdv.var    = "icdv",
  dx.var      = "dx",
  method      = "pccc_v3.1",
  flag.method = "current",
  poa         = 1,
  subconditions = TRUE
)

summary_current <- summary(pccc_sub)

stopifnot(
  inherits(summary_current, "data.frame"),
  identical(
    names(summary_current),
    c("condition",
      "subcondition",
      "count",
      "percent_of_cohort",
      "percent_of_those_with_condition")
  ),
  is.character(summary_current[["condition"]]),
  is.character(summary_current[["subcondition"]]),
  is.numeric(summary_current[["count"]]),
  is.numeric(summary_current[["percent_of_cohort"]]),
  is.numeric(summary_current[["percent_of_those_with_condition"]]),
  all(summary_current[["count"]] >= 0),
  all(summary_current[["percent_of_cohort"]] >= 0),
  all(summary_current[["percent_of_cohort"]] <= 100),
  all(is.na(summary_current[["percent_of_those_with_condition"]][is.na(summary_current[["subcondition"]])])),
  all(summary_current[["percent_of_those_with_condition"]][!is.na(summary_current[["subcondition"]]) & summary_current[["count"]] > 0] >= 0),
  all(summary_current[["percent_of_those_with_condition"]][!is.na(summary_current[["subcondition"]]) & summary_current[["count"]] > 0] <= 100)
)

################################################################################
# Condition-level counts align with the underlying comorbidity object
N <- nrow(pccc_sub[["conditions"]])

cvd_total <- sum(pccc_sub[["conditions"]][["cvd_dxpr_or_tech"]])
cvd_row <- summary_current[
  summary_current[["condition"]] == "cvd" &
  is.na(summary_current[["subcondition"]]),
]

stopifnot(
  nrow(cvd_row) == 1L,
  cvd_row[["count"]] == cvd_total,
  isTRUE(all.equal(cvd_row[["percent_of_cohort"]], 100 * cvd_total / N)),
  is.na(cvd_row[["percent_of_those_with_condition"]])
)

resp_total <- sum(pccc_sub[["conditions"]][["respiratory_dxpr_or_tech"]])
resp_row <- summary_current[
  summary_current[["condition"]] == "respiratory" &
  is.na(summary_current[["subcondition"]]),
]

stopifnot(
  nrow(resp_row) == 1L,
  resp_row[["count"]] == resp_total,
  isTRUE(all.equal(resp_row[["percent_of_cohort"]], 100 * resp_total / N)),
  is.na(resp_row[["percent_of_those_with_condition"]])
)

################################################################################
# Subcondition counts and percentages match direct aggregations
cvd_sub <- pccc_sub[["subconditions"]][["cvd"]]
cvd_hgvm <- sum(cvd_sub[["heart_and_great_vessel_malformations"]])

cvd_hgvm_idx <- which(
  summary_current[["condition"]] == "cvd" &
  summary_current[["subcondition"]] == "heart_and_great_vessel_malformations"
)
cvd_hgvm_row <- summary_current[cvd_hgvm_idx, , drop = FALSE]

stopifnot(
  nrow(cvd_hgvm_row) == 1L,
  cvd_hgvm_row[["count"]] == cvd_hgvm,
  isTRUE(all.equal(cvd_hgvm_row[["percent_of_cohort"]], 100 * cvd_hgvm / N)),
  isTRUE(all.equal(cvd_hgvm_row[["percent_of_those_with_condition"]], 100 * cvd_hgvm / cvd_total))
)

resp_sub <- pccc_sub[["subconditions"]][["respiratory"]]
resp_cystic_fibrosis <- sum(resp_sub[["cystic_fibrosis"]])

resp_cf_idx <- which(
  summary_current[["condition"]] == "respiratory" &
  summary_current[["subcondition"]] == "cystic_fibrosis"
)
resp_cf_row <- summary_current[resp_cf_idx, , drop = FALSE]

stopifnot(
  nrow(resp_cf_row) == 1L,
  resp_cf_row[["count"]] == resp_cystic_fibrosis,
  isTRUE(all.equal(resp_cf_row[["percent_of_cohort"]], 100 * resp_cystic_fibrosis / N)),
  isTRUE(all.equal(resp_cf_row[["percent_of_those_with_condition"]], 100 * resp_cystic_fibrosis / resp_total))
)

################################################################################
# A non-current flag.method emits a warning but returns the same summary
pccc_sub_cumulative <- pccc_sub
attr(pccc_sub_cumulative, "flag.method") <- "cumulative"

warn_obj <- tryCatchWarning(summary(pccc_sub_cumulative))

stopifnot(
  inherits(warn_obj, "warning"),
  identical(
    conditionMessage(warn_obj),
    "Logic for pccc_summary_table has been implemented for flag.method = 'current'.  Using this function for flag.method = 'cumulative' may not provide a meaningful summary."
  )
)

summary_cumulative <- suppressWarnings(summary(pccc_sub_cumulative))

stopifnot(identical(summary_cumulative, summary_current))

################################################################################
# No conditions flagged -> percentages should be 0/NA, not NaN/Inf
df_none <- data.frame(
  patid = 1:3,
  icdv  = 10L,
  dx    = 1L,
  code  = c("XXX1", "XXX2", "XXX3"),
  stringsAsFactors = FALSE
)

pccc_empty <- comorbidities(
  data          = df_none,
  id.vars       = "patid",
  icd.codes     = "code",
  icdv.var      = "icdv",
  dx.var        = "dx",
  method        = "pccc_v3.1",
  flag.method   = "current",
  poa           = 1,
  subconditions = TRUE
)

summary_empty <- summary(pccc_empty)

stopifnot(
  inherits(summary_empty, "data.frame"),
  all(summary_empty$count == 0),
  all(summary_empty$percent_of_cohort == 0),
  !any(is.nan(summary_empty$percent_of_cohort)),
  all(is.na(summary_empty$percent_of_those_with_condition) | summary_empty$percent_of_those_with_condition == 0),
  !any(is.nan(summary_empty$percent_of_those_with_condition))
)

################################################################################
#                                 End of File                                  #
################################################################################

## ----label = "setup", include = FALSE-----------------------------------------
# IMPORTANT SYNTAX NOTE:
#
# DO NOT USE the pipeOp `|>`
#
# While convenient, that is a R 4.1.0 feature at a minimum. Notable improvements
# to the pipeOp come in 4.2.0 and 4.2.1.  To keep this package dependent on R >=
# 3.5.0 do not use the pipeOp.

library(kableExtra)
options(qwraps2_markup = "markdown")
options(knitr.kable.NA = '')
knitr::opts_chunk$set(collapse = TRUE, fig.align = "center")

## ----label = 'medicalcoder-namespace'-----------------------------------------
library(medicalcoder)
packageVersion("medicalcoder")

## -----------------------------------------------------------------------------
str(get_elixhauser_codes())
str(get_elixhauser_index_scores())
str(get_elixhauser_poa())

## -----------------------------------------------------------------------------
# will warn because primarydx and primarydx.var are both NULL
mdcr_results0 <-
  comorbidities(
    data = mdcr,
    id.vars = "patid",
    icdv.var = "icdv",
    icd.codes = "code",
    dx.var = "dx",
    flag.method = "current",
    poa = 1,
    method = "elixhauser_ahrq2025"
  )

# no warning
mdcr_results <-
  comorbidities(
    data = mdcr,
    id.vars = "patid",
    icdv.var = "icdv",
    icd.codes = "code",
    dx.var = "dx",
    flag.method = "current",
    poa = 1,
    method = "elixhauser_ahrq2025",
    primarydx = 0
  )

identical(mdcr_results, mdcr_results0)

## -----------------------------------------------------------------------------
str(mdcr_results)

## -----------------------------------------------------------------------------
summary(mdcr_results)

## ----echo = FALSE, results = "asis"-------------------------------------------
x <- summary(mdcr_results)$conditions
tab <-
  kableExtra::kbl(
    x = x,
    format = "html",
    caption = "Counts and percentages of patients in the mdcr example data sets with the Elixhauser @quan2005 comorbidities.",
    col.names = c("", "Count", "Percentage"),
    digits = 3
  )
tab <- kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab <- kableExtra::pack_rows(tab, group_label = "Comorbidity", start_row = 1, end_row = 38)
tab <- kableExtra::pack_rows(tab, group_label = "Total Comorbidities", start_row = 39, end_row = nrow(x))
tab

source('utilities.R')
library(medicalcoder)

################################################################################
# Verify the summary table output for pccc_v3.0 with flag.method = current is as expected
rtn <- comorbidities(data = mdcr, id.vars = "patid", icd.codes = "code", poa = 1, flag.method = 'current', method = "pccc_v3.0")
rtn <- summary(rtn)

stopifnot(
  inherits(rtn, "data.frame"),
  identical(
    names(rtn),
    c("condition", "label",
      "dxpr_or_tech_count", "dxpr_or_tech_percent",
      "dxpr_only_count", "dxpr_only_percent",
      "tech_only_count", "tech_only_percent",
      "dxpr_and_tech_count", "dxpr_and_tech_percent")
  ),
  is.character(rtn[["condition"]]),
  is.character(rtn[["label"]]),
  is.integer(rtn[["dxpr_or_tech_count"]]),
  is.numeric(rtn[["dxpr_or_tech_percent"]]),
  is.integer(rtn[["dxpr_only_count"]]),
  is.numeric(rtn[["dxpr_only_percent"]]),
  is.integer(rtn[["tech_only_count"]]),
  is.numeric(rtn[["tech_only_percent"]]),
  is.integer(rtn[["dxpr_and_tech_count"]]),
  is.numeric(rtn[["dxpr_and_tech_percent"]])
)

################################################################################
# Verify the summary table output for pccc_v3.0 with flag.method = cumulative is as expected
cmb <- comorbidities(data = mdcr_longitudinal, id.vars = c("patid", "date"), icd.codes = "code", poa = 1, flag.method = 'cumulative', method = "pccc_v3.0")
rtn <- tryCatchWarning(summary(cmb))
stopifnot(inherits(rtn, "warning"))

rtn <- suppressWarnings(summary(cmb))
stopifnot(
  inherits(rtn, "data.frame"),
  identical(
    names(rtn),
    c("condition", "label",
      "dxpr_or_tech_count", "dxpr_or_tech_percent",
      "dxpr_only_count", "dxpr_only_percent",
      "tech_only_count", "tech_only_percent",
      "dxpr_and_tech_count", "dxpr_and_tech_percent")
  ),
  is.character(rtn[["condition"]]),
  is.character(rtn[["label"]]),
  is.integer(rtn[["dxpr_or_tech_count"]]),
  is.numeric(rtn[["dxpr_or_tech_percent"]]),
  is.integer(rtn[["dxpr_only_count"]]),
  is.numeric(rtn[["dxpr_only_percent"]]),
  is.integer(rtn[["tech_only_count"]]),
  is.numeric(rtn[["tech_only_percent"]]),
  is.integer(rtn[["dxpr_and_tech_count"]]),
  is.numeric(rtn[["dxpr_and_tech_percent"]])
)

################################################################################
# Verify the summary table output for pccc_v3.1 is as expected
rtn <- comorbidities(data = mdcr, id.vars = "patid", icd.codes = "code", poa = 1, flag.method = 'current', method = "pccc_v3.1")
rtn <- summary(rtn)

stopifnot(
  inherits(rtn, "data.frame"),
  identical(
    names(rtn),
    c("condition", "label",
      "dxpr_or_tech_count", "dxpr_or_tech_percent",
      "dxpr_only_count", "dxpr_only_percent",
      "tech_only_count", "tech_only_percent",
      "dxpr_and_tech_count", "dxpr_and_tech_percent")
  ),
  is.character(rtn[["condition"]]),
  is.character(rtn[["label"]]),
  is.integer(rtn[["dxpr_or_tech_count"]]),
  is.numeric(rtn[["dxpr_or_tech_percent"]]),
  is.integer(rtn[["dxpr_only_count"]]),
  is.numeric(rtn[["dxpr_only_percent"]]),
  is.integer(rtn[["tech_only_count"]]),
  is.numeric(rtn[["tech_only_percent"]]),
  is.integer(rtn[["dxpr_and_tech_count"]]),
  is.numeric(rtn[["dxpr_and_tech_percent"]])
)

################################################################################
# PCCC Version 2.0

rtn <- comorbidities(data = mdcr, id.vars = "patid", icd.codes = "code", poa = 1, flag.method = 'current', method = "pccc_v2.0")
rtn <- summary(rtn)

stopifnot(
  inherits(rtn, "data.frame"),
  identical(
    names(rtn),
    c("condition", "label", "count", "percent")
  ),
  is.character(rtn[["condition"]]),
  is.character(rtn[["label"]]),
  is.integer(rtn[["count"]]),
  is.numeric(rtn[["percent"]])
)


################################################################################
# PCCC Version 2.1

rtn <- comorbidities(data = mdcr, id.vars = "patid", icd.codes = "code", poa = 1, flag.method = 'current', method = "pccc_v2.1")
rtn <- summary(rtn)

stopifnot(
  inherits(rtn, "data.frame"),
  identical(
    names(rtn),
    c("condition", "label", "count", "percent")
  ),
  is.character(rtn[["condition"]]),
  is.character(rtn[["label"]]),
  is.integer(rtn[["count"]]),
  is.numeric(rtn[["percent"]])
)

################################################################################
# PCCC Version 2.1 with flag.method = "cumulative"

cmb <- comorbidities(data = mdcr_longitudinal, id.vars = c("patid", "date"), icd.codes = "code", poa = 1, flag.method = 'cumulative', method = "pccc_v2.1")
rtn <- tryCatchWarning(summary(cmb))
stopifnot(inherits(rtn, "warning"))
rtn <- suppressWarnings(summary(cmb))

stopifnot(
  inherits(rtn, "data.frame"),
  identical(
    names(rtn),
    c("condition", "label", "count", "percent")
  ),
  is.character(rtn[["condition"]]),
  is.character(rtn[["label"]]),
  is.integer(rtn[["count"]]),
  is.numeric(rtn[["percent"]])
)

################################################################################
# Zero-row input should summarize without NaN/Inf (v2.1)

pccc_zero <- comorbidities(
  data        = mdcr[0, ],
  id.vars     = "patid",
  icd.codes   = "code",
  poa         = 1,
  flag.method = "current",
  method      = "pccc_v2.1"
)

summary_zero <- summary(pccc_zero)

stopifnot(
  inherits(summary_zero, "data.frame"),
  all(identical(summary_zero$count, c(rep(0L, 24)))),
  !any(is.nan(summary_zero$percent)),
  all(is.na(summary_zero$percent))
)

################################################################################
# Zero-row input should summarize without NaN/Inf (v3.1)

pccc_zero <- comorbidities(
  data        = mdcr[0, ],
  id.vars     = "patid",
  icd.codes   = "code",
  poa         = 1,
  flag.method = "current",
  method      = "pccc_v3.1"
)

summary_zero <- summary(pccc_zero)

stopifnot(
  inherits(summary_zero, "data.frame"),
  all(identical(summary_zero$dxpr_or_tech_count, c(rep(0L, 24)))),
  all(identical(summary_zero$dxpr_only_count, c(rep(0L, 11), rep(NA_integer_, 13)))),
  all(identical(summary_zero$tech_only_count, c(rep(0L, 11), rep(NA_integer_, 13)))),
  all(identical(summary_zero$dxpr_and_tech_count, c(rep(0L, 11), rep(NA_integer_, 13)))),
  !any(is.nan(summary_zero$dxpr_or_tech_percent)),
  !any(is.nan(summary_zero$dxpr_only_percent)),
  !any(is.nan(summary_zero$tech_only_percent)),
  !any(is.nan(summary_zero$dxpr_and_tech_percent))
)

################################################################################
#                                 End of File                                  #
################################################################################

library(medicalcoder)
source("utilities.R")
################################################################################
# testing the internal data sets - both the internal sets and the user visible
# versions.

# The internal sets can to look for: this cannot be done dynamically
# e <- new.env()
# load("../R/sysdata.rda", envir = e)
# dput(sort(ls(all.names = TRUE, envir = e)))

internal_data_sets <-
  c(
    "..mdcr_internal_charlson_codes..",
    "..mdcr_internal_charlson_index_scores..",
    "..mdcr_internal_desc_start_stop..",
    "..mdcr_internal_elixhauser_codes..",
    "..mdcr_internal_elixhauser_index_scores..",
    "..mdcr_internal_elixhauser_poa..",
    "..mdcr_internal_icd_chapters..",
    "..mdcr_internal_icd_codes..",
    "..mdcr_internal_icd_descs..",
    "..mdcr_internal_icd_subchapters..",
    "..mdcr_internal_known_and_assignable_start_stop..",
    "..mdcr_internal_pccc_codes..",
    "..mdcr_internal_pccc_conditions.."
  )

mdcr <- getNamespace("medicalcoder")

# are all the expected sets in the medicalcoder namespace?
stopifnot(all(internal_data_sets %in% names(mdcr)))

# check that there are not unaccounted for data sets.  the ..mdcr_internal_
# prefix and .. suffix is expected.  noted in the data-raw/build_sysdata.R
stopifnot(
  all(
    grep("\\.\\.mdcr_internal_.+\\.\\.$", names(mdcr), value = TRUE) %in% internal_data_sets
  )
)

################################################################################
# verify all the sets are data.frames and only data.frames
t01 <-  all(sapply(sapply(internal_data_sets, get, envir = mdcr), inherits, "data.frame"))
t02 <- !any(sapply(sapply(internal_data_sets, get, envir = mdcr), inherits, "data.table"))
t03 <- !any(sapply(sapply(internal_data_sets, get, envir = mdcr), inherits, "tbl_df"))

stopifnot(t01, t02, t03)

################################################################################
# Verify the names and classes of the internal data sets
expected_internal_names_and_classes <-
  list(
    "..mdcr_internal_charlson_codes.." = c(code_id = "integer", condition = "character", charlson_cdmf2019 = "integer", charlson_deyo1992 = "integer", charlson_quan2005 = "integer", charlson_quan2011 = "integer"),
    "..mdcr_internal_charlson_index_scores.." = c(condition_description = "character", condition = "character", charlson_cdmf2019 = "integer", charlson_quan2011 = "integer", charlson_quan2005 = "integer", charlson_deyo1992 = "integer"),
    "..mdcr_internal_desc_start_stop.." = c(code_id = "integer", desc_id = "integer", src = "factor", desc_start = "integer", desc_end = "integer"),
    "..mdcr_internal_elixhauser_codes.." = c(code_id = "integer", poaexempt = "integer", condition = "character", elixhauser_ahrq_web = "integer", elixhauser_elixhauser1988 = "integer", elixhauser_quan2005 = "integer", elixhauser_ahrq2022 = "integer", elixhauser_ahrq2023 = "integer", elixhauser_ahrq2024 = "integer", elixhauser_ahrq2025 = "integer", elixhauser_ahrq2026 = "integer", elixhauser_ahrq_icd10 = "integer"),
    "..mdcr_internal_elixhauser_index_scores.." = c(condition = "character", index = "character", elixhauser_ahrq_web = "integer", elixhauser_elixhauser1988 = "integer", elixhauser_quan2005 = "integer", elixhauser_ahrq2022 = "integer", elixhauser_ahrq2023 = "integer", elixhauser_ahrq2024 = "integer", elixhauser_ahrq2025 = "integer", elixhauser_ahrq2026 = "integer", elixhauser_ahrq_icd10 = "integer"),
    "..mdcr_internal_elixhauser_poa.." = c(condition = "character", poa_required = "integer", elixhauser_ahrq2022 = "integer", elixhauser_ahrq2023 = "integer", elixhauser_ahrq2024 = "integer", elixhauser_ahrq2025 = "integer", elixhauser_ahrq2026 = "integer", elixhauser_ahrq_icd10 = "integer"),
    "..mdcr_internal_icd_chapters.." = c(chapter = "character", chap_id = "integer"),
    "..mdcr_internal_icd_codes.." = c(icdv = "integer", dx = "integer", full_code = "character", code = "character", code_id = "integer", chap_id = "integer", subchap_id = "integer"),
    "..mdcr_internal_icd_descs.." = c(desc = "character", desc_id = "integer"),
    "..mdcr_internal_icd_subchapters.." = c(subchapter = "character", subchap_id = "integer"),
    "..mdcr_internal_known_and_assignable_start_stop.." = c(code_id = "integer", src = "factor", known_start = "integer", known_end = "integer", assignable_start = "integer", assignable_end = "integer"),
    "..mdcr_internal_pccc_codes.." = c(code_id = "integer", condition = "character", subcondition = "character", transplant_flag = "integer", tech_dep_flag = "integer", pccc_v3.1 = "integer", pccc_v3.0 = "integer", pccc_v2.1 = "integer", pccc_v2.0 = "integer"),
    "..mdcr_internal_pccc_conditions.." = c(condition = "character", subcondition = "character", condition_label = "character", subcondition_label = "character")
  )

current_names_and_classes <- sapply(sapply(internal_data_sets, get, envir = mdcr), sapply, class)

stopifnot(identical(length(current_names_and_classes), length(expected_internal_names_and_classes)))

for(n in names(current_names_and_classes)) {
  z <- identical(current_names_and_classes[[n]], expected_internal_names_and_classes[[n]])
  if (!z) {
    stop(sprintf("user visible %s does not have the expected structure", n))
  }
}

################################################################################
# get the data sets via get_ methods

# expected methods
user_visible_get_methods <-
  c(
    "get_charlson_codes",
    "get_charlson_index_scores",
    "get_elixhauser_codes",
    "get_elixhauser_index_scores",
    "get_elixhauser_poa",
    "get_icd_codes",
    "get_pccc_codes",
    "get_pccc_conditions"
  )

# check that the expected methods are accounted for
stopifnot(all(user_visible_get_methods %in% names(mdcr)))
stopifnot(all(grep("get_.+", names(mdcr), value = TRUE) %in% user_visible_get_methods))

# all the get_* methods have zero arguments except for get_icd_codes
for (m in user_visible_get_methods) {
  f <- getExportedValue(name = m, ns = "medicalcoder")
  x <- formals(f)
  if (m != "get_icd_codes") {
    if (!is.null(x)) {
      stop(sprintf("Unexpected function argument in %s()", m))
    }
  } else {
    if (!identical(x, pairlist(with.descriptions = FALSE, with.hierarchy = FALSE))) {
      stop(sprintf("Unexpected function argument in %s()", m))
    }
  }
}

# get the datasets
user_visible <-
  list(
    charlson_codes = get_charlson_codes(),
    charlson_index_scores = get_charlson_index_scores(),
    elixhauser_codes = get_elixhauser_codes(),
    elixhauser_index_scores = get_elixhauser_index_scores(),
    elixhauser_poa = get_elixhauser_poa(),
    icd_codes = get_icd_codes(),
    icd_desc = get_icd_codes(with.descriptions = TRUE),
    icd_hier = get_icd_codes(with.hierarchy = TRUE),
    icd_dh   = get_icd_codes(with.descriptions = TRUE, with.hierarchy = TRUE),
    pccc_codes = get_pccc_codes(),
    pccc_conditions = get_pccc_conditions()
  )

# verify all the sets are data.frames and only data.frames
t04 <-  all(sapply(user_visible, inherits, "data.frame"))
t05 <- !any(sapply(user_visible, inherits, "data.table"))
t06 <- !any(sapply(user_visible, inherits, "tbl_df"))

stopifnot(t04, t05, t06)

# define the expected structure
user_visible_expected_structures <-
  list(
    charlson_codes = c(icdv = "integer", dx = "integer", full_code = "character", code = "character", condition = "character", charlson_cdmf2019 = "integer", charlson_deyo1992 = "integer", charlson_quan2005 = "integer", charlson_quan2011 = "integer"),
    charlson_index_scores = expected_internal_names_and_classes$..mdcr_internal_charlson_index_scores..,
    elixhauser_codes = c(icdv = "integer", dx = "integer", full_code = "character", code = "character", poaexempt = "integer", condition = "character", elixhauser_ahrq_web = "integer", elixhauser_elixhauser1988 = "integer", elixhauser_quan2005 = "integer", elixhauser_ahrq2022 = "integer", elixhauser_ahrq2023 = "integer", elixhauser_ahrq2024 = "integer", elixhauser_ahrq2025 = "integer", elixhauser_ahrq2026 = "integer", elixhauser_ahrq_icd10 = "integer"),
    elixhauser_index_scores = expected_internal_names_and_classes$..mdcr_internal_elixhauser_index_scores..,
    elixhauser_poa = expected_internal_names_and_classes$..mdcr_internal_elixhauser_poa..,
    icd_codes = c(icdv = "integer", dx = "integer", full_code = "character", code = "character", src = "character", known_start = "integer", known_end = "integer", assignable_start = "integer", assignable_end = "integer"),
    icd_desc = c(icdv = "integer", dx = "integer", full_code = "character", code = "character", src = "character", known_start = "integer", known_end = "integer", assignable_start = "integer", assignable_end = "integer", desc = "character", desc_start = "integer", desc_end = "integer"),
    icd_hier = c(icdv = "integer", dx = "integer", full_code = "character", code = "character", src = "character", known_start = "integer", known_end = "integer", assignable_start = "integer", assignable_end = "integer", chapter = "character", subchapter = "character", category = "character", subcategory = "character", subclassification = "character", subsubclassification = "character", extension = "character"),
    icd_dh = c(icdv = "integer", dx = "integer", full_code = "character", code = "character", src = "character", known_start = "integer", known_end = "integer", assignable_start = "integer", assignable_end = "integer", desc = "character", desc_start = "integer", desc_end = "integer", chapter = "character", subchapter = "character", category = "character", subcategory = "character", subclassification = "character", subsubclassification = "character", extension = "character"),
    pccc_codes = c(icdv = "integer", dx = "integer", full_code = "character", code = "character", condition = "character", subcondition = "character", transplant_flag = "integer", tech_dep_flag = "integer", pccc_v3.1 = "integer", pccc_v3.0 = "integer", pccc_v2.1 = "integer", pccc_v2.0 = "integer"),
    pccc_conditions = expected_internal_names_and_classes$..mdcr_internal_pccc_conditions..
  )

user_visible_current_structures <- lapply(user_visible, sapply, class)

for(n in names(user_visible_current_structures)) {
  z <- identical(user_visible_current_structures[[n]], user_visible_expected_structures[[n]])
  if (!z) {
    stop(sprintf("user visible %s does not have the expected structure", n))
  }
}

# check that the row names are just sequential integers
for (n in names(user_visible)) {
  expected <- as.character(seq_len(nrow(user_visible[[n]])))
  current  <- rownames(user_visible[[n]])
  if (!identical(expected, current)) {
    stop(sprintf("rownames(user_visible[[%s]]) are not sequential integers", n))
  }
}

################################################################################
# verify *_codes are all valid codes
for (n in grep("_codes$", names(user_visible), value = TRUE)) {
  z <- is_icd(x = user_visible[[n]][["code"]],
    icdv = user_visible[[n]][["icdv"]],
    dx   = user_visible[[n]][["dx"]],
    headerok = TRUE,
    ever.assignable = TRUE,
    warn.ambiguous = FALSE
  )
  if (!all(z)) {
    stop(sprintf("not all user_visible[['%s']][['code']] are valid ever.assignable ICD codes", n))
  }
  z <- is_icd(x = user_visible[[n]][["full_code"]],
    icdv = user_visible[[n]][["icdv"]],
    dx   = user_visible[[n]][["dx"]],
    headerok = TRUE,
    ever.assignable = TRUE,
    warn.ambiguous = FALSE
  )
  if (!all(z)) {
    stop(sprintf("not all user_visible[['%s']][['full_code']] are valid ever.assignable ICD codes", n))
  }
}
################################################################################
# PCCC specific checks
#
# * verify no more than two rows for a code by pccc variant
variants <- grep("^pccc_", names(user_visible[["pccc_codes"]]), value = TRUE)
ns <-
  stats::aggregate(
    x = user_visible[["pccc_codes"]][variants],
    by = user_visible[["pccc_codes"]][c("icdv", "dx", "code")],
    FUN = sum
  )
stopifnot(unlist(ns[variants]) %in% c(0L, 1L, 2L))

################################################################################
# ICD code specific checks

# assignable_start is NA then assignable_end should be NA
for (n in grep("^icd_", names(user_visible), value = TRUE)) {
  x <- is.na(user_visible[[n]][["assignable_start"]])
  y <- is.na(user_visible[[n]][["assignable_end"]])
  z <- identical(x, y)
  if (!z) {
    stop(sprintf("is.na status of assignable_start and assignable_end are not identical for %s", n))
  }
}

# verify that columns which are expected to be fully populated are
for (n in grep("^icd_", names(user_visible), value = TRUE)) {
  for (j in c("icdv", "dx", "full_code", "code", "known_start", "known_end", "src")) {
    z <- all(Negate(is.na)(user_visible[[n]][[j]]))
    if (!z) {
      stop(sprintf("user_visible[['%s']][['%s']] has missing values", n, j))
    }
  }
}

# Verify that the src column is as expected
for (n in grep("^icd_", names(user_visible), value = TRUE)) {
  stopifnot(
    identical(c("cdc", "cms", "who"), sort(unique(user_visible[[n]][["src"]])))
  )
}



################################################################################
#                                 End of File                                  #
################################################################################

library(medicalcoder)

# Verifies cumulative flagging behavior for Charlson when poa/poa.var is omitted.
# Expected default: first encounter for a condition is NOT flagged (poa defaults
# to 0), but the condition carries forward and is flagged on later encounters
# (poa set to 1 after first occurrence). Explicit poa = 1 should flag all
# encounters.

df <- data.frame(
  patid = c(1L, 1L),
  enc   = c(1L, 2L),
  icdv  = c(10L, 10L),
  dx    = c(1L, 1L),
  code  = c("I252", "I252"),
  stringsAsFactors = FALSE
)

# Baseline: explicit poa = 1 flags the condition under cumulative logic
explicit_poa <- comorbidities(
  data        = df,
  icd.codes   = "code",
  id.vars     = c("patid", "enc"),
  icdv.var    = "icdv",
  dx.var      = "dx",
  method      = "charlson_quan2011",
  flag.method = "cumulative",
  poa         = 1L,
  primarydx   = 0L
)

stopifnot(any(explicit_poa[["cmrb_flag"]]))
stopifnot(
  explicit_poa$cmrb_flag[explicit_poa$enc == 1L] == 1L,
  explicit_poa$cmrb_flag[explicit_poa$enc == 2L] == 1L
)

# Without specifying poa/poa.var, encounter 1 should remain unflagged and
# encounter 2 should be flagged.
default_poa <- comorbidities(
  data        = df,
  icd.codes   = "code",
  id.vars     = c("patid", "enc"),
  icdv.var    = "icdv",
  dx.var      = "dx",
  method      = "charlson_quan2011",
  flag.method = "cumulative",
  primarydx   = 0L
)

stopifnot(
  default_poa$cmrb_flag[default_poa$enc == 1L] == 0L,
  default_poa$cmrb_flag[default_poa$enc == 2L] == 1L
)

#' Get Charlson Codes
#'
#' Retrieve a copy of internal lookup tables for the ICD codes 
#' used in assessing Charlson comorbidities.
#'
#' @seealso
#' * [`get_charlson_index_scores()`] for a lookup table of the by comorbidity index scores.
#' * [`get_icd_codes()`] for the lookup table of all ICD codes.
#' * [`get_pccc_codes()`] for the lookup table of ICD codes used for the PCCC.
#' * [`get_elixhauser_codes()`] for the lookup table of ICD codes used for the Elixhauser comorbidities.
#' * [`comorbidities()`] for applying comorbidity algorithms to a data set.
#'
#' @return
#'
#' A `data.frame` with the following columns:
#' 
#' * `icdv`: Integer vector indicating if the code is from ICD-9 or ICD-10
#' * `dx`: Integer vector.  1 if the code is a diagnostic, (ICD-9-CM, ICD-10-CM, WHO, CDC Mortality), or 0 if the code is procedural (ICD-9-PCS, ICD-10-PCS)
#' * `full_code`: Character vector with the ICD code and any relevant decimal point
#' * `code`: Character vector with the compact ICD code
#' * `condition`: Character vector of the conditions
#' * `charlson_\<variant\>`: Integer vector indicating if the code is part of the \<variant\> of the Charlson comorbidities.
#'
#' @examples
#' head(get_charlson_codes())
#' str(get_charlson_codes())
#'
#' @export
get_charlson_codes <- function() {
  x <- get("charlson_codes", envir = ..mdcr_data_env.., inherits = FALSE)
  unserialize(serialize(x, connection = NULL))
}

source('utilities.R')
library(medicalcoder)

################################################################################
# Regression test: selected ICD-10 codes should map to specific Elixhauser
# comorbidities under the Quan (2005) implementation.
################################################################################

example_codes <-
  data.frame(
    patid = c("P1", "P1", "P1", "P2", "P2", "P2", "P3"),
    icdv  = 10L,
    dx    = 1L,
    code  = c("I099",  # CHF
              "I110",  # Hypertensive heart disease with heart failure (supports CHF)
              "I132",  # Hypertensive heart and kidney disease with heart failure
              "E109",  # Diabetes mellitus without complications
              "E1021", # Diabetes with renal complications
              "E6609", # Other obesity
              "N185"   # Chronic kidney disease, stage 5 (renal failure)
    ),
    stringsAsFactors = FALSE
  )

res <- comorbidities(
  data        = example_codes,
  icd.codes   = "code",
  id.vars     = "patid",
  icdv.var    = "icdv",
  dx.var      = "dx",
  method      = "elixhauser_quan2005",
  poa         = 1,
  primarydx   = 0,
  flag.method = "current"
)

# P1: congestive heart failure and HTN_C
stopifnot(res[["CHF"]][res[["patid"]] == "P1"] == 1L)
stopifnot(res[["HTN_C"]][res[["patid"]] == "P1"] == 1L)
stopifnot(res[["num_cmrb"]][res[["patid"]] == "P1"] == 2L)

# P2: diabetes (with and without complications) + obesity - the patient should
# flag both but only have with complications reported.
stopifnot(res[["DM"]][res[["patid"]] == "P2"]  == 0L)
stopifnot(res[["DMCX"]][res[["patid"]] == "P2"] == 1L)
stopifnot(res[["OBESE"]][res[["patid"]] == "P2"] == 1L)
stopifnot(res[["num_cmrb"]][res[["patid"]] == "P2"] == 2L)

# P3: renal failure only
stopifnot(res[["RENLFAIL"]][res[["patid"]] == "P3"] == 1L)
stopifnot(res[["num_cmrb"]][res[["patid"]] == "P3"] == 1L)

################################################################################
#                                 End of File                                  #
################################################################################

source('utilities.R')
################################################################################
# Tests for summarizing Charlson comorbidities
library(medicalcoder)

################################################################################
# Prepare input with an age variable to exercise age scoring logic
mdcr$age <- as.integer(substr(as.character(mdcr$patid), 1, 2))

charlson <- comorbidities(
  data       = mdcr,
  id.vars    = "patid",
  icdv.var   = "icdv",
  icd.codes  = "code",
  dx.var     = "dx",
  method     = "charlson_quan2011",
  flag.method = "current",
  poa        = 1L,
  primarydx  = 0L,
  age.var    = "age"
)

summary_current <- summary(charlson)

stopifnot(
  is.list(summary_current),
  identical(names(summary_current), c("conditions", "age_summary", "index_summary")),
  is.data.frame(summary_current$conditions),
  identical(
    names(summary_current$conditions),
    c("condition_description", "condition", "count", "percent")
  ),
  is.character(summary_current$conditions$condition_description),
  is.character(summary_current$conditions$condition),
  is.numeric(summary_current$conditions$count),
  is.numeric(summary_current$conditions$percent),
  all(summary_current$conditions$count >= 0),
  all(summary_current$conditions$percent >= 0),
  all(summary_current$conditions$percent <= 100)
)

################################################################################
# Conditions summary matches direct aggregations
aidshiv_count <- summary_current$conditions[
  !is.na(summary_current$conditions$condition) &
  summary_current$conditions$condition == "aidshiv",
  "count"
]
copd_count <- summary_current$conditions[
  !is.na(summary_current$conditions$condition) &
  summary_current$conditions$condition == "copd",
  "count"
]

stopifnot(
  aidshiv_count == sum(charlson$aidshiv),
  copd_count   == sum(charlson$copd)
)

num_ge_1 <- summary_current$conditions[
  is.na(summary_current$conditions$condition) &
  summary_current$conditions$condition_description == ">= 1",
  "count"
]
num_ge_2 <- summary_current$conditions[
  is.na(summary_current$conditions$condition) &
  summary_current$conditions$condition_description == ">= 2",
  "count"
]

stopifnot(
  num_ge_1 == sum(charlson$num_cmrb >= 1),
  num_ge_2 == sum(charlson$num_cmrb >= 2)
)

################################################################################
# Age and index summaries align with expected calculations
expected_age_summary <-
  {
    age_count <- table(charlson$age_score, useNA = "always")
    age_percent <- 100 * prop.table(age_count)
    data.frame(
      age_score = unname(names(age_count)),
      count = unname(as.integer(age_count)),
      percent = unname(as.numeric(age_percent)),
      stringsAsFactors = FALSE
    )
  }

stopifnot(identical(summary_current$age_summary, expected_age_summary))

expected_index_summary <-
  data.frame(
    min    = min(charlson$cci),
    q1     = stats::quantile(charlson$cci, prob = 0.25),
    median = stats::median(charlson$cci),
    q3     = stats::quantile(charlson$cci, prob = 0.75),
    max    = max(charlson$cci),
    row.names = NULL
  )

stopifnot(identical(summary_current$index_summary, expected_index_summary))

################################################################################
# Missing age.var creates an all-NA age_score and should still summarize cleanly
charlson_no_age <- comorbidities(
  data        = mdcr,
  id.vars     = "patid",
  icdv.var    = "icdv",
  icd.codes   = "code",
  dx.var      = "dx",
  method      = "charlson_quan2011",
  flag.method = "current",
  poa         = 1L,
  primarydx   = 0L
)

summary_no_age <- summary(charlson_no_age)

stopifnot(
  identical(summary_no_age$age_summary$age_score, NA_character_),
  identical(summary_no_age$age_summary$count, nrow(charlson_no_age)),
  identical(summary_no_age$age_summary$percent, 100),
  !anyNA(row.names(summary_no_age$age_summary))
)

################################################################################
# A non-current flag.method generates a warning but still returns the summary
charlson_cumulative <- charlson
attr(charlson_cumulative, "flag.method") <- "cumulative"

warn_obj <- tryCatchWarning(summary(charlson_cumulative))

stopifnot(
  inherits(warn_obj, "warning"),
  identical(
    conditionMessage(warn_obj),
    "Logic for charlson summary table has been implemented for flag.method = 'current'.  Using this function for flag.method = 'cumulative' may not provide a meaningful summary."
  )
)

summary_cumulative <- suppressWarnings(summary(charlson_cumulative))

stopifnot(
  identical(summary_cumulative$conditions, summary_current$conditions),
  identical(summary_cumulative$age_summary, summary_current$age_summary),
  identical(summary_cumulative$index_summary, summary_current$index_summary)
)

################################################################################
# Zero-row input should summarize without NaN/Inf
df0 <- data.frame(
  patid = integer(),
  icdv  = integer(),
  dx    = integer(),
  code  = character(),
  age   = integer(),
  stringsAsFactors = FALSE
)

charlson_zero <- comorbidities(
  data        = df0,
  id.vars     = "patid",
  icdv.var    = "icdv",
  icd.codes   = "code",
  dx.var      = "dx",
  method      = "charlson_quan2011",
  flag.method = "current",
  poa         = 1L,
  primarydx   = 0L,
  age.var     = "age"
)

summary_zero <- summary(charlson_zero)

stopifnot(
  is.list(summary_zero),
  identical(names(summary_zero), c("conditions", "age_summary", "index_summary")),
  all(summary_zero$conditions$count == 0L),
  !any(is.nan(summary_zero$conditions$percent)),
  all(is.na(summary_zero$conditions$percent)),
  all(summary_zero$index_summary == 0 | is.na(summary_zero$index_summary))
)

################################################################################
#                                 End of File                                  #
################################################################################

source('utilities.R')
library(medicalcoder)

################################################################################
# Prep data I want to test the output without loading or attaching the
# data.table or the dplyr namespaces.  So, the following commented out code is
# run once, and only run when needed to update the data, so that a data.frame,
# data.table, and dplyr are all available.
#
# To keep the disk space use down, use only a subset of the mdcr data set

### c1 <- comorbidities(data = mdcr, id.vars = "patid", icd.codes = "code", icdv.var = "icdv", dx.var = "dx", poa = 1, method = "charlson_quan2005")
### c2 <- comorbidities(data = mdcr, id.vars = "patid", icd.codes = "code", icdv.var = "icdv", dx.var = "dx", poa = 1, primarydx = 0, method = "elixhauser_quan2005")
### c3 <- comorbidities(data = mdcr, id.vars = "patid", icd.codes = "code", icdv.var = "icdv", dx.var = "dx", poa = 1, method = "pccc_v3.1")
###
### set.seed(42)
### foo <- function(n, size = 100) {
###   if (n <= 2) {
###     x <-
###       intersect(
###         intersect(which(c1$num_cmrb == n), which(c2$num_cmrb == n)),
###         intersect(which(c1$num_cmrb == n), which(c3$num_cmrb == n))
###       )
###   } else {
###     x <-
###       intersect(
###         intersect(which(c1$num_cmrb >= n), which(c2$num_cmrb >= n)),
###         intersect(which(c1$num_cmrb >= n), which(c3$num_cmrb >= n))
###       )
###   }
###   sample(x, size = min(c(size, length(x))))
### }
###
### x <- mdcr[mdcr$patid %in% c1[c(foo(0), foo(1), foo(2), foo(3), foo(4)), "patid"], ]
###
### saveRDS(x, file = "mdcr_subset_DF.rds", compress = "xz")
### saveRDS(dplyr::as_tibble(x), file = "mdcr_subset_TBL.rds", compress = "xz")
### data.table::setDT(x)
### saveRDS(x, file = "mdcr_subset_DT.rds", compress = "xz")

################################################################################
# load the data from disk
mdcrDT  <- readRDS(file = "mdcr_subset_DT.rds")
mdcrDF  <- readRDS(file = "mdcr_subset_DF.rds")
mdcrTBL <- readRDS(file = "mdcr_subset_TBL.rds")

################################################################################
# apply the comorbidities_methods to the three data sets
common_args <-
  list(id.vars = "patid", icdv.var = "icdv", icd.codes = "code", dx.var = "dx", poa = 1)

DFS  <- new.env()
TBLS <- new.env()
DTS  <- new.env()

methods <- medicalcoder:::comorbidities_methods()

set_results <- function(target_env, method, data, subconditions = FALSE) {
  if (startsWith(method, "pccc")) {
    v <- do.call(comorbidities, c(common_args, list(data = data, method = method, subconditions = subconditions)))
  } else {
    v <- do.call(comorbidities, c(common_args, list(data = data, method = method, subconditions = subconditions, primarydx = 0L)))
  }
  assign(
    x = if (subconditions) paste0(method, "_with_subconditions") else method,
    value = v,
    envir = target_env
  )
}

for (m in methods) {
  set_results(DFS,  method = m, data = mdcrDF,  subconditions = FALSE)
  set_results(DTS,  method = m, data = mdcrDT,  subconditions = FALSE)
  set_results(TBLS, method = m, data = mdcrTBL, subconditions = FALSE)

  if (grepl("pccc", m)) {
    set_results(DFS,  m, mdcrDF,  subconditions = TRUE)
    set_results(DTS,  m, mdcrDT,  subconditions = TRUE)
    set_results(TBLS, m, mdcrTBL, subconditions = TRUE)
  }
}

# same objects in each environment
stopifnot(
  identical(sort(ls(envir = DFS, all.names = TRUE)), sort(ls(envir = DTS, all.names = TRUE))),
  identical(sort(ls(envir = DFS, all.names = TRUE)), sort(ls(envir = TBLS, all.names = TRUE)))
)

# All objects should be medicalcoder_comorbidities objects
# only those with the _with_subconditions suffix should be
# medicalcoder_comorbidities_with_subconditions objects
for (obj in ls(envir = DFS, all.names = TRUE)) {
  xDF  <- DFS[[obj]]
  xDT  <- DTS[[obj]]
  xTBL <- TBLS[[obj]]

  stopifnot(inherits(xDF,  "medicalcoder_comorbidities"))
  stopifnot(inherits(xDT,  "medicalcoder_comorbidities"))
  stopifnot(inherits(xTBL, "medicalcoder_comorbidities"))

  sbcnd <- endsWith(obj, "_with_subconditions")

  stopifnot(identical(sbcnd, inherits(get(x = obj, envir = DFS),  "medicalcoder_comorbidities_with_subconditions")))
  stopifnot(identical(sbcnd, inherits(get(x = obj, envir = DTS),  "medicalcoder_comorbidities_with_subconditions")))
  stopifnot(identical(sbcnd, inherits(get(x = obj, envir = TBLS), "medicalcoder_comorbidities_with_subconditions")))
}

# All the results should be identical save the attributes (class).
for (obj in ls(envir = DFS, all.names = TRUE)) {
  xDF  <- DFS[[obj]]
  xTBL <- TBLS[[obj]]
  xDT  <- DTS[[obj]]
  stopifnot(all.equal(xDF, xTBL, check.attributes = FALSE))
  stopifnot(all.equal(xDF, xDT, check.attributes = FALSE))
}

# check that the print method returns the input object
for (obj in ls(envir = DFS, all.names = TRUE)) {
  x <- print(DFS[[obj]])
  z <- identical(x, DFS[[obj]])
  if (!z) {
    stop(sprintf("print(DFS[['%s']]) does not return identical %s", obj, obj))
  }
  x <- print(TBLS[[obj]])
  z <- identical(x, TBLS[[obj]])
  if (!z) {
    stop(sprintf("print(TBLS[['%s']]) does not return identical %s", obj, obj))
  }
  x <- print(DTS[[obj]])
  z <- identical(x, DTS[[obj]])
  if (!z) {
    stop(sprintf("print(DTS[['%s']]) does not return identical %s", obj, obj))
  }
}

################################################################################
# Now, if the data.table namespace is available, then test that the elements in
# DTS are data.tables and after setting to data.frames, then the objects are
# identical.

if (requireNamespace("data.table", quietly = TRUE)) {
  stopifnot(is.data.frame(mdcrDF))
  stopifnot(is.data.frame(mdcrDT))
  stopifnot(data.table::is.data.table(mdcrDT))
  for (obj in ls(envir = DTS, all.names = TRUE)) {
    if (grepl("_with_subconditions", obj)) {
      stopifnot(data.table::is.data.table(DTS[[obj]][["conditions"]]))
      data.table::setDF(DTS[[obj]][["conditions"]])
      for (sc in names(DTS[[obj]][["subconditions"]])) {
        stopifnot(data.table::is.data.table(DTS[[obj]][["subconditions"]][[sc]]))
        data.table::setDF(DTS[[obj]][["subconditions"]][[sc]])
      }
    } else {
      stopifnot(data.table::is.data.table(DTS[[obj]]))
      data.table::setDF(DTS[[obj]])
      class(DTS[[obj]]) <- c("medicalcoder_comorbidities", class(DTS[[obj]]))
    }
  }
}

# regardless of having data.table available, the elements should be identical to
# the results for a data.frame

for (obj in ls(envir = DFS, all.names = TRUE)) {
  xDF <- DFS[[obj]]
  xDT <- DTS[[obj]]
  stopifnot(identical(class(xDT), class(xDF)))
  stopifnot(identical(xDF, xDT))
}


################################################################################
# For tibbles

if (requireNamespace("dplyr", quietly = TRUE)) {
  stopifnot(is.data.frame(mdcrDF))
  stopifnot(is.data.frame(mdcrTBL))
  stopifnot(inherits(mdcrTBL, "tbl_df"))
  for (obj in ls(envir = TBLS, all.names = TRUE)) {
    if (grepl("_with_subconditions", obj)) {
      stopifnot(inherits(TBLS[[obj]][["conditions"]], "tbl_df"))
      TBLS[[obj]][["conditions"]] <- as.data.frame(TBLS[[obj]][["conditions"]])
      for (sc in names(TBLS[[obj]][["subconditions"]])) {
        stopifnot(inherits(TBLS[[obj]][["subconditions"]][[sc]], "tbl_df"))
        TBLS[[obj]][["subconditions"]][[sc]] <- as.data.frame(TBLS[[obj]][["subconditions"]][[sc]])
      }
    } else {
      stopifnot(inherits(TBLS[[obj]], "tbl_df"))
      TBLS[[obj]] <- as.data.frame(TBLS[[obj]])
      class(TBLS[[obj]]) <- c("medicalcoder_comorbidities", class(TBLS[[obj]]))
    }
  }
}

for (obj in ls(envir = DFS, all.names = TRUE)) {
  xDF <- DFS[[obj]]
  xTBL <- TBLS[[obj]]
  stopifnot(identical(class(xTBL), class(xDF)))
  stopifnot(identical(xTBL, xDF))
}



################################################################################
#                                 End of File                                  #
################################################################################

#' Is ICD
#'
#' Answer the question "is the character string x a valid ICD code?"
#' ICD codes should be character vectors. `is_icd` will assess for both
#' "full codes" (decimal point present when appropriate) and "compact codes"
#' (decimal point omitted).
#'
#' @template details-header-and-assignable-codes
#' @template details-fiscal-and-calendar-years
#'
#' @param x Character vector of ICD codes (full or compact form).
#' @param headerok Logical scalar. If `FALSE` (default), only assignable
#'   codes are considered valid; if `TRUE`, header codes are also accepted.
#' @param ever.assignable Logical scalar. If `TRUE` then ignore `year`
#'   and return `TRUE` if the `x` was ever an assignable code.
#' @param warn.ambiguous Logical scalar. If `TRUE` (default), warn when a
#'   code matches more than one ICD version and/or type (e.g., both CM and PCS).
#' @template params-icd-icdv
#' @template params-icd-src
#' @template params-icd-dx
#' @template params-icd-year
#' @template params-icd-form
#'
#' @seealso
#' * [`get_icd_codes()`] to retrieve the internal lookup table of ICD codes.
#' * [`lookup_icd_codes()`] for retrieving details on a specific set of ICD
#'   codes.
#' * [`icd_compact_to_full()`] converts a string from a compact format to the
#'   full format based on ICD version and type (diagnostic or procedure).
#'
#' @return A logical vector the same length as `x`.
#'
#' @example examples/is_icd.R
#'
#' @family ICD tools
#' @export
is_icd <- function(x, icdv = c(9L, 10L), dx = c(1L, 0L),
                   src = c("cms", "who", "cdc"),
                   year, headerok = FALSE,
                   ever.assignable = missing(year),
                   warn.ambiguous = TRUE,
                   full.codes = TRUE, compact.codes = TRUE) {

  # check arguments
  if (is.factor(x)) {
    x <- as.character(x)
  }
  stopifnot(is.character(x))
  stopifnot(all(icdv %in% c(9L, 10L)))
  stopifnot(all(dx %in% c(1L, 0L)))
  src <- match.arg(src, several.ok = TRUE)
  assert_scalar_logical(headerok)
  assert_scalar_logical(ever.assignable)
  assert_scalar_logical(warn.ambiguous)
  assert_scalar_logical(full.codes)
  assert_scalar_logical(compact.codes)
  stopifnot(full.codes | compact.codes)
  stopifnot(missing(year) || (length(year) == 1L & is.numeric(year)))

  # return_false will generate a vector of equal length to x and is all FALSE or
  # NA.  this is used  then there are no possible codes to check and the return
  # should just be FALSE
  return_false <- function(x) {
    rtn <- rep_len(FALSE, length(x))
    rtn[is.na(x)] <- NA
    rtn
  }

  # get the known icd codes and filter to relevent codes
  codes <- get("icd_codes", envir = ..mdcr_data_env.., inherits = FALSE)

  # keep based on icdv, dx, and src
  keep <- (codes[["icdv"]] %in% icdv) & (codes[["dx"]] %in% dx) & (codes[["src"]] %in% src)
  icdv_dx_src_msg <-
    sprintf("`icdv` = %s; `dx` = %s; and `src` = %s;",
      toString(icdv), toString(dx), toString(src)
    )
  if (!any(keep)) {
    msg <-
      paste0("The combination of ", icdv_dx_src_msg, " does not match any ICD codes in the internal medicalcoder lookup tables.")
    warning(msg, call. = FALSE)
    return(return_false(x))
  }
  codes <- codes[keep, , drop = FALSE]

  # known_start / known_end are guaranteed populated; see tests/test-get_icd.R
  min_known_start <- min(codes[["known_start"]])
  max_known_end   <- max(codes[["known_end"]])

  if (missing(year)) {
    year <- max_known_end
  } else if (year < min_known_start) {
    msg <-
      paste0(
        "The combination of ", icdv_dx_src_msg,
        " has ICD codes with a first known_start year of ", min_known_start,
        ". The input of `year` = ", year, " results in no possible positive match."
      )
    warning(msg, call. = FALSE)
    return(return_false(x))
  } else if (year > max_known_end) {
    msg <-
      paste0(
        "The combination of ", icdv_dx_src_msg,
        " has ICD codes with a max known_end year of ", max_known_end,
        ". The input of `year` = ", year, " results in no possible positive match."
      )
    warning(msg, call. = FALSE)
    return(return_false(x))
  }


  keep <- rep_len(TRUE, nrow(codes))
  if (!headerok) {
    # require assignable rows, either ever or at the given year
    keep <- keep & !is.na(codes[["assignable_start"]])
  }
  if (!ever.assignable) {
    keep <- keep &
      codes[["assignable_start"]] <= year &
      codes[["assignable_end"]] >= year
  }
  if (!any(keep)) {
    msg <-
      paste0(
        "The combination of ", icdv_dx_src_msg,
        " along with `headerok` = ", toString(headerok),
        " and `ever.assignable` = ", toString(ever.assignable),
        " results in no possible positive match."
      )
    warning(msg, call. = FALSE)
    return(return_false(x))
  }

  codes <- codes[keep, , drop = FALSE]


  # unique codes to match on
  ux <- unique(x)

  kf <- integer(0L)
  kc <- integer(0L)
  if (full.codes) {
    kf <- which(codes[["full_code"]] %in% ux)
  }
  if (compact.codes) {
    kc <- which(codes[["code"]] %in% ux)
  }

  codes <- codes[union(kf, kc), , drop = FALSE]

  if (nrow(codes) == 0L) {
    return(return_false(x))
  }

  # If you are here, there are possible valid ICD to compare to

  if (full.codes) {
    rf <- x %in% codes[["full_code"]]
  } else {
    rf <- rep_len(FALSE, length(x))
  }
  if (compact.codes) {
    rc <- x %in% codes[["code"]]
  } else {
    rc <- rep_len(FALSE, length(x))
  }

  rtn <- rf | rc
  rtn[is.na(x)] <- NA

  # ambiguity check
  if (warn.ambiguous) {

    DF <- mdcr_unique(codes[c("full_code", "code", "icdv", "dx")])

    dfcs <- DF[0, , drop = FALSE]
    dccs <- DF[0, , drop = FALSE]

    if (full.codes) {
      dfcs <- DF[DF[["full_code"]] %in% ux, c("full_code", "icdv", "dx"), drop = FALSE]
      dfcs <- dfcs[duplicated(dfcs[["full_code"]]) | duplicated(dfcs[["full_code"]], fromLast = TRUE), ]
      dfcs[["msg"]] <- sprintf("ICD %d %s full", dfcs[["icdv"]],
                               c("pr", "dx")[1L + dfcs[["dx"]]])
    }

    if (compact.codes) {
      dccs <- DF[DF[["code"]] %in% ux, c("code", "icdv", "dx"), drop = FALSE]
      dccs <- dccs[duplicated(dccs[["code"]]) | duplicated(dccs[["code"]], fromLast = TRUE), ]
      dccs[["msg"]] <- sprintf("ICD %d %s compact", dccs[["icdv"]],
                               c("pr", "dx")[1L + dccs[["dx"]]])
    }

    dfcs <- split(dfcs, dfcs[["full_code"]])
    dccs <- split(dccs, dccs[["code"]])


    for (u in ux) {
      mf <- mc <- character(0L)
      if (u %in% names(dfcs)) {
        mf <- paste(dfcs[[u]][["msg"]], collapse = "; ")
      }
      if (u %in% names(dccs)) {
        mc <- paste(dccs[[u]][["msg"]], collapse = "; ")
      }
      if (length(mf) | length(mc)) {
        msg <-
          sprintf("Input '%s' is a: %s ",
                  u, paste(c(mf, mc), collapse = "; "))
        warning(msg, call. = FALSE)
      }
    }
  }

  rtn
}

################################################################################
#                                 End of File                                  #
################################################################################

source('utilities.R')
library(medicalcoder)

################################################################################
# a code of length 1 should not be a valid code for either ICD-9 or ICD-10, that
# is because the minimum number of characters for ICD-9-CM ICD-10-CM is three,
# ICD-10-PCS is seven, and ICD-9-PCS is two.  The folowing tests verify that
# is_icd will return FALSE any reasonable one characters string.
#
# tests:
#
#   code_length_one_default: use the default call to is_icd
#   code_length_one_icd9dx:  test against just ICD-9 diagnostic codes
#   code_length_one_icd9pr:  test against just ICD-9 procedure codes
#   code_length_one_icd10dx: test against just ICD-10 diagnostic codes
#   code_length_one_icd10pr: test against just ICD-10 procedure codes
#
one_char_codes <- c(LETTERS, as.character(0:9))
default <- is_icd(one_char_codes)
icd9dx  <- is_icd(one_char_codes, icdv =  9L, dx = 1L)
icd9pr  <- is_icd(one_char_codes, icdv =  9L, dx = 0L)
icd10dx <- is_icd(one_char_codes, icdv = 10L, dx = 1L)
icd10pr <- is_icd(one_char_codes, icdv = 10L, dx = 0L)

stopifnot(
  code_length_one_default = length(default) == 36L && !any(default),
  code_length_one_icd9dx  = length(icd9dx)  == 36L && !any(icd9dx),
  code_length_one_icd9pr  = length(icd9pr)  == 36L && !any(icd9pr),
  code_length_one_icd10dx = length(icd10dx) == 36L && !any(icd10dx),
  code_length_one_icd10pr = length(icd10pr) == 36L && !any(icd10pr)
)

################################################################################
# check for early return if the input combination of icdv, dx, src excludes all
# possible codes
x <- c("7993", ".7993", "7.993", "79.93", "799.3", "7993.")
y <- rep_len(FALSE, length(x))

stopifnot(
  "failed to stop on bad icdv" = inherits(tryCatchError(is_icd(x, icdv = 8)), "error"),
  "failed to stop on bad src"  = inherits(tryCatchError(is_icd(x, src = "a")), "error"),
  "failed to stop on bad dx"   = inherits(tryCatchError(is_icd(x, dx = 2)), "error"),

  "warning for WHO and ICD 9"   = inherits(tryCatchWarning(is_icd(x, icdv = 9, src = "who")), "warning"),
  "all FALSE for WHO and ICD 9" = identical(suppressWarnings(is_icd(x, icdv = 9, src = "who")), y),

  "warning for CDC and ICD 10 pr"   = inherits(tryCatchWarning(is_icd(x, icdv = 10, src = "cdc", dx = 0)), "warning"),
  "all FALSE for CDC and ICD 10 pr" = identical(suppressWarnings(is_icd(x, icdv = 10, src = "cdc", dx = 0)), y),

  "warning for WHO, ICD-10, pr" = inherits(tryCatchWarning(is_icd(x, icdv = 10, src = "who", dx = 0)), "warning"),
  "all FALSE for WHO, ICD-10, pr" = identical(suppressWarnings(is_icd(x, icdv = 10, src = "who", dx = 0)), y)
)


################################################################################
# For ICD-9 test that the presense of a dot is considered when testing.
# Example 7993 is the simplified version of the proper ICD-9 DX code 799.3 and
# PR code 79.93.  Becuase the look up tables use 7993, the input of 7993 will be
# valid code for both dx and pr.  if a dot is present, consider it, and return
# FALSE if the dot is in the wrong place

x <- c("7993", ".7993", "7.993", "79.93", "799.3", "7993.")
f <- factor(x)
stopifnot(
  identical(is_icd(x, icdv =  9, dx = 1L), c(TRUE, FALSE, FALSE, FALSE, TRUE, FALSE)),
  identical(is_icd(x, icdv =  9, dx = 0L), c(TRUE, FALSE, FALSE, TRUE, FALSE, FALSE)),
  !any(is_icd(x, icdv = 10, dx = 1L)),
  !any(is_icd(x, icdv = 10, dx = 0L)),
  identical(is_icd(x, icdv =  9, dx = 1L), is_icd(f, icdv =  9, dx = 1L)),
  identical(is_icd(x, icdv =  9, dx = 0L), is_icd(f, icdv =  9, dx = 0L)),
  identical(is_icd(x, icdv = 10, dx = 1L), is_icd(f, icdv = 10, dx = 1L)),
  identical(is_icd(x, icdv = 10, dx = 0L), is_icd(f, icdv = 10, dx = 0L))
)

# expect a warning that 7993 is ambiguous
x <- tryCatchWarning(is_icd("7993"))
stopifnot(inherits(x, "warning"))

# For ICD-10 dx, if there is a dot, it need be the fourth character
# C44.1121 (basal cell carcinoma of skn of right upper eyelide, including
# canthus) is a valid code, so "C441121" should return TRUE and a dot anywere
# other than the fourth character should be FALSE.
x <- c("C441121",
       ".C441121",
       "C.441121",
       "C4.41121",
       "C44.1121",
       "C441.121",
       "C4411.21",
       "C44112.1",
       "C441121.")
f <- factor(x)
stopifnot(
  identical(is_icd(x, icdv = 10, dx = 1L), c(TRUE, FALSE, FALSE, FALSE, TRUE, FALSE, FALSE, FALSE, FALSE)),
  !any(is_icd(x, icdv = 9, dx = 1L)),
  !any(is_icd(x, icdv = 9, dx = 0L)),
  !any(is_icd(x, icdv = 10, dx = 0L)),
  identical(is_icd(x, icdv =  9, dx = 1L), is_icd(f, icdv =  9, dx = 1L)),
  identical(is_icd(x, icdv =  9, dx = 0L), is_icd(f, icdv =  9, dx = 0L)),
  identical(is_icd(x, icdv = 10, dx = 1L), is_icd(f, icdv = 10, dx = 1L)),
  identical(is_icd(x, icdv = 10, dx = 0L), is_icd(f, icdv = 10, dx = 0L))
  )

# another example
x <- c("Y389X2S",
       ".Y389X2S",
       "Y.389X2S",
       "Y3.89X2S",
       "Y38.9X2S",
       "Y389.X2S",
       "Y389X.2S",
       "Y389X2.S",
       "Y389X2S.")
f <- factor(x)
stopifnot(
  identical(is_icd(x), c(TRUE, FALSE, FALSE, FALSE, TRUE, FALSE, FALSE, FALSE, FALSE)),
  identical(is_icd(x, icdv = 10, dx = 1L), c(TRUE, FALSE, FALSE, FALSE, TRUE, FALSE, FALSE, FALSE, FALSE)),
  !any(is_icd(x, icdv = 9, dx = 1L)),
  !any(is_icd(x, icdv = 9, dx = 0L)),
  !any(is_icd(x, icdv = 10, dx = 0L)),
  identical(is_icd(x, icdv =  9, dx = 1L), is_icd(f, icdv =  9, dx = 1L)),
  identical(is_icd(x, icdv =  9, dx = 0L), is_icd(f, icdv =  9, dx = 0L)),
  identical(is_icd(x, icdv = 10, dx = 1L), is_icd(f, icdv = 10, dx = 1L)),
  identical(is_icd(x, icdv = 10, dx = 0L), is_icd(f, icdv = 10, dx = 0L))
  )

# ICD 10 pr codes have no dots
x <- c("0016074",
       ".0016074",
       "0.016074",
       "00.16074",
       "001.6074",
       "0016.074",
       "00160.74",
       "001607.4",
       "0016074.")
f <- factor(x)
stopifnot(
  identical(is_icd(x), c(TRUE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE)),
  identical(is_icd(x, icdv = 10, dx = 0L), c(TRUE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE)),
  !any(is_icd(x, icdv = 9, dx = 1L)),
  !any(is_icd(x, icdv = 9, dx = 0L)),
  !any(is_icd(x, icdv = 10, dx = 1L)),
  identical(is_icd(x, icdv =  9, dx = 1L), is_icd(f, icdv =  9, dx = 1L)),
  identical(is_icd(x, icdv =  9, dx = 0L), is_icd(f, icdv =  9, dx = 0L)),
  identical(is_icd(x, icdv = 10, dx = 1L), is_icd(f, icdv = 10, dx = 1L)),
  identical(is_icd(x, icdv = 10, dx = 0L), is_icd(f, icdv = 10, dx = 0L))
  )

################################################################################
# Ever assignable status of ICD-9 516.3
# lookup_icd_codes("516\\.3", regex = TRUE)
stopifnot(
    is_icd("516.3")              # TRUE, because it was assignable through 2011
  , is_icd("516.3", year = 1998) # TRUE, it was assignable
  , is_icd("516.3", year = 1999) # TRUE, it was assignable
  , is_icd("516.3", year = 2000) # TRUE, it was assignable
  , is_icd("516.3", year = 2001) # TRUE, it was assignable
  , is_icd("516.3", year = 2002) # TRUE, it was assignable
  , is_icd("516.3", year = 2003) # TRUE, it was assignable
  , is_icd("516.3", year = 2004) # TRUE, it was assignable
  , is_icd("516.3", year = 2004) # TRUE, it was assignable
  , is_icd("516.3", year = 2005) # TRUE, it was assignable
  , is_icd("516.3", year = 2006) # TRUE, it was assignable
  , is_icd("516.3", year = 2007) # TRUE, it was assignable
  , is_icd("516.3", year = 2008) # TRUE, it was assignable
  , is_icd("516.3", year = 2009) # TRUE, it was assignable
  , is_icd("516.3", year = 2010) # TRUE, it was assignable
  , is_icd("516.3", year = 2011) # TRUE, it was assignable
  , !is_icd("516.3", year = 2012) # FALSE, not assignable
  , !is_icd("516.3", year = 2013) # FALSE, not assignable
  , !is_icd("516.3", year = 2014) # FALSE, not assignable
  , !is_icd("516.3", year = 2015) # FALSE, not assignable
  , !is_icd("516.3", year = 2016) # FALSE, not assignable
)

################################################################################
#
# check
x <- c("E010", "C84.1", "Not a Code", "C04", "E010", "021", "V91.1", NA, "E010",
       "021")

default_rtn <-
  is_icd(x, headerok = TRUE, ever.assignable = TRUE, warn.ambiguous = FALSE)

stopifnot(identical(default_rtn, c(TRUE, TRUE, FALSE, TRUE, TRUE, TRUE, TRUE, NA, TRUE, TRUE)))

warn <-
  tryCatchWarning(
    is_icd(x, headerok = TRUE, ever.assignable = TRUE, warn.ambiguous = TRUE)
  )
stopifnot(inherits(warn, "warning"))

################################################################################
# Warn if user provides a year before the earliest known year.  Note: ICD-9 in
# the United states went into effect for
# fiscal year 1980 (Oct 1, 1979 - Sept 30, 1980).  Any year before 1980 will
# generate this error no matter how much data is in the internal lookup tables
# for ICD-9, at least based on the US version.  As of Oct 2025 and pre public
# release of medicalcoder, the earliest known year is 1997.
#
# This should work based on different sets of codes too.  For example, using the
# data in medicalcoder 0.0.0.9045, the warning should be thrown based on the
# version and source.
#
#   > as.data.table(get_icd_codes())[, min(known_start), by = .(src, icdv)]
#         src  icdv    V1
#      <char> <int> <int>
#   1:    cms     9  1997
#   2:    cms    10  2014
#   3:    cdc    10  2001
#   4:    who    10  2008
w1 <- tryCatchWarning(is_icd("516.3", year = 1979))
w2 <- tryCatchWarning(is_icd("516.3", year = 1979, icdv = 9))
w3 <- tryCatchWarning(is_icd("516.3", year = 2000, icdv = 9))
w4 <- tryCatchWarning(is_icd("516.3", year = 2000, icdv = 10))
w5 <- tryCatchWarning(is_icd("516.3", year = 2009, icdv = 10))
w6 <- tryCatchWarning(is_icd("516.3", year = 2009, icdv = 10, src = "cms"))

stopifnot(
  "Year before first known year generates an warning: w1" = inherits(w1, "warning"),
  "Year before first known year generates an warning: w2" = inherits(w2, "warning"),
  "year = 2000 does not generate a warning for icdv 9"    = !inherits(w3, "warning"),
  "year = 2000 generates a warning for icdv 10"           = inherits(w4, "warning"),
  "year = 2009 does not generate a warning for icdv 10"   = !inherits(w5, "warning"),
  "year = 2009 does generate a warning for icdv 10 with src = cms" = inherits(w6, "warning")
)


################################################################################
# ICD-9 516.3 is a good test for ever.assignable and year.  It was assignable
# from 1997 through 2011, and then a header code through the end of ICD-9 in
# 2015.
#
# t1:
#   default call to is_icd for code 516.3 is TRUE because year is missing and
#   thus treated as most current, and ever.billable is TRUE because year was
#   missing
#
# t2:
#   explicitly use ever.assignable = TRUE
#
# t3:
#   with ever.assignable = FALSE and year missing the return should be FALSE as
#   the code was not assignable in the most current year.
#
# t4:
#   with ever.assignable = FALSE and year = 2012 the return will be FALSE
#   because 516.3 was not assignable in 2012
#
# t5:
#   with ever.assignable = TRUE and year = 2012 the return will be TRUE
#   because 516.3 was billable prior to 2012
#
# t6:
#   with ever.assignable = FALSE and year = 2011 the return will be TRUE because
#   516.3 was assignable in 2011

stopifnot(
  "t1" =  is_icd("516.3"),
  "t2" =  is_icd("516.3", ever.assignable = TRUE),
  "t3" = !is_icd("516.3", ever.assignable = FALSE),
  "t4" = !is_icd("516.3", ever.assignable = FALSE, year = 2012),
  "t5" =  is_icd("516.3", ever.assignable = TRUE, year = 2012),
  "t6" =  is_icd("516.3", ever.assignable = FALSE, year = 2011)
  )

################################################################################
#                                 End of File                                  #
################################################################################

source('utilities.R')
library(medicalcoder)

################################################################################
# verify that there is not going to be an error if no matches are found
dat <- data.frame(patid = 1:26,
                  icd = c(letters, LETTERS),
                  pr1 = 1:26,
                  icdv = sample(9:10, size = 26, replace = TRUE))

stopifnot(sum(
    comorbidities(data = dat, icd.codes = "icd", poa = 1, flag.method = "current", method = "pccc_v3.0")
    , na.rm = TRUE) == 0)

stopifnot(sum(comorbidities(data = dat, id.vars = "patid", icd.codes = "icd",                    poa = 1, flag.method = "current", method = "pccc_v3.0"), na.rm = TRUE) == sum(1:26))
stopifnot(sum(comorbidities(data = dat, id.vars = "patid", icd.codes = "icd", icdv.var = "icdv", poa = 1, flag.method = "current", method = "pccc_v3.0"), na.rm = TRUE) == sum(1:26))
stopifnot(sum(comorbidities(data = dat, id.vars = "patid", icd.codes = "icd", icdv = 9,          poa = 1, flag.method = "current", method = "pccc_v3.0"), na.rm = TRUE) == sum(1:26))
stopifnot(sum(comorbidities(data = dat, id.vars = "patid", icd.codes = "icd", icdv = 10,         poa = 1, flag.method = "current", method = "pccc_v3.0"), na.rm = TRUE) == sum(1:26))
stopifnot(sum(comorbidities(data = dat, id.vars = "patid", icd.codes = "icd", dx = 1L,       poa = 1, flag.method = "current", method = "pccc_v3.0"), na.rm = TRUE) == sum(1:26))
stopifnot(sum(comorbidities(data = dat, id.vars = "patid", icd.codes = "icd", dx = 0L,       poa = 1, flag.method = "current", method = "pccc_v3.0"), na.rm = TRUE) == sum(1:26))

stopifnot(sum(comorbidities(data = dat, icd.codes = "icd", method = "pccc_v2.0", poa = 1, flag.method = "current"), na.rm = TRUE) == 0)

stopifnot(sum(comorbidities(data = dat, id.vars = "patid", icd.codes = "icd",                    poa = 1, flag.method = "current", method = "pccc_v2.0"), na.rm = TRUE) == sum(1:26))
stopifnot(sum(comorbidities(data = dat, id.vars = "patid", icd.codes = "icd", icdv.var = "icdv", poa = 1, flag.method = "current", method = "pccc_v2.0"), na.rm = TRUE) == sum(1:26))
stopifnot(sum(comorbidities(data = dat, id.vars = "patid", icd.codes = "icd", icdv = 9,          poa = 1, flag.method = "current", method = "pccc_v2.0"), na.rm = TRUE) == sum(1:26))
stopifnot(sum(comorbidities(data = dat, id.vars = "patid", icd.codes = "icd", icdv = 10,         poa = 1, flag.method = "current", method = "pccc_v2.0"), na.rm = TRUE) == sum(1:26))
stopifnot(sum(comorbidities(data = dat, id.vars = "patid", icd.codes = "icd", dx = 1L,       poa = 1, flag.method = "current", method = "pccc_v2.0"), na.rm = TRUE) == sum(1:26))
stopifnot(sum(comorbidities(data = dat, id.vars = "patid", icd.codes = "icd", dx = 0L,       poa = 1, flag.method = "current", method = "pccc_v2.0"), na.rm = TRUE) == sum(1:26))

################################################################################
# Verify the output of the pccc call

rtn <- comorbidities(data = mdcr, icd.codes = "code", method = "pccc_v3.0", poa = 1, flag.method = "current")

expected_rtn <-
  structure(
    list(
      congeni_genetic_dxpr_only = 1L,
      congeni_genetic_tech_only = 0L,
      congeni_genetic_dxpr_and_tech = 0L,
      congeni_genetic_dxpr_or_tech = 1L,
      cvd_dxpr_only = 0L,
      cvd_tech_only = 0L,
      cvd_dxpr_and_tech = 1L,
      cvd_dxpr_or_tech = 1L,
      gi_dxpr_only = 0L,
      gi_tech_only = 0L,
      gi_dxpr_and_tech = 1L,
      gi_dxpr_or_tech = 1L,
      hemato_immu_dxpr_only = 1L,
      hemato_immu_tech_only = 0L,
      hemato_immu_dxpr_and_tech = 0L,
      hemato_immu_dxpr_or_tech = 1L,
      malignancy_dxpr_only = 1L,
      malignancy_tech_only = 0L,
      malignancy_dxpr_and_tech = 0L,
      malignancy_dxpr_or_tech = 1L,
      metabolic_dxpr_only = 0L,
      metabolic_tech_only = 0L,
      metabolic_dxpr_and_tech = 1L,
      metabolic_dxpr_or_tech = 1L,
      misc_dxpr_only = 0L,
      misc_tech_only = 0L,
      misc_dxpr_and_tech = 1L,
      misc_dxpr_or_tech = 1L,
      neonatal_dxpr_only = 1L,
      neonatal_tech_only = 0L,
      neonatal_dxpr_and_tech = 0L,
      neonatal_dxpr_or_tech = 1L,
      neuromusc_dxpr_only = 0L,
      neuromusc_tech_only = 0L,
      neuromusc_dxpr_and_tech = 1L,
      neuromusc_dxpr_or_tech = 1L,
      renal_dxpr_only = 0L,
      renal_tech_only = 0L,
      renal_dxpr_and_tech = 1L,
      renal_dxpr_or_tech = 1L,
      respiratory_dxpr_only = 0L,
      respiratory_tech_only = 0L,
      respiratory_dxpr_and_tech = 1L,
      respiratory_dxpr_or_tech = 1L,
      any_tech_dep = 1L,
      any_transplant = 1L,
      num_cmrb = 11L,
      cmrb_flag = 1L
    ),
    row.names = c(NA, -1L),
    class = c("medicalcoder_comorbidities", "data.frame"),
    method = "pccc_v3.0",
    id.vars = "..medicalcoder_id..",
    flag.method = "current"
  )

stopifnot(
  "pccc v3 with mdcr is as all.equal" = isTRUE(all.equal(rtn, expected_rtn)),
  "pccc v3 with mdcr is as expected" = isTRUE(identical(rtn, expected_rtn))
)

################################################################################
# test longitudinal output

args <- list(data = mdcr_longitudinal, id.vars = c("patid", "date"), icdv.var = "icdv", icd.codes = "code", poa = 1)

current <-
  list(
    pccc_current_v3.1    = do.call(comorbidities, c(args, list(flag.method = 'current',    method = "pccc_v3.1"))),
    pccc_cumulative_v3.1 = do.call(comorbidities, c(args, list(flag.method = "cumulative", method = "pccc_v3.1"))),
    pccc_current_v3.0    = do.call(comorbidities, c(args, list(flag.method = 'current',    method = "pccc_v3.0"))),
    pccc_cumulative_v3.0 = do.call(comorbidities, c(args, list(flag.method = "cumulative", method = "pccc_v3.0"))),
    pccc_current_v2.1    = do.call(comorbidities, c(args, list(flag.method = 'current',    method = "pccc_v2.1"))),
    pccc_cumulative_v2.1 = do.call(comorbidities, c(args, list(flag.method = "cumulative", method = "pccc_v2.1"))),
    pccc_current_v2.1    = do.call(comorbidities, c(args, list(flag.method = 'current',    method = "pccc_v2.0"))),
    pccc_cumulative_v2.1 = do.call(comorbidities, c(args, list(flag.method = "cumulative", method = "pccc_v2.0")))
)

# if the expected results need to be updated:
### for (n in names(current)) {
###   saveRDS(
###     object = current[[n]],
###     file = file.path("expected-results-for-test-pccc", paste0(n, ".rds")),
###     compress = "xz"
###   )
### }

expected <- list.files(path = "expected-results-for-test-pccc", pattern = "\\.rds$", full.name = TRUE)
names(expected) <- sub("\\.rds", "", basename(expected))
expected <- sapply(expected, readRDS, simplify = FALSE)

for (obj in names(current)) {
  stopifnot(all.equal(expected[[obj]], current[[obj]]))
}

################################################################################
# tests for different sets of inputs
# with or with icdv
# with or without dx

# common arguments for the calls to comorbidities
cargs <-
  list(
    data = mdcr,
    id.vars = "patid",
    icd.codes = "code",
    poa = 1,
    method = "pccc_v3.1"
  )

out00 <- do.call(comorbidities, c(cargs, list(icdv.var = "icdv", dx.var = "dx")))
out01 <- do.call(comorbidities, c(cargs, list(                   dx.var = "dx")))
out02 <- do.call(comorbidities, c(cargs, list(icdv = 9,          dx.var = "dx")))
out03 <- do.call(comorbidities, c(cargs, list(icdv = 10,         dx.var = "dx")))
out04 <- do.call(comorbidities, c(cargs, list(icdv.var = "icdv"               )))
out05 <- do.call(comorbidities, c(cargs, list(icdv.var = "icdv", dx = 0       )))
out06 <- do.call(comorbidities, c(cargs, list(icdv.var = "icdv", dx = 1       )))
out07 <- do.call(comorbidities, c(cargs))

# all the outputs should be unique
stopifnot(
  !isTRUE(all.equal(out00, out01)),
  !isTRUE(all.equal(out00, out02)),
  !isTRUE(all.equal(out00, out03)),
  !isTRUE(all.equal(out00, out04)),
  !isTRUE(all.equal(out00, out05)),
  !isTRUE(all.equal(out00, out06)),
  !isTRUE(all.equal(out00, out07)),

  !isTRUE(all.equal(out01, out02)),
  !isTRUE(all.equal(out01, out03)),
  !isTRUE(all.equal(out01, out04)),
  !isTRUE(all.equal(out01, out05)),
  !isTRUE(all.equal(out01, out06)),
  !isTRUE(all.equal(out01, out07)),

  !isTRUE(all.equal(out02, out03)),
  !isTRUE(all.equal(out02, out04)),
  !isTRUE(all.equal(out02, out05)),
  !isTRUE(all.equal(out02, out06)),
  !isTRUE(all.equal(out02, out07)),

  !isTRUE(all.equal(out03, out04)),
  !isTRUE(all.equal(out03, out05)),
  !isTRUE(all.equal(out03, out06)),
  !isTRUE(all.equal(out03, out07)),

  !isTRUE(all.equal(out04, out05)),
  !isTRUE(all.equal(out04, out06)),
  !isTRUE(all.equal(out04, out07)),

  !isTRUE(all.equal(out05, out06)),
  !isTRUE(all.equal(out05, out07)),

  !isTRUE(all.equal(out06, out07))
)

# out00 should be "correct", out01 and out04 should only have false positives
stopifnot(
  all(out00$num_cmrb <= out01$num_cmrb),
  all(out00$num_cmrb <= out04$num_cmrb)
)

# for out02, out03, out05, and out06, there will be false positives and false
# negatives
stopifnot(
  !identical(out00$num_cmrb, out02$num_cmrb),
  !identical(out00$num_cmrb, out03$num_cmrb),
  !identical(out00$num_cmrb, out05$num_cmrb),
  !identical(out00$num_cmrb, out06$num_cmrb),
  !identical(out00$num_cmrb, out07$num_cmrb)
)

################################################################################
# looking for the case in v3.1 where the code is a tech and transplant
techtrans <-
  subset(get_pccc_codes(), pccc_v3.1 == 1 & transplant_flag == 1 & tech_dep_flag == 1)
techtrans[["rid"]] <- seq_len(nrow(techtrans))

techtrans_results <-
  comorbidities(
    data = techtrans,
    icd.codes = "code",
    id.vars = "rid",
    method = "pccc_v3.1",
    poa = 1
  )

for (j in colnames(techtrans_results)) {
  if (j == "rid") {
    stopifnot(techtrans_results[[j]] == 1:6)
  } else if (j %in% c("cvd_tech_only", "cvd_dxpr_or_tech", "any_tech_dep", "any_transplant", "num_cmrb", "cmrb_flag")) {
    stopifnot(techtrans_results[[j]] == rep(1L, 6))
  } else {
    stopifnot(techtrans_results[[j]] == rep(0L, 6))
  }
}

################################################################################
#                                 End of File                                  #
################################################################################

#' Construct Charlson indicator data.frame and compute CCI
#'
#' Internal.  Called from [`comorbidities()`].  Not expected to be called from
#' any other context.
#'
#' @param id.vars Character vector of ID column names.
#' @param iddf A `data.frame` of unique IDs
#' @param cmrb A `data.frame` containing at least `id.vars` and
#' `condition` columns; i.e., the 'comorbidity' `data.frame`.
#' @param primarydx.var Character (scalar) with the name of the column in
#' `cmrb` denoting if the condition was flaged as a primary diagnostic or not.
#' @param method Character scalar; name of the Charlson variant to assess
#'
#' @return A `data.frame` with `id.vars`, per-condition 0/1
#' indicators, `num_cmrb`, `cmrb_flag`, and `cci`.
#'
#' @family internal comorbidity functions
#' @noRd
#' @keywords internal
.charlson <- function(id.vars, iddf, cmrb, primarydx.var, method) {
  ccc <- mdcr_unique(mdcr_select(cmrb, cols = c(id.vars, "condition", primarydx.var)))

  # omit primary dx
  idx <- which(ccc[[primarydx.var]] == 0L)
  ccc <- mdcr_subset(ccc, i = idx)

  # get the method weights and conditions
  conditions <- mdcr_subset(..mdcr_internal_charlson_index_scores..,
                            i = !is.na(..mdcr_internal_charlson_index_scores..[[method]]),
                            cols = c("condition", method))
  needed <- conditions[["condition"]]
  cci_wt <- stats::setNames(as.integer(conditions[[method]]), needed)

  # build indicator matrix
  X <- matrix(0L, nrow = nrow(iddf), ncol = length(needed))
  colnames(X) <- needed

  key_iddf <- do.call(paste, c(iddf, sep = "\r"))
  key_ccc  <- do.call(paste, c(mdcr_select(ccc, id.vars), sep = "\r"))
  ri <- match(key_ccc, key_iddf)
  ci <- match(ccc[["condition"]], needed)

  keep <- !(is.na(ri) | is.na(ci))
  if (any(keep)) {
    X[cbind(ri[keep], ci[keep])] <- 1L
  }

  # for charlson_cdmf2019 the aids category is 1 hiv and aids are flagged
  if (method == "charlson_cdmf2019") {
    X[, "aids"] <- as.integer(X[, "aids"] * X[, "hiv"])
  }

  # set less severe conditions to zero if the more severe condition is present
  X[X[, "msld"] == 1L, "mld"] <- 0L
  X[X[, "dmc"]  == 1L, "dm"]  <- 0L
  X[X[, "mst"]  == 1L, "mal"] <- 0L

  # num_cmrb, cmrb_flag, cci
  storage.mode(X) <- "integer"
  num_cmrb  <- as.integer(rowSums(X))
  cmrb_flag <- as.integer(num_cmrb > 0L)
  cci      <- as.integer(as.vector(X %*% cci_wt))

  # build the return object
  rtn <- mdcr_cbind(iddf, as.data.frame(X, check.names = FALSE, stringsAsFactors = FALSE))
  rtn <- mdcr_set(rtn, j = "num_cmrb",  value = num_cmrb)
  rtn <- mdcr_set(rtn, j = "cmrb_flag", value = cmrb_flag)
  rtn <- mdcr_set(rtn, j = "cci",      value = cci)
  rownames(rtn) <- NULL
  rtn
}

source('utilities.R')
################################################################################
# Tests for summarizing Elixhauser comorbidities
library(medicalcoder)

################################################################################
# Build Elixhauser comorbidities and summary
elixhauser <- comorbidities(
  data        = mdcr,
  id.vars     = "patid",
  icdv.var    = "icdv",
  icd.codes   = "code",
  dx.var      = "dx",
  method      = "elixhauser_ahrq_icd10",
  flag.method = "current",
  poa         = 1,
  primarydx   = 0
)

summary_current <- summary(elixhauser)

stopifnot(
  is.list(summary_current),
  identical(names(summary_current), c("conditions", "index_summary")),
  is.data.frame(summary_current$conditions),
  identical(
    names(summary_current$conditions),
    c("condition", "count", "percent")
  ),
  is.character(summary_current$conditions$condition),
  is.numeric(summary_current$conditions$count),
  is.numeric(summary_current$conditions$percent),
  all(summary_current$conditions$count >= 0),
  all(summary_current$conditions$percent >= 0),
  all(summary_current$conditions$percent <= 100)
)

################################################################################
# Conditions summary matches direct aggregations
hf_count <- summary_current$conditions[
  summary_current$conditions$condition == "HF",
  "count"
]
diab_cx_count <- summary_current$conditions[
  summary_current$conditions$condition == "DIAB_CX",
  "count"
]

stopifnot(
  hf_count == sum(elixhauser$HF),
  diab_cx_count == sum(elixhauser$DIAB_CX)
)

num_ge_1 <- summary_current$conditions[
  summary_current$conditions$condition == ">= 1",
  "count"
]
num_ge_2 <- summary_current$conditions[
  summary_current$conditions$condition == ">= 2",
  "count"
]

stopifnot(
  num_ge_1 == sum(elixhauser$num_cmrb >= 1),
  num_ge_2 == sum(elixhauser$num_cmrb >= 2)
)

################################################################################
# Index summary aligns with expected calculations
stopifnot(
  is.data.frame(summary_current$index_summary),
  identical(
    names(summary_current$index_summary),
    c("index", "min", "q1", "median", "q3", "max")
  ),
  is.character(summary_current$index_summary$index),
  is.numeric(summary_current$index_summary$min),
  is.numeric(summary_current$index_summary$q1),
  is.numeric(summary_current$index_summary$median),
  is.numeric(summary_current$index_summary$q3),
  is.numeric(summary_current$index_summary$max)
)

expected_index_summary <-
  data.frame(
    index  = c("readmission", "mortality"),
    min    = c(min(elixhauser$readmission_index),
               min(elixhauser$mortality_index)),
    q1     = c(stats::quantile(elixhauser$readmission_index, prob = 0.25),
               stats::quantile(elixhauser$mortality_index, prob = 0.25)),
    median = c(stats::median(elixhauser$readmission_index),
               stats::median(elixhauser$mortality_index)),
    q3     = c(stats::quantile(elixhauser$readmission_index, prob = 0.75),
               stats::quantile(elixhauser$mortality_index, prob = 0.75)),
    max    = c(max(elixhauser$readmission_index),
               max(elixhauser$mortality_index)),
    row.names = NULL,
    stringsAsFactors = FALSE
  )

stopifnot(identical(summary_current$index_summary, expected_index_summary))

################################################################################
# A non-current flag.method generates a warning but still returns the summary
elixhauser_cumulative <- elixhauser
attr(elixhauser_cumulative, "flag.method") <- "cumulative"

warn_obj <- tryCatchWarning(summary(elixhauser_cumulative))

stopifnot(
  inherits(warn_obj, "warning"),
  identical(
    conditionMessage(warn_obj),
    "Logic for Elixhauser summary has been implemented for flag.method = 'current'.  Using this function for flag.method = 'cumulative' may not provide a meaningful summary."
  )
)

summary_cumulative <- suppressWarnings(summary(elixhauser_cumulative))

stopifnot(
  identical(summary_cumulative$conditions, summary_current$conditions),
  identical(summary_cumulative$index_summary, summary_current$index_summary)
)

################################################################################
# Zero-row input should summarize without NaN/Inf
df0 <- data.frame(
  patid = integer(),
  icdv  = integer(),
  dx    = integer(),
  code  = character(),
  stringsAsFactors = FALSE
)

elixhauser_zero <- comorbidities(
  data        = df0,
  id.vars     = "patid",
  icdv.var    = "icdv",
  icd.codes   = "code",
  dx.var      = "dx",
  method      = "elixhauser_ahrq_icd10",
  flag.method = "current",
  poa         = 1L,
  primarydx   = 0L
)

summary_zero <- summary(elixhauser_zero)

stopifnot(
  is.list(summary_zero),
  identical(names(summary_zero), c("conditions", "index_summary")),
  all(summary_zero$conditions$count == 0L),
  !any(is.nan(summary_zero$conditions$percent)),
  all(is.na(summary_zero$conditions$percent)),
  all(is.na(summary_zero$index_summary$min)),
  all(is.na(summary_zero$index_summary$q1)),
  all(is.na(summary_zero$index_summary$median)),
  all(is.na(summary_zero$index_summary$q3)),
  all(is.na(summary_zero$index_summary$max))
)

################################################################################
#                                 End of File                                  #
################################################################################

#' medicalcoder
#'
#' An R package for working with ICD codes and comorbidity assessments.
#'
#' medicalcoder is a lightweight, base-R package for working with ICD-9 and
#' ICD-10 diagnosis and procedure codes. It provides fast, dependency-free tools
#' to look up, validate, and manipulate ICD codes, while also implementing
#' widely used comorbidity algorithms such as Charlson, Elixhauser, and the
#' Pediatric Complex Chronic Conditions (PCCC).
#' Designed for portability and reproducibility, the package avoids external
#' dependencies—requiring only R >= 3.5.0—yet offers a rich set of curated ICD
#' code libraries from the United States' Centers for Medicare and Medicaid
#' Services (CMS), Centers for Disease Control (CDC), and the World Health
#' Organization (WHO).
#'
#' The package balances performance with elegance: its internal caching,
#' efficient joins, and compact data structures make it practical for
#' large-scale health data analyses, while its clean design makes it easy to
#' extend or audit. Whether you need to flag comorbidities, explore ICD
#' hierarchies, or standardize clinical coding workflows, medicalcoder provides
#' a robust, transparent foundation for research and applied work in biomedical
#' informatics.
#'
#' # Implementation
#'
#' The medicalcoder package was intentionally designed and built to have zero
#' dependencies beyond R version 3.5.0 (needed due to a [change in data
#' serialization](https://cran.r-project.org/src/base/NEWS.3#:~:text=R%20has%20new,to%20version%203.5.0.))
#' and zero imports. The package is completely self-contained for the purposes
#' of installation and use.
#'
#' This design choice was made for several reasons.
#'
#' 1. Ease of installation:
#'    * Only requirement is R >= 3.5.0.
#'    * No need for external files, downloads, or other packages for the ICD database.
#'
#' 2. Works well with different data paradigms
#'    * Base R `data.frames`,
#'    * [tidyverse](https://tidyverse.org/) `tibble`s, and
#'    * `data.table`s from the [data.table](https://rdatatable.gitlab.io/data.table/) package.
#'
#' One of the reasons for focusing on building a self-contained package with no
#' need for additional namespaces is to make installation and use in a
#' restricted and secured system easier. The author of this package routinely works
#' on machines with extremely limited access to the world-wide web. As
#' such, relying on any system dependencies or other R packages can become
#' difficult as the machine may or may not have the needed software. So long as
#' R >= 3.5.0 is available, medicalcoder will work.
#'
#' A great deal of thought went into performance of the methods and the size of
#' the package.  The internal data sets, for example, are not stored in a
#' structure that is easy to use for end users. When the package namespace is
#' loaded, the needed internal lookup tables are generated and cached.
#'
#' @seealso [comorbidities()], [get_icd_codes()], [is_icd()],
#'
#' * Vignette for working with ICD codes:
#'   * `vignette(topic = "icd", package = "medicalcoder")`
#' * Vignettes for applying comorbidities:
#'   * `vignette(topic = "comorbidities", package = "medicalcoder")`
#'   * `vignette(topic = "pccc", package = "medicalcoder")`
#'   * `vignette(topic = "charlson", package = "medicalcoder")`
#'   * `vignette(topic = "elixhauser", package = "medicalcoder")`
#'
#' @keywords internal
"_PACKAGE"

source('utilities.R')
################################################################################
library(medicalcoder)
set.seed(42)

################################################################################
# verify an error will be thrown if lenght(id.vars) < 2 and flag.method =
# "cumulative"

rtn <- # length(id.vars) = 0
  tryCatchError(
    comorbidities(
      data = mdcr,
      icd.codes = "code",
      poa = 1L,
      flag.method = 'cumulative',
      method = "pccc_v3.1"
    )
  )
stopifnot(inherits(rtn, "error"))

rtn <- # length(id.vars) = 1
  tryCatchError(
    comorbidities(
      data = mdcr,
      id.vars = "patid",
      icd.codes = "code",
      poa = 1L,
      flag.method = 'cumulative',
      method = "pccc_v3.1"
    )
  )
stopifnot(inherits(rtn, "error"))


################################################################################
# verify errors are thrown if a "protected" name is used for the id.vars,
# poa.var, or primarydx.var
mdcr2 <- mdcr
mdcr2[["condition"]] <- 1L

x <-
  tryCatchError(
    comorbidities(
      data = mdcr2,
      icd.codes = "code",
      id.vars = c("patid", "condition"),
      method = "pccc_v3.1"
    )
  )
stopifnot(inherits(x, "error"))

x <-
  tryCatchError(
    comorbidities(
      data = mdcr2,
      icd.codes = "code",
      id.vars = c("condition"),
      method = "pccc_v3.1"
    )
  )
stopifnot(inherits(x, "error"))

x <-
  tryCatchError(
    comorbidities(
      data = mdcr2,
      icd.codes = "code",
      poa.vars = c("condition"),
      method = "pccc_v3.1"
    )
  )
stopifnot(inherits(x, "error"))

x <-
  tryCatchError(
    comorbidities(
      data = mdcr2,
      icd.codes = "code",
      primarydx.var = "condition",
      method = "elixhauser_ahrq2025"
    )
  )
stopifnot(inherits(x, "error"))

################################################################################
# verify errors are thrown when icdv.var, dx.var, poa.var, or primarydx.var are
# non-numeric
mdcr3 <- mdcr
mdcr3[["icdv_chr"]] <- as.character(mdcr3[["icdv"]])
mdcr3[["dx_chr"]] <- as.character(mdcr3[["dx"]])
mdcr3[["poa_chr"]] <- as.character(rep(1L, nrow(mdcr3)))
mdcr3[["primarydx_chr"]] <- as.character(rep(0L, nrow(mdcr3)))

x <-
  tryCatchError(
    comorbidities(
      data = mdcr3,
      id.vars = "patid",
      icd.codes = "code",
      icdv.var = "icdv_chr",
      dx.var = "dx",
      poa = 1L,
      method = "pccc_v3.1"
    )
  )
stopifnot(
  inherits(x, "error"),
  grepl(
    "Column .*icdv_chr.* must be numeric \\(9/10/NA\\) when supplied as icdv\\.var\\.",
    gsub("['\"`\\p{Pi}\\p{Pf}]", "", x[["message"]], perl = TRUE)
  )
)

x <-
  tryCatchError(
    comorbidities(
      data = mdcr3,
      id.vars = "patid",
      icd.codes = "code",
      icdv.var = "icdv",
      dx.var = "dx_chr",
      poa = 1L,
      method = "pccc_v3.1"
    )
  )
stopifnot(
  inherits(x, "error"),
  grepl(
    "Column .*dx_chr.* must be numeric \\(0/1/NA\\) when supplied as dx\\.var\\.",
    gsub("['\"`\\p{Pi}\\p{Pf}]", "", x[["message"]], perl = TRUE)
  )
)

x <-
  tryCatchError(
    comorbidities(
      data = mdcr3,
      id.vars = "patid",
      icd.codes = "code",
      icdv.var = "icdv",
      dx.var = "dx",
      poa.var = "poa_chr",
      method = "pccc_v3.1"
    )
  )
stopifnot(
  inherits(x, "error"),
  grepl(
    "Column .*poa_chr.* must be numeric \\(0/1/NA\\) when supplied as poa\\.var\\.",
    gsub("['\"`\\p{Pi}\\p{Pf}]", "", x[["message"]], perl = TRUE)
  )
)

x <-
  tryCatchError(
    comorbidities(
      data = mdcr3,
      id.vars = "patid",
      icd.codes = "code",
      icdv.var = "icdv",
      dx.var = "dx",
      poa = 1L,
      primarydx.var = "primarydx_chr",
      method = "elixhauser_ahrq2025"
    )
  )
stopifnot(
  inherits(x, "error"),
  grepl(
    "Column .*primarydx_chr.* must be numeric \\(0/1/NA\\) when supplied as primarydx\\.var\\.",
    gsub("['\"`\\p{Pi}\\p{Pf}]", "", x[["message"]], perl = TRUE)
  )
)


################################################################################
# Test: check_and_set_*
#
# The check_and_set_* are non-exported methods.  We test them here with calls to
# comorbidities() so that we can capture these test with covr

# Verify these methods are non-exported
stopifnot(!("check_and_set_id_vars" %in% getNamespaceExports("medicalcoder")))

# The following data frames are used to test check_and_set_id_vars.
# `..medicalcoder_id..` is the initial name used in the check and set when
# creating an id column.  it needs to persist in the output if provided as an
# id.
DF0 <- data.frame(pid = NA, code = "A00")
DF1 <- data.frame(medicalcoder_id = NA, code = "A00")
DF2 <- data.frame(..medicalcoder_id.. = NA, code = "A00")

# verify that the constuction of a id.vars within the function works as
# expected.
OUT0 <- comorbidities(data = DF0, icd.codes = "code", method = "pccc_v3.0", poa = 1)
OUT1 <- comorbidities(data = DF1, icd.codes = "code", method = "pccc_v3.0", poa = 1)
OUT2 <- comorbidities(data = DF2, icd.codes = "code", method = "pccc_v3.0", poa = 1)

expected_names <-
  c(
    "congeni_genetic_dxpr_only",
    "congeni_genetic_tech_only",
    "congeni_genetic_dxpr_and_tech",
    "congeni_genetic_dxpr_or_tech",

    "cvd_dxpr_only",
    "cvd_tech_only",
    "cvd_dxpr_and_tech",
    "cvd_dxpr_or_tech",

    "gi_dxpr_only",
    "gi_tech_only",
    "gi_dxpr_and_tech",
    "gi_dxpr_or_tech",

    "hemato_immu_dxpr_only",
    "hemato_immu_tech_only",
    "hemato_immu_dxpr_and_tech",
    "hemato_immu_dxpr_or_tech",

    "malignancy_dxpr_only",
    "malignancy_tech_only",
    "malignancy_dxpr_and_tech",
    "malignancy_dxpr_or_tech",

    "metabolic_dxpr_only",
    "metabolic_tech_only",
    "metabolic_dxpr_and_tech",
    "metabolic_dxpr_or_tech",

    "misc_dxpr_only",
    "misc_tech_only",
    "misc_dxpr_and_tech",
    "misc_dxpr_or_tech",

    "neonatal_dxpr_only",
    "neonatal_tech_only",
    "neonatal_dxpr_and_tech",
    "neonatal_dxpr_or_tech",

    "neuromusc_dxpr_only",
    "neuromusc_tech_only",
    "neuromusc_dxpr_and_tech",
    "neuromusc_dxpr_or_tech",

    "renal_dxpr_only",
    "renal_tech_only",
    "renal_dxpr_and_tech",
    "renal_dxpr_or_tech",

    "respiratory_dxpr_only",
    "respiratory_tech_only",
    "respiratory_dxpr_and_tech",
    "respiratory_dxpr_or_tech",

    "any_tech_dep",
    "any_transplant",

    "num_cmrb",
    "cmrb_flag"
  )

stopifnot(identical(names(OUT0), expected_names))
stopifnot(identical(names(OUT1), expected_names))
stopifnot(identical(names(OUT2), expected_names))

OUT0 <- comorbidities(data = DF0, id.vars = "pid", icd.codes = "code", method = "pccc_v3.0", poa = 1)
OUT1 <- comorbidities(data = DF1, id.vars = "medicalcoder_id", icd.codes = "code", method = "pccc_v3.0", poa = 1)
OUT2 <- comorbidities(data = DF2, id.vars = "..medicalcoder_id..", icd.codes = "code", method = "pccc_v3.0", poa = 1)

stopifnot(identical(names(OUT0), c("pid", expected_names)))
stopifnot(identical(names(OUT1), c("medicalcoder_id", expected_names)))
stopifnot(identical(names(OUT2), c("..medicalcoder_id..", expected_names)))

################################################################################
# Check for "protected" names in id.vars.
#
# During development I had been using id.vars such as:
#
#   id.vars = c("patid", "icd_code")
#
# so I could look at which codes in a patient record where flagging.
#
# When I tried the following
#
#   id.vars = c("patid", "icd_code", "icdv")
#   icdv.var = NULL
#   icdv = NULL
#
# I'd get an error because the "icdv" name is being used in the lookup tables and
# gets drop in the methods and the joins in the code created a icdv.x and icdv.y
# variable.  The code was updated so that the suffixes would be c("", ".y") to
# fix this issue.
#
# for 0.0.0.9038 the following calls to comorbidities would error with the
# following message:
#
#    Error in `[.data.frame`(x, r, vars, drop = drop) :
#      undefined columns selected
#
# Expect that the following calls should work without error
#
# between 0.0.0.9044 and 0.0.0.9045 it was determined that there are some names
# that should not be used in the id.vars, poa.var, or the primarydx.var.  Tests
# for those are above.  The tests below would have passed for 0.0.0.9039 -
# 0.0.0.9044, but will error for 0.0.0.9045 with a useful error message.
mdcr$full_code <- "just a test"
mdcr$icd_code <- mdcr$code

args <- list(data = mdcr, icd.code = "icd_code", method = "pccc_v3.0", poa = 1)

out1 <- tryCatchError(do.call(comorbidities, c(args, list(id.vars = c("patid", "full_code")))))
out2 <- tryCatchError(do.call(comorbidities, c(args, list(id.vars = c("patid", "icdv")))))
out3 <- tryCatchError(do.call(comorbidities, c(args, list(id.vars = c("patid", "dx")))))
out4 <- tryCatchError(do.call(comorbidities, c(args, list(id.vars = c("patid", "code")))))
stopifnot(
  inherits(out1, "error"),
  inherits(out2, "error"),
  inherits(out3, "error"),
  inherits(out4, "error")
)

################################################################################
# when a primarydx.var was passed to comorbidities when not needed an error was
# thrown.  https://github.com/dewittpe/medicalcoder/issues/16
#
# This has been corrected to be a warning
x <-
  tryCatchWarning(
    comorbidities(
      data = mdcr,
      id.var = "patid",
      method = "charlson_quan2005",
      icd.codes = "code",
      poa = 1
    )
  )
stopifnot("warning for missing primarydx" = inherits(x, "warning"))

################################################################################
# Subconditions are only applicable to PCCC, so a warning should be given when
# subconditions = TRUE for any other method
args <- list(
  data = mdcr[1:10, ],
  icd.codes = "icd_code",
  icdv.var = "icdv",
  dx.var = "dx",
  poa = 1,
  subconditions = TRUE
  )
ms <- medicalcoder:::comorbidities_methods()
ms <- ms[!startsWith(ms, "pccc")]
for (m in ms) {
  x <- tryCatchWarning(do.call(comorbidities, c(args, list(method = m))))
  z <- inherits(x, "warning")
  if (!z) {
    stop(sprintf("no warning given for subconditions = TRUE with method = '%s'", m))
  }
  if (x$message != "subconditions only implemented for PCCC") {
    stop(sprintf("unexpected warning message for subcondtions = TRUE with method = '%s'", m))
  }
}

################################################################################
# age.var handling for charlson with missing/ambiguous id.vars
#
# RE: https://github.com/dewittpe/medicalcoder/issues/43
DF <-
  data.frame(
    pid = c(1, 1, 2),
    enc_id = c(1, 2, 1),
    icd_code = c("", "", ""),
    age = c(54, 55, 54),
    stringsAsFactors = FALSE
  )

common_args <-
  list(
    data      = DF,
    icd.codes = "icd_code",
    poa       = 1L,
    primarydx = 0L,
    method    = "charlson_quan2005"
  )

# id.vars = NULL should not error when age.var is supplied
# for v0.8.0 this would error.
out_no_error <-
  tryCatchError(
    suppressWarnings(
      do.call(comorbidities, c(common_args, list(id.vars = NULL, age.var = "age")))
    )
  )
stopifnot(inherits(out_no_error, "medicalcoder_comorbidities"), nrow(out_no_error) == 2L)

# if id.vars = NULL, or id.vars = "ptid", for the example, data, the return will
# have three rows becuase the ages are distinct.  This should give a warning
out0_warning <-
  tryCatchWarning(
    do.call(comorbidities, c(common_args, list(id.vars = NULL, age.var = "age")))
  )
out1_warning <-
  tryCatchWarning(
    do.call(comorbidities, c(common_args, list(id.vars = "pid", age.var = "age")))
  )
stopifnot(inherits(out0_warning, "warning"), inherits(out1_warning, "warning"))
stopifnot(
  out0_warning$message == "There is more than one unique value for age.  Since `id.vars = NULL` the expectation is there would be one unique age value.  The return will have more than one row, one for each unique age.",
  out1_warning$message == "There is at least one set of id.vars with more than one age value.  The expectation is that there is only one age value for each unique set of id.vars.  The return will have more than one row for each unique set of id.vars."
)

out0 <-
  suppressWarnings(
    do.call(comorbidities, c(common_args, list(id.vars = NULL, age.var = "age")))
  )
out1 <-
  suppressWarnings(
    do.call(comorbidities, c(common_args, list(id.vars = "pid", age.var = "age")))
  )
stopifnot(
  inherits(out0, "medicalcoder_comorbidities"), nrow(out0) == 2L,
  inherits(out1, "medicalcoder_comorbidities"), nrow(out1) == 3L
)

################################################################################
#                                 End of File                                  #
################################################################################

source('utilities.R')
if (!requireNamespace("data.table", quietly = TRUE)) {
  message("SKIP: data.table not available; skipping test-locked-bindings.R")
  quit(save = "no", status = 0, runLast = FALSE)
}

library(data.table)

# it is possible for a end user to modify the internal objects, this is a R
# problem. For example, if you use data.table::setDT on datasets::mtcars you can
# modify the object. The following example worked as of 29 Aug 2025 in R 4.5.1
# and datasets_4.5.1.
#
#> setDT(datasets::mtcars)
#> set(datasets::mtcars, j = "cyl", value = 1L)
#> stopifnot(all(mtcars[["cyl"]] == 1L))
#> stopifnot(all(datasets::mtcars[["cyl"]] == 1L))

# medicalcoder tries to prevent this for its lookup tables.  Importally, :w
#
# several user friendly data sets in .onLoad.  If a end user tries to
# modify the non-exported internal objects, only accessable via ::: then the
# namespace is loaded _before_ anything else can be done and the data sets used
# within the package should be preserved.
setDT(medicalcoder:::..mdcr_internal_icd_codes..)
x <- medicalcoder:::..mdcr_internal_icd_codes..

# bad user, don't modify things, even if you can!
x[, icdv := 8L]
stopifnot(medicalcoder:::..mdcr_internal_icd_codes..[, all(icdv == 8L)])

# but the good news is that when the namespace was loaded the data set that
# needed to be provided to the user and is called within the package via
# get_icd_codes is not modified!
x <- medicalcoder::get_icd_codes()
stopifnot(x[["icdv"]] %in% c(9L, 10L))

# now, what about getting at the built objects?
# first, the ..mdcr_data_env.. is not accessable other than by :::
library(medicalcoder)
stopifnot(medicalcoder:::..mdcr_internal_icd_codes..[, all(icdv == 8L)])  # still bad, but...

# the environment is hard to get to
x <- tryCatchError(..mdcr_data_env..)
stopifnot(inherits(x, "error"))
x <- tryCatchError(medicalcoder::..mdcr_data_env..)
stopifnot(inherits(x, "error"))

x <- medicalcoder:::..mdcr_data_env..
stopifnot(is.environment(x))
# and modifing the data will error
t <- tryCatchError(x$icd_codes[["icdv"]] <- 11L)
stopifnot(inherits(t, "error"))

# data.table will also fail, at first.  Note that accessing the icd_codes right
# now it is just a data.frame
stopifnot(
   is.data.frame(medicalcoder:::..mdcr_data_env..$icd_codes),
  !is.data.table(medicalcoder:::..mdcr_data_env..$icd_codes)
)

# an error is thrown, but the class has been modified and it is now a data.table
t <- tryCatchError(setDT(x$icd_codes))
stopifnot(inherits(t, "error"))
stopifnot(
   is.data.frame(medicalcoder:::..mdcr_data_env..$icd_codes),
   is.data.table(medicalcoder:::..mdcr_data_env..$icd_codes)
)

# so now, end user could modify the object and the return from get_icd_codes()
# will reflect this change
stopifnot(all(get_icd_codes()[["icdv"]] %in% c(9L, 10L)))
medicalcoder:::..mdcr_data_env..$icd_codes[, icdv := 98L]
stopifnot(all(get_icd_codes()[["icdv"]] == 98L))

# so, yeah, end users could mess things up but if someone does this that is on
# them.  Importantly, the get_x functions use
#
# unserialize(serialize(x, connection = NULL))
#
# to ensure end users only get deep copies of the chached data sets.  One quick
# check.
detach("package:medicalcoder", unload = TRUE)
library(medicalcoder)
x <- get_icd_codes()
setDT(x)
y <- data.table::copy(x)
x[, icdv := 42L]
z <- get_icd_codes()

stopifnot(
  all(y[["icdv"]] %in% c(9L, 10L)),
  all(z[["icdv"]] %in% c(9L, 10L)),
  all(x[["icdv"]] == 42L)
)

################################################################################
#                                 End of File                                  #
################################################################################

#' Pediatric Complex Chronic Condition and Subconditions
#'
#' Retrieve a copy of internal lookup tables for the syntax valid and human
#' readable labels of the Pediatric Complex Chronic Conditions (PCCC) conditions
#' and subconditions.
#'
#' @seealso
#' * [`get_pccc_codes()`] for the lookup table of ICD codes used for the PCCC.
#' * [comorbidities()] for applying comorbidity algorithms to a data set.
#'
#' @return a `data.frame` with the following columns
#' * `condition`: (character) syntax valid name for the condition
#' * `subcondition`: (character) syntax valid name for the subcondition
#' * `condition_label`: (character) human readable label for the condition
#' * `subcondition_label`: (character) human readable label for the
#'   subcondition
#'
#' @examples
#' get_pccc_conditions()
#'
#' @export
get_pccc_conditions <- function() {
  unserialize(serialize(..mdcr_internal_pccc_conditions.., connection = NULL))
}

################################################################################
#                                 End of File                                  #
################################################################################

# define some helper functions for the testing suite
tryCatchError <- function(expr, ...) {
  tryCatch(expr, ..., error = function(e) {e})
}

tryCatchWarning <- function(expr, ...) {
  tryCatch(expr, ..., warning = function(w) {w})
}

library(medicalcoder)

# POA-NA should still flag POA-exempt Elixhauser conditions (post-2022).
method <- "elixhauser_ahrq_icd10"

codes <- get_elixhauser_codes()
codes <- codes[!is.na(codes[[method]]), , drop = FALSE]
codes[["patid"]] <- seq_len(nrow(codes))
codes[["poa"]] <- NA_integer_

rtn <- comorbidities(
  data = codes,
  icd.codes = "code",
  id.vars = "patid",
  icdv.var = "icdv",
  dx.var = "dx",
  poa.var = "poa",
  primarydx = 0L,
  flag.method = "current",
  method = method
)

rtn <-
  merge(
    x = codes[, c("patid", "poaexempt", "condition")],
    y = rtn,
    by = "patid"
  )

# verify
# ICD codes which are POA exempt should all flag to 1
poaexempt_codes <- subset(rtn, poaexempt == 1L)
for (cn in unique(rtn[["condition"]])) {
  x <- subset(poaexempt_codes, condition == cn)
  stopifnot(all(x[[cn]] == 1L))
}

# now for the conditions
rtn <- subset(rtn, poaexempt == 0L)


poa_tbl <- get_elixhauser_poa()

poa_required_conds <- 
  poa_tbl[["condition"]][poa_tbl[[method]] == 1L & poa_tbl$poa_required == 1L]

poa_not_required_conds <- 
  poa_tbl[["condition"]][poa_tbl[[method]] == 1L & poa_tbl$poa_required == 0L]

for (cn in poa_required_conds) {
  stopifnot(all(rtn[[cn]] == 0L))
}

for (cn in poa_not_required_conds) {
  x <- subset(rtn, condition == cn)
  stopifnot(all(x[[cn]] == 1L))
}

#' Lookup ICD Codes
#'
#' Functions for working with ICD codes.
#'
#' ICD codes should be character vectors. These tools work with either
#' "full codes" (decimal point present when appropriate) or "compact codes"
#' (decimal point omitted).
#'
#' @template details-header-and-assignable-codes
#' @template details-fiscal-and-calendar-years
#'
#' @param x Character vector of ICD codes (full or compact form). When
#'   `regex = TRUE`, `x` must contain at least one non-empty, non-missing
#'   string.
#' @param regex Logical scalar. If `TRUE`, treat `x` as regular
#'   expressions; if `FALSE`, use exact matching.
#' @template params-icd-form
#'
#' @param ... Passed to [base::grep()] when `regex = TRUE`
#'
#' @seealso
#' * [`get_icd_codes()`] to retrieve the internal lookup table of ICD codes.
#' * [`is_icd()`] to test if a string is a known ICD code.
#' * [`icd_compact_to_full()`] converts a string from a compact format to the
#'   full format based on ICD version and type (diagnostic or procedure).
#'
#' @return A `data.frame` with one or more rows per input, including columns
#' * `match_type`: did the input match a full or compact code
#' * `icdv`: icd version (9 or 10)
#' * `dx`: diagnostic code (1) or procedure code (0)
#' * `full_code`: the full code string
#' * `code`: the compact codes string
#' * `src`: the source - CMS, CDC, or WHO.
#' * year ranges (`known_*`, `assignable_*`).
#'
#' @family ICD tools
#' @export
lookup_icd_codes <- function(x, regex = FALSE, full.codes = TRUE, compact.codes = TRUE, ...) {
  if (is.factor(x)) {
    x <- as.character(x)
  }
  stopifnot(inherits(x, "character"))
  assert_scalar_logical(regex)
  assert_scalar_logical(full.codes)
  assert_scalar_logical(compact.codes)
  stopifnot(isTRUE(full.codes | compact.codes))
  if (isTRUE(regex) && (length(x) == 0L || any(is.na(x) | nchar(x) == 0L))) {
    stop("When regex = TRUE, x must be non-empty strings.")
  }

  ICDCODES <- get("icd_codes", envir = ..mdcr_data_env.., inherits = FALSE)
  lookup_order <- seq_len(nrow(ICDCODES))

  if (regex) {
    if(full.codes) {
      fcmatches <- lapply(x, grep, x = ICDCODES[["full_code"]], value = TRUE, ...)
      fcmatches <- data.frame(
        input_seq   = rep(seq_along(x), times = sapply(fcmatches, length)),
        input_regex = rep(x, times = sapply(fcmatches, length)),
        input_code  = do.call(c, fcmatches),
        stringsAsFactors = FALSE
      )
    }
    if (compact.codes) {
      ccmatches <- lapply(x, grep, x = ICDCODES[["code"]], value = TRUE, ...)
      ccmatches <- data.frame(
        input_seq   = rep(seq_along(x), times = sapply(ccmatches, length)),
        input_regex = rep(x, times = sapply(ccmatches, length)),
        input_code  = do.call(c, ccmatches),
        stringsAsFactors = FALSE
      )
    }
    if (full.codes & compact.codes) {
      input <- rbind(fcmatches, ccmatches)
    } else if (full.codes & !compact.codes) {
      input <- fcmatches
    } else if (!full.codes & compact.codes) {
      input <- ccmatches
    }
  } else {
    input <- data.frame(
      input_seq = seq_along(x),
      input_code = x,
      stringsAsFactors = FALSE
    )
  }

  if (full.codes) {
    on_full_code <-
      merge(
        x = input,
        y = data.frame(ICDCODES,
                       matched_full_code = ICDCODES[["full_code"]],
                       lookup_order = lookup_order,
                       zzz = 1L,
                       stringsAsFactors = FALSE),
        by.x = "input_code",
        by.y = "full_code",
        sort = FALSE
      )
  }

  if (compact.codes) {
    on_compact_code <-
      merge(
        x = input,
        y = data.frame(ICDCODES,
                       matched_code = ICDCODES[["code"]],
                       lookup_order = lookup_order,
                       zzz = 1L,
                       stringsAsFactors = FALSE),
        by.x = "input_code",
        by.y = "code",
        sort = FALSE
      )
  }

  if (full.codes & compact.codes) {
    # Prefer full-code matches when both forms are requested so the
    # same record does not appear twice; drop compact-code rows whose
    # join key already matched on the full-code pass.
    keys <- c("input_seq", "input_code", "icdv", "dx", "src")
    ckey <- do.call(paste, c(on_compact_code[keys], sep = "\r"))
    fkey <- do.call(paste, c(on_full_code[keys], sep = "\r"))
    keep <- is.na(match(ckey, unique(fkey)))
    on_compact_code <- on_compact_code[keep, , drop = FALSE]
  }

  if (regex) {
    col_order <- c("input_regex")
  } else {
    col_order <- c("input_code")
  }
  col_order <- c(col_order, "input_seq", "match_type", "icdv", "dx", "full_code", "code", "src", "known_start", "known_end", "assignable_start", "assignable_end")
  sort_order <- c(col_order[1:2], "match_rank", "lookup_order")

  ofc <- exists("on_full_code") && nrow(on_full_code)
  occ <- exists("on_compact_code") && nrow(on_compact_code)

  if (ofc) {
    on_full_code[["match_type"]] <- "full_code"
    on_full_code[["match_rank"]] <- 1L
    names(on_full_code) <- sub("matched_", "", names(on_full_code))
    on_full_code <- mdcr_select(on_full_code, cols = c(col_order, "match_rank", "lookup_order"))
  }

  if (occ) {
    on_compact_code[["match_type"]] <- "compact_code"
    on_compact_code[["match_rank"]] <- 2L
    names(on_compact_code) <- sub("matched_", "", names(on_compact_code))
    on_compact_code <- mdcr_select(on_compact_code, cols = c(col_order, "match_rank", "lookup_order"))
  }

  if (ofc & occ) {
    matches <- rbind(on_full_code, on_compact_code)
  } else if (ofc & !occ) {
    matches <- on_full_code
  } else if (!ofc & occ) {
    matches <- on_compact_code
  } else {
    matches <- as.data.frame(
      matrix(NA_character_, nrow = 1L, ncol = length(col_order), dimnames = list("", col_order)),
      stringsAsFactors = FALSE
    )
  }

  rtn <- merge(x = input, y = matches, all.x = TRUE, by = col_order[1:2], sort = FALSE)

  if (regex) {
    rtn[["input_code"]] <- NULL
  }

  rtn <- mdcr_setorder(rtn, by = sort_order[sort_order %in% names(rtn)])
  rtn[["match_rank"]] <- NULL
  rtn[["lookup_order"]] <- NULL
  rtn[["input_seq"]] <- NULL

  mdcr_unique(rtn)
}

################################################################################
#                                 End of File                                  #
################################################################################

---
title: "Charlson Comorbidities"
output:
 rmarkdown::html_vignette:
   toc: true
   number_sections: false
bibliography: references.bib
vignette: >
 %\VignetteIndexEntry{Charlson Comorbidities}
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8}
---

```{r, label = "setup", include = FALSE}
# IMPORTANT SYNTAX NOTE:
#
# DO NOT USE the pipeOp `|>`
#
# While convenient, that is a R 4.1.0 feature at a minimum. Notable improvements
# to the pipeOp come in 4.2.0 and 4.2.1.  To keep this package dependent on R >=
# 3.5.0 do not use the pipeOp.

library(kableExtra)
options(qwraps2_markup = "markdown")
options(knitr.kable.NA = '')
knitr::opts_chunk$set(collapse = TRUE, fig.align = "center")
```

```{r, label = 'medicalcoder-namespace'}
library(medicalcoder)
packageVersion("medicalcoder")
```

# Introduction

The medicalcoder package implements several variants of the Charlson
comorbidities algorithm.

* `charlson_deyo1992`: Deyo's original set of codes [@deyo1992adapting;@quan2005]
* `charlson_quan2005` and `charlson_quan2011`: Codes and index scoring [@quan2005;@quan2011]
* `charlson_cdmf2019`: [@glasheen2019]

# ICD Codes and Index Scores

End users can access the lookup tables for the mapping between ICD-9 and ICD-10
codes to conditions and the index scoring in `data.frame`s from the
`get_charlson_codes()` and `get_charlson_index_scores()` calls respectively.

```{r}
str(get_charlson_codes())
str(get_charlson_index_scores())
```

# Applying Charlson

Example: applying the @quan2005 variant of the Charlson comorbidities to the
`mdcr` data can be done as follows.

```{r}
mdcr_results <-
  comorbidities(
    data = mdcr,
    id.vars = "patid",
    icdv.var = "icdv",
    icd.codes = "code",
    dx.var = "dx",
    flag.method = "current",
    poa = 1,
    primarydx = 0,
    method = "charlson_quan2005"
  )
```

The return object is a `data.frame` with 0/1 integer indicator columns for the
relevant conditions, the id.vars (if applicable), `age_score` (if age, in years,
is supplied), `num_cmrb` the number of comorbidities, `cmrb_flag` a 0/1
indicator for presence of at least one comorbidity, and `cci` the Charlson
Comorbidity Index.

```{r}
str(mdcr_results)
```

Call `summary()` on the return object for a list of summary objects.  These can be
used to generate output tables to the end user's liking.

```{r}
str(summary(mdcr_results))
```

```{r echo = FALSE, results = "asis"}
x <- summary(mdcr_results)$conditions[, c("condition_description", "count", "percent")]
tab <-
  kableExtra::kbl(
    x = x,
    format = "html",
    caption = "Counts and percentages of patients in the mdcr example data sets with the @quan2005 comorbidities.",
    col.names = c("", "Count", "Percentage"),
    digits = 3
  )
tab <- kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab <- kableExtra::pack_rows(tab, group_label = "Comorbidity", start_row = 1, end_row = 17)
tab <- kableExtra::pack_rows(tab, group_label = "Total Comorbidities", start_row = 18, end_row = nrow(x))
tab
```

# References

<!-- ----------------------------------------------------------------------- -->
<!--                              End of File                                -->
<!-- ----------------------------------------------------------------------- -->

---
title: "ICD Codes"
output:
 rmarkdown::html_vignette:
   toc: true
   toc_depth: 1
   number_sections: false
vignette: >
 %\VignetteIndexEntry{ICD Codes}
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8}
---

```{r label = "setup", include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, fig.align = "center")
```

There are four functions in the medicalcoder package specifically for working
with International Classification of Diseases (ICD) codes.

1. `get_icd_codes()`: returns a lookup table of ICD codes as a `data.frame`.
2. `lookup_icd_codes()`: returns details on specific ICD codes.
3. `is_icd()`: returns `TRUE` or `FALSE` for a vector of codes while considering
   ICD version, type, and billable status.
4. `icd_compact_to_full()`: insert a decimal point into a string to be
   consistent with ICD-9 diagnostic, ICD-9 procedure, or ICD-10 diagnostic
   codes.  (ICD-10 procedure codes do not have decimal places.)  NOTE: this will
   not validate the return as a valid ICD code, just format the input string
   appropriately.

# `get_icd_codes()`

A lookup table for the ICD codes has been built as internal data sets within the
medicalcoder package.  The sources for these lookup tables come from the Centers
for Disease Control (CDC) and from the Centers for Medicare & Medicaid Services
(CMS) and World Health Organization (WHO).  The specific links to the source
data sets can be found in the source code for the medicalcoder package on
[GitHub](https://github.com/dewittpe/medicalcoder).

End users can get a `data.frame` with
ICD-9 diagnostic, ICD-9 procedure, ICD-10 diagnostic, and ICD-10 procedure codes.

```{r label = "get-icd-codes"}
library(medicalcoder)
icd_codes <- get_icd_codes()
str(icd_codes)
```
The columns of this data.frame are:

* `icdv`: integer value 9 or 10 indicating ICD-9 or ICD-10
* `dx`: 1 if the code is a diagnostic code, i.e., from the ICD-9-CM or ICD-10-CM
  standard.  This also covers codes from the World Health Organization (WHO) and
  the Center for Disease Control and Prevention (CDC) Mortality codes.
  `dx` will be 0 if the code is a procedure code, i.e., from the ICD-9-PCS or
  ICD-10-PCS standard.
* `full_code`: the full ICD code with the decimal point if applicable.
* `code`: compact code, any applicable decimal point has been omitted.
* `src`: a character vector denoting the source of the information.
* `known_start`: The first year that the medicalcoder package has data for this
  code.
  * For codes based on the ICD-9-CM, ICD-9-PCS, ICD-10-CM, ICD-10-PCS standards
    the year is the _fiscal year_ for the United States Federal Government;
    October 1 - September 30.
    For example, fiscal year 2018 started October 1, 2017 and ended on September
    30, 2018.

  * For codes from the World Health Organization (WHO) and the Centers for
    Disease Control and Prevention (CDC) Mortality coding, the year is
    _calendar year_.
  * For ICD-9, CDC extracts in medicalcoder span fiscal years 1997--2012 and
    CMS extracts span fiscal years 2006--2015.

* `known_end`: the last year the code was part of the standard, or that the
  medicalcoder package has data for.
  * ICD-9 CMS extracts run through FY 2015; CDC ICD-9 extracts end at FY 2012.
  * ICD-10 is active.  The current version of medicalcoder has details on ICD-10
    codes through FY `r max(icd_codes[["known_end"]][icd_codes[["icdv"]] == 10L])`.

```{r, label = "example-for-assignable", include = FALSE}
d4 <- lookup_icd_codes(x = "^C84\\.6", regex = TRUE, compact.codes = FALSE)
d4 <- subset(d4, src %in% c("cms", "who"), select = c("src", "full_code"))
d4 <- unique(d4)
d4
```

* `assignable_start`: The first year (fiscal or calendar based on src) a code
  was assignable. `NA` indicates the code was never assignable.  Assignable
  status can vary from source to source. For example the code "C84.6" is an
  assignable code using the WHO ICD-10 codes because there is no code with more
  granularity.  The same code is not assignable under ICD-10-CM because codes
  `r paste(d4$full_code[nchar(d4$full_code) > 5], collapse = ", ")`
  exist.  Codes that are not assignable are called header codes.  Ideally codes
  are reported with the greatest level of granularity, but that is not always the
  case.

* `assignable_end`: The last year the code was assignable.

## with.descriptions

To get the descriptions of the ICD codes call `get_icd_codes()` with
`with.descriptions = TRUE`.
```{r, label = "icd-codes-with-descriptions"}
str(get_icd_codes(with.descriptions = TRUE))
```
The return has the additional columns:

* `desc`: the description of the code
* `desc_start`: the first fiscal_year that the description was documented
* `desc_end`: the last fiscal_year that the description was documented

Over time the descriptions for some ICD codes were modified within sources.
There are also many differences between sources.  The tables below have several
examples.

```{r, label = "deltas-in-desc", results = "hide"}
delta_in_desc <-
  subset(
    get_icd_codes(with.descriptions = TRUE),
    subset = full_code %in% c("Z88.7", "010.93", "V76.49"),
    select = c("full_code", "src", "desc", "desc_start", "desc_end")
  )
```

The only difference in the description for 010.93 is a comma.

```{r, label = "deltas-in-desc-show-010.93", echo = FALSE, results = "asis"}
knitr::kable(
  subset(delta_in_desc, subset = full_code == "010.93"),
  row.names = FALSE
)
```

ICD-10-CM Z88.7 has differences in the description over time within `cms` source
and between `cms` and `who`.

```{r, label = "deltas-in-desc-show-Z88.7", echo = FALSE, results = "asis"}
knitr::kable(
  subset(delta_in_desc, subset = full_code == "Z88.7"),
  row.names = FALSE
)
```

ICD-9-CM V79.49 had the description of 'other' which would require exploration
of the header codes to understand. Even the most verbose description may
still require consideration of the header codes to fully understand.

```{r, label = "deltas-in-desc-show-v79.49", echo = FALSE, results = "asis"}
knitr::kable(
  subset(delta_in_desc, subset = full_code == "V76.49"),
  row.names = FALSE
)
```

## with.hierarchy

Lastly, the `get_icd_codes()` function includes the argument `with.hierarchy`
which will provide additional details for the codes.

```{r label = "get-icd-descs-with-hierarchy"}
str(get_icd_codes(with.hierarchy = TRUE))
```
The additional columns, in order of hierarchy, are:

* chapter
* subchapter
* category
* subcategory
* subclassification
* subsubclassification
* extension

To keep the install size of medicalcoder under the size
limits for CRAN, the stored data is structured in a way that several joins and
other operations are needed to have a data set that is end user friendly.
Several data sets are generated and cached when the namespace is loaded.

# `lookup_icd_codes()`

A related function, `lookup_icd_codes()`, allows the user to look up specific ICD
codes. The return is a `data.frame`.  The columns report the input code, if it
was matched as a full code (with an applicable decimal point) or a compact code
(applicable decimal point omitted) along with the ICD version, type, and when
the code was assignable.

```{r, label = "lookup-icd-code-example"}
codes <- c("0011", "7329", "732", "73291", "not a code", "001.1", "A9248", "A924", "Z00")
knitr::kable(lookup_icd_codes(codes), row.names = FALSE)
```

It is possible to restrict the lookup to just full or compact codes.  The
default, as shown above, is to consider full and compact codes.  Set `full.codes
= FALSE` so only compact codes are considered.
```{r, label = "lookup-compact-icd-codes"}
knitr::kable(
  lookup_icd_codes(codes, full.codes = FALSE),
  row.names = FALSE
)
```
And set `compact.codes = FALSE` to only consider full codes.
```{r, label = "lookup-full-icd-codes"}
knitr::kable(
  lookup_icd_codes(codes, compact.codes = FALSE),
  row.names = FALSE
)
```

By default, `lookup_icd_codes()` considers the input to be a string and a direct
match to the internal lookup table is made.

`lookup_icd_codes()` can also accept regular expressions.  By providing a vector
of regular expression patterns for the codes (passed to `grep()`)

```{r, label = "lookup-icd-code-by-regex"}
knitr::kable(
  lookup_icd_codes(x = "^C84\\.6[0-1A-Z]", regex = TRUE),
  row.names = FALSE
)
```

# `is_icd()`

By convention, ICD codes are generally reported without decimal points. Under
this convention discriminating between ICD-9 and ICD-10, and between diagnostic
and procedure codes can be difficult.

Is "7993" a valid code?  It is not a valid ICD-10 code as a four digit code
could not be an ICD-10 procedure code, and all ICD-10 diagnostic codes start
with a letter, not a number.  So this string could only be a ICD-9 code.  It is
a valid ICD-9 diagnostic code, and a valid ICD-9 procedure code.

```{r label = "icd-7993"}
is_icd(x = "7993")
is_icd(x = "7993", icdv =  9, dx = 1)
is_icd(x = "7993", icdv =  9, dx = 0)
is_icd(x = "7993", icdv = 10, dx = 1)
is_icd(x = "7993", icdv = 10, dx = 0)
lookup_icd_codes("7993")
```

A vector of possible codes:
```{r}
x <- c("7993", "A924", "7993", "A924", "no", "A92", "516", "5163", "51631", "A00")
is_icd(x)
```

If you have codes with decimal points then discriminating between ICD-9
diagnostic and procedure codes can be done.

```{r}
x <- c("7993",  # valid dx and pr code
       ".7993", # not a valid code
       "7.993", # not a valid code
       "79.93", # invalid dx code; valid pr code
       "799.3", # valid dx code; invalid pr code
       "7993.") # not a valid code
data.frame(x = x,
           icd9_dx = is_icd(x, icdv = 9, dx = 1, warn.ambiguous = FALSE),
           icd9_pr = is_icd(x, icdv = 9, dx = 0, warn.ambiguous = FALSE))
```

### Assignable codes

Ideally, codes are reported with the greatest level of detail.  While there is
always a chance for incomplete coding, it is possible that an assignable code in
one year becomes a header code in a subsequent year.  Let's look at the
ICD-9 DX code 516.3 and five digit codes 516.30 through 516.39 (not all of these
are valid, as we'll see in the examples.)

Given the default settings, we have the following results for testing if these
strings are valid ICD-9 dx codes.

By default, if no year is provided in the `is_icd()` call then return will
be `TRUE` if the code was ever assignable.

```{r, results = "asis"}
x <- paste0("516.3", c("", 0:9))
tab <-
  data.frame(
    code       = x,
    default    = is_icd(x, icdv = 9, dx = 1),
    assignable_1997_cdc = is_icd(x, src = "cdc", icdv = 9, dx = 1, year = 1997),
    assignable_2010_cms = is_icd(x, src = "cms", icdv = 9, dx = 1, year = 2010),
    assignable_2011_cms = is_icd(x, src = "cms", icdv = 9, dx = 1, year = 2011),
    assignable_2012_cdc = is_icd(x, src = "cdc", icdv = 9, dx = 1, year = 2012),
    assignable_2012_cms = is_icd(x, src = "cms", icdv = 9, dx = 1, year = 2012),
    assignable_2015_cms = is_icd(x, src = "cms", icdv = 9, dx = 1, year = 2015),
    assignable_ever_cdc = is_icd(x, src = "cdc", icdv = 9, dx = 1, ever.assignable = TRUE)
  )
knitr::kable(tab)
```

Similar information can be quickly and easily retrieved via `lookup_icd_codes()`.
```{r, results = "asis"}
knitr::kable(lookup_icd_codes(x))
```

For fiscal years `r lookup_icd_codes("516.3")$assignable_start` through
`r lookup_icd_codes("516.3")$assignable_end` the code 516.3 was assignable.  In
`r lookup_icd_codes("516.30")$assignable_start` 516.3 was not assignable due to the
introduction of the five digit codes 516.30, 516.31, 516.32, 516.33, 516.34,
516.35, 516.36, and 516.37.  Codes 526.38 and 516.39 were never in the ICD-9-CM
standard.  When looking at retrospective data over several years the use of the
`ever.assignable` argument will simplify the testing for valid codes.

### Header codes

There is also an option to considering header codes to be valid.  As seen below,
the code "516" is a header, it was never assignable in ICD-9-CM.
By setting `headerok = TRUE` "516" will be flagged as a valid code.  An ICD-10
header "A00" will be FALSE in the following checks of ICD-9 codes.

```{r}
x <- c("516", "5163", "51631", "A00")
tab <-
  data.frame(
    code     = x,
    default  = is_icd(x, icdv = 9, dx = 1, src = "cms", headerok = FALSE, ever.assignable = FALSE, warn.ambiguous = FALSE),
    ever     = is_icd(x, icdv = 9, dx = 1, src = "cms", headerok = FALSE, ever.assignable = TRUE,  warn.ambiguous = FALSE),
    headerok = is_icd(x, icdv = 9, dx = 1, src = "cms", headerok = TRUE,                           warn.ambiguous = FALSE)
  )
knitr::kable(tab)
```

A more complex situation is ICD-9-CM code 719.7 and the five digit codes
719.70, 719.75, 719.76, 719.77, 719.78, and 719.79.  The five digit codes were
assignable codes through FY 2004.  Starting in FY 2004 the five digit codes were
removed from the standard and the four digit code became assignable.  This is a
rare example of a header code becoming assignable.
```{r label="7197"}
x <- paste0("719.7", c("", "0", 5:9))
tab <-
  data.frame(
    code            = x,
    default         = is_icd(x, src = "cms", icdv = 9, dx = 1),
    assignable_2002 = is_icd(x, src = "cms", icdv = 9, dx = 1, year = 2002),
    assignable_2003 = is_icd(x, src = "cms", icdv = 9, dx = 1, year = 2003),
    assignable_2004 = is_icd(x, src = "cms", icdv = 9, dx = 1, year = 2004),
    assignable_2005 = is_icd(x, src = "cms", icdv = 9, dx = 1, year = 2005),
    assignable_ever = is_icd(x, src = "cms", icdv = 9, dx = 1, ever.assignable = TRUE)
  )
knitr::kable(tab)
```

# `icd_compact_to_full()`

To go from a full code to a compact code is simple, omit any decimal point in
the string.

To go from a compact code to a full code requires knowing if the code is from
version 9 or 10, and if it is a diagnostic or a procedure code.
`icd_compact_to_full()` will format a string appropriately, within reason.  This
method only formats the strings and will not validate the return.

For example, the compact code "E1234" is in the format expected for a ICD-9
diagnostic code or ICD-10 diagnostic code.  It could not be a procedure code as
ICD-9 procedure codes are all numeric values and ICD-10 procedure codes are
seven characters long.  The actual code E1234 is not a valid ICD code.  We use
this string as an example.

```{r}
icd_compact_to_full("E1234", icdv =  9, dx = 1)
icd_compact_to_full("E1234", icdv = 10, dx = 1)

lookup_icd_codes(c("E1234", "E123.4", "E12.34"))[, c("input_code", "match_type")]
```

Notice that no change to the string is made when trying to convert to a full
procedure code.

```{r}
icd_compact_to_full("E1234", icdv =  9, dx = 0)
icd_compact_to_full("E1234", icdv = 10, dx = 0)
```

# General Notes on ICD Code Structure

All four sets of codes have a hierarchical structure.  The first level of the
hierarchy is the chapter which groups codes by disease category, body system,
and/or condition.  Following that are subchapters for all but the ICD-9
procedure codes.  After the subchapter, depending on the ICD variant, are the
category, subcategory, subclassification, subsubclassification, and extension.

## ICD-9 Diagnostic Codes
ICD-9 Diagnostic codes are organized by a hierarchy of five levels:

1. chapter,
2. subchapter,
3. category,
4. subcategory, and
5. subclassification.

ICD-9 diagnostic codes are three to five digits, not counting a decimal point,
numeric or alpha numeric strings.  The first three digits are the category with
numeric code 000 through 999 (leading zeros are part of the numeric code), or
V00-V99, or E000-E999.  When the category does not provide sufficient detail,
a fourth numeric digit, separated from the category by a decimal point, is used.
Lastly, when the subcategory is insufficient detail, then a fifth numeric digit
is used, save for the E categories.

## ICD-9 Procedure Codes
ICD-9 Procedure codes are organized by a hierarchy of four levels:

1. chapter,
2. category,
3. subcategory, and
4. subclassification.

The codes are numeric strings of four digits with a decimal point between the
second and third digits.  The first two digits are the category, the third digit
is the subcategory, and the fourth digit is the subclassification.

## ICD-10 Diagnostic Codes

ICD-10 diagnostic codes are up to seven alphanumeric codes with a hierarchy of

1. chapter,
2. subchapter,
3. category,
4. subcategory,
5. subclassification,
6. subsubclassification, and
7. extension.

The category describes the general type of disease of injury, with the
subcategory, subclassification and subsubclassification providing detail on the
cause, manifestation, location, severity, and type of disease or injury.
Finally, the extension specifies the type of encounter, i.e., initial or
subsequent encounter, or sequela for encounters related to prior disease or
injury.

## ICD-10 Procedure Codes

In general, ICD-10 procedure codes are seven digits.  In medicalcoder, the three
digit (chapter, subchapter, _category_) and the seven digit codes are in the
data base.

<!-- ----------------------------------------------------------------------- -->
<!--                              End of File                                -->
<!-- ----------------------------------------------------------------------- -->

---
title: "Elixhauser Comorbidities"
output:
 rmarkdown::html_vignette:
   toc: true
   number_sections: false
bibliography: references.bib
vignette: >
 %\VignetteIndexEntry{Elixhauser Comorbidities}
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8}
---

```{r, label = "setup", include = FALSE}
# IMPORTANT SYNTAX NOTE:
#
# DO NOT USE the pipeOp `|>`
#
# While convenient, that is a R 4.1.0 feature at a minimum. Notable improvements
# to the pipeOp come in 4.2.0 and 4.2.1.  To keep this package dependent on R >=
# 3.5.0 do not use the pipeOp.

library(kableExtra)
options(qwraps2_markup = "markdown")
options(knitr.kable.NA = '')
knitr::opts_chunk$set(collapse = TRUE, fig.align = "center")
```

```{r, label = 'medicalcoder-namespace'}
library(medicalcoder)
packageVersion("medicalcoder")
```

# Introduction

The medicalcoder package implements several variants of the Elixhauser
comorbidity algorithm.

* Based on codes provided by the Agency for Healthcare Research and Quality (AHRQ) for fiscal years 2022 through 2026 [@ahrqicd10]
  *  `elixhauser_ahrq2022`
  *  `elixhauser_ahrq2023`
  *  `elixhauser_ahrq2024`
  *  `elixhauser_ahrq2025`
  *  `elixhauser_ahrq2026`
  *  `elixhauser_ahrq_icd10` - All ICD-10 codes ever listed by AHRQ

* Codes from Table 2 of @quan2005
  * `elixhauser_elixhauser1988`: [@elixhauser1998;@quan2005]
  * `elixhauser_ahrq_web`: [@quan2005;@ahrq2017]
  * `elixhauser_quan2005`: [@quan2005]

**IMPORTANT NOTE:** Elixhauser 1998 and AHRQ Web used diagnosis-related
group (DRG) codes as part of the methods.  The medicalcoder
package _does not_ use DRG codes.  This is consistent with the way these
methods were implemented in @quan2005.

# ICD Codes and Index Scores

End users may access three lookup tables relevant to Elixhauser variants.

```{r}
str(get_elixhauser_codes())
str(get_elixhauser_index_scores())
str(get_elixhauser_poa())
```

* `get_elixhauser_codes()` returns a `data.frame` of the ICD codes mapping to
  each condition with indicators for the inclusion in specific variants.

* `get_elixhauser_index_scores()` returns a `data.frame` with points scored per
  condition for both mortality and readmission indices by variant.

* `get_elixhauser_poa()` returns a `data.frame` which reports which conditions
  are required and not required to be present-on-admission for the conditions to
  be classified as a comorbidity by variant.  For the variants not listed the
  default is as with the Charlson and PCCC methods, the code needs to be
  present-on-admission for the condition to be flagged.

# Applying Elixhauser

The Elixhauser method considers if the diagnostic codes are for primary or
secondary diagnoses. As such, the following call will throw a warning that the
`primarydx.var` or `primarydx` argument was not set and that all conditions will
be assumed to be secondary.  To suppress this warning pass an appropriate value
for either argument.

```{r}
# will warn because primarydx and primarydx.var are both NULL
mdcr_results0 <-
  comorbidities(
    data = mdcr,
    id.vars = "patid",
    icdv.var = "icdv",
    icd.codes = "code",
    dx.var = "dx",
    flag.method = "current",
    poa = 1,
    method = "elixhauser_ahrq2025"
  )

# no warning
mdcr_results <-
  comorbidities(
    data = mdcr,
    id.vars = "patid",
    icdv.var = "icdv",
    icd.codes = "code",
    dx.var = "dx",
    flag.method = "current",
    poa = 1,
    method = "elixhauser_ahrq2025",
    primarydx = 0
  )

identical(mdcr_results, mdcr_results0)
```

The return object is a `data.frame` with 0/1 integer indicator columns for the
relevant conditions, the id.vars (if applicable), `num_cmrb`, the number of
comorbidities, `cmrb_flag`, a 0/1 indicator for presence of at least one
comorbidity, and the mortality and readmission index scores.

```{r}
str(mdcr_results)
```

Call `summary()` to get a list of summaries which can be used to generate summary
tables.

```{r}
summary(mdcr_results)
```

```{r echo = FALSE, results = "asis"}
x <- summary(mdcr_results)$conditions
tab <-
  kableExtra::kbl(
    x = x,
    format = "html",
    caption = "Counts and percentages of patients in the mdcr example data sets with the Elixhauser @quan2005 comorbidities.",
    col.names = c("", "Count", "Percentage"),
    digits = 3
  )
tab <- kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab <- kableExtra::pack_rows(tab, group_label = "Comorbidity", start_row = 1, end_row = 38)
tab <- kableExtra::pack_rows(tab, group_label = "Total Comorbidities", start_row = 39, end_row = nrow(x))
tab
```


# References

<!-- ----------------------------------------------------------------------- -->
<!--                              End of File                                -->
<!-- ----------------------------------------------------------------------- -->

---
title: "Pediatric Complex Chronic Conditions"
output:
 rmarkdown::html_vignette:
   toc: true
   number_sections: false
bibliography: references.bib
vignette: >
 %\VignetteIndexEntry{Pediatric Complex Chronic Conditions}
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8}
---

```{r, label = "setup", include = FALSE}
# IMPORTANT SYNTAX NOTE:
#
# DO NOT USE the pipeOp `|>`
#
# While convenient, that is a R 4.1.0 feature at a minimum. Notable improvements
# to the pipeOp come in 4.2.0 and 4.2.1.  To keep this package dependent on R >=
# 3.5.0 do not use the pipeOp.

library(kableExtra)
options(qwraps2_markup = "markdown")
options(knitr.kable.NA = '')
knitr::opts_chunk$set(collapse = TRUE, fig.align = "center")
```

```{r, label = 'medicalcoder-namespace'}
library(medicalcoder)
```

# Introduction

The Pediatric Complex Chronic Condition (PCCC) coding schema version 2 was
published in 2014 [@feudtner2014pediatric] and updated to version 3 in 2024
[@feinstein2024pediatric]. Both versions identify 11 conditions, each with
multiple subconditions.

```{r, label = "tbl-syntactically-valid-conditions", echo = FALSE, results = "asis"}
CNDS <- subset(get_pccc_conditions(), select = c("condition", "condition_label"))
data.table::setDT(CNDS)
data.table::setkey(CNDS, condition)
CNDS <- unique(CNDS)

tab <-
  kbl(
    CNDS,
    caption = "Syntactically valid names for complex chronic conditions",
    row.names = TRUE
  )
tab <- kable_styling(tab, bootstrap_options = c("striped"), full_width = FALSE, font_size = 10)
tab
```

The PCCC system provides a standardized approach to identifying children with
complex chronic conditions using administrative data. This has several important
implications:

- **Research comparability**: Standard definitions allow findings to be compared
  across studies and institutions.
- **Policy and planning**: Accurate identification of populations with high
  medical needs supports resource allocation and health policy decisions.
- **Clinical insight**: Coding consistency helps clinicians and researchers
  understand disease burden and outcomes in pediatric populations.

Without a common framework such as the PCCC, studies of chronic pediatric
conditions would be fragmented, limiting their impact on both research and
practice.

# PCCC Version 2.0 vs PCCC Version 3.0

Versions 2 and 3 differ mainly in how technology dependence is treated. Many ICD
codes map to both a primary condition and either technology dependence or
transplant.

In both versions, transplant-related codes indicate organ system failure. A
patient with such a code is flagged as having both a transplant and the related
condition.

Technology dependence, however, diverges between versions. In version 2, the
presence of a technology dependence code classifies the patient as having both
the associated condition and technology dependence. For example, ICD-10 Z46.81
is both a metabolic and technology dependence code. A patient with this code is
classified as having a metabolic condition and technology dependence.

Version 3 refines this rule: technology dependence codes are assessed
conditionally, recognizing that many do not reflect chronic conditions.

Example: ICD-10 Z46.81
(`r subset(get_icd_codes(with.descriptions = TRUE), code == "Z4681")$desc`) is a
metabolic and technology dependence code.  If a patient had this code in their
medical records then they would be classified as having a metabolic condition
and a tech dependence.  Under version 2, this patient would be flagged as having
a metabolic condition and technology dependence.  Under version 3, the patient
would only be flagged with a metabolic condition and technology dependence if at
least one non-technology condition is flagged.

## ICD Codes for PCCC
Let's look at the codes that are in the PCCC schema.  Calling `get_pccc_codes()`
returns a data.frame.
```{r, label = "get-pccc-codes"}
pccc_codes <- get_pccc_codes()
str(pccc_codes)
```

The columns are:

* `icdv`: integer, ICD version
* `dx`: 1 for diagnostic (ICD-9-CM or ICD-10-CM) codes, 0 for procedure (ICD-9-PCS
  or ICD-10-PCS) codes.
* `full_code`: character, the ICD code retaining any applicable decimal point.
* `code`: character, the compact ICD code; any applicable decimal point omitted.
  Examples: ICD-9-CM full code 553.3 is represented as 5533 as a compact code.
  ICD-10-CM full code C96.9 is represented as compact code C969.
* `condition`: character, the pccc condition with syntactically valid names.
* `subcondition`: character, the pccc subcondition with syntactically valid
  names.
* `transplant_flag`: integer, 1L if the ICD code is associated with a
  transplant, 0L otherwise
* `tech_dep_flag`: integer, 1L if the ICD code is associated with technology
  dependence, 0L otherwise.
* `pccc_vX.Y`: integer, 1L if the code is part of variant `X.Y`, 0L otherwise

## Examples

Example: Consider a patient with the following four diagnostic and two procedure
ICD-9 codes:
```{r, label = "define-pat1"}
pat1 <-
  data.frame(
    dx = c(1, 1, 1, 1, 0, 0),
    icdv = 9L,
    code = c("34590", "78065", "3432", "78065", "9929", "8606")
  )
```

An inner join between the `pccc_codes` and `pat1` will yield the conditions
this patient has.

```{r, label = "inner-join-pat1-pccc-codes"}
merge(x = pccc_codes, y = pat1, all = FALSE, by = c("icdv", "dx", "code"))
```

For all PCCC variants, there is one matching dx code, 343.2, for
infantile cerebral palsy, which matches a neuromuscular condition.  The
procedure code 86.06 matches a technology-dependent metabolic condition.

Under version 2.0 of PCCC (variants `pccc_v2.0` and `pccc_v2.1`), this patient
has two conditions: neuromuscular and metabolic.  This patient also has a flag
for device and technology use.
```{r, label = "pat1-pccc-v2"}
pat1_pccc_v2.0 <-
  comorbidities(
    data = pat1,
    icd.codes = "code",
    dx.var = "dx",
    icdv = 9,
    method = "pccc_v2.0",
    flag.method = "current", # default
    poa = 1                  # default for flag.method = 'current'
  )

pat1_pccc_v2.1 <-
  comorbidities(
    data = pat1,
    icd.codes = "code",
    dx.var = "dx",
    icdv = 9,
    method = "pccc_v2.1",
    flag.method = "current",
    poa = 1
  )

all.equal(pat1_pccc_v2.0, pat1_pccc_v2.1, check.attributes = FALSE)
pat1_pccc_v2.1
```

Under version 3 of the PCCC, this patient has two conditions: neuromuscular and
metabolic.  The technology dependence flags are also 1 for this patient, but are
not counted in the total number of conditions.

```{r, label = "pat1-pccc-v3"}
pat1_pccc_v3.0 <-
  comorbidities(
    data = pat1,
    icd.codes = "code",
    dx.var = "dx",
    icdv = 9,
    method = "pccc_v3.0",
    flag.method = 'current',
    poa = 1
  )

pat1_pccc_v3.1 <-
  comorbidities(
    data = pat1,
    icd.codes = "code",
    dx.var = "dx",
    icdv = 9,
    method = "pccc_v3.1",
    flag.method = 'current',
    poa = 1
  )

all.equal(pat1_pccc_v3.0, pat1_pccc_v3.1, check.attributes = FALSE)

# retain the needed columns, there are four columns for each condition in v3
pat1_pccc_v3.0[, grep("^(cmrb_flag|num_cmrb|neuromus|metabolic|tech_dep_flag)", names(pat1_pccc_v3.0))]
```

In the output from version 3, we have four 0/1 indicator columns for each of the
conditions.

* `<condition>_dxpr_only`: the `<condition>` has been flagged due to diagnostic
  (dx) or procedure (pr) codes only.

* `<condition>_tech_only`: the `<condition>` has been flagged due to the
  presence of a technology dependence code _and_ at least one other condition
  has been flagged by dx or pr codes.

* `<condition>_dxpr_and_tech`: the `<condition>` has been flagged due to the
  presence of a dx or pr code that is not associated with a technology
  dependence _and_ another dx or pr code which is associated with technology
  dependence.

* `<condition>_dxpr_or_tech`: the `<condition>` has been flagged.  **These
  columns answer the question "does `<condition>` exist for this
  patient/encounter."**

The details in the list above might be easier to understand in a tabular form of
possible sets.  In the case of no conditions, only the
`<condition>_dxpr_or_tech` columns are flagged as 0/1 with the
`<condition>_dxpr_only`, `<condition>_tech_only`, and
`<condition>_dxpr_and_tech` columns set to `NA`.  When at least one condition
is flagged, all the columns will be populated as 0/1.

```{r, echo = FALSE, results = "asis"}
tab <- data.table::fread(colClass = character(), text = "
cmrb_flag | num_cmrb | <condition>_dxpr_or_tech | <condition>_dxpr_only | <condition>_tech_only | <condition>_dxpr_and_tech | <other condition(s)>_dxpr_or_tech
0         | 0        | 0                        | 0                     | 0                     | 0                         | 0
1         | 1        | 0                        | 0                     | 0                     | 0                         | 1
1         | 1        | 1                        | 1                     | 0                     | 0                         | 0
1         | 1        | 1                        | 1                     | 1                     | 1                         | 0
1         | >1       | 0                        | 0                     | 0                     | 0                         | 1
1         | >1       | 1                        | 0                     | 1                     | 0                         | 1
1         | >1       | 1                        | 1                     | 0                     | 0                         | 1
1         | >1       | 1                        | 1                     | 1                     | 1                         | 1                                  ")

tab <-
  kbl(
    x = tab,
    align = rep("c", ncol(tab)),
    col.names = c("cmrb flag", "num cmrb", "_dxpr_or_tech", "_dxpr_only", "_tech_only", "_dxpr_and_tech", "_dxpr_or_tech")
  )
tab <- kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab <- add_header_above(tab, c("", "", "<condition>" = 4, "<other condition(s)>" = 1))
tab
```

Now, consider another patient, pat2, with the same codes as pat1 except for
3432, the code mapping to a neuromuscular condition.
```{r, label = 'define-pat2'}
pat2 <- subset(pat1, code != "3432")
```
Under version 2 of the PCCC this patient will still have metabolic and
technology dependence conditions because of the code
86.06 is in the record, but will not have the neuromuscular condition.
```{r, label = "pat2-pccc-v2"}
pat2_pccc_v2.1 <-
  comorbidities(
    data = pat2,
    icd.codes = "code",
    dx.var = "dx",
    icdv = 9,
    method = "pccc_v2.1",
    flag.method = 'current',
    poa = 1
  )
Filter(f = function(x) x > 0, pat2_pccc_v2.1)
```

Under version 3 of the PCCC, this patient will have no conditions. This is
because no condition was identified based on non-technology dependent codes and
thus the one technology dependent code is ignored.

```{r, label = "pat2-pccc-v3"}
pat2_pccc_v3.1 <-
  comorbidities(
    data = pat2,
    icd.codes = "code",
    dx.var = "dx",
    icdv = 9,
    method = "pccc_v3.1",
    flag.method = 'current',
    poa = 1
  )
Filter(f = function(x) x > 0, pat2_pccc_v3.1)
```

# Expected Data Structures

The expected input data format for `comorbidities()` is a "long" format.  The only
mandatory column is one column of ICD codes.  These codes can be full codes
(include the decimal point) or compact codes (omitting the decimal point).
Additionally, column(s) for identifying patients, encounters, and any other
important groups are encouraged.  A column to indicate the ICD version
(9 or 10), and another column for identifying the code as a diagnostic or
procedure code are also encouraged.  The example `mdcr` data set has three
columns, a patient id (patid), the ICD compact codes (code), and a column to
indicate if the ICD code is a diagnostic or procedure code, (dx: 1 for
diagnostic, 0 for procedure).

The `mdcr` data is provided with columns for

* patient id,
* ICD version,
* compact code, and
* diagnostic/procedure indicator.

```{r, label = "mdcr-data"}
head(mdcr)
str(mdcr)
```

# Assessing PCCC

## Ignoring ICD Version and diagnostic/procedure codes

Applying `pccc_v2.1` and `pccc_v3.1` methods to `mdcr` without accounting for
ICD version or the diagnostic/procedure status of the code can be done as follows:

```{r, label = "mdcr-results-01"}
mdcr_results_v2.1_01 <-
  comorbidities(
    data = mdcr,
    icd.codes = "code",
    id.vars = "patid",
    poa = 1,
    flag.method = 'current',
    method = "pccc_v2.1"
  )

mdcr_results_v3.1_01 <-
  comorbidities(
    data = mdcr,
    icd.codes = "code",
    id.vars = "patid",
    poa = 1,
    flag.method = 'current',
    method = "pccc_v3.1"
  )
```

A useful summary of the object returned from `comorbidities()` is gained by
calling `summary()`. The return is a `data.frame` with columns for the count
and percentages.  For `pccc_v2.0` and `pccc_v2.1` the condition, label, count,
and percentage, are reported.  For `pccc_v3.0` and `pccc_v3.1` the columns are
extended to provide the counts and percentages for `_dxpr_or_tech`, `_dxpr_only`,
`_tech_only`, and `_dxpr_and_tech`.

```{r, label = "comorbidities-summary-table-str"}
str(summary(mdcr_results_v2.1_01))
str(summary(mdcr_results_v3.1_01))
```

Since the summary tables are `data.frame`s the end user may manipulate the
results for reporting as they want, see the following table.

```{r, label = "mdcr-results-01-summary"}
x <-
  merge(
    summary(mdcr_results_v2.1_01),
    summary(mdcr_results_v3.1_01),
    all = TRUE,
    by = c("condition", "label"),
    sort = FALSE
  )
x[["condition"]] <- NULL
```

```{r, label = "mdcr-results-01-summary-kable", echo = FALSE, results = "asis"}
tab <-
  kableExtra::kbl(
    x,
    digits = 1,
    col.names = c("", rep(c("count", "%"), times = 5)),
    caption = "Summary Table for `mdcr_results_v2.1_01` and `mdcr_results_v3.1_01`."
  )
tab <-
  kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab <-
  kableExtra::pack_rows(tab, group_label = c("Conditions"), start_row = 1, end_row = 11)
tab <-
  kableExtra::pack_rows(tab, group_label = c("Flags"), start_row = 12, end_row = 13)
tab <-
  kableExtra::pack_rows(tab, group_label = c("Total Conditions"), start_row = 14, end_row = 24)
tab <-
  kableExtra::add_header_above(tab, c("", "v2.1" = 2, "dxpr or tech" = 2, "dxpr only" = 2, "tech only" = 2, "dxpr and tech" = 2))
tab <-
  kableExtra::add_header_above(tab, c("", "", "", "v3.1" = 8))
tab
```

## Accounting for Diagnostic/Procedure

There are additional details we should consider with respect to the ICD codes.
The ICD version, 9 or 10, and if the code is a diagnostic or a procedure code.
For example, code ICD-9 diagnostic code 332.1 has the same compact code as ICD-9
procedure code 33.21, 3321.  In the case of the `mdcr` data where we have only
compact codes, the need to distinguish between ICD-9 diagnostic and ICD-9
procedure is critically important.  In the `mdcr` data the code 3321 does appear
as both diagnostic and procedure.

```{r, label = "note-dx-pr-in-mdcr"}
pccc_codes[pccc_codes$code == "3321", ]
table(mdcr[mdcr$code == "3321", "dx"])
```

To account for the diagnostic or procedure status of the codes specify a value
for the `dx.var` argument.

```{r, label = "mdcr-results-02"}
mdcr_results_v2.1_02 <-
  comorbidities(
    data = mdcr,
    id.vars = "patid",
    icd.codes = "code",
    dx.var = "dx",
    flag.method = 'current',
    poa = 1,
    method = "pccc_v2.1"
  )

mdcr_results_v3.1_02 <-
  comorbidities(
    data = mdcr,
    id.vars = "patid",
    icd.codes = "code",
    dx.var = "dx",
    flag.method = 'current',
    poa = 1,
    method = "pccc_v3.1"
  )
```

Specificity is increased by using the diagnostic/procedure flag.
Using `pccc_v2.1` there are `r sum(mdcr_results_v2.1_02$cmrb_flag != mdcr_results_v2.1_01$cmrb_flag)`
false positive flags when the diagnostic/procedure flag is omitted from the
`comorbidities()` call.
Using `pccc_v3.1` there are `r sum(mdcr_results_v3.1_02$cmrb_flag != mdcr_results_v3.1_01$cmrb_flag)`
false positive flags when the diagnostic/procedure flag is omitted from the
`comorbidities()` call.

```{r}
# verify that the cmrb_flag and number of conditions is the same or less after
# accounting for the diagnostic/procedure flag in the comorbidities call
stopifnot(all(mdcr_results_v2.1_02$cmrb_flag <= mdcr_results_v2.1_01$cmrb_flag))
stopifnot(all(mdcr_results_v2.1_02$num_cmrb  <= mdcr_results_v2.1_01$num_cmrb))

sum(mdcr_results_v2.1_02$cmrb_flag != mdcr_results_v2.1_01$cmrb_flag)
sum(mdcr_results_v2.1_02$num_cmrb  != mdcr_results_v2.1_01$num_cmrb)

stopifnot(all(mdcr_results_v3.1_02$cmrb_flag <= mdcr_results_v3.1_01$cmrb_flag))
stopifnot(all(mdcr_results_v3.1_02$num_cmrb  <= mdcr_results_v3.1_01$num_cmrb))

sum(mdcr_results_v3.1_02$cmrb_flag != mdcr_results_v3.1_01$cmrb_flag)
sum(mdcr_results_v3.1_02$num_cmrb  != mdcr_results_v3.1_01$num_cmrb)
```

Let's explore the record for patient 87420.

```{r, include = FALSE}
# this chunk is not included, just test that the results for patient 87420 has
# not changed.
#
# NOTE: vignettes are built in one R session.  As a result, in a prior version
# of this package, mdcr was modified in the icd.Rmd script. Those changes
# persisted into this vignette and resulted in an error.  The fix was to use an
# object in the icd.Rmd vignette called mdcr_copy.
subset(mdcr, patid %in% mdcr[mdcr$code == "5641" & mdcr$dx == 1, "patid"])
pat87420 <- subset(mdcr, patid == 87420)
stopifnot(
  isTRUE(
    all.equal(
      pat87420,
      structure(list(patid = c(87420L, 87420L), icdv = c(9L, 9L), code = c("78321", "5641"), dx = c(1L, 1L)), row.names = 4073:4074, class = "data.frame"),
      check.attributes = FALSE
    )
  )
)
```

```{r}
subset(mdcr, patid == "87420")
subset(get_pccc_codes(), code %in% c("78321", "5641"))
```

```{r}
subset(mdcr_results_v2.1_01, patid == "87420", select = c("cmrb_flag", "renal"))
subset(mdcr_results_v2.1_02, patid == "87420", select = c("cmrb_flag", "renal"))

subset(mdcr_results_v3.1_01, patid == "87420", select = c("cmrb_flag", "renal_dxpr_or_tech"))
subset(mdcr_results_v3.1_02, patid == "87420", select = c("cmrb_flag", "renal_dxpr_or_tech"))

subset(get_icd_codes(with.descriptions = TRUE), full_code %in% c("56.41", "564.1"))
```

In the above, the compact code "5641" matches procedure code 56.41 for a renal
condition.  In `mdcr_results_v2.1_01` and `mdcr_results_v3.1_01` where no
distinction was made between diagnostic and procedure codes this patient was
flagged as having a renal condition. However, when reviewing the patient
record, the compact code "5641" is listed as a diagnostic criteria and the full
code 564.1 is for
`r subset(get_icd_codes(with.descriptions = TRUE), full_code == "564.1" & desc_end == 2015)$desc`.
This is an example of where discriminating between diagnostic and procedure
codes is critically important when looking for complex chronic conditions.

If we explicitly look at an inner join between this patient's data and the pccc
lookup table we see that the code 5641 matches the procedure code in the pccc
lookup table.  By not accounting for diagnostic and procedure codes, the
overlaps between the two coding structures can lead to false positives.

```{r}
merge(
  x = subset(mdcr, patid == "87420"),
  y = pccc_codes,
  by.x = c("code"),
  by.y = c("code"),
  suffixes = c(".mdcr", ".pccc_codes")
)
```

## Full and Compact ICD codes

Using full codes can prevent false positives too.  Here are several different
ways that `comorbidities()` could be called resulting in different outcomes.

Note: this is a good example of how medicalcoder can handle full and compact
codes within a single record.

```{r}
DF <-
  data.frame(
    id = c("full dx", "full pr", "compact dx", "compact pr"),
    code = c("564.1", "56.41", "5641", "5641"),
    dx = c(1, 0, 1, 0)
  )

# ideal: using the dx/pr status and matching on full and compact codes.
comorbidities(
  data = DF,
  id.vars = "id",
  dx.var = "dx",
  icd.codes = "code",
  poa = 1,
  method = "pccc_v3.1"
)[, c("id", "cmrb_flag", "renal_dxpr_or_tech")]

# false positive for the compact dx
comorbidities(
  data = DF,
  id.vars = "id",
  icd.codes = "code",
  poa = 1,
  method = "pccc_v3.1"
)[, c("id", "cmrb_flag", "renal_dxpr_or_tech")]

# false negative for compact pr
comorbidities(
  data = DF,
  id.vars = "id",
  icd.codes = "code",
  poa = 1,
  full.code = TRUE,
  compact.codes = FALSE,
  method = "pccc_v3.1"
)[, c("id", "cmrb_flag", "renal_dxpr_or_tech")]

# false positive for compact dx
comorbidities(
  data = DF,
  id.vars = "id",
  icd.codes = "code",
  poa = 1,
  full.code = FALSE,
  compact.codes = TRUE,
  method = "pccc_v3.1"
)[, c("id", "cmrb_flag", "renal_dxpr_or_tech")]

# false negatives for compact and full pr
comorbidities(
  data = DF,
  id.vars = "id",
  icd.codes = "code",
  dx.var = "dx",
  poa = 1,
  full.code = FALSE,
  compact.codes = TRUE,
  method = "pccc_v3.1"
)[, c("id", "cmrb_flag", "renal_dxpr_or_tech")]
```

## Accounting for ICD Version

Another consideration is the version of ICD, 9 or 10.

The record for patid 95471 is a great example of the problem that a compact code
can cause.  "E030" matches ICD-9 dx compact and full code E030 (no decimal
point), and matches the ICD-10 dx compact code for full code E03.0 with only the
ICD-10 version being related to a chronic complex condition.

Inputs to the `comorbidities()` call for the ICD version will impact the output.
When calling `comorbidities()` with a variable to indicate the ICD version `NA`
values will not be joined against and the codes are ignored resulting in no
condition being flagged.  If we know that we only want to compare against ICD-9
or ICD-10 values then using the `icdv` argument can simplify the call and in
this case, no condition for ICD-9 and a condition is flagged for ICD-10.

```{r, label = "patid95471"}
subset(mdcr, patid == "95471")

# no flag becuse icdv = 9 which treats all input codes as ICD-9
comorbidities(
  data = subset(mdcr, patid == "95471"),
  icd.codes = "code",
  id.vars = 'patid',
  dx.var = "dx",
  icdv = 9L,
  poa = 1,
  method = "pccc_v3.1"
)[, c('patid', 'cmrb_flag')]

# flag because icdv = 10 - same as using `icdv.var = "icdv"`
comorbidities(
  data = subset(mdcr, patid == "95471"),
  icd.codes = "code",
  id.vars = 'patid',
  dx.var = "dx",
  icdv = 10L,
  poa = 1,
  method = "pccc_v3.1"
)[, c('patid', 'cmrb_flag')]

comorbidities(
  data = subset(mdcr, patid == "95471"),
  icd.codes = "code",
  id.vars = 'patid',
  dx.var = "dx",
  icdv.var = "icdv",
  poa = 1,
  method = "pccc_v3.0"
)[, c('patid', 'cmrb_flag')]
```

Lastly, it should be noted that a lot of the ambiguity resulting from compact
codes can be avoided when full codes are available.  medicalcoder can handle
both forms. In the example below we again use the "E030" and assess it against
all full and compact codes (default), against only full codes, and lastly
against only compact codes.  Note in this example that we are not specifying the
ICD version nor the diagnostic/procedure status of the code.

```{r}
lookup_icd_codes("E030")
data <- data.frame(id = c("Ambiguous compact code", "Full ICD-9 code", "Full ICD-10 code"),
                   code  = c("E030", "E030", "E03.0"))
data

args <-
  list(
    data = data,
    id.vars = "id",
    icd.codes = "code",
    poa = 1,
    method = "pccc_v3.1"
  )

default <-
  do.call(comorbidities, c(args, list(full.codes = TRUE,  compact.codes = TRUE )))
full_only <-
  do.call(comorbidities, c(args, list(full.codes = TRUE,  compact.codes = FALSE)))
compact_only <-
  do.call(comorbidities, c(args, list(full.codes = FALSE, compact.codes = TRUE )))

default[,       c("id", "cmrb_flag")]
full_only[,     c("id", "cmrb_flag")]
compact_only[,  c("id", "cmrb_flag")]
```

With no information about the "E030" being ICD-9 or ICD-10, full or compact,
(can only be a diagnostic code in either ICD-9 or ICD-10) we get different
flags.  The default, the most liberal approach flags this example patient as
having a condition in all cases.  When only considering the code to be a full code,
then only the ICD-10 version matches.  When only considering the compact codes
the flag is true for the ambiguous version and the ICD-9 full version since
ICD-9 E030 is a full code with the same compact form.


# Longitudinal Conditions

The medicalcoder package includes the example data set, `mdcr_longitudinal`,
with ICD-9 and ICD-10 codes for `r length(unique(mdcr_longitudinal$patid))`
synthetic patients with multiple encounters. Each row has a date (encounter) for
when the ICD code was reported.

```{r}
head(mdcr_longitudinal)
```

## PCCC V2

Let's look at the `pccc_v2.1` flags for each patient, using all the information from
all the encounters.  This can easily be done by specifying `id.vars =
c("patid")` such that the `comorbidities()` method considers all codes as
occurring on one encounter.

```{r, results = 'asis'}
longitudinal_v2_patid <-
  comorbidities(
    data = mdcr_longitudinal,
    icd.codes = "code",
    id.vars = c("patid"),
    icdv.var = "icdv",
    method = "pccc_v2.1",
    flag.method = "current",
    poa = 1
  )
tab <- kableExtra::kbl(longitudinal_v2_patid)
tab <- kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab
```

We can look at the conditions flagged at each encounter by specifying the
`id.vars = c("patid", "date")`.

```{r}
longitudinal_v2_patid_date <-
  comorbidities(data = mdcr_longitudinal,
    icd.codes = "code",
    id.vars = c("patid", "date"),
    icdv.var = "icdv",
    method = "pccc_v2.1",
    flag.method = "current",
    poa = 1
  )
```

```{r, echo = FALSE, results = 'asis'}
tab <-
  kableExtra::kbl(
    subset(longitudinal_v2_patid_date, patid == "9663901"),
    row.names = FALSE
  )
tab <- kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab
```

Looking at patid 9663901 at an encounter level we see that the conditions occur
at different moments in time and the condition the patient has change overtime.
Because these are chronic conditions, once the condition is observed, it should
be considered to exist in perpetuity.

For `pccc_v2.1` a simple carry-forward method can be applied to the data set
to mark the presence of a condition at the time of reporting and thereafter.


```{r}
longitudinal_v2_patid_date_cumulative_poa0 <-
  comorbidities(
    data = mdcr_longitudinal,
    icd.codes = "code",
    id.vars = c("patid", "date"),
    icdv.var = "icdv",
    method = "pccc_v2.1",
    flag.method = "cumulative",
    poa = 0
  )
```

```{r, echo = FALSE, results = 'asis'}
tab <-
  kableExtra::kbl(
    subset(longitudinal_v2_patid_date_cumulative_poa0, patid == "9663901"),
    row.names = FALSE
  )
tab <- kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab
```

```{r}
longitudinal_v2_patid_date_cumulative_poa1 <-
  comorbidities(
    data = mdcr_longitudinal,
    icd.codes = "code",
    id.vars = c("patid", "date"),
    icdv.var = "icdv",
    method = "pccc_v2.1",
    flag.method = "cumulative",
    poa = 1
  )
```

```{r, echo = FALSE, results = 'asis'}
tab <-
  kableExtra::kbl(
    subset(longitudinal_v2_patid_date_cumulative_poa1, patid == "9663901"),
    row.names = FALSE
  )
tab <- kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab
```

## PCCC V3

For `pccc_v3.0` and `pccc_v3.1` a simple carry-forward method would not be easy
to use as information about technology dependent codes is omitted when
non-technology dependent codes do not exist.

Let's use three ICD-10 diagnostic codes for this example and we will explore
all six possible permutations of the codes.  We'll generate a data set with
seven encounters and one code appearing on each of encounters 2, 4, and 6.

The codes we'll use are:

* H49.811: metabolic (other metabolic disorders),
* J84.111: respiratory (chronic respiratory diseases), and
* Z96.41:  metabolic (device and technology use).

```{r}
codes <- c("H49.811", "J84.111", "Z96.41")
subset(get_pccc_codes(), full_code %in% codes)
```

The constructed data and permutations are:

```{r}
permutations <-
  data.table::data.table(
    permutation = rep(1:6, each = 7),
    encounter_id = rep(1:7, times = 6),
    code =
      codes[c(NA, 1, NA, 2, NA, 3, NA,
              NA, 1, NA, 3, NA, 2, NA,
              NA, 2, NA, 1, NA, 3, NA,
              NA, 2, NA, 3, NA, 1, NA,
              NA, 3, NA, 1, NA, 2, NA,
              NA, 3, NA, 2, NA, 1, NA)]
  )

permutations[, plabel := paste(na.omit(code), collapse = ", "), by = .(permutation)]
permutations[, plabel := paste0("Permutation ", permutation, ": ", plabel)]
str(permutations, vec.len = 1)
```

```{r, echo = FALSE, results = "asis"}
cat(paste("*", permutations[, unique(plabel)]), sep = "\n")
```

We'll apply the `pccc_v3.1` to this code set with `flag.method = "cumulative"`
and all codes considered to be present-on-admission.

```{r}
rtn <-
  comorbidities(
    data = permutations,
    icd.codes = "code",
    id.vars = c("permutation", "plabel", "encounter_id"),
    icdv = 10L,
    compact.codes = FALSE,
    method = "pccc_v3.1",
    flag.method = "cumulative",
    poa = 1
  )
```

```{r, label = "setup-rtn-for-discussion", include = FALSE}
rtn_wide <-
  data.table::dcast(
    encounter_id ~ plabel,
    data = rtn,
    value.var = c("metabolic_dxpr_or_tech", "metabolic_dxpr_only",
                  "metabolic_tech_only", "metabolic_dxpr_and_tech",
                  "respiratory_dxpr_or_tech", "respiratory_dxpr_only",
                  "respiratory_tech_only", "respiratory_dxpr_and_tech",
                  "cmrb_flag", "num_cmrb")
  )
```

```{r, label = "define-pkbl", include = FALSE}
pkbl <- function(permutation = 1) {
  stopifnot(length(permutation) == 1L)

  x <- rtn_wide[
        ,
        .SD,
        .SDcols = c("encounter_id",
                    grep(paste0("Permutation ", permutation), names(rtn_wide), value = TRUE))
       ]

  pl <- rtn[["plabel"]][rtn$permutation == permutation][1]

  tab <-
    kableExtra::kbl(
      x,
      col.names = c("encounter_id", rep(c("dxpr or tech", "dxpr only", "tech only", "dxpr and tech"), times = 2), "ccc flag", "num ccc")
    )
  tab <- kableExtra::kable_styling(kable_input = tab, bootstrap_options = c("striped"), font_size = 10)
  tab <- kableExtra::add_header_above(kable_input = tab, header = c("", c("Metabolic" = 4, "Respiratory" = 4), "", ""))
  tab <- kableExtra::add_header_above(kable_input = tab, header = c("", setNames(10, pl)))
  tab
}
```

Let's walk through the results for each permutation.

**Permutation 1**

```{r, echo = FALSE, results = "asis"}
pkbl(1)
```

The first code to appear in this permutation is H49.811, metabolic (other).
This is a diagnostic code and will flag the metabolic condition for encounters 2
through 7 as `_dxpr_or_tech`.  The Z96.41 code, metabolic (tech), appears on
encounter 6.  Thus, for encounters 2 through 5 metabolic should be flagged as
`_dxpr_or_tech = 1`, `dxpr_only = 1`, `tech_only = 0`, and `dxpr_and_tech = 0`.
Encounters 6 and 7 then have `dxpr_only = 0` and `tech_only = 0` with
`dxpr_and_tech = 1`.
The J84.111 for respiratory is a non-tech code appearing on encounter 4 and
should flag as `dxpr_or_tech = 1`, `dxpr_only = 1`, `tech_only = 0`, and
`dxpr_and_tech = 0` for encounters 4 through 7.

**Permutation 2**
```{r, echo = FALSE, results = "asis"}
pkbl(2)
```

As with permutation 1, having the non-tech dependent metabolic code H49.811
appearing on encounter 2 means that metabolic is flagged for encounters 2
through 7.  What should differ is that `dxpr_only` is 1 for encounters 2 and 3,
with `dxpr_and_tech` flagging for encounters 4 through 7.  Lastly, the non-tech
code J84.111 for respiratory condition flagging as `dxpr_or_tech = dxpr_only = 1`
for encounters 6 and 7.

**Permutation 3**
```{r, echo = FALSE, results = "asis"}
pkbl(3)
```
Permutation three has respiratory flagged for encounters 2 through 7.  The
non-tech metabolic code on encounter 4 results in the flagging of metabolic for
encounters 4 through 7.

**Permutation 4**
```{r, echo = FALSE, results = "asis"}
pkbl(4)
```

Permutation 4 is notable as presence of the respiratory condition on encounters 2
through 7 means that when the technology dependent metabolic code appears on
encounter 4, a metabolic is flagged for encounters 4 through 7.  Compare this
with permutations 5 and 6.

**Permutation 5**
```{r, echo = FALSE, results = "asis"}
pkbl(5)
```

For permutation 5 the first code is a tech dependent metabolic code on encounter 2.
Because the _only_ code for flagging a condition is a technology dependent code
the PCCC version 3 algorithm results in _no_ condition being flagged for
encounters 2 and 3. On encounter 4, when the non-tech metabolic code appears
then the metabolic condition is flagged and the past history of the technology
dependent code persists.

**Permutation 6**
```{r, echo = FALSE, results = "asis"}
pkbl(6)
```

As with permutation 5, since the only code in the record for encounter 2 and 3
is the technology dependent metabolic code, there is no flagged condition. On
encounter 4, when the dxpr code for a respiratory condition is reported then the
respiratory condition _and_ the metabolic condition is flagged as technology
dependent. Note that technology only conditions are flagged if at least one
other condition is flagged.

# Subconditions

The documentation for PCCC version 2 [@feudtner2014pediatric] and version 3
[@feinstein2024pediatric] include subconditions under each of the major
conditions.  However, to our knowledge, no software prior to medicalcoder
implemented flagging for the subconditions.

The subconditions for each condition are shown in the next table.

```{r "tbl-syntactically-valid-subconditions", echo = FALSE, results = "asis"}
SCNDS <- get_pccc_conditions()
data.table::setDT(SCNDS)
data.table::setkey(SCNDS, condition, subcondition)
SCNDS[, condition := paste(condition, condition_label, sep = ": ")]

tab <-
  kableExtra::kbl(SCNDS[, .(subcondition, subcondition_label)],
      capttion = "Syntactically valid names for subconditions",
      row.names = FALSE
  )
tab <-
  kableExtra::pack_rows(tab, index = table(SCNDS$condition))
tab <-
  kableExtra::kable_styling(tab, bootstrap_options = c('striped'), fixed_thead = TRUE, font_size = 10)
tab <-
  kableExtra::scroll_box(tab, height = "300px")
tab
```

To get the subconditions all you need to do is use the `subconditions = TRUE`
argument in the `comorbidities()` call.  For this example we will apply
`pccc_v3.1` with and without comorbidities.

```{r}
without_subconditions <-
  comorbidities(
    data = mdcr,
    id.vars = "patid",
    icd.codes = "code",
    icdv.var = "icdv",
    dx.var = "dx",
    poa = 1,
    method = "pccc_v3.1",
    subconditions = FALSE
  )

with_subconditions <-
  comorbidities(
    data = mdcr,
    id.vars = "patid",
    icd.codes = "code",
    icdv.var = "icdv",
    dx.var = "dx",
    poa = 1,
    method = "pccc_v3.1",
    subconditions = TRUE
  )
```

The structure of the return object `with_subconditions` is a list with two
elements.  The first element, `conditions`, is identical to the results
of calling `comorbidities()` with `subconditions = FALSE`.

```{r}
with_subconditions

all.equal(
  with_subconditions$conditions,
  without_subconditions,
  check.attributes = FALSE
)
```

The second element of `with_subconditions` is list of `data.frame`s, one for
each condition, with indicators for only those with the condition.

A quick and easy way to get a summary of the subconditions is to call
`summary()`.

```{r}
str(summary(with_subconditions))
```

The subconditions are available for all pccc variants.  A summary is presented
in the following tables.

```{r include = FALSE}
args <-
  list(
    data = mdcr,
    id.vars = "patid",
    icd.codes = "code",
    icdv.var = "icdv",
    dx.var = "dx",
    poa = 1,
    subconditions = TRUE
  )
with_subconditions_v2.0 <- do.call(comorbidities, c(args, list(method = "pccc_v2.0")))
with_subconditions_v2.1 <- do.call(comorbidities, c(args, list(method = "pccc_v2.1")))
with_subconditions_v3.0 <- do.call(comorbidities, c(args, list(method = "pccc_v3.0")))
with_subconditions_v3.1 <- do.call(comorbidities, c(args, list(method = "pccc_v3.1")))
```

```{r include = FALSE}
rslts <-
  merge(
    merge(
      summary(with_subconditions_v2.0),
      summary(with_subconditions_v2.1),
      by = c("condition", "subcondition"),
      suffixes = c("_v2.0", "_v2.1"),
      sort = FALSE
    ),
    merge(
      summary(with_subconditions_v3.0),
      summary(with_subconditions_v3.1),
      by = c("condition", "subcondition"),
      suffixes = c("_v3.0", "_v3.1"),
      sort = FALSE
    ),
    by = c("condition", "subcondition"),
    sort = FALSE
  )

rslts$idx <- 1:nrow(rslts)

rslts <-
  merge(rslts,
        unique(get_pccc_conditions()[c("condition", "condition_label")]),
        all = TRUE,
        by = "condition",
        sort = FALSE)
rslts <-
  merge(rslts,
        unique(get_pccc_conditions()[c("subcondition", "subcondition_label")]),
        all = TRUE,
        by = "subcondition",
        sort = FALSE)
rslts$lab <- rslts$subcondition_label
rslts$lab[is.na(rslts$subcondition)] <- rslts$condition_label[is.na(rslts$subcondition)]
rslts <- rslts[order(rslts$idx), ]

rslts <- split(rslts, f = rslts[["condition"]])

cols_to_keep <-
  c(
    "lab",
    "count_v2.0",
    "percent_of_cohort_v2.0",
    "percent_of_those_with_condition_v2.0",
    "count_v2.1",
    "percent_of_cohort_v2.1",
    "percent_of_those_with_condition_v2.1",
    "count_v3.0",
    "percent_of_cohort_v3.0",
    "percent_of_those_with_condition_v3.0",
    "count_v3.1",
    "percent_of_cohort_v3.1",
    "percent_of_those_with_condition_v3.1"
  )

tabs <- lapply(rslts, function(x) {x[, cols_to_keep]})

tabs <-
  lapply(tabs,
    kableExtra::kbl,
    col.names = c("", rep(c("count", "% of cohort", "% of those with condition"), 4)),
    row.names = FALSE,
    digits = 1
  )
tabs <-
  Map(
    f = function(t,r) {
          x <- kableExtra::column_spec(t, column = 1, bold = is.na(r$subcondition))
          kableExtra::add_indent(x, which(!is.na(r$subcondition)))
        },
    t = tabs,
    r = rslts
  )
tabs <-
  lapply(tabs, kableExtra::kable_styling, bootstrap_options = "striped", font_size = 10)
tabs <-
  lapply(tabs, kableExtra::add_header_above, header = c("", "v2.0" = 3, "v2.1" = 3, "v3.0" = 3, "v3.1" = 3))
tabs
```

## Congenital or Genetic
```{r echo = FALSE, results = "asis"}
tabs[["congeni_genetic"]]
```

## Cardiovascular
```{r echo = FALSE, results = "asis"}
tabs[["cvd"]]
```

## Gastrointestinal
```{r echo = FALSE, results = "asis"}
tabs[["gi"]]
```

## Hematologic
```{r echo = FALSE, results = "asis"}
tabs[["hemato_immu"]]
```

## Malignancy
```{r echo = FALSE, results = "asis"}
tabs[["malignancy"]]
```

## Metabolic
```{r echo = FALSE, results = "asis"}
tabs[["metabolic"]]
```

## Miscellaneous
```{r echo = FALSE, results = "asis"}
tabs[["misc"]]
```

## Neonatal
```{r echo = FALSE, results = "asis"}
tabs[["neonatal"]]
```

## Neuromuscular
```{r echo = FALSE, results = "asis"}
tabs[["neuromusc"]]
```

## Renal
```{r echo = FALSE, results = "asis"}
tabs[["renal"]]
```

## Respiratory
```{r echo = FALSE, results = "asis"}
tabs[["respiratory"]]
```

## Longitudinal
The longitudinal assessment for subconditions work as well.  Using the same
`permutations` data set from above we will look at the metabolic and respiratory
conditions and subconditions.

```{r}
rslts <-
  comorbidities(
    data = permutations,
    icd.codes = "code",
    id.vars = c("permutation", "plabel", "encounter_id"),
    icdv = 10L,
    compact.codes = FALSE,
    method = "pccc_v3.1",
    flag.method = "cumulative",
    poa = 1,
    subconditions = TRUE
  )
```

Let's start by looking at the respiratory results.  The only subcondition that
should be, and is, flagged is chronic respiratory diseases.  A reminder: the
`data.frame` for a subcondition only report rows for when the primary condition
was flagged.  We see in the following encounters where the chronic
respiratory disease is flagged is consistent with when the primary respiratory
condition is flagged.

```{r}
all(rslts$subconditions$respiratory$chronic_respiratory_diseases == 1)
sapply(rslts$subconditions$respiratory[, -(1:3)], max)

# which encounters flag for primary condition respiratory?
cnd <-
  rslts$conditions[
    respiratory_dxpr_or_tech == 1,
    .(cencid = paste(encounter_id, collapse = ", ")),
    by = .(plabel)
  ]


# which encounters flag for the subcondition chronic_respiratory_diseases?
scnd <-
  rslts$subconditions$respiratory[
    ,
    .(sencid = paste(encounter_id, collapse = ", ")),
    by = .(plabel)
  ]
```

```{r, echo = FALSE, results = "asis"}
tab <-
  kableExtra::kbl(
    merge(cnd, scnd, all = TRUE, by = "plabel"),
    caption = "Encounters flagging for respiratory condition and the chronic respiratory disease subcondition.",
    col.names = c("", "Condition", "Subcondition")
  )
tab <- kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab <- kableExtra::add_header_above(tab, c("", "Encounters" = 2))
tab
```

For the metabolic condition we have two subconditions to look at, 1) device and
technology use, and 2) other metabolic disorders.

```{r}
# which encounters flag for primary condition metabolic?
cnd <-
  rslts$conditions[
    metabolic_dxpr_or_tech == 1,
    .(cencid = paste(encounter_id, collapse = ", ")),
    by = .(plabel)
  ]

# which encounters flag for the subconditions?
scnd <-
  data.table::melt(
    rslts$subconditions$metabolic,
    id.vars = c("plabel", "encounter_id"),
    measure.vars = c("device_and_technology_use", "other_metabolic_disorders"),
    variable.factor = FALSE,
    variable.name = "subcondition"
  )
scnd <- scnd[value == 1]
scnd <-
  scnd[
    ,
    .(sencid = paste(encounter_id, collapse = ", ")),
    by = .(plabel, subcondition)
  ]

scnd <-
  data.table::dcast(
    scnd,
    plabel ~ subcondition,
    value.var = "sencid"
  )
```

```{r, echo = FALSE, results = "asis"}
tab <-
  kableExtra::kbl(
    x = merge(cnd, scnd, all = TRUE, by = "plabel"),
    caption = "Encounters flagging for a metabolic condition and the encounters flagging for subconidtions device and technology use and/or other metabolic disorders.",
    col.names = c("", "Condition", "Device and Technology Use", "Other Metabolic Disorders")
  )
tab <- kableExtra::kable_styling(tab, bootstrap_options = c("striped"), font_size = 10)
tab <-
  kableExtra::add_header_above(tab, c("", "Encounters" = 3))
tab
```

# References

<!-- ----------------------------------------------------------------------- -->
<!--                              End of File                                -->
<!-- ----------------------------------------------------------------------- -->

---
title: "Comorbidities"
output:
 rmarkdown::html_vignette:
   toc: true
   number_sections: false
bibliography: references.bib
vignette: >
 %\VignetteIndexEntry{Comorbidities}
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8}
---

```{r, label = "setup", include = FALSE}
# IMPORTANT SYNTAX NOTE:
#
# DO NOT USE the pipeOp `|>`
#
# While convenient, that is a R 4.1.0 feature at a minimum. Notable improvements
# to the pipeOp come in 4.2.0 and 4.2.1.  To keep this package dependent on R >=
# 3.5.0 do not use the pipeOp.

library(kableExtra)
knitr::opts_chunk$set(collapse = TRUE, fig.align = "center")
options(qwraps2_markup = "markdown")
```

```{r, label = 'medicalcoder-namespace'}
library(medicalcoder)
packageVersion("medicalcoder")
```

# Comorbidity Algorithms

There are three comorbidity algorithms, each with several variants,
implemented in the medicalcoder package:

1. Pediatric Complex Chronic Condition System (PCCC)
   a. Version 2.0 [@feudtner2014pediatric]
       i. `pccc_v2.0` is consistent with the older R package pccc (v1.0.7) [@feinstein2018pccc;@dewitt2026pccc].
       ii. `pccc_v2.1` modifies the set of ICD codes to be more consistent with documentation and other implementations of v2.0.

   b. Version 3.0 [@feinstein2024pediatric]
       i. `pccc_v3.0` is consistent with the SAS software published on the [Children's Hospital Association](https://www.childrenshospitals.org/content/analytics/toolkit/complex-chronic-conditions) website in conjunction with @feinstein2024pediatric.
       ii. `pccc_v3.1` modifies the set of ICD codes to be more consistent with documentation.

2. Charlson
   a. `charlson_deyo1992`: Deyo's original set of codes [@deyo1992adapting;@quan2005]
   b. `charlson_quan2005` and `charlson_quan2011`: Codes and index scoring [@quan2005;@quan2011]
   c. `charlson_cdmf2019`: [@glasheen2019]

3. Elixhauser
   a. Based on codes provided by the Agency for Healthcare Research and Quality (AHRQ) for fiscal years 2022 through 2026 [@ahrqicd10]
      i. `elixhauser_ahrq2022`
      ii. `elixhauser_ahrq2023`
      iii. `elixhauser_ahrq2024`
      iv. `elixhauser_ahrq2025`
      v. `elixhauser_ahrq2026`
      vi. `elixhauser_ahrq_icd10`: uses all codes from all the specific years

   b. Codes from Table 2 of @quan2005
      i. `elixhauser_elixhauser1988`: [@elixhauser1998;@quan2005]
      ii. `elixhauser_ahrq_web`: [@quan2005;@ahrq2017]
      iii. `elixhauser_quan2005`: [@quan2005]

**IMPORTANT NOTE:** Elixhauser 1998 and AHRQ Web used diagnosis-related
group (DRG) codes as part of the methods.  The medicalcoder
package _does not_ use DRG codes.  This is consistent with the way these
methods were implemented in @quan2005.

A list of the valid methods for the package can be accessed via a non-exported
function.  In general, the methods are listed in the form of
`<algorithm>_<version>`.
```{r label = "valid-methods"}
medicalcoder:::comorbidities_methods()
```

Vignettes for each of the major methods are available.

```{r echo = TRUE, eval = FALSE}
vignette(topic = "pccc",       package = "medicalcoder")
vignette(topic = "charlson",   package = "medicalcoder")
vignette(topic = "elixhauser", package = "medicalcoder")
```

The focus of this vignette is to highlight the general use of the
`comorbidities()` function.

```{r, label = 'test-that-the-args-have-not-changed', include = FALSE}
# IF THIS FAILS YOU NEED TO MAKE SURE THE DOCUMENTATION IN THIS VIGNETTE IS
# UPTO DATE
# dput(capture.output(args(comorbidities)))
stopifnot(
  capture.output(args(comorbidities))  ==
    c("function (data, icd.codes, method, id.vars = NULL, icdv.var = NULL, ",
      "    icdv = NULL, dx.var = NULL, dx = NULL, poa.var = NULL, poa = NULL, ",
      "    age.var = NULL, primarydx.var = NULL, primarydx = NULL, flag.method = c(\"current\", ",
      "        \"cumulative\"), full.codes = TRUE, compact.codes = TRUE, ",
      "    subconditions = FALSE) ", "NULL")
)
```

Details of the function arguments call are in the manual.
```{r label = "read-the-manual", eval = FALSE}
help(topic = "comorbidities", package = "medicalcoder")
```

```{r label = "comorbidities-args"}
args(comorbidities)
```

We highlight a general concept for the arguments.  Note that several arguments
are in pairs, e.g., `dx.var` and `dx` (used for denoting if codes are
diagnostic or procedural), or `poa.var` and `poa` (used for denoting if a codes
are present on admission).  The `.var` version is the name of a variable within
the `data.frame` passed into the `data` argument.  The version without `.var` is
a default value to be applied to the entirety of `data`. We will see some
examples where this is useful.

The `data` element is expected to be a `data.frame`, or at least something that
inherits the `data.frame` class.  The format is expected to be a 'long' format:
one ICD code per row.  Two example data sets in the package show the general
expected form of the `data`.

```{r label = "example-data"}
head(mdcr)
head(mdcr_longitudinal)
```

# When are conditions flagged?

Whether or not the code is present on admission (POA) is useful when applying
the comorbidity algorithms and considering if the patient has a comorbidity at
the start of an encounter, or if the condition is a result of the current
hospitalization.

Implementation of Elixhauser comorbidities for 2022 and beyond [@ahrqicd10]
explicitly define the use of present on admission flags for specific conditions
(see the `poa_required` flag reported in the data set returned by
`get_elixhauser_poa()`).
```{r}
str(get_elixhauser_poa())
```

For Charlson comorbidities, from @quan2011:

> We defined comorbidities in the previous admissions using major and secondary
> diagnoses, without consideration of diagnosis type. The presence of a comorbid
> condition was assigned to a patient when it was present in index or previous
> admission records. Otherwise, the absence of the condition was assigned to the
> patient.

From @quan2005:

> The decision of whether to include or exclude specific codes or conditions
> from a coding algorithm depends to a large extent on a given study’s
> objectives. The original Charlson index used conditions present in
> hospitalized medical patients (regardless of whether the condition was present
> at baseline or arose after admission) to predict survival over an ensuing
> year. For such a study, a decision to exclude conditions arising after
> admission would not be ideal, as it would result in a loss of prognostic
> information relevant to long-term survival, and an 'under-adjustment' in
> risk-adjusted survival analyses. In contrast, in the context of studying
> in-hospital outcomes of a surgical procedure, researchers would be best
> advised to confine their risk adjustment to variables that are predominantly
> present at baseline. In jurisdictions that have diagnosis type indicators, the
> methodological decision is simply one of deciding, based on study objectives,
> whether to use or not use the indicators. In regions or countries without
> diagnosis type indicators, meanwhile, the data that we present ... can help
> researchers make decisions on a condition-by-condition basis of whether to
> include particular variables, depending on their study objectives.

PCCC does not explicitly note the if POA is required.

medicalcoder has been built to consider POA for all comorbidity algorithms.

End users can use a 0/1 indicator variable in the data set to report
which codes are POA via the function argument `poa.var`.  If all the codes are
to be considered POA or not, the functional argument `poa` can be used to set a
common status without adding a column to the input data set.

Additionally, medicalcoder provides a `flag.method` argument for longitudinal
data sets.

**Example:** Let's assume we have a patient record for six encounters.  We use
ICD-10 diagnostic codes C78.4 and I50.40 which maps to a cancer and heart
failure (cardiovascular disease) comorbidity respectively for PCCC, Charlson,
and Elixhauser. For demonstration, we also flag POA with the second report of
I50.40 intentionally marked as not present on admission.

```{r, label = "example-setup", include = FALSE}
lookup_icd_codes(c("C78.4", "I50.40"))

codes <- c("C78.4", "I50.40")
cols  <- c("icdv", "dx", "code", "full_code", "condition")

subset(
  get_pccc_codes(),
  subset = full_code %in% codes,
  select = c(cols, "pccc_v3.0")
)

subset(
  get_charlson_codes(),
  subset = full_code %in% codes,
  select = c(cols, "charlson_quan2011")
)

subset(
  get_elixhauser_codes(),
  subset = full_code %in% codes & elixhauser_ahrq2025 == 1L,
  select = c(cols, "elixhauser_ahrq2025")
)

record <-
  structure(
    list(
      patid = c("A", "A", "A", "A", "A", "A", "A"),
      encid = c(1L, 2L, 3L, 4L, 5L, 5L, 6L),
      code = c(NA, "C78.4", "I50.40", NA, "C78.4", "I50.40", NA),
      poa = c(NA, 0L, 1L, NA, 1L, 0L, NA)),
    row.names = c(NA, -7L),
    class = "data.frame"
  )
```

We will call `comorbidities()` for the three methods using static POA flags and
dynamic POA flags, and both flag methods.  Results are shown in the following
table.

```{r, include = FALSE}
data.table::setDT(record)
args <-
  list(data = record,
       icd.codes = "code",
       id.vars = c("patid", "encid"),
       icdv = 10L,
       dx = 1
  )
args_current_poa0    <- c(args, poa = 0L,        flag.method = "current")
args_current_poa1    <- c(args, poa = 1L,        flag.method = "current")
args_current_poav    <- c(args, poa.var = "poa", flag.method = "current")
args_cumulative_poa0 <- c(args, poa = 0L,        flag.method = "cumulative")
args_cumulative_poa1 <- c(args, poa = 1L,        flag.method = "cumulative")
args_cumulative_poav <- c(args, poa.var = "poa", flag.method = "cumulative")

left_cols <-
  cbind(encid = 1L:6L,
        ICD = c("", "C78.4", paste0("I50.40", footnote_marker_symbol(1L)),
                "", paste0("C78.4", footnote_marker_symbol(1L), "; I50.40"), ""))

rtn <-
  rbind(
    do.call(cbind,
            list(
              left_cols,
              do.call(comorbidities, c(args_current_poa0,                 method = "pccc_v3.0"))[,           .(CVD = cvd_dxpr_or_tech, CANCER = malignancy_dxpr_or_tech)],
              do.call(comorbidities, c(args_current_poa0, primarydx = 0L, method = "charlson_quan2011"))[,   .(CVD = chf,              CANCER = mst)],
              do.call(comorbidities, c(args_current_poa0, primarydx = 0L, method = "elixhauser_ahrq2025"))[, .(CVD = HF,               CANCER = CANCER_METS)],
              do.call(comorbidities, c(args_current_poa1,                 method = "pccc_v3.0"))[,           .(CVD = cvd_dxpr_or_tech, CANCER = malignancy_dxpr_or_tech)],
              do.call(comorbidities, c(args_current_poa1, primarydx = 0L, method = "charlson_quan2011"))[,   .(CVD = chf,              CANCER = mst)],
              do.call(comorbidities, c(args_current_poa1, primarydx = 0L, method = "elixhauser_ahrq2025"))[, .(CVD = HF,               CANCER = CANCER_METS)],
              do.call(comorbidities, c(args_current_poav,                 method = "pccc_v3.0"))[,           .(CVD = cvd_dxpr_or_tech, CANCER = malignancy_dxpr_or_tech)],
              do.call(comorbidities, c(args_current_poav, primarydx = 0L, method = "charlson_quan2011"))[,   .(CVD = chf,              CANCER = mst)],
              do.call(comorbidities, c(args_current_poav, primarydx = 0L, method = "elixhauser_ahrq2025"))[, .(CVD = HF,               CANCER = CANCER_METS)]
    ))
  ,
    do.call(cbind,
            list(
              left_cols,
              do.call(comorbidities, c(args_cumulative_poa0,                 method = "pccc_v3.0"))[,           .(CVD = cvd_dxpr_or_tech, CANCER = malignancy_dxpr_or_tech)],
              do.call(comorbidities, c(args_cumulative_poa0, primarydx = 0L, method = "charlson_quan2011"))[,   .(CVD = chf,              CANCER = mst)],
              do.call(comorbidities, c(args_cumulative_poa0, primarydx = 0L, method = "elixhauser_ahrq2025"))[, .(CVD = HF,               CANCER = CANCER_METS)],
              do.call(comorbidities, c(args_cumulative_poa1,                 method = "pccc_v3.0"))[,           .(CVD = cvd_dxpr_or_tech, CANCER = malignancy_dxpr_or_tech)],
              do.call(comorbidities, c(args_cumulative_poa1, primarydx = 0L, method = "charlson_quan2011"))[,   .(CVD = chf,              CANCER = mst)],
              do.call(comorbidities, c(args_cumulative_poa1, primarydx = 0L, method = "elixhauser_ahrq2025"))[, .(CVD = HF,               CANCER = CANCER_METS)],
              do.call(comorbidities, c(args_cumulative_poav,                 method = "pccc_v3.0"))[,           .(CVD = cvd_dxpr_or_tech, CANCER = malignancy_dxpr_or_tech)],
              do.call(comorbidities, c(args_cumulative_poav, primarydx = 0L, method = "charlson_quan2011"))[,   .(CVD = chf,              CANCER = mst)],
              do.call(comorbidities, c(args_cumulative_poav, primarydx = 0L, method = "elixhauser_ahrq2025"))[, .(CVD = HF,               CANCER = CANCER_METS)]
    ))
  )
```

```{r, echo = FALSE, results = "asis"}
tab <-
  kbl(
    rtn,
    row.names = FALSE,
    escape = FALSE,
    align = rep("c", nrow(rtn)),
    caption = "Indicators for when a comorbidity is flagged based on the algorithm, present on admission (poa), and flag.method. The two ICD codes, C78.4 and I50.40, map to cancer and cardiovascular disease respectively."
  )
tab <-
  footnote(
    tab,
    symbol = c("Present on Admission"),
    general = "C78.4 does not need to be POA to count for Elixhauser. I50.40 does need to be POA to count for Elixhauser."
  )

tab <- kable_styling(tab, bootstrap_options = c("striped", "bordered"), full_width = FALSE, font_size = 8)
#tab <- scroll_box(tab, height = "600px")
tab <- pack_rows(tab, "flag.method = 'current'", 1L, 6L)
tab <- pack_rows(tab, "flag.method = 'cumulative'", 7L, 12L)
tab <- add_header_above(tab, c(" " = 2L, rep(c("PCCC" = 2L, "Charlson" = 2L, "Elixhauser" = 2L), 3L)))
tab <- add_header_above(tab, c(" " = 2L, c("POA = 0" = 6L, "POA = 1" = 6L, "poa.var = 'poa'" = 6L)))
tab
```

### Flag method and POA defaults

When `flag.method = "cumulative"` and you do *not* supply `poa` or `poa.var`,
`comorbidities()` treats the first encounter where a condition appears as
`poa = 0` and carries that condition forward with `poa = 1` on later encounters.

When `flag.method = "current"` and you do *not* supply `poa` or `poa.var`,
`comorbidities()` treats all ICD codes as `poa = 1`.

# Mapping ICD Codes to Comorbidities

End users can quickly assess the lookup table for all the ICD codes associated
with a comorbidity algorithm using the `get_<comorbidity>_codes` functions.
Each `data.frame` has columns for the ICD version, diagnostic or procedure flag,
the compact code, and the full code.  A column for the condition and other
method specific flags are provided.  Lastly, there are indicator columns for the
variant of each method

```{r, label = "get-codes"}
str(get_pccc_codes())
str(get_charlson_codes())
str(get_elixhauser_codes())
```

End users should be aware that just because an ICD code exists in a data
set does not mean that the patient has the condition.  For Elixhauser, the
presence on admission is important to consider.  For PCCC version 3.0 and 3.1,
tech dependencies on their own are insufficient to flag a condition (see
`vignette(topic = "pccc", package = "medicalcoder")`).

For the `charlson_cdmf2019` method [@glasheen2019], the AIDS categories are
defined by the presence of HIV and an opportunistic infection.  In the
following example, if only considering ICD codes which flag 'aids',
there would be several thousand cases of AIDS, but only six cases
of HIV.

<!-- NOTE: the following example is also in tests/test-charlson_cdmf2019.R -->
```{r}
cdmf_eg <-
  merge(x = mdcr,
        y = subset(get_charlson_codes(),
                   condition %in% c("aids", "hiv") &
                   charlson_cdmf2019 == 1),
        by = c("icdv", "dx", "code"))
data.table::setDT(cdmf_eg)

cdmf_eg <-
  data.table::dcast(data = cdmf_eg,
                    patid ~ condition,
                    value.var = "charlson_cdmf2019",
                    fun.aggregate = function(x) {as.integer(sum(x) > 0)})

cdmf_eg[, .N, keyby = .(hiv, aids)]
```

When calling `comorbidities()` we get the expected result.

```{r}
cmdf_mdcr <-
  comorbidities(data = mdcr,
                icd.codes = "code",
                id.vars = "patid",
                icdv.var = "icdv",
                dx.var = "dx",
                method = "charlson_cdmf2019",
                flag.method = "current",
                primarydx = 0L,
                poa = 1L)
data.table::setDT(cmdf_mdcr)

cmdf_mdcr[, .N, keyby = .(hiv, aids)]
```

# References

<!-- ----------------------------------------------------------------------- -->
<!--                              End of File                                -->
<!------------------------------------------------------------------------- -->

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.medicalcoder_comorbidities_with_subconditions}
\alias{summary.medicalcoder_comorbidities_with_subconditions}
\title{Summaries of Comorbidities with Subconditions}
\usage{
\method{summary}{medicalcoder_comorbidities_with_subconditions}(object, ...)
}
\arguments{
\item{object}{a \code{medicalcoder_comorbidities_with_subconditions} object generated by calling
\code{\link[=comorbidities]{comorbidities()}} with \code{subconditions = TRUE}.  This is currently only
applicable to PCCC.}

\item{...}{additional parameters, not currently used}
}
\value{
a \code{data.frame} with five columns.
\enumerate{
\item \code{condition} the primary condition
\item \code{subcondition} the subcondition(s) within the \code{condition}.  There will be
a row where \code{subcondition} is \code{NA} which is used to report the \code{count} and
\code{percent_of_cohort} for the \code{condition} overall.
\item \code{count} the number of rows in \code{object} with the applicable \code{condition} and
\code{subcondition}.
\item \code{percent_of_cohort}: a numeric value within [0, 100] for the percent of
rows in \code{object} with the flagged \code{condition} and \code{subcondition}.
\item \code{percent_of_those_with_condition}: a numeric value within [0, 100] for the
subset of rows in \code{object} with the primary \code{condition} and the flagged
\code{subcondition}.  Will be \code{NA} for the primary \code{condition}.
}
}
\description{
Build summaries (counts and percentages) for each Pediatric Complex Chronic
Condition (PCCC) condition and subcondition.
}
\examples{
pccc_v3.1_subcondition_results <-
  comorbidities(data = mdcr,
                icd.codes = "code",
                id.vars = "patid",
                dx.var = "dx",
                method = "pccc_v3.1",
                flag.method = 'current',
                poa = 1,
                subconditions = TRUE)
summary(pccc_v3.1_subcondition_results)

}
\seealso{
\code{\link[=comorbidities]{comorbidities()}},
\code{vignette(topic = "pccc", package = "medicalcoder")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lookup_icd_codes.R
\name{lookup_icd_codes}
\alias{lookup_icd_codes}
\title{Lookup ICD Codes}
\usage{
lookup_icd_codes(
  x,
  regex = FALSE,
  full.codes = TRUE,
  compact.codes = TRUE,
  ...
)
}
\arguments{
\item{x}{Character vector of ICD codes (full or compact form). When
\code{regex = TRUE}, \code{x} must contain at least one non-empty, non-missing
string.}

\item{regex}{Logical scalar. If \code{TRUE}, treat \code{x} as regular
expressions; if \code{FALSE}, use exact matching.}

\item{full.codes}{Logical scalar. If \code{TRUE} (default), match codes that
include the decimal point where applicable.}

\item{compact.codes}{Logical scalar. If \code{TRUE} (default), match codes without
the decimal point.}

\item{...}{Passed to \code{\link[base:grep]{base::grep()}} when \code{regex = TRUE}}
}
\value{
A \code{data.frame} with one or more rows per input, including columns
\itemize{
\item \code{match_type}: did the input match a full or compact code
\item \code{icdv}: icd version (9 or 10)
\item \code{dx}: diagnostic code (1) or procedure code (0)
\item \code{full_code}: the full code string
\item \code{code}: the compact codes string
\item \code{src}: the source - CMS, CDC, or WHO.
\item year ranges (\verb{known_*}, \verb{assignable_*}).
}
}
\description{
Functions for working with ICD codes.

ICD-10 code "C00" is a header code because the four-character codes
C00.0, C00.1, C00.2, C00.3, C00.4, C00.5, C00.6, C00.7, C00.8, and C00.9
exist. Those four-character codes are assignable (as of 2025) because no
five-character descendants (e.g., C00.40) exist.

When the source is the World Health Organization (WHO) or CDC Mortality,
years refer to calendar years. CDC/CMS sources use the U.S. federal fiscal
year, which starts on October 1 (e.g., fiscal year 2024 runs 2023-10-01 to
2024-09-30).
}
\details{
ICD codes should be character vectors. These tools work with either
"full codes" (decimal point present when appropriate) or "compact codes"
(decimal point omitted).

Similarly for ICD-9-CM: "055" is a header for measles; 055.0, 055.1,
055.2, 055.8, and 055.9 are assignable. Codes 055.3–055.6 do not exist.
Code 055.7 is a header because 055.71 and 055.72 exist.

Some codes change status across years. For example, ICD-9-CM 516.3 was
assignable in fiscal years 1997–2011 for the CDC extracts (2006–2011 for CMS)
and became a header in 2012–2015.
}
\seealso{
\itemize{
\item \code{\link[=get_icd_codes]{get_icd_codes()}} to retrieve the internal lookup table of ICD codes.
\item \code{\link[=is_icd]{is_icd()}} to test if a string is a known ICD code.
\item \code{\link[=icd_compact_to_full]{icd_compact_to_full()}} converts a string from a compact format to the
full format based on ICD version and type (diagnostic or procedure).
}

Other ICD tools:
\code{\link[=icd_compact_to_full]{icd_compact_to_full()}},
\code{\link[=is_icd]{is_icd()}}
}
\concept{ICD tools}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_elixhauser_codes.R
\name{get_elixhauser_codes}
\alias{get_elixhauser_codes}
\title{Get Elixhauser Codes}
\usage{
get_elixhauser_codes()
}
\value{
A \code{data.frame} with the following columns:
\itemize{
\item \code{icdv}: Integer vector indicating if the code is from ICD-9 or ICD-10
\item \code{dx}: Integer vector.  1 if the code is a diagnostic, (ICD-9-CM, ICD-10-CM, WHO, CDC Mortality), or 0 if the code is procedural (ICD-9-PCS, ICD-10-PCS)
\item \code{full_code}: Character vector with the ICD code and any relevant decimal point
\item \code{code}: Character vector with the compact ICD code omitting any relevant decimal point
\item \code{condition}: Character vector of the conditions
\item \verb{elixhauser_<variant>}: Integer vector indicating if the code is part of the \verb{<variant>} of the Elixhauser comorbidities.
}
}
\description{
Retrieve  copy of internal lookup tables for the ICD codes used in assessing
Elixhauser comorbidities.
}
\examples{
head(get_elixhauser_codes())
str(get_elixhauser_codes())

}
\seealso{
\itemize{
\item \code{\link[=get_elixhauser_index_scores]{get_elixhauser_index_scores()}} for the lookup table of the condition by
condition scores for mortality and readmission indices.
\item \code{\link[=get_elixhauser_poa]{get_elixhauser_poa()}} for the lookup table of the conditions which do an
do not require associated ICD codes to be present-on-admission to flag the
comorbidity.
\item \code{\link[=get_icd_codes]{get_icd_codes()}} for the lookup table of all ICD codes.
\item \code{\link[=get_pccc_codes]{get_pccc_codes()}} for the lookup table of ICD codes used for the PCCC.
\item \code{\link[=get_charlson_codes]{get_charlson_codes()}} for the lookup table of ICD codes used for the Charlson comorbidities.
\item \code{\link[=comorbidities]{comorbidities()}} for applying comorbidity algorithms to a data set.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/medicalcoder-pkg.R
\docType{package}
\name{medicalcoder-package}
\alias{medicalcoder}
\alias{medicalcoder-package}
\title{medicalcoder}
\description{
An R package for working with ICD codes and comorbidity assessments.
}
\details{
medicalcoder is a lightweight, base-R package for working with ICD-9 and
ICD-10 diagnosis and procedure codes. It provides fast, dependency-free tools
to look up, validate, and manipulate ICD codes, while also implementing
widely used comorbidity algorithms such as Charlson, Elixhauser, and the
Pediatric Complex Chronic Conditions (PCCC).
Designed for portability and reproducibility, the package avoids external
dependencies—requiring only R >= 3.5.0—yet offers a rich set of curated ICD
code libraries from the United States' Centers for Medicare and Medicaid
Services (CMS), Centers for Disease Control (CDC), and the World Health
Organization (WHO).

The package balances performance with elegance: its internal caching,
efficient joins, and compact data structures make it practical for
large-scale health data analyses, while its clean design makes it easy to
extend or audit. Whether you need to flag comorbidities, explore ICD
hierarchies, or standardize clinical coding workflows, medicalcoder provides
a robust, transparent foundation for research and applied work in biomedical
informatics.
}
\section{Implementation}{
The medicalcoder package was intentionally designed and built to have zero
dependencies beyond R version 3.5.0 (needed due to a \href{https://cran.r-project.org/src/base/NEWS.3#:~:text=R\%20has\%20new,to\%20version\%203.5.0.}{change in data serialization})
and zero imports. The package is completely self-contained for the purposes
of installation and use.

This design choice was made for several reasons.
\enumerate{
\item Ease of installation:
\itemize{
\item Only requirement is R >= 3.5.0.
\item No need for external files, downloads, or other packages for the ICD database.
}
\item Works well with different data paradigms
\itemize{
\item Base R \code{data.frames},
\item \href{https://tidyverse.org/}{tidyverse} \code{tibble}s, and
\item \code{data.table}s from the \href{https://rdatatable.gitlab.io/data.table/}{data.table} package.
}
}

One of the reasons for focusing on building a self-contained package with no
need for additional namespaces is to make installation and use in a
restricted and secured system easier. The author of this package routinely works
on machines with extremely limited access to the world-wide web. As
such, relying on any system dependencies or other R packages can become
difficult as the machine may or may not have the needed software. So long as
R >= 3.5.0 is available, medicalcoder will work.

A great deal of thought went into performance of the methods and the size of
the package.  The internal data sets, for example, are not stored in a
structure that is easy to use for end users. When the package namespace is
loaded, the needed internal lookup tables are generated and cached.
}

\seealso{
\code{\link[=comorbidities]{comorbidities()}}, \code{\link[=get_icd_codes]{get_icd_codes()}}, \code{\link[=is_icd]{is_icd()}},
\itemize{
\item Vignette for working with ICD codes:
\itemize{
\item \code{vignette(topic = "icd", package = "medicalcoder")}
}
\item Vignettes for applying comorbidities:
\itemize{
\item \code{vignette(topic = "comorbidities", package = "medicalcoder")}
\item \code{vignette(topic = "pccc", package = "medicalcoder")}
\item \code{vignette(topic = "charlson", package = "medicalcoder")}
\item \code{vignette(topic = "elixhauser", package = "medicalcoder")}
}
}
}
\author{
\strong{Maintainer}: Peter DeWitt \email{peter.dewitt@cuanschutz.edu} (\href{https://orcid.org/0000-0002-6391-0795}{ORCID}) [cover designer]

Authors:
\itemize{
  \item Peter DeWitt \email{peter.dewitt@cuanschutz.edu} (\href{https://orcid.org/0000-0002-6391-0795}{ORCID}) [cover designer]
}

Other contributors:
\itemize{
  \item Tell Bennett \email{tell.bennett@cuanschutz.edu} (\href{https://orcid.org/0000-0003-1483-4236}{ORCID}) [contributor]
  \item Seth Russell \email{seth.russell@cuanschutz.edu} (\href{https://orcid.org/0000-0002-2436-1367}{ORCID}) [contributor]
  \item Meg Rebull \email{meg.rebull@cuanschutz.edu} (\href{https://orcid.org/0000-0003-0334-4223}{ORCID}) [contributor]
  \item Vincent Rubinetti \email{vincent.rubinetti@cuanschutz.edu} (\href{https://orcid.org/0000-0002-4655-3773}{ORCID}) [cover designer]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_charlson_index_scores.R
\name{get_charlson_index_scores}
\alias{get_charlson_index_scores}
\title{Get Charlson Index Scores}
\usage{
get_charlson_index_scores()
}
\value{
A \code{data.frame} with the following columns:
\itemize{
\item \code{condition}: Character vector of the conditions
\item \code{index}: Character vector indicating if the score is for the mortality or the readmission index score
\item \verb{charlson_<variant>}:  the index scores for the variant
}
}
\description{
Retrieve a copy of internal lookup tables of index scores used in assessing
Charlson comorbidities.
}
\examples{
head(get_charlson_index_scores())
str(get_charlson_index_scores())

}
\seealso{
\itemize{
\item \code{\link[=get_charlson_codes]{get_charlson_codes()}} for a lookup table of the ICD codes mapping to the
Charlson comorbidities.
\item \code{\link[=comorbidities]{comorbidities()}} for applying comorbidity algorithms to a data set.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_pccc_codes.R
\name{get_pccc_codes}
\alias{get_pccc_codes}
\title{Pediatric Complex Chronic Conditions ICD Codes}
\usage{
get_pccc_codes()
}
\value{
a \code{data.frame} with the following columns
\itemize{
\item \code{icdv}: Integer vector indicating if the code is from ICD-9 or ICD-10.
\item \code{dx}: Integer vector.  1 if the code is a diagnostic, (ICD-9-CM,
ICD-10-CM, WHO, CDC Mortality), or 0 if the code is procedural (ICD-9-PCS,
ICD-10-PCS).
\item \code{full_code}: Character vector with the ICD code and any relevant decimal point.
\item \code{code}: Character vector with the compact ICD code omitting any relevant decimal point.
\item \code{condition}: Character vector of the conditions.
\item \code{subcondition}: Character vector of the subconditions.
\item \code{transplant_flag}: Integer vector indicating if the code is associated with a transplant.
\item \code{tech_dep_flag}: Integer vector indicating if the code is associated with technology dependence.
\item \verb{pccc_<variant>}: Integer vector indicating if the code is part of the
v2.0, v2.1, v3.0, or v3.1 variant.
}
}
\description{
Retrieve a copy of internal lookup tables for the ICD codes mapping to the
Pediatric Complex Chronic Conditions (PCCC) conditions and subconditions by
variant.
}
\examples{
head(get_pccc_codes())
str(get_pccc_codes())
}
\seealso{
\itemize{
\item \code{\link[=get_pccc_conditions]{get_pccc_conditions()}} for a reference of the PCCC conditions and
subconditions.
\item \code{\link[=get_icd_codes]{get_icd_codes()}} for the lookup table of all ICD codes.
\item \code{\link[=comorbidities]{comorbidities()}} for applying comorbidity algorithms to a data set.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.medicalcoder_comorbidities}
\alias{summary.medicalcoder_comorbidities}
\title{Summaries of Comorbidities}
\usage{
\method{summary}{medicalcoder_comorbidities}(object, ...)
}
\arguments{
\item{object}{a \code{medicalcoder_comorbidities} object generated by calling
\code{\link[=comorbidities]{comorbidities()}}}

\item{...}{additional parameters, not currently used}
}
\value{
either a list or a data \code{data.frame}
}
\description{
Build summaries (counts and percentages) for each comorbidity and other
summary statistics by method.
}
\examples{
pccc_v3.1_results <-
  comorbidities(data = mdcr,
                icd.codes = "code",
                id.vars = "patid",
                dx.var = "dx",
                method = "pccc_v3.1",
                flag.method = 'current',
                poa = 1)
summary(pccc_v3.1_results)

charlson_results <-
  comorbidities(data = mdcr,
                icd.codes = "code",
                id.vars = "patid",
                dx.var = "dx",
                method = "charlson_quan2011",
                flag.method = 'current',
                poa = 1)
summary(charlson_results)

elixhauser_results <-
  comorbidities(data = mdcr,
                icd.codes = "code",
                id.vars = "patid",
                dx.var = "dx",
                method = "elixhauser_ahrq2025",
                primarydx = 1,
                flag.method = 'current',
                poa = 1)
summary(elixhauser_results)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{mdcr}
\alias{mdcr}
\title{Synthetic Data}
\format{
\code{mdcr} is a \code{data.frame} with 4 columns, Each row is for one ICD id.
\itemize{
\item \code{patid}: patient identifier, integer values
\item \code{icdv}: ICD version; integer values, 9 or 10
\item \code{dx}: indicator column for ICD diagnostic (1) or procedure (0) codes
\item \code{code}: ICD code; character values
}
}
\usage{
mdcr
}
\description{
Synthetic Data
}
\seealso{
Other datasets:
\code{\link{mdcr_longitudinal}}
}
\concept{datasets}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_icd.R
\name{is_icd}
\alias{is_icd}
\title{Is ICD}
\usage{
is_icd(
  x,
  icdv = c(9L, 10L),
  dx = c(1L, 0L),
  src = c("cms", "who", "cdc"),
  year,
  headerok = FALSE,
  ever.assignable = missing(year),
  warn.ambiguous = TRUE,
  full.codes = TRUE,
  compact.codes = TRUE
)
}
\arguments{
\item{x}{Character vector of ICD codes (full or compact form).}

\item{icdv}{Integer vector of allowed ICD versions. Use \code{9L} and/or
\code{10L}. Defaults to both.}

\item{dx}{Integer vector indicating allowed code type(s): \code{1L} for
diagnostic (ICD-9-CM, ICD-10-CM, CDC mortality, WHO), \code{0L} for procedural
(ICD-9-PCS, ICD-10-PCS). Defaults to both.}

\item{src}{Character vector of code sources. One or more of \code{"cms"}, \code{"who"},
\code{"cdc"}. Defaults to all.}

\item{year}{Numeric scalar. Calendar or fiscal year to reference. Default
is the most current year available per source. For ICD-9, CMS data run
through fiscal year 2015 and CDC extracts through 2012; ICD-10 sources are
updated annually. Calendar year for WHO and CDC mortality. Fiscal year for
CMS.}

\item{headerok}{Logical scalar. If \code{FALSE} (default), only assignable
codes are considered valid; if \code{TRUE}, header codes are also accepted.}

\item{ever.assignable}{Logical scalar. If \code{TRUE} then ignore \code{year}
and return \code{TRUE} if the \code{x} was ever an assignable code.}

\item{warn.ambiguous}{Logical scalar. If \code{TRUE} (default), warn when a
code matches more than one ICD version and/or type (e.g., both CM and PCS).}

\item{full.codes}{Logical scalar. If \code{TRUE} (default), match codes that
include the decimal point where applicable.}

\item{compact.codes}{Logical scalar. If \code{TRUE} (default), match codes without
the decimal point.}
}
\value{
A logical vector the same length as \code{x}.
}
\description{
Answer the question "is the character string x a valid ICD code?"
ICD codes should be character vectors. \code{is_icd} will assess for both
"full codes" (decimal point present when appropriate) and "compact codes"
(decimal point omitted).

ICD-10 code "C00" is a header code because the four-character codes
C00.0, C00.1, C00.2, C00.3, C00.4, C00.5, C00.6, C00.7, C00.8, and C00.9
exist. Those four-character codes are assignable (as of 2025) because no
five-character descendants (e.g., C00.40) exist.

When the source is the World Health Organization (WHO) or CDC Mortality,
years refer to calendar years. CDC/CMS sources use the U.S. federal fiscal
year, which starts on October 1 (e.g., fiscal year 2024 runs 2023-10-01 to
2024-09-30).
}
\details{
Similarly for ICD-9-CM: "055" is a header for measles; 055.0, 055.1,
055.2, 055.8, and 055.9 are assignable. Codes 055.3–055.6 do not exist.
Code 055.7 is a header because 055.71 and 055.72 exist.

Some codes change status across years. For example, ICD-9-CM 516.3 was
assignable in fiscal years 1997–2011 for the CDC extracts (2006–2011 for CMS)
and became a header in 2012–2015.
}
\examples{
################################################################################
# Some ICD-9 diagnostic codes
x <- c("136.2", "718.60", "642.02")

is_icd(x, icdv =  9, dx = 1)
is_icd(x, icdv =  9, dx = 0)
is_icd(x, icdv = 10, dx = 1)
is_icd(x, icdv = 10, dx = 0)

is_icd(x, icdv = 9, dx = 1, headerok = TRUE)
is_icd(x, icdv = 9, dx = 1, year = 2006)

################################################################################
# ICD code with, or without a dot.  The ICD-9 diagnostic code 799.3 and ICD-9
# procedure code 79.93 both become 7993 when assessed against the ICD code look
# up tables.  As such "7993" is a valid ICD-9 diagnostic and procedure code,
# whereas 799.3 is only a valid dx code, and 79.93 is only a valid pr code.
# Further, codes such as ".7993", "7.993", "7993.", are all non-valid codes.

x <- c("7993", ".7993", "7.993", "79.93", "799.3", "7993.")
data.frame(
  x,
  dx = is_icd(x, icdv = 9, dx = 1),
  pr = is_icd(x, icdv = 9, dx = 0)
)

################################################################################
# example of a ICD-9 code that was assignable, but became a header when
# more descriptive codes were introduced: ICD-9 diagnostic code 516.3
lookup_icd_codes(paste0("516.3", c("", as.character(0:9))))

# ICD-9 code 516.3 was an assignable code through fiscal year 2011.
is_icd("516.3")

# If `year` is omitted, and `ever.assignable = FALSE` then the `year` is
# implied to be the max `known_end` year for ICD codes matched by `icdv`,
# `dx`, and `src`.
is_icd("516.3", ever.assignable = FALSE)

# when `year` is provided then `ever.assignable` is `FALSE` by default and
# the return is TRUE when 516.3 was assignable and FALSE otherwise.
is_icd("516.3", year = 2015)
is_icd("516.3", year = 2011)

# when year is a non-assignable year, but `ever.assignable = TRUE` the return
# will be TRUE.  Useful if you know the data is retrospective and collected
# through fiscal year 2015.
is_icd("516.3", year = 2015, ever.assignable = TRUE)

################################################################################
# Consiser the string E010
#   - This could be a ICD-9-CM full code
#   - Could be a ICD-10-CM compact code
lookup_icd_codes("E010")
subset(get_icd_codes(with.descriptions = TRUE), grepl("^E010$", code))

is_icd("E010")
is_icd("E010", icdv = 9) # FALSE because it is a header code and was never assignable
is_icd("E010", icdv = 9, ever.assignable = TRUE) # FALSE
is_icd("E010", icdv = 9, headerok = TRUE) # TRUE
}
\seealso{
\itemize{
\item \code{\link[=get_icd_codes]{get_icd_codes()}} to retrieve the internal lookup table of ICD codes.
\item \code{\link[=lookup_icd_codes]{lookup_icd_codes()}} for retrieving details on a specific set of ICD
codes.
\item \code{\link[=icd_compact_to_full]{icd_compact_to_full()}} converts a string from a compact format to the
full format based on ICD version and type (diagnostic or procedure).
}

Other ICD tools:
\code{\link[=icd_compact_to_full]{icd_compact_to_full()}},
\code{\link[=lookup_icd_codes]{lookup_icd_codes()}}
}
\concept{ICD tools}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_elixhauser_index_scores.R
\name{get_elixhauser_index_scores}
\alias{get_elixhauser_index_scores}
\title{Get Elixhauser Index Scores}
\usage{
get_elixhauser_index_scores()
}
\value{
A \code{data.frame} with the following columns:
\itemize{
\item \code{condition}: Character vector of the conditions
\item \code{index}: Character vector indicating if the score is for the mortality or the readmission index score
\item \verb{elixhauser_<variant>}: integer vector of the scores
}
}
\description{
Functions to get a copy of internal lookup tables for the ICD codes and index
scores used in assessing Elixhauser comorbidities.
}
\examples{
head(get_elixhauser_index_scores())
str(get_elixhauser_index_scores())

}
\seealso{
\itemize{
\item \code{\link[=get_elixhauser_codes]{get_elixhauser_codes()}} for the lookup table of ICD codes mapping to the
Elixhauser comorbidities.
\item \code{\link[=get_elixhauser_poa]{get_elixhauser_poa()}} for the lookup table of the conditions which do an
do not require associated ICD codes to be present-on-admission to flag the
comorbidity.
\item \code{\link[=comorbidities]{comorbidities()}} for applying comorbidity algorithms to a data set.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/comorbidities.R
\name{comorbidities}
\alias{comorbidities}
\title{Comorbidities}
\usage{
comorbidities(
  data,
  icd.codes,
  method,
  id.vars = NULL,
  icdv.var = NULL,
  icdv = NULL,
  dx.var = NULL,
  dx = NULL,
  poa.var = NULL,
  poa = NULL,
  age.var = NULL,
  primarydx.var = NULL,
  primarydx = NULL,
  flag.method = c("current", "cumulative"),
  full.codes = TRUE,
  compact.codes = TRUE,
  subconditions = FALSE
)
}
\arguments{
\item{data}{A \code{data.frame} in a "long" format. The input \code{data.frame} is
expected to have one column of ICD codes (one code per row) with additional
(optional) columns for patient/encounter ids, ICD version,
diagnostic/procedure status, present-on-admission flags, primary
diagnostic flags, or age.}

\item{icd.codes}{Character scalar naming the column in \code{data} that contains
ICD codes (character strings). Codes may be provided in full form (with
decimal points, e.g., C84.2), compact form (dots omitted, e.g., C842), or
any mix of the two. Matching against lookup tables is governed by
\code{icdv.var}/\code{icdv}, \code{dx.var}/\code{dx}, and the \code{full.codes} / \code{compact.codes}
flags.}

\item{method}{Character string indicating the comorbidity algorithm to
apply to \code{data}.}

\item{id.vars}{Optional character vector of column names. When
missing, the entire input \code{data} is treated as a single encounter from a
single patient. If you want to set \code{flag.method = "cumulative"} then
\code{length(id.vars) >= 2} is expected. The last element should be the
encounter order (must be sortable).}

\item{icdv.var}{Character scalar naming the column in \code{data} that indicates
the ICD version (9 or 10). If present it must be integer values \code{9} or
\code{10}.  \code{icdv.var} takes precedence over \code{icdv} if both are provided.}

\item{icdv}{An integer value of \code{9L} or \code{10L} indicating that all
\code{data[[icd.codes]]} are ICD version 9 or 10, respectively. Ignored
(with a warning) if \code{icdv.var} is provided.}

\item{dx.var}{Character scalar naming the column in \code{data} that indicates
diagnostic (\code{1}) vs procedural (\code{0}) codes. If present it must be integer
values \code{0} or \code{1}.  \code{dx.var} takes precedence over \code{dx} if both are
provided.}

\item{dx}{An integer indicating that all \code{data[[icd.codes]]} are
diagnostic (\code{1}) or procedure (\code{0}) codes. Ignored (with a
warning) if \code{dx.var} is provided.}

\item{poa.var}{Character scalar naming the column with present-on-admission
flags: integer \code{1L} (present), \code{0L} (not present), or \code{NA}.
PCCC and Charlson will only flag conditions when the code is
present-on-admission. Elixhauser has a mix of conditions; some require
present-on-admission while others do not. \code{poa.var} takes precedence over
\code{poa} if both are provided.}

\item{poa}{Integer scalar \code{0} or \code{1}. Use when all \code{icd.codes} share the same
present-on-admission status. Ignored with a warning if \code{poa} and \code{poa.var}
are both provided.}

\item{age.var}{Character scalar naming the column in \code{data} that contains
patient age in years. Only applicable to Charlson comorbidities.}

\item{primarydx.var}{Character scalar naming the column in \code{data} that
indicates whether \code{data[[icd.codes]]} are primary diagnostic codes (\code{1L})
or not (\code{0L}). Primary diagnosis is used only for Elixhauser and Charlson
comorbidities and is ignored when the method is a PCCC variant.
\code{primarydx.var} takes precedence over \code{primarydx} if both are provided.}

\item{primarydx}{An integer value of \code{0} or \code{1}. If \code{0},
treat all codes as non-primary diagnoses; if \code{1}, treat all codes as
primary diagnoses. Ignored, with a warning, if \code{primarydx.var} is provided.}

\item{flag.method}{When \code{flag.method = 'current'} (default) only codes
associated with the current \code{id.vars} are considered when flagging
comorbidities.  When \code{flag.method = 'cumulative'} then all prior encounters
are considered when flagging comorbidities. See \strong{Details}.}

\item{full.codes, compact.codes}{Logical; when \code{TRUE} compare
\code{data[[icd.codes]]} against full and/or compact ICD codes in the
method’s lookup tables. Full ICD codes include a decimal point (when
applicable) and compact codes omit the decimal point. For example:
\code{B95.0} is the full ICD-10-CM diagnostic code for “Streptococcus,
group A, as the cause of disease classified elsewhere,” whereas \code{B950}
is the associated compact code.}

\item{subconditions}{Logical scalar; when \code{TRUE}, report both conditions and
subconditions (PCCC only).}
}
\value{
The return object will be slightly different depending on the value of
\code{method} and \code{subconditions}.
\itemize{
\item When \code{subconditions = FALSE}, a \code{medicalcoder_comorbidities} object (a
\code{data.frame} with attributes) is returned.  Column(s) for \code{id.vars}, if
defined in the function call.  For all methods there will be the following
columns:
\itemize{
\item \code{num_cmrb} a count of comorbidities/conditions flagged
\item \code{cmrb_flag} a 0/1 integer indicator for at least one
comorbidity/condition.
}

Additional columns:
\itemize{
\item PCCC methods:
\itemize{
\item For \code{method = "pccc_v2.0"} and \code{method = "pccc_v2.1"}, there is one
indicator column per condition.
\item For \code{method = "pccc_v3.0"} and \code{method = "pccc_v3.1"},
there are four columns per condition:
\itemize{
\item \verb{<condition>_dxpr_or_tech}: the condition was flagged due to the
presence of either a diagnostic or procedure code, or was flagged due to
the presence of a technology dependence code along with at least one
comorbidity being flagged by a diagnostic or procedure code.
\item \verb{<condition>_dxpr_only}: the condition was flagged due to the
presence of a non-technology dependent diagnostic or procedure code
only.
\item \verb{<condition>_tech_only}: the condition was flagged due to the
presence of a technology dependent code only and at least one other
comorbidity was flagged by a non-technology dependent code.
\item \verb{<condition>_dxpr_and_tech}: The patient had both diagnostic or
procedure codes and a technology dependence code for the condition.
}
}
\item For Charlson variants, indicator columns are returned for the relevant
conditions, \code{cci} (Charlson Comorbidity Index), and \code{age_score}.
\item For Elixhauser variants, indicator columns are returned for all relevant
comorbidities, mortality, and readmission indices.
}
\item When \code{subconditions = TRUE} and the method is a PCCC variant,
a list of length two is returned: the first element contains condition
indicators; the second element is a named list of \code{data.frame}s with
indicators for subconditions within each condition.
}
}
\description{
Apply established comorbidity algorithms to ICD-coded data. Supported
methods include several variants of the Charlson comorbidity system,
Elixhauser, and the Pediatric Complex Chronic Conditions (PCCC).
}
\details{
When \code{flag.method = "current"}, only codes from the index encounter
contribute to flags. When a longitudinal method is selected (e.g.,
\code{"cumulative"}), prior encounters for the same \code{id.vars}
combination may contribute to condition flags. For the cumulative method to
work, \code{id.vars} needs to be a character vector of length 2 or more. The last
element is treated as the encounter identifier and must be sortable. For
example, say you have data with a hospital, patient, and encounter id. The
\code{id.vars} could be one of two entries: \code{c("hospital", "patient", "encounter")}
or \code{c("patient", "hospital", "encounter")}. In both cases the return will be
the same because the encounter identifier is unchanged regardless of whether
hospital or patient is listed first.

It is critically important that the \code{data[[tail(id.vars, 1)]]} variable can
be sorted.  Just because your data is sorted in temporal order does not mean
that the results will be correct if the \code{tail(id.vars, 1)} is not in the same
order as the data.  For example, say you had the following:\tabular{ccc}{
   patid \tab enc_id \tab date \cr
   P1 \tab 10823090 \tab Aug 2023 \cr
   P1 \tab 10725138 \tab Jul 2025 \cr
}


\code{id.vars = c("patid", "enc_id")} will give the wrong result as enc_id
10725138 would be sorted to come before enc_id 10823090.  \code{id.vars = c("patid", "date")} would be sufficient input, assuming that \code{date} has been
correctly stored. Adding a column \code{enc_seq}, e.g.,\tabular{cccc}{
   patid \tab enc_id \tab date \tab enc_seq \cr
   P1 \tab 10823090 \tab Aug 2023 \tab 1 \cr
   P1 \tab 10725138 \tab Jul 2025 \tab 2 \cr
}


and calling \code{comorbidities()} with \code{id.vars = c("patid", "enc_seq")} will
have better performance than using the date and will clear up any possible
issues with non-sequential encounter ids from the source data.

\strong{Cumulative + POA defaults:}

When \code{flag.method = "cumulative"} and neither
\code{poa} nor \code{poa.var} is supplied, the first encounter for a condition is
treated as \code{poa = 0}. Subsequent encounters for that condition are flagged as
\code{poa = 1}.

When \code{flag.method = "current"} and neither \code{poa} nor \code{poa.var} is supplied,
then all codes will be considered present-on-admission.  If poa was assumed
to be 0, then in this case the only conditions that could be flagged are the
Elixhauser conditions which are poa-exempt.
}
\examples{
pccc_v3.1_results <-
  comorbidities(data = mdcr,
                icd.codes = "code",
                id.vars = "patid",
                dx.var = "dx",
                method = "pccc_v3.1",
                flag.method = 'current',
                poa = 1)
summary(pccc_v3.1_results)

pccc_v3.1_subcondition_results <-
  comorbidities(data = mdcr,
                icd.codes = "code",
                id.vars = "patid",
                dx.var = "dx",
                method = "pccc_v3.1",
                flag.method = 'current',
                poa = 1,
                subconditions = TRUE)
summary(pccc_v3.1_subcondition_results)

charlson_results <-
  comorbidities(data = mdcr,
                icd.codes = "code",
                id.vars = "patid",
                dx.var = "dx",
                method = "charlson_quan2011",
                flag.method = 'current',
                poa = 1)
summary(charlson_results)

elixhauser_results <-
  comorbidities(data = mdcr,
                icd.codes = "code",
                id.vars = "patid",
                dx.var = "dx",
                method = "elixhauser_ahrq2025",
                primarydx = 1,
                flag.method = 'current',
                poa = 1)
summary(elixhauser_results)
}
\references{
\itemize{
\item Pediatric Complex Chronic Conditions:
\itemize{
\item Feudtner, C., Feinstein, J.A., Zhong, W. et al. Pediatric complex
chronic conditions classification system version 2: updated for ICD-10
and complex medical technology dependence and transplantation. BMC
Pediatr 14, 199 (2014). https://doi.org/10.1186/1471-2431-14-199
\item Feinstein JA, Hall M, Davidson A, Feudtner C. Pediatric Complex Chronic
Condition System Version 3. JAMA Netw Open. 2024;7(7):e2420579.
https://doi.org/10.1001/jamanetworkopen.2024.20579
}
\item Charlson Comorbidities:
\itemize{
\item Mary E. Charlson, Peter Pompei, Kathy L. Ales, C.Ronald
MacKenzie, A new method of classifying prognostic comorbidity in
longitudinal studies: Development and validation, Journal of Chronic
Diseases, Volume 40, Issue 5, 1987, Pages 373-383, ISSN 0021-9681,
https://doi.org/10.1016/0021-9681(87)90171-8.
\item Deyo RA, Cherkin DC, Ciol MA. Adapting a clinical comorbidity index
for use with ICD-9-CM administrative databases. J Clin Epidemiol. 1992
Jun;45(6):613-9. https://doi.org/10.1016/0895-4356(92)90133-8. PMID: 1607900.
\item Quan H, Sundararajan V, Halfon P, Fong A, Burnand B, Luthi JC,
Saunders LD, Beck CA, Feasby TE, Ghali WA. Coding algorithms for defining
comorbidities in ICD-9-CM and ICD-10 administrative data. Med Care. 2005
Nov;43(11):1130-9. https://doi.org/10.1097/01.mlr.0000182534.19832.83. PMID:
16224307.
\item Quan H, Li B, Couris CM, Fushimi K, Graham P, Hider P, Januel JM,
Sundararajan V. Updating and validating the Charlson comorbidity index
and score for risk adjustment in hospital discharge abstracts using
data from 6 countries. Am J Epidemiol. 2011 Mar 15;173(6):676-82.
https://doi.org/10.1093/aje/kwq433. Epub 2011 Feb 17. PMID: 21330339.
\item Glasheen WP, Cordier T, Gumpina R, Haugh G, Davis J, Renda A.
Charlson Comorbidity Index: ICD-9 Update and ICD-10 Translation. Am Health
Drug Benefits. 2019 Jun-Jul;12(4):188-197. PMID: 31428236; PMCID:
PMC6684052.
}
\item Elixhauser Comorbidities:
\itemize{
\item Agency for Healthcare Research and Quality (AHRQ). Elixhauser
Comorbidity Software Refined for ICD-10-CM Diagnoses, v2026.1 [Internet].
2026. Available from:
https://www.hcup-us.ahrq.gov/toolssoftware/comorbidityicd10/comorbidity_icd10.jsp
}
}
}
\seealso{
\itemize{
\item \code{vignettes(topic = "comorbidities", package = "medicalcoder")}
\item \code{vignettes(topic = "pccc", package = "medicalcoder")}
\item \code{vignettes(topic = "charlson", package = "medicalcoder")}
\item \code{vignettes(topic = "elixhauser", package = "medicalcoder")}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_pccc_conditions.R
\name{get_pccc_conditions}
\alias{get_pccc_conditions}
\title{Pediatric Complex Chronic Condition and Subconditions}
\usage{
get_pccc_conditions()
}
\value{
a \code{data.frame} with the following columns
\itemize{
\item \code{condition}: (character) syntax valid name for the condition
\item \code{subcondition}: (character) syntax valid name for the subcondition
\item \code{condition_label}: (character) human readable label for the condition
\item \code{subcondition_label}: (character) human readable label for the
subcondition
}
}
\description{
Retrieve a copy of internal lookup tables for the syntax valid and human
readable labels of the Pediatric Complex Chronic Conditions (PCCC) conditions
and subconditions.
}
\examples{
get_pccc_conditions()

}
\seealso{
\itemize{
\item \code{\link[=get_pccc_codes]{get_pccc_codes()}} for the lookup table of ICD codes used for the PCCC.
\item \code{\link[=comorbidities]{comorbidities()}} for applying comorbidity algorithms to a data set.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_icd_codes.R
\name{get_icd_codes}
\alias{get_icd_codes}
\title{Get ICD Codes}
\usage{
get_icd_codes(with.descriptions = FALSE, with.hierarchy = FALSE)
}
\arguments{
\item{with.descriptions}{Logical scalar, if \code{TRUE} include the
description of the codes.}

\item{with.hierarchy}{Logical scalar, if \code{TRUE} include the ICD
hierarchy.}
}
\value{
a \code{data.frame}

The default return has the following columns:
\itemize{
\item \code{icdv}: Integer vector indicating if the code is from ICD-9 or ICD-10
\item \code{dx}: Integer vector.  1 if the code is a diagnostic, (ICD-9-CM, ICD-10-CM, WHO, CDC Mortality), or 0 if the code is procedural (ICD-9-PCS, ICD-10-PCS)
\item \code{full_code}: Character vector with the ICD code and any relevant decimal point
\item \code{code}: Character vector with the compact ICD code omitting any relevant decimal point
\item \code{src}: Character vector reporting the source of the information.  See Details.
\item \code{known_start}: Integer vector reporting the first known year of use.  See Details.
\item \code{known_end}: Integer vector reporting the last known year of use.  See Details.
\item \code{assignable_start}: Integer vector reporting the first known year the code was assignable.  See Details.
\item \code{assignable_end}: Integer vector reporting the last known year the code was assignable.  See Details.
}

When \code{with.descriptions = TRUE} there are the following additional columns:
\itemize{
\item \code{desc}: Character vector of descriptions.  For \code{cms} codes descriptions from CMS are used preferentially over CDC.
\item \code{desc_start}: Integer vector of the first year the description was used.
\item \code{desc_end}: Integer vector of the last year the description was used.
}

When \code{with.hierarchy = TRUE} there are the following additional columns:
\itemize{
\item \code{chapter}
\item \code{subchapter}
\item \code{category}
\item \code{subcategory}
\item \code{subclassification}
\item \code{subsubclassification}
\item \code{extension}
}
}
\description{
Retrieve a copy of the internal lookup table for all known ICD codes.
}
\details{
\subsection{Sources}{

There are three sources of ICD codes.
\itemize{
\item \code{cms}: Codes from the ICD-9-CM, ICD-9-PCS, ICD-10-CM, and ICD-10-PCS standards.
\item \code{who}: Codes from World Health Organization.
\item \code{cdc}: Codes from CDC Mortality coding standard.
}
}

\subsection{Fiscal and Calendar Years}{

When reporting years there is a mix of fiscal and calendar years.

Fiscal years are the United States Federal Government fiscal years, running
from October 1 to September 30. For example, fiscal year 2013 started October
1 2012 and ended on September 30 2013.

Calendar years run January 1 to December 31.

Within the ICD data there are columns
\code{known_start}, \code{known_end}, \code{assignable_start}, \code{assignable_end},
\code{desc_start} and \code{desc_end}.  For ICD codes with \code{src == "cms"}, these are
fiscal years.  For codes with \code{src == "cdc"} or \code{src == "who"} these are
calendar years.

\code{known_start} is the first fiscal or calendar year (depending on source) that
the medicalcoder package as definitive source data for.  ICD-9-CM started in
the United States in fiscal year 1980.  The CDC extracts included in
medicalcoder span fiscal years 1997--2012; the CMS ICD-9-CM/PCS extracts
start in fiscal year 2006 and run through fiscal year 2015.  As such 1997 is
the earliest "known start" for ICD-9 within medicalcoder.

\code{known_end} is the last fiscal or calendar year (depending on source)
for which we have definitive source data for.  For ICD-9-CM and ICD-9-PCS,
CMS provides data through fiscal year 2015, while the CDC extracts stop at
fiscal year 2012.  For ICD-10-CM and ICD-10-PCS, which are active, it is just
the last year of known data.  ICD-10 from the WHO ends in 2019.
}

\subsection{Header and Assignable Codes}{

"Assignable" indicates that the code is the most granular for the source.
Ideally codes are reported with the greatest level of detail but that is not
always the case.  Also, the greatest level of detail can differ between
sources.
Example: C86 is a header code for \code{cms} and \code{who} because codes C86.0, C86.1,
C86.2, C86.3, C86.4, C86.5, and C86.6 all exist in both standards.  No code
with a fifth digit exists in the \code{who} so all these four digit codes are
'assignable.'  In the \code{cms} standard, C86.0 was assignable through fiscal
year 2024. In fiscal year 2025 codes C86.00 and C86.01 were added making
C86.0 a header code and C86.00 and C86.01 assignable codes.
}
}
\examples{
icd_codes <- get_icd_codes()
str(icd_codes)

# Explore the change in the assignable year for C86 code between CMS and
# WHO
subset(get_icd_codes(), grepl("^C86$", full_code))
subset(get_icd_codes(), grepl("^C86\\\\.\\\\d$", full_code))
subset(get_icd_codes(), grepl("^C86\\\\.0(\\\\d|$)", full_code))

is_icd("C86", headerok = FALSE) # FALSE
is_icd("C86", headerok = TRUE)  # TRUE
is_icd("C86", headerok = TRUE, src = "cdc") # Not a CDC mortality code

lookup_icd_codes("^C86\\\\.0\\\\d*", regex = TRUE)
}
\seealso{
\code{\link[=is_icd]{is_icd()}}, \code{\link[=lookup_icd_codes]{lookup_icd_codes()}},
\code{vignette(topic = "icd", package = "medicalcoder")}
}
\concept{icd tools}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{mdcr_longitudinal}
\alias{mdcr_longitudinal}
\title{Synthetic Longitudinal Data}
\format{
\code{mdcr_longitudinal} is a \code{data.frame} with 4 columns.  The codes are
expected to be treated as diagnostic codes. Warning: there are a few ICD-9
codes which could match to procedure codes.
\itemize{
\item \code{patid}: patient identifier, integer values
\item \code{date}: date the diagnostic code was recorded
\item \code{icdv}: ICD version 9 or 10, integer valued
\item \code{code}: ICD codes; character values
}
}
\usage{
mdcr_longitudinal
}
\description{
Synthetic Longitudinal Data
}
\seealso{
Other datasets:
\code{\link{mdcr}}
}
\concept{datasets}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_elixhauser_poa.R
\name{get_elixhauser_poa}
\alias{get_elixhauser_poa}
\title{Get Elixhauser Present-on-Admission Requirements}
\usage{
get_elixhauser_poa()
}
\value{
A \code{data.frame} with the following columns:
\itemize{
\item \code{condition}: Character vector of the conditions
\item \code{desc}: Character vector with a verbose description of the condition
\item \code{poa_required}: Integer indicators if the code needs to present on admission to be considered a comorbidity
\item \verb{elixhauser_<variant>}: indicators for the Elixhauser \verb{<variant>}
}
}
\description{
Retrieve a copy of internal lookup table with details on which Elixhauser
comorbidities do and do not require the associated ICD codes to be
present-on-admission to be flagged.
}
\examples{
head(get_elixhauser_poa())
str(get_elixhauser_poa())

}
\seealso{
\itemize{
\item \code{\link[=get_elixhauser_index_scores]{get_elixhauser_index_scores()}} for the lookup table of the condition by
condition scores for mortality and readmission indices.
\item \code{\link[=get_elixhauser_codes]{get_elixhauser_codes()}} for the lookup table of ICD codes mapping to the
Elixhauser comorbidities.
\item \code{\link[=comorbidities]{comorbidities()}} for applying comorbidity algorithms to a data set.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/icd_compact_to_full.R
\name{icd_compact_to_full}
\alias{icd_compact_to_full}
\title{Convert ICD Compact Codes to Full Codes}
\usage{
icd_compact_to_full(x, icdv, dx)
}
\arguments{
\item{x}{Character vector}

\item{icdv}{Integer vector of allowed ICD versions. Use \code{9L} and/or
\code{10L}. Defaults to both.}

\item{dx}{Integer vector indicating allowed code type(s): \code{1L} for
diagnostic (ICD-9-CM, ICD-10-CM, CDC mortality, WHO), \code{0L} for procedural
(ICD-9-PCS, ICD-10-PCS). Defaults to both.}
}
\value{
A character vector the same length as \code{x}.
}
\description{
Take an assumed ICD compact code string and convert to a full code based on
the ICD version (9 or 10) and type (diagnostic or procedure).  This method
only formats strings and does not validate the code(s).
}
\seealso{
\itemize{
\item \code{\link[=get_icd_codes]{get_icd_codes()}} to retrieve the internal lookup table of ICD codes.
\item \code{\link[=lookup_icd_codes]{lookup_icd_codes()}} for retrieving details on a specific set of ICD
codes.
\item \code{\link[=is_icd]{is_icd()}} to test if a string is a known ICD code.
}

Other ICD tools:
\code{\link[=is_icd]{is_icd()}},
\code{\link[=lookup_icd_codes]{lookup_icd_codes()}}
}
\concept{ICD tools}
